
local Badge = require "widgets/badge"
local TpComposed = require "screens/tp_composed_panel"
local TpUpdate = require "screens/tp_update_panel"
local Text = require "widgets/text"
local TpLevel = require "screens/tp_level_panel"
local TpCheck = require "screens/tp_check_panel"
local TpTeach = require "screens/tp_teach_panel"
local TpSale = require "screens/tp_sale_panel"
local TpResource = require "screens/tp_resource_panel"
local TpBuffBar = require "widgets/tp_buff_bar"
local UIAnim = require "widgets/uianim"
local ImageButton = require "widgets/imagebutton"
local Image = require "widgets/image"
local SpecialSlot = require "widgets/specialslot"


local MadBadge = Class(Badge, function(self, owner)
	Badge._ctor(self, "beaver_meter", owner)
end)

local EnergyBadge = Class(Badge, function(self, owner)
	Badge._ctor(self, "livingartifact_meter", owner)
end)


local function resolve_img_path(img)
	local atlas = "images/inventoryimages"
	if string.find(img, "#") then
		img = string.sub(img, 1, -2)
		atlas = atlas.."/"..img
	elseif img > "torch" then
		atlas = atlas.."_2"
	end
	return atlas..".xml", img..".tex"
end

AddClassPostConstruct("widgets/statusdisplays", function(self)




	local function add_word(ui, txt)
        ui.word = ui:AddChild(Text(TITLEFONT, 20))
        ui.word:SetPosition(0, -30, 0)
        ui.word:SetString(txt)
	end
	local function get_position(start, pad, len, idx)
		local x, y = start[1], start[2]
		local pad_x, pad_y = pad[1], pad[2]
		local pos_x = x + (idx%len)*pad_x
		local pos_y = y + math.floor(idx/len)*pad_y
		return pos_x, pos_y
	end
	local function set_position(ui, x, y, z)
		local scale = WARGON.get_screen_size()
		ui:SetPosition(x*scale, y*scale, z)
	end
	if self.owner.prefab ~= nil then
		self.madvalue = self:AddChild(MadBadge(self.owner))

	    self.madvalue:SetPosition(-80, -115, 0)
	    self.madvalue:SetPercent(self.owner.components.tpmadvalue:GetPercent(), 
	    	self.owner.components.tpmadvalue.max)
	    self.inst:ListenForEvent("tp_madvalue_delta", function(inst, data)
	    	self:MadValueDelta(data)
		end, self.owner)
		
	    function self:MadValueDelta(data)
	    	self.madvalue:SetPercent(data.new_per, self.owner.components.tpmadvalue.max) 
			if not data.no_flash then
				if data.new_per > data.old_per then
					self.madvalue:PulseGreen()
					TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/sanity_up")
				elseif data.new_per < data.old_per then
					TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/sanity_down")
					self.madvalue:PulseRed()
				end
			end
		end

		self.energy = self:AddChild(EnergyBadge(self.owner))
		self.energy:SetPosition(-150, -115, 0)

		self.energy:Hide()
		local vehicle = self.owner.components.tpvehicleowner and self.owner.components.tpvehicleowner.vehicle
		if vehicle and vehicle.components.tpenergy then
			self.energy:SetPercent(vehicle.components.tpenergy:GetPercent(),
				vehicle.components.tpenergy.max)

			self.energy:Show()
		end
		self.inst:ListenForEvent("tp_vehicle_energy_delta", function(inst, data)
			self:EnergyDelta(data)
		end, self.owner)
		self.inst:ListenForEvent("tp_up_car", function(inst, data)
			if data and data.vehicle
			and data.vehicle.components.tpenergy then
				self.energy:Show()
			end
		end, self.owner)
		self.inst:ListenForEvent("tp_down_car", function(inst, data)
			self.energy:Hide()
		end, self.owner)

		function self:EnergyDelta(data)
			self.energy:SetPercent(data.new_p, data.max)
		end

		self.tp_version_txt = self:AddChild(Text(TITLEFONT, 30))
		self.tp_version_txt:SetString(WARGON.version)	
		self.tp_version_txt:SetPosition(-180, 0, 0)










		local state_imgs = {
			v = {"minimap/minimap_data.xml", "wickerbottom.png"},
			m = {"minimap/minimap_data.xml", "wilson.png"},
			s = {"minimap/minimap_data.xml", "woodlegs.png"},
			f = {"minimap/minimap_data.xml", "wathgrithr.png"},
			i = {"minimap/minimap_data.xml", "wendy.png"},
			b = {"minimap/minimap_data.xml", "waxwell.png"},
			w = {"images/map_icons/esctemplate.xml", "esctemplate.tex"},
		}








		self.tp_state_img = self:AddChild(Image(
			unpack(state_imgs.v)
		))
		self.tp_state_img:Hide()
		if self.owner.components.sciencemorph then
			local cur = self.owner.components.sciencemorph.cur
			self.tp_state_img:SetTexture(unpack(state_imgs[cur]))
		end
		self.tp_state_img:SetPosition(-180, -40, 0)

		self.inst:ListenForEvent("tp_morph", function(inst, data)
			if data and data.cur then

				self.tp_state_img:SetTexture(unpack(state_imgs[data.cur]))
			end
		end, self.owner)


		self.icons = {}
		local icons = {
			{
				img = "book_brimstone",
				txt = "更新公告",
				fn = function()
					TheFrontEnd:PushScreen(TpUpdate())
				end
			},
			{
				img = "book_meteor",
				txt = "合成图鉴",
				fn = function()
					TheFrontEnd:PushScreen(TpComposed())
				end,
			},
			{
				img = "book_gardening",
				txt = "等级提升",
				fn = function()
					if c_find("tp_train_center") then
						TheFrontEnd:PushScreen(TpLevel())
					else
						GetPlayer().components.talker:Say("需要建造一个训练中心")
					end
				end,
			},
			{
				img = "book_birds",
				txt = "搜索世界",
				fn = function()
					TheFrontEnd:PushScreen(TpCheck())
				end,
			},
			{
				img = "book_sleep",
				txt = "帮助信息",
				fn = function()
					TheFrontEnd:PushScreen(TpTeach())
				end,
			},
			{
				img = "book_tentacles",
				txt = "充值商城",
				fn = function()


					if c_find("tp_shop") then
						TheFrontEnd:PushScreen(TpSale())
					else
						GetPlayer().components.talker:Say("需要建造一个自动售货机")
					end



				end,
			},
			{
				img ={"tp_structures_2", "tp_resource_transporter"},
				txt = "资源中心",
				fn = function()
					local player = GetPlayer()
					local center = c_find("tp_resource_transporter")
					if center then
						if center.components.container.open then
							center.components.container:Close(player)
						else
							center.components.container:Open(player)
						end



					end
				end
			},
		}
		for k, v in pairs(icons) do
			local wgimg = WgImg(v.img)
			self.icons[k] = self:AddChild(ImageButton(

				wgimg.atlas, wgimg.img
			))
			local pos_x, pos_y = get_position(
				{-260-70, 140}, {70, -70}, 3, k-1
			)
			self.icons[k]:SetPosition(pos_x, pos_y, 0)

			self.icons[k]:SetOnClick(function()
				local click_fn = v.fn
				click_fn()
			end)
			add_word(self.icons[k], v.txt)
		end
		

		self.skills = {}
		local skills = {






















































		}
		for k, v in pairs(skills) do
			local wgimg = WgImg(v.img)
			self.skills[k] = self:AddChild(ImageButton(

				wgimg.atlas, wgimg.img
			))
			local pos_x, pos_y = get_position(
				{-1205, -580}, {70, 70}, 8, k-1
			)
			self.skills[k]:SetPosition(pos_x, pos_y, 0)
			set_position(self.skills[k], pos_x, pos_y, 0)
			self.skills[k]:SetOnClick(function()
				local click_fn = v.fn
				click_fn()
			end)
			add_word(self.skills[k], v.txt)
		end
	end
end)

AddClassPostConstruct("widgets/itemtile", function(self)

	if self.item.components.tprecharge then
		self.tp_recharge_frame = self:AddChild(UIAnim())
		self.tp_recharge_frame:GetAnimState():SetBank("recharge_meter_wargon")
		self.tp_recharge_frame:GetAnimState():SetBuild("recharge_meter_wargon")
		self.tp_recharge_frame:GetAnimState():PlayAnimation("frame")
		self.tp_recharge_frame:Hide()

		self.tp_recharge = self:AddChild(UIAnim())
		self.tp_recharge:GetAnimState():SetBank("recharge_meter_wargon")
		self.tp_recharge:GetAnimState():SetBuild("recharge_meter_wargon")
		self.tp_recharge:SetClickable(false)

		self.inst:ListenForEvent("tp_recharge_change", function(item, data)
			self:SetTpRechargePercent(data.per)
		end, self.item)
	end
	function self:SetTpRechargePercent(p)
		if p < 1 then
			self.tp_recharge:GetAnimState():SetPercent("recharge", p)
			local owner = self.item.components.inventoryitem:GetGrandOwner()
			if self.item and owner.components.inventory 
			and not self.item == owner.components.inventory.activeitem then
				self.tp_recharge_frame:Show()
			end
		else
			if not self.tp_recharge:GetAnimState():IsCurrentAnimation("frame_pst") then
				self.tp_recharge:GetAnimState():PlayAnimation("frame_pst")
			end


			self.tp_recharge_frame:Hide()
		end
	end

	if self.item.components.tpplantable
	and self.item.components.tpplantable.item_img then
		self.tp_info_image = self:AddChild(
			Image(
				self.item.components.tpplantable:GetImage()
			)
		)
		self.tp_info_image:SetScale(.5, .5, .5)
	end
	if self.item.components.tppotion
	and self.item.components.tppotion.item_img then
		self.tp_info_image = self:AddChild(
			Image(
				self.item.components.tppotion:GetImage()
			)
		)
		self.tp_info_image:SetScale(.5, .5, .5)
	end
	local old_fn = self.StartDrag
	function self:StartDrag(...)
		old_fn(self, ...)
		if self.tp_recharge then
			self.tp_recharge:Hide()
		end
		if self.tp_recharge_frame then
			self.tp_recharge_frame:Hide()
		end
		if self.tp_info_image then
			self.tp_info_image:Hide()
		end
	end

	if self.item.components.tpbullets then
		self:SetQuantity(self.item.components.tpbullets:GetNum())
		self.inst:ListenForEvent("tp_bullet_change", function(item, data)
			self:SetQuantity(self.item.components.tpbullets:GetNum())
			self:ScaleTo(self.basescale*2, self.basescale, .25)
		end, self.item)
	end
end)


AddClassPostConstruct('widgets/recipepopup', function(self)
	local old_refresh = self.Refresh
	function self:Refresh()
		old_refresh(self)
		if self.recipe then
			for k, v in pairs(self.recipe.ingredients) do
				if v.image then
					local ing = self.ing[k]
					ing.ing:SetTexture(v.atlas, v.image)
				end
			end
		end
	end
end)

AddClassPostConstruct("brains/leifbrain", function(self)
	require "behaviours/chaseandattack"
	require "behaviours/runaway"
	require "behaviours/wander"
	require "behaviours/doaction"
	require "behaviours/attackwall"
	require "behaviours/follow"

	local MIN_FOLLOW_DIST = 2
	local TARGET_FOLLOW_DIST = 5
	local MAX_FOLLOW_DIST = 9

	local function GetLeader(inst)
	    return inst.components.follower and inst.components.follower.leader 
	end

	function self:OnStart()

	    local clock = GetClock()

	    local root =
	        PriorityNode(
	        {
	            AttackWall(self.inst),
	            ChaseAndAttack(self.inst),
	            Follow(self.inst, GetLeader, MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
	            Wander(self.inst)            
	        },1)
	    
	    self.bt = BT(self.inst, root)
	end
end)

AddClassPostConstruct("screens/playerhud", function(self)
	local old_update_leaves = self.UpdateLeaves
	function self:UpdateLeaves(dt)
		if WARGON.is_dlc(3) then
			old_update_leaves(self, dt)
		else
			if self.leavesTop then
			    if not self.leavestop_intensity then
			    	self.leavestop_intensity = 0
			    end	 
				local player = GetPlayer()
				if player:HasTag("tp_spear_wind") then
					self.leavestop_intensity = math.min(1,self.leavestop_intensity+(1/30) )
				else			
				 	self.leavestop_intensity = math.max(0,self.leavestop_intensity-(1/30) )
				end	

				if self.leavestop_intensity == 0 then
			    	self.leavesTop:Hide()
			    else
			    	self.leavesTop:Show()
					if self.leavestop_intensity == 1 then
				    	if not self.leavesfullyin then
				    		self.leavesTop:GetAnimState():PlayAnimation("idle", true)	
				    		self.leavesfullyin = true

				    	else	
					    	if GetPlayer().sg:HasStateTag("moving") then
					    		if not self.leavesmoving then
					    			self.leavesmoving = true
					    			self.leavesTop:GetAnimState():PlayAnimation("run_pre")	
					    			self.leavesTop:GetAnimState():PushAnimation("run_loop", true)					    					    	
					    		end
					    	else
					    		if self.leavesmoving then
					    			self.leavesmoving = nil
					    			self.leavesTop:GetAnimState():PlayAnimation("run_pst")	
					    			self.leavesTop:GetAnimState():PushAnimation("idle", true)	
					    			self.leaves_olddir = nil
					    		end
					    	end
				    	end
				    else
				    	self.leavesfullyin = nil
				    	self.leavesmoving = nil
				    	self.leavesTop:GetAnimState():SetPercent("zoom_in", self.leavestop_intensity)
					end	    	
			    end	    
			end
		end
	end
end)

AddGlobalClassPostConstruct('entityscript', 'EntityScript', function(self)
	self.tp_tag_num = {}
	function self:AddTagNum(tag, n)
		if self.tp_tag_num[tag] == nil then
			self.tp_tag_num[tag] = 0
		end
		self.tp_tag_num[tag] = math.max(0, self.tp_tag_num[tag]+n)
		if self.tp_tag_num[tag] > 0 then
			self:AddTag(tag)
			print("entity add tag", tag)
		else
			self:RemoveTag(tag)
			print("entity rm tag", tag)
		end
	end
end)

AddClassPostConstruct("widgets/hoverer",function(self)
    local old_SetString = self.text.SetString
    self.text.SetString = function(text,str)
        local target = TheInput:GetHUDEntityUnderMouse()
        if target ~= nil then
            target = target.widget ~= nil and target.widget.parent ~= nil and target.widget.parent.item
        else
            target = TheInput:GetWorldEntityUnderMouse()
        end
        if target and target.entity ~= nil and target.prefab ~= nil then
        	if target.components.tpenchantment then
        		local attr = target.components.tpenchantment.attribute
	        	local word = STRINGS.TP_STR.tp_enchantment
        		for k, v in pairs(attr) do
        			str = v > 0 and str.."\n"..word[k]..":"..v or str

        		end
            end
           	if target.components.tppowerfruit
           	and target.components.tppowerfruit.enable
           	and target.components.tppowerfruit.power
           	and target.components.tppowerfruit.quality then
           		local name = target.components.tppowerfruit:GetName()
           		local quality = target.components.tppowerfruit:GetQuality()
	           	local word = STRINGS.TP_STR.tp_fruit_quality

	           	local txt = word[quality+1]
				str = str.."\n"..name..": "..txt
           	end
           	if target.components.tppower
           	and target.components.tppower.enable
           	and target.components.tppower.power then
	           	local word = STRINGS.TP_STR.tp_element_power
	           	local txt = word[target.components.tppower.power]
	           	str = str.."\n"..txt
           	end
           	if target.components.tppowernew
           	and target.components.tppowernew.enable
           	and target.components.tppowernew.power then
          		local name = target.components.tppowernew:GetName()
          		str = str.."\n"..name
          	end
          	if target.components.tppowertree
          	and not target:HasTag("stump")
          	and target.components.tppowertree.enable
          	and target.components.tppowertree.power then
          		local name = target.components.tppowertree:GetName()
          		str = str.."\n"..name
          	end
          	if target.components.tppowerboss then
          		local name = target.components.tppowerboss:GetName()
          		str = str.."\n"..name
          	end
          	if target.components.tpenergy then
          		local max = target.components.tpenergy.max
          		local cur = target.components.tpenergy.current
          		local txt = string.format("%.0d/%d", cur, max)
          		str = str.."\n"..txt
          	end
			if target:HasTag("tp_generator") and target.components.tpvalue then
				local max = target.components.tpvalue.max
          		local cur = target.components.tpvalue.current
				local txt = string.format("%.0d/%d", cur, max)
				str = str.."\n"..txt
			end
			if target.components.tpelectricity then
				local max = target.components.tpelectricity.max
          		local cur = target.components.tpelectricity.current
				local txt = string.format("%.0d/%d", cur, max)
				str = str.."\n"..txt
			end
			if target.components.tpbattery then
				local max = target.components.tpbattery.max
          		local cur = target.components.tpbattery.current
				local txt = string.format("%.0d/%d", cur, max)
				str = str.."\n"..txt
			end
			if target.components.tpsword then
				local s = target.components.tpsword:GetWargonString()
				str = str.."\n"..s
			end
			if target.components.tpornament then
				local s = target.components.tpornament:GetWargonString()
				str = str.."\n"..s
			end
    	end
        old_SetString(text,str)
    end
end)

AddClassPostConstruct("widgets/inventorybar", function(self)
	local W = 68
	local SEP = 12
	local YSEP = 8


	self.buff_bar = self.root:AddChild(TpBuffBar())

	self.buff_bar:SetPosition(0, 220, 0)
	self.buff_bar:SetScale(2, 2, 2)
	self.inst:ListenForEvent("tp_add_buff", function(inst, data)
		if data.name then
			local buff_shelf = WARGON.DATA.tp_data_buff
			local buff = buff_shelf:GetBuff(data.name)
			if buff.img then
				self.buff_bar:Add(buff, data.max_time)
			end
		end
	end, self.owner)
	self.inst:ListenForEvent("tp_done_buff", function(inst, data)
		self.buff_bar:Delete(data.buff)
	end, self.owner)

	local buff_shelf = WARGON.DATA.tp_data_buff
	for k, v in pairs(self.owner.components.tpbuff.buff) do
		local buff_name = k
		local time = v.left - GetTime()
		local buff = buff_shelf:GetBuff(buff_name)
		if buff.img then
			local buff_slot = self.buff_bar:Add(buff)
			buff_slot:SetTime(buff.time)
		end
	end

	self.tp_equip_uis = {}

	local equip_uis = require "datas/equip_skill"
	local equip_ui_pos = {
		hand = 455,
		body = 555,
		head = 655,
		hand2= 455,
		body2= 555,
		head2= 655,
		hand3= 455,
		body3= 555,
		head3= 655,
	}
	local equip_ui_y = W/2+YSEP/2+120
	local equip_y_cnt = 0
	for k, v in pairs(equip_ui_pos) do
		self.tp_equip_uis[k] = self.root:AddChild(ImageButton(
			"images/inventoryimages.xml", "ash.tex"
		))
		local dt_y = equip_y_cnt%3*80
		self.tp_equip_uis[k]:SetPosition(v, equip_ui_y+dt_y, 0)
		self.tp_equip_uis[k]:Hide()
		self.tp_equip_uis[k].word = self.tp_equip_uis[k]:AddChild(Text(TITLEFONT, 30))
		self.tp_equip_uis[k].word:SetPosition(0, -30, 0)
		equip_y_cnt = equip_y_cnt+1
	end

	local equip_skill_slot = {
		hands = "hand",
		head = "head",
		body = "body",
	}
	local function show_equip_ui(item)
		if equip_uis[item.prefab] then
			local info = equip_uis[item.prefab]
			local slot = "hand"
			if item.components.equippable then





				slot = equip_skill_slot[item.components.equippable.equipslot]
			end

			local btns = {self.tp_equip_uis[slot]}
			if info.fn2 then
				btns[2] = self.tp_equip_uis[slot.."2"]
			end
			if info.fn3 then
				btns[3] = self.tp_equip_uis[slot.."3"]
			end

			local atlas = item.components.inventoryitem:GetAtlas()
			local img = item.components.inventoryitem:GetImage()




			for k, btn in pairs(btns) do
				btn:SetTextures(

					atlas, img
				)
				local btn_fn = info.fn
				local btn_txt = info.txt
				if k == 2 then
					btn_fn = info.fn2
					btn_txt = info.txt2
				elseif k == 3 then
					btn_fn = info.fn3
					btn_txt = info.txt3
				end
				btn:SetOnClick(function()
					btn_fn(item, self.owner)
				end)
				btn.word:SetString(btn_txt)
				btn:Show()
			end
		end
	end
	local function hide_equip_ui(item)
		if equip_uis[item.prefab] then
			local info = equip_uis[item.prefab]
			local slot = "hand"
			if item.components.equippable then





				slot = equip_skill_slot[item.components.equippable.equipslot]
			end
			local btn = self.tp_equip_uis[slot]

			btn:Hide()
			local btn2 = self.tp_equip_uis[slot.."2"]
			btn2:Hide()
			local btn3 = self.tp_equip_uis[slot.."3"]
			btn3:Hide()
		end
	end

	local equip_slots = {"head", "hand", "body"}
	for k, v in pairs(equip_slots) do
		local equip = self.owner:get_equip_item(v)
		if equip then
			show_equip_ui(equip)
		end
	end

	self.inst:ListenForEvent("equip", function(inst, data)
		if data and data.item then
			show_equip_ui(data.item)
		end
	end, self.owner)
	self.inst:ListenForEvent("unequip", function(inst, data)
		if data and data.item then
			hide_equip_ui(data.item)
		end
	end, self.owner)

	self.tp_morph_uis = {}
	local morph_uis = {
		blood_prince = {
			img = "pig_shop_arcane",
			txt = "夜视",
			fn = function()


				if not self.owner:HasTag("shadow_state_night_eye") then



					self.owner:AddTag("shadow_state_night_eye")
					self.owner:add_hunger_rate("shadow_state_night_eye", .5)
					self.owner.Light:Enable(true)
					self.owner:RemoveTag("lightsource")
					GetWorld().components.colourcubemanager:SetOverrideColourCube("images/colour_cubes/beaver_vision_cc.tex")


				else


					self.owner:RemoveTag("shadow_state_night_eye")
					self.owner:rm_hunger_rate("shadow_state_night_eye")
					GetWorld().components.colourcubemanager:SetOverrideColourCube(nil)
					self.owner.Light:Enable(false)

				end
			end,
		}
	}
	for k, v in pairs(morph_uis) do
		self.tp_morph_uis[k] = self.root:AddChild(ImageButton(
			resolve_img_path(v.img)
		))
		self.tp_morph_uis[k]:SetPosition(755, W/2+YSEP/2+120, 0)
		self.tp_morph_uis[k]:SetOnClick(function()
			v.fn()
		end)
		self.tp_morph_uis[k].word = self.tp_morph_uis[k]:AddChild(Text(TITLEFONT, 30))
		self.tp_morph_uis[k].word:SetPosition(0, -30, 0)
		self.tp_morph_uis[k].word:SetString(v.txt)
		self.tp_morph_uis[k]:Hide()
	end
	self.inst:ListenForEvent("tp_morph_ui_show", function(inst, data)
		if data and data.ui then
			self.tp_morph_uis[data.ui]:Show()
		end
	end, self.owner)
	self.inst:ListenForEvent("tp_morph_ui_hide", function(inst, data)
		if data and data.ui then
			self.tp_morph_uis[data.ui]:Hide()
		end
	end, self.owner)

	self.inst:ListenForEvent("mountboat", function(inst, data)
		for k, v in pairs(self.tp_equip_uis) do
			local pos = v:GetPosition()
			v:SetPosition(pos.x, pos.y+80, pos.z)
		end
		for k, v in pairs(self.tp_morph_uis) do
			local pos = v:GetPosition()
			v:SetPosition(pos.x, pos.y+80, pos.z)
		end
	end, self.owner)
	self.inst:ListenForEvent("dismountboat", function(inst, data)
		for k, v in pairs(self.tp_equip_uis) do
			local pos = v:GetPosition()
			v:SetPosition(pos.x, pos.y-80, pos.z)
		end
		for k, v in pairs(self.tp_morph_uis) do
			local pos = v:GetPosition()
			v:SetPosition(pos.x, pos.y-80, pos.z)
		end
	end, self.owner)


	self.car_uis = {}
	local car_ui_count = 0
	for i = 0, 2 do
		for j = 0, 3 do
			car_ui_count = car_ui_count + 1
			self.car_uis[car_ui_count] = self:AddChild(ImageButton(
				resolve_img_path("ash")
			))
			self.car_uis[car_ui_count]:SetPosition(-455-100*j, W/2+YSEP/2+200+i*100, 0)
			self.car_uis[car_ui_count].word = self.car_uis[car_ui_count]:AddChild(Text(TITLEFONT, 30))
			self.car_uis[car_ui_count].word:SetPosition(0, -30, 0)
			self.car_uis[car_ui_count]:Hide()
		end
	end
	local car_skills = require "datas/car_skill"
	local function show_car_ui(vehicle)
		local skills = car_skills[vehicle.prefab] or {}
		for k, v in pairs(skills) do
			self.car_uis[k]:SetTextures(
				resolve_img_path(v.img)
			)
			self.car_uis[k]:SetOnClick(function()
				v.fn(vehicle, self.owner)
			end)
			self.car_uis[k].word:SetString(v.txt)
			self.car_uis[k]:Show()
		end
	end
	self.inst:ListenForEvent("tp_up_car", function(inst, data)
		if data and data.vehicle then











			show_car_ui(data.vehicle)
		end
	end, self.owner)
	self.inst:ListenForEvent("tp_down_car", function(inst, data)
		for k, v in pairs(self.car_uis) do
			v:Hide()
		end
	end, self.owner)

	if self.owner.components.tpvehicleowner then
		local vehicle = self.owner.components.tpvehicleowner.vehicle 
		if vehicle then
			show_car_ui(vehicle)
		end
	end






























end)

AddGlobalClassPostConstruct('entityscript', 'EntityScript', function(self)
	local get_display_name = self.GetDisplayName
	function self:GetDisplayName()
		local name = get_display_name(self)
		if self:HasTag("tp_forge_weapon") then

			name = STRINGS.TP_STR.tp_forge_weapon..name
		end
		if self:HasTag("tp_giant") then

			name = STRINGS.TP_STR.tp_giant..name
		end
		return name
	end
end)

















AddClassPostConstruct("widgets/iceover", function(self)
	local on_update = self.OnUpdate
	function self:OnUpdate(dt)
		if self.owner and self.owner:HasTag("tp_not_cold_damage") then
			self:Hide()
		else
			on_update(self, dt)
		end
	end
end)

AddClassPostConstruct("widgets/heatover", function(self)
	local on_update = self.OnUpdate
	function self:OnUpdate(dt)
		if self.owner and self.owner:HasTag("tp_not_hot_damage") then
			self:Hide()
		else
			on_update(self, dt)
		end
	end
end)

AddClassPostConstruct("widgets/crafttabs", function(self)
    self.HandleMultiCraftingStationTabs = function(self,valid_tabs)

		local tabcounts = {}
		local multitab 
		for i,v in pairs(self.tabbyfilter) do
			if v ~= self.multitab then
				local isValid = valid_tabs[v]
				if isValid then
					local index = i.sort
					tabcounts[index] = tabcounts[index] or {}
					table.insert(tabcounts[index],{i,v})
				end
			end
		end


		for i,v in pairs(tabcounts) do
			if #v > 1 then

				if self.multitab then


					for i,v in pairs(v) do   
						valid_tabs[v[2]] = false
					end

					valid_tabs[self.multitab] = true


					self.multitab_crafting_stations = {}


					table.sort(v, function(a, b) 
										local tabname_a = a[1].str
										local tabname_b = b[1].str
										local tabdef_a = RECIPETABS[tabname_a]
										local tabdef_b = RECIPETABS[tabname_b]
										return tabdef_a.priority < tabdef_b.priority 
									end)
					
					local highlighted = false
					local alternatehighlighted = false
					local overlayshow = false

					for i,v in pairs(v) do

						local tabdef = v[1]
						local actualTab = v[2]

						local name = tabdef.str
						local tabname = name
						local imagename = actualTab.icon

						if imagename:endsWith(".tex") then
							imagename = imagename:left(-4)
						end
						local tooltip = STRINGS.TABS[string.upper(name)]
						local category = RecipeCategory(tabname, RECIPETABS[name], RECIPETABS.CRAFTINGSTATIONS, TECH.NONE, RECIPE_GAME_TYPE.COMMON, imagename, tooltip)

						category.atlas = actualTab.icon_atlas or actualTab.atlas
						category.imageScale = 0.5
						category.imageNudge = -6
						category.skipCategoryCheck = true

						self.multitab_crafting_stations[tabname] = true

						category.alternatehighlighted = actualTab.alternatehighlighted
						alternatehighlighted = alternatehighlighted or actualTab.alternatehighlighted

						category.highlighted = actualTab.highlighted
						highlighted = highlighted or actualTab.highlighted

						category.overlayshow = actualTab.overlayshow
						overlayshow = overlayshow or actualTab.overlayshow
					end

					if highlighted or alternatehighlighted then
						if highlighted then
							self.multitab:Highlight(1)
						else
							self.multitab:UnHighlight(true)
							self.multitab:AlternateHighlight(1)
						end
					else
						self.multitab:UnHighlight()
					end
					if overlayshow then
						self.multitab:Overlay()
					else
						self.multitab:HideOverlay()
					end

					local recipes = GetAllRecipes(true)
				end
			end
		end
		

	end
end)

AddClassPostConstruct("widgets/containerwidget", function(self)
	local OnUpdate = self.OnUpdate
	function self:OnUpdate(...)
		if self.container:HasTag("tp_not_auto_close") then
		else
			OnUpdate(self, ...)
		end
	end
end)

AddClassPostConstruct("screens/mainscreen", function(self)
end)