local function tp_potion_shadow_on_attack(inst, data)
	if inst.components.tpbuff then
		inst.components.tpbuff:DoneBuff("tp_potion_shadow")
	end
end

local function create_speech(inst, str, colour)
	local fx = SpawnPrefab("tp_fx_speech")
	fx.Transform:SetPosition(0,0,0)
	fx.tp_str = str
	fx.tp_colour = {x=colour[1], y=colour[2], z=colour[3]}
	inst:AddChild(fx)
end

local function nightstick_attacked(inst, data)
    if inst:HasTag("tp_nightstick_5") and data and data.attacker then
        if WARGON.can_thorns(nil, data) then
            WARGON.get_attacked(inst, data.attacker, 20, nil, "throns")
        end
    end
end

local buffs = {
	tp_ballhat = {
		time = 30,
		add = function(inst, cmp)
			inst.components.combat:AddDamageModifier("tp_ballhat", .2)
			if cmp.fxs["tp_ballhat"] == nil then

				cmp.fxs["tp_ballhat"] = WARGON.make_fx2("alive_sparklefx", inst)
				inst:AddChild(cmp.fxs["tp_ballhat"])
			end
		end,
		rm = function(inst, cmp)
			inst.components.combat:RemoveDamageModifier("tp_ballhat")
			if cmp.fxs["tp_ballhat"] then

				cmp.fxs["tp_ballhat"]:Recycle()
				cmp.fxs["tp_ballhat"] = nil
			end
		end,

		img = "footballhat",
		desc = "增加百分之二十的攻击力",
	},
	scroll_pig_armorex = {
		time = 80,
		add = function(inst, cmp)
			inst.components.health:AddAbsorbModifier("scroll_pig_armorex", .8)

			if cmp.fxs["scroll_pig_armorex"] == nil then
				cmp.fxs["scroll_pig_armorex"] = SpawnPrefab('tp_fx_scroll_pig_buff')
				cmp.fxs["scroll_pig_armorex"].fx_name = "tp_fx_scroll_pig_armorex"
				inst:AddChild(cmp.fxs["scroll_pig_armorex"])
				cmp.fxs["scroll_pig_armorex"].Transform:SetPosition(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			inst.components.health:RemoveAbsorbModifier("scroll_pig_armorex")

			if cmp.fxs["scroll_pig_armorex"] then
				cmp.fxs["scroll_pig_armorex"]:Remove()
				cmp.fxs["scroll_pig_armorex"] = nil
			end
		end,
	},
	scroll_pig_damage = {
		time = 80,
		add = function(inst, cmp)
			inst.components.combat:AddDamageModifier('scroll_pig_damage', 1)
			if cmp.fxs["scroll_pig_damage"] == nil then
				cmp.fxs["scroll_pig_damage"] = SpawnPrefab('tp_fx_scroll_pig_buff')
				cmp.fxs["scroll_pig_damage"].fx_name = 'tp_fx_scroll_pig_damage'
				inst:AddChild(cmp.fxs["scroll_pig_damage"])
				cmp.fxs["scroll_pig_damage"].Transform:SetPosition(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			inst.components.combat:RemoveDamageModifier('scroll_pig_damage')
			if cmp.fxs["scroll_pig_damage"] then
				cmp.fxs["scroll_pig_damage"]:Remove()
				cmp.fxs["scroll_pig_damage"] = nil
			end
		end,
	},
	scroll_pig_speed = {
		time = 80,
		add = function(inst, cmp)
			inst.components.locomotor:AddSpeedModifier_Mult('scroll_pig_speed', .5)
			if cmp.fxs["scroll_pig_speed"] == nil then
				cmp.fxs["scroll_pig_speed"] = SpawnPrefab('tp_fx_scroll_pig_buff')
				cmp.fxs["scroll_pig_speed"].fx_name = 'tp_fx_scroll_pig_speed'
				inst:AddChild(cmp.fxs["scroll_pig_speed"])
				cmp.fxs["scroll_pig_speed"].Transform:SetPosition(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			inst.components.locomotor:RemoveSpeedModifier_Mult('scroll_pig_speed')
			if cmp.fxs["scroll_pig_speed"] then
				cmp.fxs["scroll_pig_speed"]:Remove()
				cmp.fxs["scroll_pig_speed"] = nil
			end
		end,
	},
	scroll_pig_heal = {
		time = 80,
		add = function(inst, cmp)
			inst.components.health:DoDelta(500)

			if cmp.tasks["scroll_pig_heal"] then
				cmp.tasks["scroll_pig_heal"] = inst:per_task(1, function()
					inst.components.health:DoDelta(50)
				end)
			end
			if cmp.fxs["scroll_pig_heal"] == nil then
				cmp.fxs["scroll_pig_heal"] = SpawnPrefab('tp_fx_scroll_pig_buff')
				cmp.fxs["scroll_pig_heal"].fx_name = 'tp_fx_scroll_pig_heal'
				inst:AddChild(cmp.fxs["scroll_pig_heal"])
				cmp.fxs["scroll_pig_heal"].Transform:SetPosition(0,0,0)
			end
		end,
		rm = function(inst, cmp)

			if cmp.tasks["scroll_pig_heal"] then
				cmp.tasks["scroll_pig_heal"]:Cancel()
				cmp.tasks["scroll_pig_heal"] = nil
			end
			if cmp.fxs["scroll_pig_heal"] then
				cmp.fxs["scroll_pig_heal"]:Remove()
				cmp.fxs["scroll_pig_heal"] = nil
			end
		end,
	},
	scroll_pig_health = {
		time = 180,
		add = function(inst, cmp)
			inst.components.health:SetMaxHealthSp(600)
			inst.components.health:DoDelta(0)
			if cmp.fxs["scroll_pig_health"] == nil then
				cmp.fxs["scroll_pig_health"] = SpawnPrefab('tp_fx_scroll_pig_buff')
				cmp.fxs["scroll_pig_health"].fx_name = 'tp_fx_scroll_pig_health'
				inst:AddChild(cmp.fxs["scroll_pig_health"])
				cmp.fxs["scroll_pig_health"].Transform:SetPosition(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			local cur = inst.components.health.currenthealth
			local max_hp = TUNING.PIG_HEALTH
			if inst:HasTag("werepig") then
				max_hp = TUNING.WEREPIG_HEALTH
			end
			inst.components.health:SetMaxHealthSp(max_hp)
			inst.components.health.currenthealth = cur
			inst.components.health:DoDelta(0)
			if cmp.fxs["scroll_pig_health"] then
				cmp.fxs["scroll_pig_health"]:Remove()
				cmp.fxs["scroll_pig_health"] = nil
			end
		end,
	},
	scroll_wind = {
		time = 240,
		add = function(inst, cmp)
			inst.components.locomotor:AddSpeedModifier_Mult("scroll_wind", .3)
			if cmp.tasks["scroll_wind"] == nil then
				cmp.tasks["scroll_wind"] = WARGON.per_task(inst, .5, function()
					if inst.sg:HasStateTag("moving") then
						local fx = WARGON.make_fx(inst, "tp_fx_leaf_"..math.random(4))
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			inst.components.locomotor:RemoveSpeedModifier_Mult("scroll_wind")
			if cmp.tasks["scroll_wind"] then
				cmp.tasks["scroll_wind"]:Cancel()
				cmp.tasks["scroll_wind"] = nil
			end
		end,
		img = "scroll_wind#",
		desc = "增加百分之三十的移速",
	},
	tp_pack_catcoon = {
		time = 1,
		add = function(inst, cmp)
			inst.components.locomotor:AddSpeedModifier_Mult("tp_pack_catcoon", .3)
			if cmp.tasks["tp_pack_catcoon"] == nil then
				cmp.tasks["tp_pack_catcoon"] = WARGON.per_task(inst, .1, function()
					if inst.sg:HasStateTag("moving") then
						local fx = WARGON.make_fx(inst, "tp_fx_wilson_run")
						fx.master = inst
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			inst.components.locomotor:RemoveSpeedModifier_Mult("tp_pack_catcoon")
			if cmp.tasks["tp_pack_catcoon"] then
				cmp.tasks["tp_pack_catcoon"]:Cancel()
				cmp.tasks["tp_pack_catcoon"] = nil
			end
		end,
		desc = "增加百分之三十的移速",
	},
	tp_spear_speed = {
		time = 3,
		add = function(inst, cmp)
			inst.components.locomotor:AddSpeedModifier_Mult("tp_spear_speed", .25)
			if cmp.tasks["tp_spear_speed"] == nil then

			end
		end,
		rm = function(inst, cmp)
			inst.components.locomotor:RemoveSpeedModifier_Mult("tp_spear_speed")
			if cmp.tasks["tp_spear_speed"] then
				cmp.tasks["tp_spear_speed"]:Cancel()
				cmp.tasks["tp_spear_speed"] = nil
			end
		end,

		img = {"tp_core_ring", "tp_core_ring_1"},
		desc = "增加百分之二十五的移速",
	},
	tp_potion_health_small = {
		time = 10,
		add = function(inst, cmp)
			if cmp.tasks["tp_potion_health_small"] == nil then
				cmp.tasks["tp_potion_health_small"] = WARGON.per_task(inst, 1, function()
					if inst.components.health then
						inst.components.health:DoDelta(3)
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["tp_potion_health_small"] then
				cmp.tasks["tp_potion_health_small"]:Cancel()
				cmp.tasks["tp_potion_health_small"] = nil
			end
		end,

		img = "blue_cap",
		desc = "持续回复生命",
	},
	tp_potion_sanity_small = {
		time = 10,
		add = function(inst, cmp)
			if cmp.tasks["tp_potion_sanity_small"] == nil then
				cmp.tasks["tp_potion_sanity_small"] = WARGON.per_task(inst, 1, function()
					if inst.components.sanity then
						inst.components.sanity:DoDelta(4)
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["tp_potion_sanity_small"] then
				cmp.tasks["tp_potion_sanity_small"]:Cancel()
				cmp.tasks["tp_potion_sanity_small"] = nil
			end
		end,

		img = "green_cap",
		desc = "持续回复理智",
	},
	tp_potion_crazy = {
		time = 60,
		add = function(inst, cmp)
			WARGON.add_speed_rate(inst, "tp_potion_crazy", .2)







		end,
		rm = function(inst, cmp)
			WARGON.remove_speed_rate(inst, "tp_potion_crazy")




		end,

		img = "petals_evil",
		desc = "增加百分之二十的移速",
	},
	tp_potion_shine = {
		time = 240,
		add = function(inst, cmp)
			if cmp.fxs["tp_potion_shine"] == nil then
				cmp.fxs["tp_potion_shine"] = SpawnPrefab("tp_fx_light")
				inst:AddChild(cmp.fxs["tp_potion_shine"])
			end
		end,
		rm = function(inst, cmp)
			if cmp.fxs["tp_potion_shine"] then
				cmp.fxs["tp_potion_shine"]:Remove()
				cmp.fxs["tp_potion_shine"] = nil
			end
		end,

		img = "algae",
		desc = "发光",
	},
	tp_potion_dry = {
		time = 240,
		add = function(inst, cmp)
			inst.components.moisture.baseDryingRate = 0.9
		end,
		rm = function(inst, cmp)
			inst.components.moisture.baseDryingRate = 0
		end,

		img = "flint",
		desc = "干燥得更快",
	},
	tp_potion_smell = {
		time = 240,
		add = function(inst, cmp)
			inst:AddTag("houndfriend")
			inst:AddTagNum("beefalo", 1)
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("houndfriend")
			inst:AddTagNum("beefalo", -1)
		end,

		img = "petals",
		desc = "不会引起猎犬和发情的牛的仇恨",
	},
	tp_potion_warth = {
		time = 60,
		add = function(inst, cmp)
			local fx = SpawnPrefab("tp_fx_speech")
			fx.Transform:SetPosition(0,0,0)
			fx.tp_str = "阿\n攻"
			fx.tp_colour = {x=1, y=0, z=0}
			inst:AddChild(fx)
			WARGON.add_dmg_rate(inst, "tp_potion_warth", .2)







		end,
		rm = function(inst, cmp)
			WARGON.remove_dmg_rate(inst, "tp_potion_warth")




		end,

		img = "meat_dried",
		desc = "增加百分之二十攻击力",
	},
	tp_potion_killer = {
		time = 60,
		add = function(inst, cmp)
			local fx = SpawnPrefab("tp_fx_speech")
			fx.Transform:SetPosition(0,0,0)
			fx.tp_str = "夜\n叉\n戮"
			fx.tp_colour = {x=1, y=0, z=0}
			inst:AddChild(fx)
			WARGON.add_dmg_rate(inst, "tp_potion_killer", .6)
			if inst:HasTag("player") then
				if inst.components.tpbody then

					inst.components.tpbody:AddHealthPenaltyModifier("tp_pesticide", 1)
				end
				if inst.components.health then
					inst.components.health:RecalculatePenalty()
				end
			end
		end,
		rm = function(inst, cmp)
			WARGON.remove_dmg_rate(inst, "tp_potion_killer")
			if inst.components.tpbody then


				inst.components.tpbody:RemoveHealthPenaltyModifier("tp_pesticide")
			end
			if inst.components.health then
				inst.components.health:RecalculatePenalty()
			end
		end,

		img = "obsidian",
		desc = "增加百分之六十攻击力，降低生命上限",
	},
	tp_potion_shadow = {
		time = 60,
		add = function(inst, cmp)
			local fx = SpawnPrefab("tp_fx_speech")
			fx.Transform:SetPosition(0,0,0)
			fx.tp_str = "月\n隐"
			fx.tp_colour = {x=0, y=1, z=1}
			inst:AddChild(fx)
			if inst:HasTag("player") and not inst:HasTag("notarget") then
				inst:AddTagNum("notarget", 1)
				inst.AnimState:SetMultColour(1, 1, 1, .5)
				inst:ListenForEvent("onhitother", tp_potion_shadow_on_attack)







			end
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("notarget", -1)
			inst.AnimState:SetMultColour(1, 1, 1, 1)
			inst:RemoveEventCallback("onhitother", tp_potion_shadow_on_attack)




		end,

		img = "bluegem",
		desc = "隐身",
	},
	tp_potion_iron = {
		time = 60,
		add = function(inst, cmp)
			local fx = SpawnPrefab("tp_fx_speech")
			fx.Transform:SetPosition(0,0,0)
			fx.tp_str = "哞\n护"
			fx.tp_colour = {x=0, y=0, z=1}
			inst:AddChild(fx)
			if inst.components.tpbody then
				inst.components.tpbody:AddAbsorbModifier("tp_potion_iron", .2)
			end







		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:RemoveAbsorbModifier("tp_potion_iron")
			end




		end,

		img = "iron",
		desc = "增加百分之二十防御力",
	},
	tp_potion_metal = {
		time = 60,
		add = function(inst, cmp)
			local fx = SpawnPrefab("tp_fx_speech")
			fx.Transform:SetPosition(0,0,0)
			fx.tp_str = "刚\n躯"
			fx.tp_colour = {x=1, y=1, z=0}
			inst:AddChild(fx)
			inst:AddTagNum("not_hit_stunned", 1)







		end,
		rm = function(inst, cmp)
			inst:AddTagNum("not_hit_stunned", -1)




		end,

		img = "tentaclespots",
		desc = "霸体",
	},
	tp_potion_cool = {
		time = 480,
		add = function(inst, cmp)
			if cmp.fxs["tp_potion_cool"] == nil then
				cmp.fxs["tp_potion_cool"] = SpawnPrefab("tp_fx_cool")
				inst:AddChild(cmp.fxs["tp_potion_cool"])
				cmp.fxs["tp_potion_cool"]:set_pos(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			if cmp.fxs["tp_potion_cool"] then
				cmp.fxs["tp_potion_cool"]:Remove()
				cmp.fxs["tp_potion_cool"] = nil
			end
		end,

		img = "watermelon",
		desc = "清凉",
	},
	tp_potion_frozen = {
		time = 180,
		add = function(inst, cmp)
			inst:AddTagNum("tp_not_freezable", 1)
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("tp_not_freezable", -1)
		end,

		img = "cave_banana",
		desc = "免疫冰冻",
	},
	tp_potion_warm = {
		time = 480,
		add = function(inst, cmp)
			if cmp.fxs["tp_potion_warm"] == nil then
				cmp.fxs["tp_potion_warm"] = SpawnPrefab("tp_fx_warm")
				inst:AddChild(cmp.fxs["tp_potion_warm"])
				cmp.fxs["tp_potion_warm"]:set_pos(0,0,0)
			end
		end,
		rm = function(inst, cmp)
			if cmp.fxs["tp_potion_warm"] then
				cmp.fxs["tp_potion_warm"]:Remove()
				cmp.fxs["tp_potion_warm"] = nil
			end
		end,

		img = "durian",
		desc = "保温",
	},
	tp_potion_fire = {
		time = 180,
		add = function(inst, cmp)
			inst:AddTagNum("tp_not_fire_damage", 1)
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("tp_not_fire_damage", -1)
		end,

		img = "eggplant",
		desc = "免疫火烧",
	},
	tp_potion_holy = {
		time = 60,
		add = function(inst, cmp)
			inst:AddTag("tp_potion_holy")
			cmp:ClearDebuff()
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_potion_holy")
		end,

		img = "dragonfruit",
		desc = "清除所有负面效果，持续时间内不受劣质食品影响",
	},
	tp_fast_work = {
		time = 30,
		add = function(inst, cmp)
			inst:AddTag("tp_fast_work")







		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_fast_work")




		end,
		img = "shovel",
		desc = "采集速度变快",
	},
	tp_harvester_king = {
		time = 60,
		add = function(inst, cmp)
			inst:AddTag("tp_harvester_king")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_harvester_king")
		end,
		img = "goldenshovel",
		desc = "采集时会采集周围的作物",
	},
	tp_combat_fruit = {
		time = 60,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_combat_fruit", .5)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_combat_fruit")
		end,
		img = "spear_wathgrithr",
		desc = "大幅增加攻击力",
	},
	tp_wind_seed = {
		time = 120,
		add = function(inst, cmp)
			inst:AddTag("tp_tornado_power")
			inst:add_speed_rate("tp_wind_seed", .6)
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_tornado_power")
			inst:rm_speed_rate("tp_wind_seed")
		end,
		desc = "大幅提升移动速度，免疫春季之王和雨季之王飓风的伤害",
		img = "sail_stick",
	},
	tp_spear_combat = {
		time = 5,
		add = function(inst, cmp)
			cmp.stacks["tp_spear_combat"] = cmp.stacks["tp_spear_combat"] or 0
			cmp.stacks["tp_spear_combat"] = math.min(cmp.stacks["tp_spear_combat"]+1,10)
			local rate = cmp.stacks["tp_spear_combat"] * .15
			inst:add_dmg_rate("tp_spear_combat", rate)
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_spear_combat"] = 0
			inst:rm_dmg_rate("tp_spear_combat")
		end,

		img = {"tp_core_blade", "tp_core_blade11"},
		desc = "每层增加百分之十五的攻击力",
	},
	tp_spear_conqueror = {
		time = 5,
		add = function(inst, cmp)


			cmp.stacks["tp_spear_conqueror"] = cmp.stacks["tp_spear_conqueror"] or 0
			cmp.stacks["tp_spear_conqueror"] = math.min(cmp.stacks["tp_spear_conqueror"]+1,5)
			if cmp.stacks["tp_spear_conqueror"] >= 5 then
				cmp:AddBuff("tp_spear_conqueror_buff")
			end
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_spear_conqueror"] = 0
		end,
	},
	tp_spear_conqueror_buff = {
		time = 5,
		add = function(inst, cmp)

			inst:AddTag("tp_spear_conqueror")
		end,
		rm = function(inst, cmp)

			inst:RemoveTag("tp_spear_conqueror")
		end,

		img = {"tp_core_blade", "tp_core_blade4"},
		desc = "攻击时回复生命和理智，攻击造成额外伤害",
	},
	tp_spear_cooldown = {
		time = 5,
		add = function(inst, cmp)
			cmp.stacks["tp_spear_cooldown"] = cmp.stacks["tp_spear_cooldown"] or 0
			cmp.stacks["tp_spear_cooldown"] = math.min(cmp.stacks["tp_spear_cooldown"]+1,5)
			if cmp.stacks["tp_spear_cooldown"] >= 5 then
				cmp:AddBuff("tp_spear_cooldown_buff")
			end
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_spear_cooldown"] = 0
		end,
	},
	tp_ancient_hulk_buff = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_ancient_hulk_recharging")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_ancient_hulk_buff")
			inst:RemoveTag("tp_ancient_hulk_recharging")
		end,
	},
	tp_ancient_hulk_damage = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_damage"] == nil then
				cmp.fxs["tp_ancient_hulk_damage"] = SpawnPrefab("tp_fx_food_follower")
				inst:AddChild(cmp.fxs["tp_ancient_hulk_damage"])
				cmp.fxs["tp_ancient_hulk_damage"]:set_pos(0, 0, 0)
				local atlas, img = WARGON.resolve_img_path("ratatouille")
				cmp.fxs["tp_ancient_hulk_damage"].components.tpfollowimage:SetImage(atlas, img)
			end
			inst.components.combat:AddDamageModifier("tp_ancient_hulk_damage", .5)
		end,
		rm = function(inst, cmp)

			cmp:AddBuff("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_damage"] then
				cmp.fxs["tp_ancient_hulk_damage"]:Remove()
				cmp.fxs["tp_ancient_hulk_damage"] = nil
			end
			inst.components.combat:RemoveDamageModifier("tp_ancient_hulk_damage")
		end,
	},
	tp_ancient_hulk_defense = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_defense"] == nil then
				cmp.fxs["tp_ancient_hulk_defense"] = SpawnPrefab("tp_fx_food_follower")
				inst:AddChild(cmp.fxs["tp_ancient_hulk_defense"])
				cmp.fxs["tp_ancient_hulk_defense"]:set_pos(0, 0, 0)
				local atlas, img = WARGON.resolve_img_path("steamedhamsandwich")
				cmp.fxs["tp_ancient_hulk_defense"].components.tpfollowimage:SetImage(atlas, img)
			end
			inst.components.health:SetAbsorptionAmount(.5)
		end,
		rm = function(inst, cmp)

			cmp:AddBuff("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_defense"] then
				cmp.fxs["tp_ancient_hulk_defense"]:Remove()
				cmp.fxs["tp_ancient_hulk_defense"] = nil
			end
			inst.components.health:SetAbsorptionAmount(0)
		end,
	},
	tp_ancient_hulk_heal = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_heal"] == nil then
				cmp.fxs["tp_ancient_hulk_heal"] = SpawnPrefab("tp_fx_food_follower")
				inst:AddChild(cmp.fxs["tp_ancient_hulk_heal"])
				cmp.fxs["tp_ancient_hulk_heal"]:set_pos(0, 0, 0)
				local atlas, img = WARGON.resolve_img_path("tea")
				cmp.fxs["tp_ancient_hulk_heal"].components.tpfollowimage:SetImage(atlas, img)
			end
			inst.components.health:StartRegen(100, 1)
		end,
		rm = function(inst, cmp)

			cmp:AddBuff("tp_ancient_hulk_buff")
			if cmp.fxs["tp_ancient_hulk_heal"] then
				cmp.fxs["tp_ancient_hulk_heal"]:Remove()
				cmp.fxs["tp_ancient_hulk_heal"] = nil
			end
			inst.components.health:StopRegen()
		end,
	},
	tp_spear_perseus = {
		time = 10,
		add = function(inst, cmp)
			inst:AddTag("tp_perseus")
			inst:AddTagNum("tp_not_hot_damage", 1)
			inst:AddTagNum("tp_not_cold_damage", 1)
			inst:AddTagNum("tp_not_fire_damage", 1)
			inst:AddTagNum("tp_not_poison_damage", 1)
			inst:AddTagNum("tp_not_gas_damage", 1)
			inst:AddTagNum("tp_not_freezable", 1)

		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_perseus")
			inst:AddTagNum("tp_not_hot_damage", -1)
			inst:AddTagNum("tp_not_cold_damage", -1)
			inst:AddTagNum("tp_not_fire_damage", -1)
			inst:AddTagNum("tp_not_poison_damage", -1)
			inst:AddTagNum("tp_not_gas_damage", -1)
			inst:AddTagNum("tp_not_freezable", -1)

		end,

		img = {"tp_swords", "tp_good_sword"},
		desc = "免疫冰冻、烧伤、中毒、冲击波、过热、过冷",

	},
	tp_incitant = {
		time = 60,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_incitant", .1)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_incitant", .1)
		end,
		img = "spear",
		desc = "增加百分之十攻击力",
	},
	tp_glow = {
		time = 90,
		add = function(inst, cmp)
			if cmp.fxs["tp_glow"] == nil then
				cmp.fxs["tp_glow"] = SpawnPrefab("tp_fx_glow")
				inst:AddChild(cmp.fxs["tp_glow"])
				cmp.fxs["tp_glow"]:set_pos(0, 0, 0)
			end
		end,
		rm = function(inst, cmp)
			if cmp.fxs["tp_glow"] then
				cmp.fxs["tp_glow"]:Remove()
				cmp.fxs["tp_glow"] = nil
			end
		end,
		img = "wormlight",
		desc = "发光",
	},
	tp_spear_valkyrie_buff = {
		time = 5,
		add = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:AddAbsorbModifier("tp_spear_valkyrie_buff", .2)
			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:RemoveAbsorbModifier("tp_spear_valkyrie_buff")
			end
		end,

		img = "yellowstaff",
		desc = "增加防御力",
	},
	tp_cane_buff = {
		time = 30,
		add = function(inst, cmp)
			inst:AddTag("tp_cane_buff")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_cane_buff")
		end,
		img = "cane",
		desc = "移动时装备手杖，攻击时装备武器",
	},
	tp_torch_buff = {
		time = 90,
		add = function(inst, cmp)
			if cmp.fxs["tp_torch_buff"] == nil then
				cmp.fxs["tp_torch_buff"] = SpawnPrefab("torchfire")
				fx = cmp.fxs["tp_torch_buff"]
				inst:AddChild(fx)
				fx.Transform:SetPosition(0, 0, 0)
			end
		end,
		rm = function(inst, cmp)
			if cmp.fxs["tp_torch_buff"] then
				cmp.fxs["tp_torch_buff"]:Remove()
				cmp.fxs["tp_torch_buff"] = nil
			end
		end,
		img = "torch",
		desc = "一路火花带闪电",
	},
	tp_spear_single = {
		time = .5,
		add = function(inst, cmp)
			if cmp.stacks["tp_spear_single"] == nil then
				cmp.stacks["tp_spear_single"] = 1
			else
				local n  = cmp.stacks["tp_spear_single"] + 1
				cmp.stacks["tp_spear_single"] = n

			end
			inst:add_dmg_rate("tp_spear_single", cmp.stacks["tp_spear_single"]*.25)
			inst:do_task(.5, function()
				if cmp.stacks["tp_spear_single"] then
					cmp.stacks["tp_spear_single"] = cmp.stacks["tp_spear_single"] - 1
					if cmp.stacks["tp_spear_single"] < 0 then
						cmp.stacks["tp_spear_single"] = 0
					end
				end
			end)
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_spear_single"] = 0
			inst:rm_dmg_rate("tp_spear_single")
		end,
		img = "tp_spear_single#",
		desc = "每层增加百分之二十五攻击力",
	},
	tp_ruinbat_deerclops = {
		time = 5,
		add = function(inst, cmp)
			if cmp.stacks["tp_ruinbat_deerclops"] == nil then
				cmp.stacks["tp_ruinbat_deerclops"] = 1
			else
				cmp.stacks["tp_ruinbat_deerclops"] = cmp.stacks["tp_ruinbat_deerclops"] + 1
				if cmp.stacks["tp_ruinbat_deerclops"] == 4 then
					inst:AddTag("tp_ruinbat_deerclops")
				elseif cmp.stacks["tp_ruinbat_deerclops"] > 4 then
					cmp.stacks["tp_ruinbat_deerclops"] = 1
					inst:RemoveTag("tp_ruinbat_deerclops")
				end
			end
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_ruinbat_deerclops"] = 0
			inst:RemoveTag("tp_ruinbat_deerclops")
		end,
		img = "tp_ruinbat_deerclops#",
		desc = "四层时将会强化下一次攻击或技能",
	},
	tp_spear_shell = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_spear_shell")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_spear_shell")
		end,

		img = "spidergland",
		desc = "免疫中毒",
	},
	tp_spear_wave = {
		time = 5,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_spear_wave", .2)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_spear_wave")
		end,
		img = "cutlass",
		desc = "增加攻击力和移速"
	},
	tp_tonic = {
		time = 30,
		add = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:AddMaxHealthModifier("tp_tonic", 20)

			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:RemoveMaxHealthModifier("tp_tonic")

			end
		end,
		img = "turkeydinner",
		desc = "增加最大生命上限",
	},
	tp_dinner = {
		time = 30,
		add = function(inst, cmp)
			if inst.components.sanity then

				if inst.components.tpbody then
					local need = false
					if inst.components.tpbody.max_sanity_mods["tp_dinner"] == nil then
						need = true
					end
					inst.components.tpbody:AddMaxSanityModifier("tp_dinner", 20)

					if need then
						inst.components.sanity:DoDelta(20)
					end
				end
			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:RemoveMaxSanityModifier("tp_dinner", 20)

			end
		end,
		img = "flowersalad",
		desc = "增加最大理智上限",
	},
	tp_pickle = {
		time = 30,
		add = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:AddMaxHungerModifier("tp_pickle", 20)

			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then
				inst.components.tpbody:RemoveMaxHungerModifier("tp_pickle")

			end
		end,
		img = "asparagussoup",
		desc = "增加最大饥饿上限",
	},
	tp_coffee = {
		time = 60,
		add = function(inst, cmp)
		end,
		rm = function(inst, cmp)
		end,
		img = "coffeebeans_cooked",
		desc = "增加移动速度",
	},
	tp_armor_ancient = {
		time = 5,
		add = function(inst, cmp)
			if cmp.stacks["tp_armor_ancient"]==nil then
				cmp.stacks["tp_armor_ancient"] = 1
			else
				local n = cmp.stacks["tp_armor_ancient"]
				cmp.stacks["tp_armor_ancient"] = n + 1
				if n > 5 then
					cmp.stacks["tp_armor_ancient"] = 5
					cmp:AddBuff("tp_armor_defense")
				end
			end
		end,
		rm = function(inst, cmp)
			cmp.stacks["tp_armor_ancient"] = nil
		end,
	},
	tp_armor_defense = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_armor_defense")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_armor_defense")
		end,

		img = "armormarble",
		desc = "降低敌人对你造成的伤害",
	},
	tp_armor_ancient4 = {
		time = 5,
		add = function(inst, cmp)
			if inst.components.health then
				inst.components.health:SetInvincible(true, "tp_armor_ancient4")
			end
			if cmp.fxs["tp_armor_ancient4"] == nil then
				local fx = SpawnPrefab("tp_fx_force_field")
				inst:AddChild(fx)
				fx:set_pos(0, 0, 0)
				cmp.fxs["tp_armor_ancient4"] = fx
			end
		end,
		rm = function(inst, cmp)
			if inst.components.health then
				inst.components.health:SetInvincible(false, "tp_armor_ancient4")
			end
			if cmp.fxs["tp_armor_ancient4"] then
				cmp.fxs["tp_armor_ancient4"]:Remove()
				cmp.fxs["tp_armor_ancient4"] = nil
			end
		end,

		img = "armorruins",
		desc = "无敌时多么寂寞",
	},
	tp_armor_health = {
		time = 5,
		add = function(inst, cmp)
			if cmp.tasks["tp_armor_health"] == nil then
				cmp.tasks["tp_armor_health"] = inst:per_task(1, function()
					if inst.components.health then
						inst.components.health:DoDelta(2, nil, "tp_health_equip")
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["tp_armor_health"] then
				cmp.tasks["tp_armor_health"]:Cancel()
				cmp.tasks["tp_armor_health"] = nil
			end
		end,

		img = {"tp_core_necklace", "tp_core_necklace2"},
		desc = "持续回复生命",
	},
	tp_armor_shadow = {
		time = 5,
		add = function(inst, cmp)
			if cmp.tasks["tp_armor_shadow"] == nil then
				cmp.tasks["tp_armor_shadow"] = inst:per_task(1, function()
					if inst.components.sanity then
						inst.components.sanity:DoDelta(2)
					end
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["tp_armor_shadow"] then
				cmp.tasks["tp_armor_shadow"]:Cancel()
				cmp.tasks["tp_armor_shadow"] = nil
			end
		end,
		img = "tp_armor_shadow#",
		desc = "持续回复理智",
	},
	tp_armor_thunder = {
		time = 150,
		add = function(inst, cmp)
			inst:add_speed_rate("tp_armor_thunder", .25)
			if cmp.fxs["tp_armor_thunder"] == nil then
				local fx = SpawnPrefab("tp_fx_glow")
				inst:AddChild(fx)
				fx:set_pos(0, 0, 0)
				cmp.fxs["tp_armor_thunder"] = fx
			end
		end,
		rm = function(inst, cmp)
			inst:rm_speed_rate("tp_armor_thunder")
			if cmp.fxs["tp_armor_thunder"] then
				cmp.fxs["tp_armor_thunder"]:Remove()
				cmp.fxs["tp_armor_thunder"] = nil
			end
		end,

		img = {"tp_core_ring", "tp_core_ring6"},
		desc = "持续发光，加速",
	},
	tp_templar = {
		time = 6,
		add = function(inst, cmp)
			cmp:AddBuff("tp_armor_ancient4", 3)
			if cmp.fxs["tp_templar"] == nil then
				local fx = WARGON.make_fx2("spear_lance_array", inst, {owner=inst})
				cmp.fxs["tp_templar"] = fx 
			end
		end,
		rm = function(inst, cmp)
			local fx = cmp.fxs["tp_templar"]
			if fx then
				fx:Recycle()
				cmp.fxs["tp_templar"] = nil
			end
		end,
	},
	tp_spear_a = {
		time = 20,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_spear_a", .2)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_spear_a")
		end,
		img = "spear",
		desc = "攻击力提升百分之二十",
	},
	tp_spear_a_2 = {
		time = 25,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_spear_a", .3)
			inst:add_speed_rate("tp_spear_a", .15)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_spear_a")
			inst:rm_speed_rate("tp_spear_a")
		end,
		img = "spear",
		desc = "攻击力提升百分之三十，移速增加百分之十五",
	},
	tp_spear_a_3 = {
		time = 30,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_spear_a", .4)
			inst:add_speed_rate("tp_spear_a", .2)
			inst:AddTag("tp_spear_a_3_buff")
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_spear_a")
			inst:rm_speed_rate("tp_spear_a")
			inst:RemoveTag("tp_spear_a_3_buff")
		end,
		img = "spear",
		desc = "攻击力提升百分之四十，移速增加百分之二十",
	},
	tp_tophat_7 = {
		time = 10,
		add = function(inst, cmp)
			inst:AddTagNum("tp_hat_witch", 1)
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("tp_hat_witch", -1)
		end,
		img = "tophat",
		desc = "提升冰火法杖的攻击力",
	},
	tp_core_hunter = {
		time = 5,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_core_hunter", .1)
			inst:add_speed_rate("tp_core_hunter", .1)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_core_hunter")
			inst:rm_speed_rate("tp_core_hunter")
		end,
		img = {"tp_core_gun", "tp_core_gun3"},
		desc = "提升攻击力和移动速度",
	},
	tp_nightstick_5 = {
		time = 30,
		add = function(inst, cmp)
			if inst.components.health then
				inst.components.health:AddAbsorbModifier("tp_nightstick_5", .3)
			end
			if cmp.fxs["tp_nightstick_5"] == nil then
				cmp.fxs["tp_nightstick_5"] = WARGON.make_fx2("thunder_element", inst, {follow=true})
			end
			if not inst:HasTag("tp_nightstick_5") then
				inst:AddTag("tp_nightstick_5")
				inst:ListenForEvent("attacked", nightstick_attacked)
			end
		end,
		rm = function(inst, cmp)
			if inst.components.health then
				inst.components.health:RemoveAbsorbModifier("tp_nightstick_5")
			end
			if cmp.fxs["tp_nightstick_5"] then
				cmp.fxs["tp_nightstick_5"]:Recycle()
				cmp.fxs["tp_nightstick_5"] = nil
			end
			inst:RemoveTag("tp_nightstick_5")
			inst:RemoveEventCallback("attacked", nightstick_attacked)
		end,
		img = "nightstick",
		desc = "减免30%的攻击伤害，对攻击者造成20点伤害",
	},
	tp_potion_trunk = {
		time = 90,
		add = function(inst, cmp)
			inst:AddTagNum("groundpoundimmune", 1)
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("groundpoundimmune", -1)
		end,
		img = "trunk_summer",
		desc = "免疫冲击波"
	},

	tp_beefalohat_spawned = {
		time = 30,
		add = function(inst, cmp)
			inst:AddTag("tp_beefalohat_spawned")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_beefalohat_spawned")
		end,
		img = "beefalohat",
		desc = "不能使用牛角帽召唤猪人",
		debuff = true,
	},
	tp_hat_wolf = {
		time = 30,
		add = function(inst, cmp)
			inst:AddTag("tp_hat_wolf")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_hat_wolf")
		end,

		img = "footballhat",
		desc = "不能使用奔狼头盔",
		debuff = true,
	},
	tp_armor_airi = {
		time = 10,
		add = function(inst, cmp)
			inst:AddTag("tp_armor_airi")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_armor_airi")
		end,
		img = "tp_armor_airi#",
		desc = "不能使用艾黎",
		debuff = true,
	},
	tp_spear_turret = {
		time = 20,
		add = function(inst, cmp)
			inst:AddTag("tp_spear_turret")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_spear_turret")
		end,
		img = "tp_spear_turret#",
		desc = "不能使用防御者",
		debuff = true,
	},
	tp_spear_cooldown_buff = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_spear_cooldown")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_spear_cooldown")
		end,
		debuff = true,
		img = "tp_spear_cooldown#",
		desc = "西西弗斯不造成伤害",
	},
	tp_wound = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_wound")
			if cmp.fxs["tp_wound"] == nil then
				create_speech(inst, "重\n伤", {.8,0,0})
				cmp.fxs["tp_wound"] = SpawnPrefab("poisonbubble")
				WARGON.set_scale(cmp.fxs["tp_wound"], .5)
				inst:AddChild(cmp.fxs["tp_wound"])
				cmp.fxs["tp_wound"].Transform:SetPosition(0, 0, 0)
				cmp.fxs["tp_wound"].AnimState:SetMultColour(1,.1,.1,1)
				WARGON.no_save(cmp.fxs["tp_wound"])
			end
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_wound")
			if cmp.fxs["tp_wound"] then
				cmp.fxs["tp_wound"]:Remove()
				cmp.fxs["tp_wound"] = nil
			end
		end,
		debuff = true,
		img = "spidergland",
		desc = "生命回复效果减半",
	},
	tp_hurt = {
		time = 10,
		add = function(inst, cmp)
			if cmp.tasks["tp_hurt"] == nil then
				create_speech(inst, "流\n血", {.8,0,0})
				cmp.tasks["tp_hurt"] = WARGON.per_task(inst, 1, function()
					local mult = inst.components.health.poison_damage_scale
					local dt = 2 * mult
					inst.components.health:DoDelta(-dt, false, "tp_hurt")
					local fx = WARGON.make_fx(inst, "poisonbubble")
					fx.AnimState:SetMultColour(1,.1,.1,1)
					WARGON.no_save(fx)
					WARGON.do_task(fx, 1, function()
						fx:Remove()
					end)
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["tp_hurt"] then
				cmp.tasks["tp_hurt"]:Cancel()
				cmp.tasks["tp_hurt"] = nil
			end
		end,
		debuff = true,
		img = "health_down",
		desc = "持续失去生命",
	},
	tp_armor_broken = {
		time = 5,
		add = function(inst, cmp)
			inst:AddTag("tp_armor_broken")
			if cmp.fxs["tp_armor_broken"] == nil then
				create_speech(inst, "破\n甲", {0,0,.8})
				cmp.fxs["tp_armor_broken"] = SpawnPrefab("tp_fx_armor_broken")
				inst:AddChild(cmp.fxs["tp_armor_broken"])
				cmp.fxs["tp_armor_broken"].Transform:SetPosition(0, 0, 0)
				if cmp.fxs["tp_armor_broken"].components.tpfollowimage then
					cmp.fxs["tp_armor_broken"].components.tpfollowimage:SetImage(
						"images/inventoryimages/tp_armor_broken.xml",
						"tp_armor_broken.tex")
				end
			end
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_armor_broken")
			if cmp.fxs["tp_armor_broken"] then
				cmp.fxs["tp_armor_broken"]:Remove()
				cmp.fxs["tp_armor_broken"] = nil
			end
		end,
		debuff = true,
		img = "tp_armor_broken#",
		desc = "护甲收益减半",
	},
	tp_unhygienic = {
		time = 30,
		add = function(inst, cmp)
			inst:add_dmg_rate("tp_unhygienic", -.15)
		end,
		rm = function(inst, cmp)
			inst:rm_dmg_rate("tp_unhygienic")
		end,
		debuff = true,
		img = "peg_leg",
		desc = "降低百分之十五攻击力",
	},
	tp_parasite = {
		time = 30,
		add = function(inst, cmp)
			inst:AddTag("tp_parasite")
		end,
		rm = function(inst, cmp)
			inst:RemoveTag("tp_parasite")
		end,
		debuff = true,
		img = "stinger",
		desc = "攻击时会失去生命值",
	},
	tp_pesticide = {
		time = 30,
		add = function(inst, cmp)
			if inst:HasTag("player") then
				if inst.components.tpbody then

					inst.components.tpbody:AddHealthPenaltyModifier("tp_pesticide", 1)
				end
				if inst.components.health then
					inst.components.health:RecalculatePenalty()
				end
			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then


				inst.components.tpbody:RemoveHealthPenaltyModifier("tp_pesticide")
			end
			if inst.components.health then
				inst.components.health:RecalculatePenalty()
			end
		end,
		debuff = true,
		img = "health_max",
		desc = "降低生命上限",
	},
	tp_overdue = {
		time = 30,
		add = function(inst, cmp)
			if inst:HasTag("player") then
				if inst.components.tpbody then

					inst.components.tpbody:AddSanityPenaltyModifier("tp_overdue", 30)
				end
				if inst.components.sanity then
					inst.components.sanity:RecalculatePenalty()
				end
			end
		end,
		rm = function(inst, cmp)
			if inst.components.tpbody then

				inst.components.tpbody:RemoveSanityPenaltyModifier("tp_overdue")
			end
			if inst.components.sanity then
				inst.components.sanity:RecalculatePenalty()
			end
		end,
		debuff = true,
		img = "sanity_max",
		desc = "降低理智上限",
	},
	tp_ice_iron = {
		time = 10,
		add = function(inst, cmp)
			if inst.components.combat then
				inst.components.combat:AddPeriodModifier("tp_ice_iron", .2)
			end
		end,
		rm = function(inst, cmp)
			if inst.components.combat then
				inst.components.combat:RemovePeriodModifier("tp_ice_iron")
			end
		end,
		debuff = true,
		img = "armor_metalplate",
		desc = "降低攻击速度",
	},
	tp_ice_heart = {
		time = 10,
		add = function(inst, cmp)
			if inst.components.combat then
				inst.components.combat:AddPeriodModifier("tp_ice_heart", .3)
			end
		end,
		rm = function(inst, cmp)
			if inst.components.combat then
				inst.components.combat:RemovePeriodModifier("tp_ice_heart")
			end
		end,
		debuff = true,
		img = "metalplatehat",
		desc = "降低攻击速度",
	},
	tp_junkfood = {
		time = 60,
		add = function(inst, cmp)
		end,
		rm = function(inst, cmp)
		end,
		debuff = true,
		img = "wetgoop",
		desc = "降低移动速度",
	},
	tp_justice_honor = {
		time = 5,
		add = function(inst, cmp)
		end,
		rm = function(inst, cmp)
		end,
		debuff = true,
		img = "armor_snakeskin_scaly",
		desc = "降低移动速度",
	},
	tp_ice_fight = {
		time = 5,
		add = function(inst, cmp)
		end,
		rm = function(inst, cmp)
		end,
		debuff = true,
		img = "icestaff",
		desc = "降低移动速度",
	},
	tp_board_armor = {
		time = 5,
		add = function(inst, cmp)
		end,
		rm = function(inst, cmp)
		end,
		debuff = true,
		img = "armorwood",
		desc = "降低移动速度",
	},
}








































































































local BuffData = Class(function(self)
	self.name = nil
	self.time = nil
	self.add = nil
	self.rm = nil
	self.img = nil
	self.desc = nil
	self.debuff = nil
	self.stack = nil
	self.limit = 19
end)

function BuffData:SplitString(str)
	local len = string.len(str)/3
	local substr = nil
	if len > self.limit then
		substr = string.sub(str, self.limit*3+1, -1)
		str = string.sub(str, 1, self.limit*3)
	end
	return str, substr
end

function BuffData:GetString()
	if not self.desc then
		return
	end

	local str, substr = WARGON.split_string(self.desc, self.limit)
	local sentence = str

	while substr do

		str, substr = WARGON.split_string(substr, self.limit)
		sentence = sentence.."\n"..str
	end
	return sentence
end

local BuffShelf = Class(function(self)
	self.buffs = {}

end)

function BuffShelf:AddBuff(buff)
	self.buffs[buff.name] = buff

end

function BuffShelf:GetBuff(buff_name)
	return self.buffs[buff_name]
end

function BuffShelf:SetStack(buff_name, stack)
	self.buffs[buff_name].stack = stack
end

local buff_shelf = BuffShelf()
for k, v in pairs(buffs) do
	local buff = BuffData()
	buff.name = k
	buff.time = v.time
	buff.img = v.img
	buff.add = v.add
	buff.rm = v.rm
	buff.debuff = v.debuff
	buff.desc = v.desc
	buff_shelf:AddBuff(buff)
end

WARGON.DATA.tp_data_buff = buff_shelf