local Anims = {
	pigking = {"Pig_King", "Pig_King", "idle", },
	bearger = {"bearger", "bearger_build", "ground_pound", },
	moose = {"goosemoose", "goosemoose_build", "honk", },
	dragonfly = {"dragonfly", "dragonfly_fire_build", "taunt_pre", },
	deerclops = {"deerclops", "deerclops_build", "atk", },
	teatree_nut = {"teatree_nut", "teatree_nut", "idle", },
	jungletreeseed = {"jungletreeseed", "jungletreeseed", "idle", },
	pinecone = {"pinecone", "pinecone", "idle", },
	coconut = {"coconut", "coconut", "idle", },
	acorn = {"acorn", "acorn", "idle", },
	green_leaf = {"tree_leaf_fx", "tree_leaf_fx_green", "fall", },
	green_leaf2 = {"tree_leaf_fx", "tree_leaf_fx_green", "chop", },
	green_leaf3 = {"chop_mangrove", "chop_mangrove", "chop", },
	green_leaf4 = {"chop_mangrove", "chop_mangrove", "fall", },
	fall_jungle = {"chop_jungle", "chop_jungle", "fall", },
	fall_leaf = {"pine_needles", "pine_needles", "fall", },
	chop_leaf = {"pine_needles", "pine_needles", "chop", },
	chop_jungle = {"chop_jungle", "chop_jungle", "chop", },
	dragonfruit = {"dragonfruit", "dragonfruit", "idle", },
	hambat = {"ham_bat", "ham_bat", "idle", },
	armorwood = {"armor_wood", "armor_wood", "anim", },
	coffeebeans = {"coffeebeans", "coffeebeans", "idle", },
	bandage = {"bandage", "bandage", "idle", },
	football = {"footballhat", "hat_football", "anim", },
	has_item = {"tp_has_item", "tp_has_item", "idle", },
	meat = {"meat", "meat", "raw", },
	catcoon = {"catcoon", "catcoon_build", "taunt_pst", },
	blood = {"wortox_soul_heal_fx", "wortox_soul_heal_fx", "heal", },
	sparkle = {"sparklefx", "sparklefx", "sparkle", },
	stafflight = {"star", "star", "appear", },
	small_meat = {"meat_small", "meat_small", "raw", },
	wilson_lunge = {"wilson", "esctemplate", "lunge_pst", },
	poison_hole = {"poison_hole", "poison_hole", "pop", },
	woodie = {"wilsonbeefalo", "woodie", "idle_loop", },
	snow_ball = {"firefighter_projectile", "firefighter_projectile", "spin_loop", },
	warg = {"warg", "warg_build", "idle_loop", },
	bishop_charge = {"bishop_attack", "bishop_attack", "idle", },
	spore_blue = {"tp_spore_blue", "tp_spore_blue", "cough_out", },
	spore = {"tp_spore", "tp_spore", "cough_out", },
	birdcage = {"birdcage", "birdcage_curly", "idle", },
	sign = {"sign_home", "sign_home", "place", },
	forcefield = {"forcefield", "forcefield", "idle_loop"},
	bramble = {"bramblefx", "bramblefx", "idle"},
	arrow = {"tracker_pointer", "tracker_pointer", "idle"},
	spear_lance = {"tp_spear_lance", "tp_spear_lance", "throw",},
	collapse_big = {"collapse", "structure_collapse_fx", "collapse_large"},
	collapse_small = {"collapse", "structure_collapse_fx", "collapse_small"},
	groundpoundring_fx = {"bearger_ring_fx", "bearger_ring_fx", "idle"},
	impact = {"impact", "impact", "idle"},
	laser = {"laser_hits_sparks", "laser_hit_sparks_fx", "hit_1"},
	laserscorch = {"laser_burntground", "burntground", "idle"},
	lasertrail = {"laser_smoke_fx", "laser_smoke_fx", "idle"},
	star = {"star", "star", "appear"},
	wave = {"wave_ripple", "wave_ripple", "idle"},
}

local easing = require("easing")

local function add_physics(inst, phy)
	if inst.Physics then
		return
	end
	if phy == 'inv' then
		MakeInventoryPhysics(inst)
	elseif phy == 'obs' then
		MakeObstaclePhysics(inst, .5)
	else
		MakeCharacterPhysics(inst, 1, .5)
	end
	inst.Physics:ClearCollisionMask()
	inst.Physics:CollidesWith(COLLISION.GROUND)
end

local function set_proj(inst)
	if inst.components.tpfxproj == nil then
		add_physics(inst, "inv")
		inst:AddComponent("tpfxproj")
		inst.components.tpfxproj:SetSpeed(25)
		inst.components.tpfxproj:SetLaunchOffset(Vector3(0, 0, 0))
		inst.components.tpfxproj:SetOnMissFn(function()
			inst:Recycle()
		end)
		inst.components.tpfxproj:SetOnHitFn(function(inst, attacker, target)
			if attacker and attacker:is_alive()
			and attacker.components.combat and target
			and attacker.components.combat:CanTarget(target) then
				target.components.combat:GetAttacked(attacker, inst.damage or 10, nil, inst.stimuli)
				if inst.on_hit then
					inst:on_hit(attacker, attacker)
				end
			end
			inst:Recycle()
		end)
	end
end

local fx_fns = {
sign = function(inst)
    inst:set_anim(Anims.sign)
    inst:ListenForEvent("animover", function()
        inst:Recycle()
    end)
end,
shadow_spawn = function(inst)
	inst:do_task(0, function()
		local pt = inst:GetPosition()
		for i = 1, 6 do
			local angle = PI/180 * 360/6 * i
			local radius = 5
			local offset = Vector3(math.cos(angle)*radius, 0, math.sin(angle)*radius)
			local fx = WARGON.make_fx2("shadow_line", pt+offset)
			fx:ForceFacePoint(pt)
		end
	end)
	inst:do_task(.5, function()
		local pt = inst:GetPosition()
		for i = 1, 18 do
			local angle = PI/180 * 360/18 * i
			local radius = 10
			local offset = Vector3(math.cos(angle)*radius, 0, math.sin(angle)*radius)
			local fx = WARGON.make_fx2("shadow_line_2", pt)
			fx:ForceFacePoint(pt+offset)
		end
		WARGON.make_fx2("statue_transition_2", inst)
		inst:Recycle()
	end)
end,
shadow_line_2 = function(inst)
	if inst.Physics == nil then
		add_physics(inst)
	end
	inst.Physics:SetMotorVel(2, 0, 0)
	inst.task = inst:per_task(.5, function()
		WARGON.make_fx2("shadow_dust", inst)
	end)
	inst:do_task(2, function()




		inst:Recycle()
	end)
end,
shadow_line = function(inst)
	add_physics(inst)
	inst.Physics:SetMotorVel(10, 0, 0)
	inst.task = inst:per_task(.1, function()
		WARGON.make_fx2("shadow_dust", inst)
	end)
	WARGON.do_task(inst, .5, function()




		inst:Recycle()
	end)
end,
lightning = function(inst)
	if inst.Light == nil then
		inst.entity:AddLight()
		inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
		inst.AnimState:SetLightOverride(1)
		inst.Transform:SetScale(2,2,2)
		
		inst.AnimState:SetBank("lightning")
		inst.AnimState:SetBuild("lightning")
	end
	
	inst.AnimState:PlayAnimation("anim")
	inst:ListenForEvent("animover", function() inst:Recycle() end)

    inst:DoTaskInTime(0, function()
		GetClock():DoLightningLighting(.5)
		GetPlayer().SoundEmitter:PlaySound("dontstarve/rain/thunder_close")
		GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, .5, 40)
    end)
end,
collapse_big = function(inst)
	inst:set_anim(Anims.collapse_big)
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_smoke")
	inst:do_task(1, inst.Recycle)
end,
collapse_small = function(inst)
	inst:set_anim(Anims.collapse_small)
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_smoke",nil,.25)
	inst:do_task(1, inst.Recycle)
end,
has_item = function(inst)
	if inst.components.tpfollowimage == nil then
		WARGON.set_scale(inst, 4)
		inst:AddComponent("tpfollowimage")
		inst:set_anim(Anims.has_item)
	end
	inst:ListenForEvent("tp_recycle", function()
		inst.components.tpfollowimage:Hide()
	end)
end,
snow_ball = function(inst)
	inst:set_anim(Anims.snow_ball)
	if inst.Physics == nil then
		local physics = inst.entity:AddPhysics()
		physics:SetMass(1)
		physics:SetCapsule(0.2, 0.2)
		inst.Physics:SetFriction(10)
		inst.Physics:SetDamping(5)
		inst.Physics:SetCollisionGroup(COLLISION.CHARACTERS)
		inst.Physics:ClearCollisionMask()
		inst.Physics:CollidesWith(GetWorldCollision())
		inst.Physics:CollidesWith(COLLISION.INTWALL)
		inst:AddComponent("locomotor")
		inst:AddComponent("complexprojectile")
		inst.components.complexprojectile:SetOnHit(function(inst)
			local dist = 4
			local x,y,z = inst:GetPosition():Get()
			local ents = TheSim:FindEntities(x,y,z, dist, nil, 
				{"FX", "DECOR", "INLIMBO"})
			for k,v in pairs(ents) do
				if v then
					print("testing",v.prefab)
					if v.components.burnable then
						print("testing 2",v.prefab)
						if v.components.burnable:IsBurning() then
							print("testing 3",v.prefab)
							v.components.burnable:Extinguish(true, TUNING.FIRESUPPRESSOR_EXTINGUISH_HEAT_PERCENT)
						elseif v.components.burnable:IsSmoldering() then
							print("testing 4",v.prefab)
							v.components.burnable:Extinguish(true)
						end
					end
					if not v:HasTag("tp_snow_power") then
						if v.components.freezable then
							print("testing 5",v.prefab)
							v.components.freezable:AddColdness(2) 
						elseif v.components.health 
						and not v.components.health:IsDead() then
							v.components.health:DoDelta(-20)
						end
						if v.components.temperature then
							print("testing 6",v.prefab)
							local temp = v.components.temperature:GetCurrent()
							v.components.temperature:SetTemperature(temp - TUNING.FIRE_SUPPRESSOR_TEMP_REDUCTION)
						end
						if inst.diff and v.components.health then
							local delta = v:HasTag("player") and 10 or 20
							v.components.health:DoDelta(-delta)
						end
					end
					if inst.diff and v:HasTag("tp_snow_power") then
						if v.components.health and not v.components.health:IsDead() then
							v.components.health:DoDelta(20)
						end
					end
				end
			end
			inst.SoundEmitter:PlaySound("dontstarve_DLC001/common/firesupressor_impact")

			WARGON.make_fx2("splash_snow_fx", inst)
			inst:Recycle()
		end)
		inst.components.complexprojectile.yOffset = 2.5
	end
end,
snow_ball_shoot = function(inst)
	local function launch_projectile(inst, targetpos)
		local x, y, z = inst.Transform:GetWorldPosition()

		local projectile = WARGON.make_fx2("snow_ball", inst)
	    projectile.Transform:SetPosition(x, y, z)
	    local dx = targetpos.x - x
	    local dz = targetpos.z - z
	    local rangesq = dx * dx + dz * dz
	    local maxrange = TUNING.FIRE_DETECTOR_RANGE
	    local speed = easing.linear(rangesq, 15, 3, maxrange * maxrange)
	    projectile.components.complexprojectile:SetHorizontalSpeed(speed)
	    projectile.components.complexprojectile:SetGravity(-25)
	    projectile.components.complexprojectile:Launch(targetpos, inst, inst)
	    projectile.owner = inst
	end
	WARGON.do_task(inst, 0, function()
		local pt = inst:GetPosition()
		inst.Transform:SetPosition(pt.x, 4, pt.z)
		local pos = WARGON.around_land(inst, 
			math.random(TUNING.FIRE_DETECTOR_RANGE/2)+math.random())
		if pos then
			launch_projectile(inst, pos)
		end
		inst:Recycle()
	end)
end,
groundpoundring_fx = function(inst)
	inst:set_anim(Anims.groundpoundring_fx)
	if inst.tp_fx_inited == nil then
		inst.tp_fx_inited = true
		inst.AnimState:SetFinalOffset(-1)
		inst.AnimState:SetOrientation( ANIM_ORIENTATION.OnGround )
		inst.AnimState:SetLayer( LAYER_BACKGROUND )
		inst.AnimState:SetSortOrder( 3 )
	end
	inst:ListenForEvent("animover", inst.Recycle)
end,
small_ground_pound = function(inst)
	inst:set_anim(Anims.groundpound_fx)
	inst:do_task(0, function()
		local s = inst.scale or .5
		inst.Transform:SetScale(s, s, s)
		inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/bearger/dustpoof")
		inst:ListenForEvent("animover", inst.Recycle)
	end)
end,
small_ground_pound_ring = function(inst)
	inst:set_anim(Anims.groundpoundring_fx)
	if inst.tp_fx_inited == nil then
		inst.tp_fx_inited = true
		local s = inst.scale or .5
		inst.Transform:SetScale(s, s, s)
		inst.AnimState:SetFinalOffset(-1)
		inst.AnimState:SetOrientation( ANIM_ORIENTATION.OnGround )
		inst.AnimState:SetLayer( LAYER_BACKGROUND )
		inst.AnimState:SetSortOrder( 3 )
	end
	inst:ListenForEvent("animover", inst.Recycle)
end,
bearger = function(inst)
	inst:set_anim(Anims.bearger)
	if inst.tp_fx_inited == nil then
		inst.tp_fx_inited = true
		inst.Transform:SetFourFaced()
		inst.AnimState:SetMultColour(1,1,1,.5)



		inst:AddComponent("combat")
		inst.components.combat:SetDefaultDamage(TUNING.BEARGER_DAMAGE)
		inst:AddComponent("tpgroundpounder")
		inst:AddComponent("groundpounder")




		local cmp = inst.components.tpgroundpounder
		cmp.destroyer = true
		cmp.damageRings = 3
		cmp.destructionRings = 4
		cmp.numRings = 5
		local cmp2 = inst.components.groundpounder
		cmp2.destroyer = true
		cmp2.damageRings = 3
		cmp2.destructionRings = 4
		cmp2.numRings = 5
	end
    WARGON.do_task(inst, 20*FRAMES, function()
	    local cmp = inst.components.tpgroundpounder
	    if inst:HasTag("tp_boss_angry") then

	    	inst.components.combat:SetDefaultDamage(75)
	    	cmp.destroyer = false
	    	cmp.tags = {"player"}
	    	cmp.noTags = nil
	    elseif inst:HasTag("tp_boss_shadow") then

			cmp = inst.components.tpgroundpounder



			cmp.noTags = inst.components.groundpounder.noTags
	    	cmp.damageRings = 0
	    	local x, y, z = inst:GetPosition():Get()
	        local ents = TheSim:FindEntities(x, y, z, 12, nil, cmp.noTags)
	        if ents then
	            for k2,v2 in pairs(ents) do
	                if v2 and v2.components.health and not v2.components.health:IsDead() and 
	                inst.components.combat:CanTarget(v2) then

	                    local dmg = inst.components.combat.defaultdamage
	                    v2.components.combat:GetAttacked(inst, dmg, nil, nil)
	                end
	            end
	        end
	    end
		cmp:GroundPound()
		WARGON.shake_camera(inst)
		inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/bearger/groundpound")
	end)
	inst:ListenForEvent("animover", inst.Recycle)
end,
bearger_line = function(inst)
	if inst.Physics == nil then
		add_physics(inst)
	end
	inst:do_task(0, function()
		inst.Physics:SetMotorVel(35, 0, 0)
		for i = 1, 2 do
			WARGON.do_task(inst, .3*i, function()

				local fx = WARGON.make_fx2("bearger", inst)
				fx:AddTag("tp_boss_shadow")
				if inst.master and inst.master:IsValid() then
					local rot = inst.master:GetRotation()
					fx.Transform:SetRotation(rot)
				end
				if i == 2 then
					inst:Recycle()
				end
			end)
		end
	end)
end,
pointer_arrow = function(inst)
	if inst.tp_fx_inited == nil then
		inst.tp_fx_inited = true
		inst:set_anim(Anims.arrow)
		local anim = inst.AnimState
		anim:SetOrientation( ANIM_ORIENTATION.OnGround )
		anim:SetLayer( LAYER_BACKGROUND )
		inst.AnimState:SetSortOrder( 4 )
		

		inst.UpdateRotation = function(inst, target)
			local x, y, z = target:GetPosition():Get()
			if inst.pos_target then
				inst:FacePoint(x,y,z)
				inst.Transform:SetRotation(inst.Transform:GetRotation() + 90 - inst.pos_target.Transform:GetRotation())
			end
		end
	end
end,
no_electric = function(inst)
	if inst.components.tpfollowimage == nil then
		WARGON.set_scale(inst, 4)
		inst:AddComponent("tpfollowimage")
		inst.components.tpfollowimage:SetImage(WARGON.resolve_img_path("tp_no_electric#"))

	end
	inst:ListenForEvent("tp_recycle", function()
		inst.components.tpfollowimage:Hide()
	end)
end,
impact = function(inst, data)
	inst:set_anim(Anims.impact)
	inst.AnimState:SetFinalOffset(-1)
	inst:ListenForEvent("animover", function()
		inst:Recycle()
	end)
	if inst.followr == nil then
		local follower = inst.entity:AddFollower()
		inst.follower = follower
	end
	if data.target then
		inst.follower:FollowSymbol(data.target.GUID, data.target.components.combat.hiteffectsymbol, 0, 0, 0 )
		inst:FacePoint(data.target:GetPosition():Get())
	end
end,
wilson_lunge = function(inst)
	inst:set_anim(Anims.wilson_lunge)
	if inst.inited == nil then
		inst.inited = true
		inst.AnimState:SetMultColour(.1, .1, 1, .8)
		inst.Transform:SetFourFaced()
	end
	WARGON.do_task(inst, .3, function()
		inst:Recycle()
	end)
end,
spore = function(inst, data)
	if data.num == 2 then
		inst:set_anim(Anims.spore)
	else
		inst:set_anim(Anims.spore_blue)
		if data.num == 3 then
			inst.AnimState:SetMultColour(1, 1, .3, 1)
		end
	end
	inst.AnimState:PushAnimation('idle_flight_loop', true)
	if data and data.owner then
		inst.owner = data.owner
	end
	if inst.Physics == nil then
		inst.Transform:SetScale(3, 3, 3)
		add_physics(inst, "char")
		inst:AddComponent("tpfollowimage")
		inst.kill = function(inst)
			inst.AnimState:PlayAnimation("land", false)
			inst:ListenForEvent("animover", function()
				inst:Recycle()
				inst:RemoveTag("tp_moving_star")
				inst.AnimState:SetMultColour(1, 1, 1, 1)
			end)
		end
	end
	inst:per_task(.1, function()
		WARGON.face_target(inst, inst.owner)
		if not inst:IsNear(inst.owner, 20) then
			inst.Transform:SetPosition(inst.owner:GetPosition():Get())
		elseif not inst:IsNear(inst.owner, 6) then
			if not inst:HasTag("tp_moving_star") then
				inst.Physics:SetMotorVel(6/3, 0, 0)
				inst:AddTag("tp_moving_star")
			end
		elseif inst:IsNear(inst.owner, 3)
		and inst:HasTag("tp_moving_star") then
			inst:RemoveTag("tp_moving_star")
			inst.Physics:SetMotorVel(0, 0, 0)
		end
	end)
end,
star = function(inst, data)
	inst:set_anim(Anims.star)
	if data and data.num == 2 then
		inst.AnimState:SetMultColour(.3, 1, .3, 1)
	elseif data and data.num == 1 then
		inst.AnimState:SetMultColour(.3, .3, 1, 1)
	end
	inst.AnimState:PushAnimation("idle_loop", true)
	if data and data.owner then
		inst.owner = data.owner
	end
	if inst.Physics == nil then
		inst.Transform:SetScale(1.5, 1.5, 1.5)
		add_physics(inst, "char")
		inst:AddComponent("tpfollowimage")
		inst.kill = function(inst)
			inst.AnimState:PlayAnimation("disapper", false)
			inst:ListenForEvent("animover", function()
				inst:Recycle()
				inst:RemoveTag("tp_moving_star")
				inst.AnimState:SetMultColour(1, 1, 1, 1)
			end)
		end
	end
	inst:per_task(.1, function()
		WARGON.face_target(inst, inst.owner)
		if not inst:IsNear(inst.owner, 20) then
			inst.Transform:SetPosition(inst.owner:GetPosition():Get())
		elseif not inst:IsNear(inst.owner, 6) then
			if not inst:HasTag("tp_moving_star") then
				inst.Physics:SetMotorVel(6/3*2, 0, 0)
				inst:AddTag("tp_moving_star")
			end
		elseif inst:IsNear(inst.owner, 3)
		and inst:HasTag("tp_moving_star") then
			inst:RemoveTag("tp_moving_star")
			inst.Physics:SetMotorVel(0, 0, 0)
		end
	end)
end,
spear_lance = function(inst)
	inst:set_anim(Anims.spear_lance)
	if inst.components.tpfxproj == nil then
		add_physics(inst, "inv")
		inst:AddComponent("tpfxproj")
		inst.components.tpfxproj:SetSpeed(25)
		inst.components.tpfxproj:SetLaunchOffset(Vector3(0, 0, 0))
		inst.components.tpfxproj:SetOnMissFn(function()
			inst:Recycle()
		end)
		inst.components.tpfxproj:SetOnHitFn(function(inst, attacker, target)
			if attacker and attacker:is_alive()
			and attacker.components.combat and target
			and attacker.components.combat:CanTarget(target) then
				target.components.combat:GetAttacked(attacker, 34, nil, "tp_fx_spear_lance")
				attacker.components.health:DoDelta(20, nil, nil, true)
			end
			inst:Recycle()
		end)
	end
end,
spear_lance_surround = function(inst)
	inst:set_anim(Anims.spear_lance)
	inst.angle = 0
	inst.dist = 0
	inst.task = inst:per_task(0, function()
		local angle = inst.angle
		local dist = inst.dist
		inst.dist = math.min(3, dist+1/30*3)
		inst.angle = angle+1/30*360
		local offset = Vector3(math.cos(angle*DEGREES)*dist, 0, math.sin(angle*DEGREES)*dist)
		inst:set_pos(offset:Get())
	end)
	if inst.func == nil then
		inst.func = function(master, owner, target)
			if owner and target and owner.components.combat
			and owner.components.combat:CanTarget(target) then

				local pt = inst:GetPosition()
				local fx = WARGON.make_fx2("spear_lance", pt)
				fx.components.tpfxproj:Throw(owner, target, owner)
				inst:Recycle()
			end
		end
	end
end,
spear_lance_array = function(inst, data)
	local owner = data.owner
	if owner then
		inst.owner = owner
		inst:per_task(0, function()
			local pt = inst.owner:GetPosition()
			inst:set_pos(pt:Get())
		end)
	end
	inst.spears = {}
	for i = 1, 4 do
		inst:do_task(i*.3, function()
			inst.spears[i] = WARGON.make_fx2("spear_lance_surround", inst, {follow=true})
			inst.spears[i]:do_task(2, function()
				if inst.spears[i] then
					if inst.owner:is_alive() then
						local target = inst.owner.components.combat and inst.owner.components.combat.target
						if target and inst.owner.components.combat:CanTarget(target) then
							inst.spears[i].func(inst, inst.owner, target)
						end
					end
				end
			end)
		end)
	end
	inst:ListenForEvent("tp_recycle", function()
		inst.owner = nil
		for k, v in pairs(inst.spears) do
			if v and v.activate then
				v:Recycle()
			end
		end
		inst.spears = nil
	end)
end,
laser = function(inst)
	inst:set_anim(Anims.laser)
	inst.AnimState:PlayAnimation("hit_"..math.random(5))
	if inst.inited == nil then
		inst.inited = true
		inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
	end
	inst:do_task(inst.AnimState:GetCurrentAnimationLength()+2*FRAMES, inst.Recycle)
end,
laserscorch = function(inst)
	inst:set_anim(Anims.laserscorch)
	inst.Transform:SetRotation(math.random() * 360)
	if inst.inited == nil then
		inst.AnimState:SetLayer(LAYER_BACKGROUND)
		inst.AnimState:SetSortOrder(3)
		inst.inited = true
	end
	inst.alpha = 0
	inst:per_task(0, function()
		inst.alpha = math.max(0, inst.alpha - (1/90) )
		inst.AnimState:SetMultColour(1, 1, 1,  inst.alpha)
		if inst.alpha == 0 then
			inst:Recycle()
		end
	end)
end,
lasertrail = function(inst)
	inst:set_anim(Anims.lasertrail)
	if inst.inited == nil then
		inst.AnimState:SetAddColour(1, 0, 0, 1)
		inst.AnimState:SetMultColour(1, 0, 0, 1)
		inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
	end
	inst:do_task(inst.AnimState:GetCurrentAnimationLength()+2*FRAMES, inst.Recycle)
end,
laser_line = function(inst)
	if inst.Physics == nil then
		add_physics(inst, "inv")
		inst.Physics:SetMotorVel(20, 0, 0)
	end
	inst:per_task(.05, function()
		WARGON.make_fx2("laser", inst)
		WARGON.make_fx2("laserscorch", inst)
		WARGON.make_fx2("lasertrail", inst)
	end, 0.1)
	inst:do_task(.5, function()
		inst:Recycle()
	end)
end,
hell_fire_ignite_line = function(inst)
	if inst.Physics == nil then
		add_physics(inst, "inv")
		inst.Physics:SetMotorVel(20, 0, 0)
	end
	inst:per_task(.1, function()
		local ents = inst:wg_finds(2, nil, {"fire", "player"})
		for k, v in pairs(ents) do
			WARGON.fire_prefab(v)
		end
	end, .1)
	inst:do_task(.5, function()
		inst:Recycle()
	end)
end,
poison_element = function(inst)
	if inst.inited == nil then
		inst.inited = true
		inst.AnimState:SetBank("poison")
		inst.AnimState:SetBuild("poison")
		inst.AnimState:PlayAnimation("level2_pre")
		inst.AnimState:PushAnimation("level2_loop", true)
		inst.AnimState:SetFinalOffset(2)
	end
end,
blood_element = function(inst)
	if inst.inited == nil then
		inst.inited = true
		inst.AnimState:SetBank("poison")
		inst.AnimState:SetBuild("poison")
		inst.AnimState:PlayAnimation("level2_pre")
		inst.AnimState:PushAnimation("level2_loop", true)
		inst.AnimState:SetMultColour(1, .1, .1, 1)
		inst.AnimState:SetFinalOffset(2)
	end
end,
fire_element = function(inst)
	inst:per_task(1, function()
		WARGON.make_fx2("dragoon_charge_fx", inst)
	end)
end,
thunder_element = function(inst)
	if inst.fx == nil then
		inst.fx = WARGON.make_fx2("tesla_held", inst)
		inst:AddChild(inst.fx)
		inst.fx.Transform:SetPosition(0, 1, 0)
	end
end,
shadow_element = function(inst)
	inst:per_task(.5, function()
		WARGON.make_fx2("shadow_dust", inst)
	end)
end,
ice_element = function(inst)
	inst:per_task(.5, function()
		local fx = WARGON.make_fx2("icespike_fx_"..math.random(4), inst)
		fx:AddTag("tp_no_sound")
	end)
end,
sheild_element = function(inst)
	inst.AnimState:SetBank("forcefield")
    inst.AnimState:SetBuild("forcefield")
    inst.AnimState:PlayAnimation("idle_loop", true)
	inst.AnimState:SetMultColour(1, 1, 1, .2)
end,
one_sparklefx = function(inst)
	inst:set_anim(Anims.sparkle)
	inst:ListenForEvent("animover", function()
		inst:Recycle()
	end)
end,
alive_sparklefx = function(inst)
	inst:per_task(1, function()
		WARGON.make_fx2("one_sparklefx", inst)
	end)
end,
small_meat = function(inst)
	inst:set_anim(Anims.small_meat)
	local s = math.random()*.6 + 0.4
	inst.Transform:SetScale(s, s, s)



	if inst.Physics == nil then
		add_physics(inst, "inv")
	end
	WARGON.do_task(inst, .5, function()
		local judge = math.random() < (inst.rand and inst.rand*.1+.05 or .05)
		if s > .9 and judge then

			WARGON.make_spawn(inst, "smallmeat")
		end

		inst:Recycle()
	end)
end,
many_small_meat = function(inst)
	WARGON.do_task(inst, 0, function()
		for i = 1, 4 do

			local fx = WARGON.make_fx2("small_meat", inst)
			local angle = PI/180 * 90 * i + math.random() * 10
			local sp = 4
			fx.Physics:SetVel(sp*math.cos(angle), 10, sp*math.sin(angle))
			if inst.rand then
				fx.rand = inst.rand
			end
		end

		inst:Recycle()
	end)
end,
minotaur_shadow = function(inst, data)
	inst:set_anim({"rook", "rook_rhino", "atk"})
	if data and data.owner then
		inst.owner = data.owner
	end
	if inst.Physics == nil then
		inst.AnimState:SetMultColour(1, 1, 1, .5)
		add_physics(inst, "inv")
		inst.Physics:SetMotorVel(TUNING.MINOTAUR_RUN_SPEED+3, 0, 0)
		inst.Transform:SetFourFaced()
	end
	inst.ents = {}
	inst:per_task(.1, function()
		if inst.owner then
			local ents = inst:wg_finds(4, nil, {"tp_shadow_power"})
			for k, v in pairs(ents) do
				if inst.ents[v] == nil and v ~= inst.owner then
					if v.components.combat then

						local dmg = 50
						WARGON.get_attacked(inst.owner, v, dmg, nil, nil)
						inst.ents[v] = true
					elseif v.components.workable then
						WARGON.make_fx2("collapse_small", v)
						v.components.workable:Destroy(inst.owner)
					end
				end
			end
		end
	end)
	inst:ListenForEvent("animover", function()
		inst:Recycle()
	end)



end,
minotaur_shadow_spawner = function(inst, data)
	if data and data.owner then
		inst.owner = data.owner
	end
	inst:do_task(.2, function()
		if inst.owner then
			if inst.owner.prefab == "minotaur" then
				WARGON.face_target(inst, inst.owner)
			else
				local rot = inst.owner.Transform:GetRotation()
				inst.Transform:SetRotation(rot)
			end
			local pt = inst:GetPosition()
            local angle = inst.Transform:GetRotation()
            local vals = {
                {6, -60}, {5, -30}, {4, 0}, {6, 30}, {8, 60}
            }
            for i = 1, 5 do
                local radius = vals[i][1]

                local angle2 = angle
                local rot = angle2 + vals[i][2]*PI/180
                local x = pt.x + math.cos(rot)*radius
                local z = pt.z + math.sin(rot)*radius
                local fx = WARGON.make_fx2("minotaur_shadow", Vector3(x, 0, z), {owner=inst.owner})
                fx.Transform:SetRotation(angle)
            end
		end
	end)
end,
teatree_nut_sleeping = function(inst, data)
	inst:set_anim(Anims.teatree_nut)
	if inst.Physics == nil then
		add_physics(inst)
		inst.Physics:SetMotorVel(0, 0, 5)
	end
	if data and data.target then
		inst.target = data.target
	end
	WARGON.do_task(inst, 0, function()
		local pt = inst:GetPosition()
		pt.y = 1
		inst.Transform:SetPosition(pt:Get())
	end)
	WARGON.per_task(inst, 0, function()
		if inst.target then
			inst:ForceFacePoint(inst.target:GetPosition())
		end
	end)
	WARGON.per_task(inst, .1, function()


		WARGON.make_fx2("teatree_nut_shadow", inst)
	end)
	WARGON.do_task(inst, 1.5, function()
		inst:Recycle()
	end)
end,
boss_spawn = function(inst)
	inst:do_task(0, function()
		local pos = inst:GetPosition()
		WARGON.make_fx2("shadow_spawn", pos)
		for i = 0, 4 do
			local angle = DEGREES*360/5*i
			local radius = 6
			local offset = pos + Vector3(math.cos(angle)*radius, 0, math.sin(angle)*radius)

			inst:do_task(.5*i, function()
				WARGON.make_fx2("lightning", offset)
			end)
		end

		inst:do_task(5*.5, function()
			WARGON.make_fx2("collapse_big", pos)
		end)
	end)
end,
tigershark_wave = function(inst, data)
	inst:set_anim(Anims.wave)
	inst.AnimState:PlayAnimation("idle", true)
	if data and data.owner then
		inst.owner = data.owner
	end
	if inst.Physics == nil then
		add_physics(inst, 'inv')
		inst.Transform:SetFourFaced()
		inst.Physics:SetMotorVel(5, 0, 0)
	end
	inst:per_task(.05, function()
		local wave = inst:wg_find(3, nil, {"tp_cutlass_wave"})
		if wave then
			WARGON.make_fx2("splash_water_drop", wave)
			inst:Recycle()
			wave:Recycle()
		else
			local ent = inst:wg_find(3, nil, nil, {"tp_tsunami_power"})
			if ent and inst.owner
			and inst.owner.components.combat:CanTarget(ent) then
				WARGON.get_attacked(inst.owner, ent, 20)
				if ent.components.moisture then
					ent.components.moisture:DoDelta(10)
				end
				WARGON.make_fx2("splash_water_drop", ent)
				inst:Recycle()
			end
		end
	end)
	inst:do_task(4, function()
		WARGON.make_fx2("splash_water_drop", inst)
		inst:Recycle()
	end)
end,
tigershark_wave_maker = function(inst, data)
	if data and data.owner then
		inst.owner = data.owner
	end
	inst:do_task(0, function()
		if inst.owner then
			for i = 1, 8 do
				local angle = i*360/8
				local fx = WARGON.make_fx2("tigershark_wave", inst, {owner=inst.owner})
				fx.Transform:SetRotation(angle)
			end
		end
		inst:Recycle()
	end)
end,
cutlass_wave = function(inst, data)
	inst:set_anim(Anims.wave)
	inst.AnimState:PlayAnimation("idle", true)
	if data and data.owner then
		inst.owner = data.owner
	end
	if inst.Physics == nil then
		inst:AddTag("tp_cutlass_wave")
		add_physics(inst, 'inv')
		inst.Transform:SetFourFaced()
		inst.Physics:SetMotorVel(8, 0, 0)
	end
	inst:per_task(.05, function()
		local ent = inst:wg_find(3)


		if ent and WARGON.can_enemy(inst.owner, ent) then
			WARGON.get_attacked(inst.owner, ent, 68)
			if ent.components.moisture then
				ent.components.moisture:DoDelta(10)
			end
			WARGON.make_fx2("splash_water_drop", ent)
			inst:Recycle()
		end
	end)
	inst:do_task(1.5, function()
		WARGON.make_fx2("splash_water_drop", inst)
		inst:Recycle()
	end)
end,
cutlass_wave_maker = function(inst, data)
	inst.owner = data and data.owner
	inst.target = data and data.target
	inst:do_task(0, function()
		if inst.owner and inst.target then
			WARGON.face_target(inst, inst.target)
			local rot = inst.Transform:GetRotation()
			for i = -2, 2 do
				local fx = WARGON.make_fx2("cutlass_wave", inst, {owner=inst.owner})
				fx.Transform:SetRotation(rot+30*i)
			end
		end
		inst:Recycle()
	end)
end,
proj = function(inst)
	inst:set_anim(Anims.bishop_charge)
	inst.AnimState:PlayAnimation(Anims.bishop_charge[3], true)
	inst.damage = TUNING.BISHOP_DAMAGE
	if inst.components.tpfxproj == nil then
		add_physics(inst, "inv")
		inst:AddComponent("tpfxproj")
		inst.components.tpfxproj:SetSpeed(25)
		inst.components.tpfxproj:SetLaunchOffset(Vector3(0, 0, 0))
		inst.components.tpfxproj:SetOnMissFn(function()
			inst:Recycle()
		end)
		inst.components.tpfxproj:SetOnHitFn(function(inst, attacker, target)
			if attacker and attacker:is_alive()
			and attacker.components.combat and target
			and attacker.components.combat:CanTarget(target) then
				target.components.combat:GetAttacked(attacker, inst.damage, nil, "tp_fx_spear_lance")

			end
			inst:Recycle()
		end)
	end
end,
bishop_charge = function(inst)
	inst:set_anim(Anims.bishop_charge)
	inst.AnimState:PlayAnimation(Anims.bishop_charge[3], true)
	inst.damage = TUNING.BISHOP_DAMAGE
	set_proj(inst)
end,
ancient_hulk_bullet = function(inst, data)	
	inst:set_anim(Anims.bishop_charge)
	inst.AnimState:PlayAnimation(Anims.bishop_charge[3], true)
	inst.owner = data and data.owner
	inst.scale = .1
	inst.Transform:SetScale(.1, .1, .1)
	inst:per_task(.05, function()
		inst.scale = inst.scale + .05
		inst.Transform:SetScale(inst.scale, inst.scale, inst.scale)
		if inst.scale >= 1 then
			if inst.owner and inst.owner:is_alive() then
				local target = inst.owner.components.combat.target
				if target then
					local fx = WARGON.make_fx2("bishop_charge", inst)
					fx.damage = 50
					fx.components.tpfxproj:Throw(inst.owner, target, inst.owner)
				end
			end
			inst:Recycle()
		end
	end)
end,
blunderbuss_proj = function(inst)
	inst:set_anim({"amo01", "blunderbuss_ammo", "idle"})
	inst.damage = 50
	set_proj(inst)
	inst.components.tpfxproj:SetSpeed(50)
end,
}

local tree_seeds = {
	"acorn", "pinecone", "jungletreeseed", "teatree_nut", "coconut",
}
for k, v in pairs(tree_seeds) do
	fx_fns[v.."_shadow"] = function(inst)
		inst:set_anim(Anims[v])
		inst.alpha = .7
		inst:per_task(.1, function()
			inst.AnimState:SetMultColour(1, 1, 1, inst.alpha)
			inst.alpha = inst.alpha - .3
		end)
		inst:do_task(.3, function()
			inst:Recycle()
		end)
	end
end


local other_fx_fns = {
	"shadow_dust_fx",
	"bat_fx",
	"sparkle_fx",
	"tesla_fx",
}
for k, v in pairs(other_fx_fns) do
	local fns = require("datas/"..v)
	for k2, v2 in pairs(fns) do
		fx_fns[k2] = v2
	end
end





local original_fxs = require("fx")
for k, t in pairs(original_fxs) do
    fx_fns[t.name] = function(inst)
        if not t.twofaced then
            inst.Transform:SetFourFaced()
        else
            inst.Transform:SetTwoFaced()
        end
        if type(t.anim) ~= "string" then
            t.anim = t.anim[math.random(#t.anim)]
        end
        if t.fnc and t.fntime then
            inst:DoTaskInTime(t.fntime, t.fnc)
        end
        if t.sound then
            inst:DoTaskInTime(t.sounddelay or 0, function() 
				if not inst:HasTag("tp_no_sound") then
					inst.SoundEmitter:PlaySound(t.sound) 
				end
			end)
        end
        if t.sound2 then
            inst:DoTaskInTime(t.sounddelay2 or 0, function() 
				if not inst:HasTag("tp_no_sound") then
					inst.SoundEmitter:PlaySound(t.sound2) 
				end
			end)
        end
        inst.AnimState:SetBank(t.bank)
        inst.AnimState:SetBuild(t.build)
        inst.AnimState:PlayAnimation(t.anim, false)
        if t.tint or t.tintalpha then
            inst.AnimState:SetMultColour((t.tint and t.tint.x) or (t.tintalpha or 1),(t.tint and t.tint.y)  or (t.tintalpha or 1),(t.tint and t.tint.z)  or (t.tintalpha or 1), t.tintalpha or 1)
        end

        if t.transform then
            inst.AnimState:SetScale(t.transform:Get())
        end









        if t.bloom then
            inst.bloom = true
            inst.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
        end
        inst:AddTag("FX")

        inst:ListenForEvent("animover", function() 
            if inst.bloom then inst.AnimState:ClearBloomEffectHandle() end
            inst:Recycle()
        end)
    end
end

GLOBAL.WARGON.DATA.tp_data_fx = fx_fns
