local ResourceData = Class(function(self, name, img)
	self.name = name
	self.img = img or ""
	self.book = ""
	self.limit = 19
end)

function ResourceData:SplitString(str)
	local len = string.len(str)/3
	local substr = nil
	if len > self.limit then
		substr = string.sub(str, self.limit*3+1, -1)
		str = string.sub(str, 1, self.limit*3)
	end
	return str, substr
end

function ResourceData:GetString()

	local str, substr = WARGON.split_string(self.book, self.limit)
	local sentence = str

	while substr do
		str, substr = WARGON.split_string(substr, self.limit)

		sentence = sentence.."\n"..str
	end
	return sentence
end

function ResourceData:__tostring()
	return string.format("ShelfData %s - %s", self.name, self.price)
end

local ResourceManager = Class(function(self)
    self.center = "tp_resource_transporter"
end)

function ResourceManager:Launch(slot, center)

    if center then
        center:transport(slot)
    end
end

function ResourceManager:GetItems(center)

    if center then
        local slots = center.components.container.slots
        local items = {}
        for i = 1, 12 do
            items[i] = ResourceData("tp_no_resource", 
                    {"minimap/minimap_data.xml", "xspot.png", true})
        end
        for k, v in pairs(slots) do
            if v then
                local img = v.prefab
                local atlas = v.components.inventoryitem.atlasname
                local image =  v.components.inventoryitem.imagename
                if atlas and image then
                    img = {atlas, image..".tex", true}
                end
                items[k] = ResourceData(v.prefab, img)
            end
        end
        return items
    end
end

function ResourceManager:GivePlane()
    local plane = c_find("tp_transport_plane")
    local player = GetPlayer()
    if plane and not plane:HasTag("launched") then
        local owner = plane.components.inventoryitem.owner
        if owner and owner ~= player then
            local cmp = owner.components.inventory or owner.components.container
            local p = cmp:RemoveItem(plane)
            player.components.inventory:GiveItem(p)
        end
    else

        c_give("tp_transport_plane")
    end
end

local ResourcePanelData = Class(ShelfData, function(self, center)
    ShelfData._ctor(self, "资源中心", 20)

    self.center = center
    self.manager = ResourceManager()
	self.tip = "运输火箭以到达"
	self.tip2 = ""
end)

function ResourcePanelData:Init()
    if self.center == nil then
        self.center = c_find("tp_resource_transporter")
    end
    if self.center then
        self:ClearShelf()
        local items = self.manager:GetItems(self.center)
        for k, v in pairs(items) do
            self:AddItem(v)
        end
    end
end





function ResourcePanelData:Launch(k)
    self.manager:Launch(k, self.center)
end

local resource_panel_data = ResourcePanelData()
WARGON.DATA.resource_panel_data = resource_panel_data