local not_entity_tags = {
	"falling", "FX", "NOCLICK", "DECOR", "INLIMBO", 
}
local not_enemy_tags = {
	"falling", "FX", "NOCLICK", "DECOR", "INLIMBO", "eyeturret", "companion", "player", "wall",
}

local function add_asset(assets, asset, asset_type)
	local type_tbl = {img="IMAGE", anim="ANIM", atlas="ATLAS", a_b="ATLAS_BUILD", snd="SOUND",}
	local pro_tbl = {img="images/", anim="anim/", atlas="images/", a_b="images/", snd="sound/",}
	local ext_tbl = {img=".tex", anim=".zip", atlas=".xml", a_b=".xml", snd=".fsb",}
	local a = Asset(type_tbl[asset_type], pro_tbl[asset_type]..asset..ext_tbl[asset_type])
	if asset_type == "a_b" then
		a = Asset(type_tbl[asset_type], pro_tbl[asset_type]..asset..ext_tbl[asset_type], 256)
	end
	table.insert(assets, a)




	




end

local function add_recipe(name, ingd, tab, tech, game, placer, aquatic)
	local rec_tab = GLOBAL.RECIPETABS or RECIPETABS
	local tab_tbl = {
		lig = rec_tab.LIGHT,
		tow = rec_tab.TOWN,
		far = rec_tab.FARM,
		sur = rec_tab.SURVIVAL,
		too = rec_tab.TOOLS,
		sci = rec_tab.SCIENCE,
		mag = rec_tab.MAGIC,
		ref = rec_tab.REFINE,
		war = rec_tab.WAR,
		dre = rec_tab.DRESS,
		anc = rec_tab.ANCIENT,
		nau = rec_tab.NAUTICAL,
		arc = rec_tab.ARCHAEOLOGY,
		cit = rec_tab.CITY,
		obs = rec_tab.OBSIDIAN,
	}
	local g_tech = GLOBAL.TECH or TECH
	local tech_tbl = {
		none = g_tech.NONE,
		s_1 = g_tech.SCIENCE_ONE,
		s_2 = g_tech.SCIENCE_TWO,
		s_3 = g_tech.SCIENCE_THREE,
		s_9 = {SCIENCE=9},
		m_2 = g_tech.MAGIC_TWO,
		m_3 = g_tech.MAGIC_THREE,
		a_2 = g_tech.ANCIENT_TWO,
		a_3 = g_tech.ANCIENT_THREE,
		a_4 = g_tech.ANCIENT_FOUR,
		o_2 = g_tech.OBSIDIAN_TWO,
		h_2 = g_tech.HOME_TWO,
		city = g_tech.CITY,
		lost = g_tech.LOST,
	}
	local rg_type = GLOBAL.RECIPE_GAME_TYPE or RECIPE_GAME_TYPE
	local game_tbl = {
		com = rg_type.COMMON,
		rog = rg_type.ROG,
		sw  = rg_type.SHIPWRECKED,
		ham = rg_type.PORKLAND,
	}
	local game_type = {}

	if type(game) == 'string' then
		game_type = game_tbl[game]
	elseif game == nil then
		game_type = game_tbl.com
	else
		for _, v in pairs(game) do
			table.insert(game_type, game_tbl[v])
		end
	end
	local ingredients = {}
	for i, v in pairs(ingd) do
		if i == 'hp' then i = CHARACTER_INGREDIENT.HEALTH
		elseif i == "max_hp" then i = CHARACTER_INGREDIENT.MAX_HEALTH
		elseif i == 'san' then i = CHARACTER_INGREDIENT.SANITY
		elseif i == "max_san" then i = CHARACTER_INGREDIENT.MAX_SANITY end
		if type(v) == "table" then
			local ingd = GLOBAL.Ingredient(i, v[1], v[2])
			ingd.image = v[3]
			table.insert(ingredients, ingd)
		else
			table.insert(ingredients, GLOBAL.Ingredient(i, v))
		end
	end
	local rcp = GLOBAL.Recipe(name, ingredients, tab_tbl[tab] or tab, tech_tbl[tech], game_type, placer)
	if aquatic then
		rcp = GLOBAL.Recipe(name, ingredients, tab_tbl[tab] or tab, tech_tbl[tech], game_type, placer, nil, nil, nil, true, 4)
	end



	return rcp
end

local function add_map(atlas)
	AddMinimapAtlas("images/"..atlas..".xml")
end

local function add_print(str, x)
	print(str..'-'..x)
	return x + 1
end

local function add_str(name, str1, str2, str3)
	local names = GLOBAL.STRINGS.NAMES or STRINGS.NAMES
	local generic = GLOBAL.STRINGS.CHARACTERS.GENERIC.DESCRIBE or STRINGS.CHARACTERS.GENERIC.DESCRIBE
	local desc = GLOBAL.STRINGS.RECIPE_DESC or STRINGS.RECIPE_DESC
	name = string.upper(name)
	if str1 then names[name] = str1 end
	if str2 then generic[name] = str2 end
	if str3 then desc[name] = str3 end
end

local function add_tags(inst, tags)
	for i, v in pairs(tags) do
		inst:AddTag(v)
	end
end

local function remove_tags(inst, tags)
	for i, v in pairs(tags) do
		inst:RemoveTag(v)
	end
end

local function add_listen(inst, listeners)
	for i, v in pairs(listeners) do
		inst:ListenForEvent(i, v)
	end
end

local function make_prefab(anims, float_anim, phy, shadows, faced, fn)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local snd = inst.entity:AddSoundEmitter()

	if anims then
		local bank = anims and anims[1] or ''
		local build = anims and anims[2] or ''
		local anim_name = anims and anims[3] or ''
		anim:SetBank(bank)
		anim:SetBuild(build)
		anim:PlayAnimation(anim_name)
		if float_anim then
			if MakeInventoryFloatable then
				MakeInventoryFloatable(inst, float_anim, anim_name)
			end
		end
	end
	local make_phy_tbl = {
		char = MakeCharacterPhysics,
		inv = MakeInventoryPhysics,
		obs = MakeObstaclePhysics,
	}
	if phy then
		if type(phy) == "string" then
			make_phy_tbl[phy](inst)
		else
			make_phy_tbl[phy[1]](inst, phy[2], phy[3])
		end
	end
	if shadows then
		local shadow = inst.entity:AddDynamicShadow()
		shadow:SetSize( shadows[1], shadows[2] )
	end
	if faced == 2 then
		trans:SetTwoFaced()
	elseif faced == 4 then
		trans:SetFourFaced()
	elseif faced == 6 then
		trans:SetSixFaced()
	elseif faced == 8 then
		trans:SetEightFaced()
	end

	if fn then
		fn(inst)
	end
	return inst
end

local function create_prefab(data)
	local anims = data.anim or {}
	local float_anim = anims[4]
	return make_prefab(anims, float_anim, data.phy, data.shadow, data.faced, data.fn)
end

local function make_placer(name, anims, ...)
	return MakePlacer("common/"..name.."_placer", anims[1], anims[2], anims[3], ...)
end

local function add_placer_test(placer, test_fn)





end

local function create_fx(pos, anims, loop, fn)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local snd = inst.entity:AddSoundEmitter()

	local bank = anims and anims[1] or ''
	local build = anims and anims[2] or ''
	local anim_name = anims and anims[3] or ''
	anim:SetBank(bank)
	anim:SetBuild(build)
	anim:PlayAnimation(anim_name, loop)

	if pos.GetPosition then
		trans:SetPosition(pos:GetPosition():Get())
	elseif pos.Get then
		trans:SetPosition(pos:Get())
	end

	inst:AddTag('FX')
	inst:AddTag('NOCLICK')
	inst.persists = false

	if fn then
		fn(inst)
	end

	return inst
end

local function make_fx(pos, fx_name, is_follow)
	if type(pos) == "string" and type(fx_name) == "table" then
		local tmp = pos
		pos = fx_name
		fx_name = tmp
	end
	local fx = SpawnPrefab(fx_name)
	local pt = nil
	if fx then
		if pos.GetPosition then
			fx.Transform:SetPosition(pos:GetPosition():Get())
		elseif pos.Get then
			fx.Transform:SetPosition(pos:Get())
		end
		if is_follow and pos.entity then
			pos:AddChild(fx)
			fx.Transform:SetPosition(0, 0, 0)
		end
	end
	return fx
end

local function make_light(inst, falloff, intensity, radius, colour, enable)
	local light = inst.entity:AddLight()
	light:SetFalloff(falloff)
	light:SetIntensity(intensity)
	light:SetRadius(radius)
	light:SetColour(colour[1], colour[2], colour[3])
	light:Enable(enable)
end

local function make_map(inst, img, typ)
	local minimap = nil
	if inst.MiniMapEntity then
		minimap = inst.MiniMapEntity
	else
		minimap = inst.entity:AddMiniMapEntity()
	end
	if typ then
		minimap:SetIcon( img.."."..typ )
	else
		minimap:SetIcon(img)
	end
end

local function make_burn(inst, size, ...)
	if size == 'small' then
		MakeSmallBurnable(inst, ...)
	elseif size == 'med' then
		MakeMediumBurnable(inst, ...)
	elseif size == 'large' then
		MakeLargeBurnable(inst, ...)
	elseif size == 'c_small' then
		MakeSmallBurnableCharacter(inst, ...)
	elseif size == 'c_med' then
		MakeMediumBurnableCharacter(inst, ...)
	elseif size == "c_large" then
		MakeLargeBurnableCharacter(inst, ...)
	end
end

local function burn_bait(inst, n)
	if inst.components.burnable then
		inst.components.burnable:MakeDragonflyBait(n)
	end
end

local function make_prop(inst, size)
	if size == 'small' then
		MakeSmallPropagator(inst)
	elseif size == 'med' then
		MakeMediumPropagator(inst)
	elseif size == 'large' then
		MakeLargePropagator(inst)
	end
end

local function make_free(inst, size, ...)
	if size == 'tiny' then
		MakeTinyFreezableCharacter(inst, ...)
	elseif size == 'small' then
		MakeSmallFreezableCharacter(inst, ...)
	elseif size == 'med' then
		MakeMediumFreezableCharacter(inst, ...)
	elseif size == 'large' then
		MakeLargeFreezableCharacter(inst, ...)
	elseif size == 'huge' then
		MakeHugeFreezableCharacter(inst, ...)
	end
end

local function make_blow(inst, ...)
	MakeBlowInHurricane(inst, ...)
end

local function make_poi(inst, ...)
	MakePoisonableCharacter(inst, ...)
end

local function on_water(inst, pt)
	if pt then
		return inst:GetIsOnWater(pt.x, pt.y, pt.z)
	end
	return inst:GetIsOnWater()
end

local function on_land(inst, pt)
	if pt then
		return inst:GetIsOnLand(pt.x, pt.y, pt.z)
	end
	return inst:GetIsOnLand()
end

local function is_dlc(n)
	if SaveGameIndex then
		local tbl = {
			SaveGameIndex:IsModeSurvival(),
			SaveGameIndex:IsModeShipwrecked(),
			SaveGameIndex:IsModePorkland(),
		}
		return tbl[n]
	end
end

local function can_dlc(n)
	if GLOBAL.IsDLCEnabled then
		return GLOBAL.IsDLCEnabled(n)
	else
		return IsDLCEnabled(n)
	end
end

local function find(inst, range, fn, tags, no_tags)
	return FindEntity(inst, range, fn, tags, no_tags)
end

local function finds(inst, range, tags, no_tags)
	local x, y, z
	if inst.Get then
		x, y, z = inst:Get()
	elseif inst.GetPosition then
		x, y, z = inst:GetPosition():Get()
	elseif type(inst)=="table" then
		x, y, z = inst[1], inst[2], inst[3]
	end
	if x and y and z then
		return TheSim:FindEntities(x, y, z, range, tags, no_tags)
	end
end

local function find_close(inst, tag, range)
	return GetClosestInstWithTag(tag, inst, range)
end

local function do_task(inst, ...)
	return inst:DoTaskInTime(...)
end

local function per_task(inst, ...)
	return inst:DoPeriodicTask(...)
end

local function around_land(inst, dist, attemp)
	local theta = math.random() * 2 * PI
    local pt = inst:GetPosition()

    local radius = dist
    local offset = FindWalkableOffset(pt, theta, radius, attemp or 12, true)
    if offset then
        local pos = pt + offset
        if WARGON.get_tile(pos) == GROUND.IMPASSABLE 
        or WARGON.get_tile(pos) == GROUND.INVALID then
        	return
        end
        return pos
    end
end

local function no_save(inst)
	inst.persists = false
end

local function key_down(key, fn)
	TheInput:AddKeyDownHandler(key, fn)
end

local function key_up(key, fn)
	TheInput:AddKeyUpHandler(key, fn)
end

local function area_dmg(inst, range, attacker, dmg, reason, fn, target)
	local ents = WARGON.finds(inst, range, nil, WARGON.not_enemy_tags)

	local attacker = attacker or inst
	for i, v in pairs(ents) do
		if attacker then
			if target ~= v and not v.components.follower 
			or v.components.follower.leader ~= attacker then
				if v.components.combat and v.components.health
				and attacker.components.combat:CanTarget(v) then
					v.components.combat:GetAttacked(attacker, dmg, inst, reason)
					if fn then fn(inst, attacker, v) end
				end
			end
		end
	end
end

local function area_dmg2(inst, data)
	WARGON.area_dmg(data.range, data.attacker, data.dmg, data.reason, data.fn, data.target)
end

local function other_area_dmg(inst, range, attacker, dmg, reason, fn)
	local ents = WARGON.finds(inst, range, nil, WARGON.not_entity_tags)
	local attacker = attacker or inst
	for i, v in pairs(ents) do
		if attacker then
			if not v.components.follower or v.components.follower.leader ~= attacker then
				if v.components.combat and v.components.health
				and attacker.components.combat:CanTarget(v) then
					v.components.combat:GetAttacked(attacker, dmg, inst, reason)
					if fn then fn(inst, attacker, v) end
				end
			end
		end
	end
end

local function can_enemy(inst, target)
	if not inst:has_tags( WARGON.not_enemy_tags) then

		if not target.components.follower or target.components.follower.leader ~= inst then
			if target:IsValid() and target.components.combat 
			and target.components.health
			and not target.components.health:IsDead()
			and inst.components.combat
			and inst.components.combat:CanTarget(target) then
				return true
			end
		end
	end
end

local function fire_prefab(target, attacker)
	if target.components.burnable and not target.components.burnable:IsBurning() then
        if target.components.freezable and target.components.freezable:IsFrozen() then           
            target.components.freezable:Unfreeze()            
        else            
            if target.components.fueled and target:HasTag("campfire") and target:HasTag("structure") then

                local fuel = SpawnPrefab("cutgrass")
                if fuel then target.components.fueled:TakeFuelItem(fuel) end
            else
                target.components.burnable:Ignite(true)
            end
        end   
    end

    if target:HasTag("aquatic") and not target.components.burnable then 
        local pt = target:GetPosition()
        local smoke = SpawnPrefab("smoke_out")
        smoke.Transform:SetPosition(pt:Get())

         if target.SoundEmitter then 
            target.SoundEmitter:PlaySound("dontstarve_DLC002/common/fire_weapon_out") 
        end 
    end 

    if target.components.freezable then
        target.components.freezable:AddColdness(-1)
        if target.components.freezable:IsFrozen() then
            target.components.freezable:Unfreeze()            
        end
    end

    if target.components.sleeper and target.components.sleeper:IsAsleep() then
        target.components.sleeper:WakeUp()
    end

    if target:HasTag("player") then
    	target.components.health:DoFireDamage(4, nil, true)
    end

    if attacker then
	    target:PushEvent("attacked", { attacker = attacker, damage = 0 })
    end
end

local function frozen_prefab(target, attacker, num)
	if not target:IsValid() then
        return
    end
    
    if target.components.sleeper and target.components.sleeper:IsAsleep() then
        target.components.sleeper:WakeUp()
    end
    if target.components.burnable then
        if target.components.burnable:IsBurning() then
            target.components.burnable:Extinguish()
        elseif target.components.burnable:IsSmoldering() then
            target.components.burnable:SmotherSmolder()
        end
    end

    if attacker then
	    if target.sg ~= nil and not target.sg:HasStateTag("frozen") then
	        target:PushEvent("attacked", { attacker = attacker, damage = 0 })
	    end
    end

    if target.components.freezable then
        target.components.freezable:AddColdness(num or 1)
        target.components.freezable:SpawnShatterFX()
    end
end

local function poison_prefab(target)
	if target.components.poisonable and target:HasTag("poisonable") then
        target.components.poisonable:Poison()
    end 
end

local function sleep_prefab(target)
	if target.components.sleeper then
	    target.components.sleeper:AddSleepiness(10, TUNING.PANFLUTE_SLEEPTIME, target)
	end
end

local function shake_camera(inst)
	GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, 2, 40)
end

local function pigking_throw(inst, nug)

    local pt = Vector3(inst.Transform:GetWorldPosition()) + Vector3(0,4.5,0)
    
    nug.Transform:SetPosition(pt:Get())
    local down = TheCamera:GetDownVec()
    local angle = math.atan2(down.z, down.x) + (math.random()*60-30)*DEGREES

    local sp = math.random()*4+2
    nug.Physics:SetVel(sp*math.cos(angle), math.random()*2+8, sp*math.sin(angle))
    if nug.components.inventoryitem then
	    nug.components.inventoryitem:OnStartFalling()
    end
end

local function set_scale(inst, ...)
	local args = {...}
	if inst.Transform then
		if #args == 1 then
			local s = args[1]
			inst.Transform:SetScale(s,s,s)
		else
			inst.Transform:SetScale(...)
		end
	end
end

local function set_pos(inst, ...)
	local args = {...}
	if inst.Transform then
		if #args == 1 then
			local x = args[1]
			inst.Transform:SetPosition(x,x,x)
		else
			inst.Transform:SetPosition(...)
		end
	end
end

local function add_speed_rate(inst, ...)
	if inst.components.locomotor then
		inst.components.locomotor:AddSpeedModifier_Mult(...)
	end
end

local function remove_speed_rate(inst, ...)
	if inst.components.locomotor then
		inst.components.locomotor:RemoveSpeedModifier_Mult(...)
	end
end

local function add_dmg_rate(inst, ...)
	if inst.components.combat then
		inst.components.combat:AddDamageModifier(...)
	end	
end

local function remove_dmg_rate(inst, ...)
	if inst.components.combat then
		inst.components.combat:RemoveDamageModifier(...)
	end
end

local function add_hunger_rate(inst, ...)
	if inst.components.hunger then
		inst.components.hunger:AddBurnRateModifier(...)
	end
end

local function remove_hunger_rate(inst, ...)
	if inst.components.hunger then
		inst.components.hunger:RemoveBurnRateModifier(...)
	end
end

local function add_san_rate(inst, ...)
	if inst.components.sanity then
		inst.components.sanity:AddRateModifier(...)
	end
end

local function remove_san_rate(inst, ...)
	if inst.components.sanity then
		inst.components.sanity:RemoveRateModifier(...)
	end
end

local function burn_save(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
		data.burnt = true
	end
end

local function burn_load(inst, data)
	if data and data.burnt and inst.components.burnable then
		inst.components.burnable.onburnt(inst)
	end
end

local function get_tile(pt)
	return GetGroundTypeAtPosition(pt)
end

local function can_deploy(inst, pt, range, no_tags, dist)
	local ents = WARGON.finds(pt, range, nil, no_tags)
	local min = dist or inst.components.deployable and inst.components.deployable.min_spacing
	for k, v in pairs(ents) do
		if v ~= inst and v:IsValid() and v.entity:IsVisible()
		and not v.components.placer and v.parent == nil then
			if distsq(Vector3(v:GetPosition():Get()), pt) < min*min then
				return false
			end
		end
	end
	return true
end

local function seed_save(inst, data)
	if inst.growtime then
        data.growtime = inst.growtime - GetTime()
    end
end

local function seed_load(inst, data)
	if data and data.growtime then
		WARGON.TREE.treeseed_plant(inst, data.growtime, inst.tree)
    end
end

local function is_monster(inst)
	return inst:HasTag("monster") or inst:HasTag("hound") 
		or inst:HasTag("spider_monkey")
end

local function play_snd(inst, ...)
	if inst.SoundEmitter then
		inst.SoundEmitter:PlaySound(...)
	end
end

local function get_divide_point(inst, n, r)
	local pos = nil
	if inst.GetPosition then
		pos = inst:GetPosition()
	elseif inst.Get then
		pos = inst
	end
	if pos then
		local pts = {}
		local radius = r or 1
		for i = 1, n do
			local angle = PI/180 * 360/n * i
			local pt = pos
			pt.x = pt.x + math.cos(angle)*radius
			pt.z = pt.z + math.sin(angle)*radius
			table.insert(pts, pt)
		end
		return pts
	end
end

local function get_config(key)
	return GetModConfigData(key)
end

local function has_tag(inst, tags)
	for k, v in pairs(tags) do
		if inst:HasTag(v) then
			return true
		end
	end
	return false
end

local function has_tags(inst, tags)
	for k, v in pairs(tags) do
		if not inst:HasTag(v) then
			return false
		end
 	end
 	return true
end

local function get_years()
	local days = GetClock():GetNumCycles()
	local years = math.floor(days/70) + 1
	return years
end

local function get_days()
	local clock = GetClock()
	if clock then
		local days = clock:GetNumCycles()
		return days
	end
end

local function get_dist(inst, target)
	return inst:GetDistanceSqToInst(target)
end

local function is_full_moon()
	local clock = GetClock()
	if clock:IsNight() and clock:GetMoonPhase() == "full" then
		return true
	end
end

local function face_target(inst, target)
	if target.GetPosition then
		inst:ForceFacePoint(target:GetPosition())
	elseif target.Get then
		inst:ForceFacePoint(target)
	end
end

local function can_thorns(inst, data)
	if (data.weapon == nil or (not data.weapon:HasTag("projectile") 
	and data.weapon.projectile == nil)) and data.attacker 
	and data.attacker.components.combat and data.stimuli ~= "thorns" 
	and not data.attacker:HasTag("thorny") 
	and (data.attacker.components.combat == nil 
	or (data.attacker.components.combat.defaultdamage > 0))
	and data.damage and data.damage > 0 then
		return true
	end
end

local function resolve_img_path(img)
	if img then
		if type(img) == "table" then
			if img[3] then
				return img[1], img[2]
			else
				local atlas = "images/inventoryimages/"..img[1]..".xml"
				return atlas, img[2]..".tex"
			end
		else
			local atlas = "images/inventoryimages"
			if string.find(img, "#") then
				img = string.sub(img, 1, -2)
				atlas = atlas.."/"..img
			else

				if img == "cutlichen" then
					img = "algae"
				end
				if img > "torch" then
					atlas = atlas.."_2"
				end
			end
			return atlas..".xml", img..".tex"
		end
	end
end

local function get_screen_size()
	local scr_w, scr_h = TheSim:GetScreenSize()
	return scr_w/1920
end

local function spawn_tornado(inst, target, no_attack_player)
	local function getspawnlocation(inst, target)
	    local tarPos = target:GetPosition()
	    local pos = inst:GetPosition()
	    local vec = tarPos - pos
	    vec = vec:Normalize()
	    local dist = pos:Dist(tarPos)
	    return pos + (vec * (dist * .15))
	end
	target = target or inst
	local tornado = SpawnPrefab("tornado")
	tornado:AddTag("tp_wind_attack_target")
	if no_attack_player then
		tornado:AddTag("tp_no_attack_player")
		tornado:SetStateGraph("SGtp_tornado")
		tornado:no_save()
	end
	tornado.WINDSTAFF_CASTER = inst
	local spawnPos = inst:GetPosition() + TheCamera:GetDownVec()
	local totalRadius = target.Physics and target.Physics:GetRadius() or 0.5 + tornado.Physics:GetRadius() + 0.5
	local targetPos = target:GetPosition() + (TheCamera:GetDownVec() * totalRadius)
	tornado.Transform:SetPosition(getspawnlocation(inst, target):Get())
	tornado.components.knownlocations:RememberLocation("target", targetPos)
end

local function flc_get(fn,name,maxlevel,max,level,file)
	if type(fn) ~= "function" then
		return
	end
	local maxlevel = maxlevel or 5
	local level = level or 0
	local max = max or 20
	for i=1,max,1 do
		local upname,upvalue = debug.getupvalue(fn,i)
		if upname and upname == name then
			if file and type(file) == "string" then
				local fninfo = debug.getinfo(fn)
				if fninfo.source and fninfo.source:match(file) then
					return upvalue
				end
			else
				return upvalue
			end
		end
		if level < maxlevel and upvalue and type(upvalue) == "function" then
			local upupvalue = flc_get(upvalue,name,maxlevel,max,level+1,file)
			if upupvalue then
				return upupvalue
			end
		end
	end
end

local function trans_to_boss(boss_name)
	local boss = c_find(boss_name)
	if boss then
		GetPlayer():set_pos(boss:GetPosition():Get())
	end
end

local function around_ground(pt, dist)
	local pos = FindGroundOffset(pt, math.random() * 2 * math.pi, dist, 18)
	if pos then
		local offset = pt + pos
		return offset
	end
end

local function add_inventory_item_atlas_lookup(image, atlas)
	local _, inventoryItemAtlasLookup = debug.getupvalue(GetInventoryItemAtlas, 1)
	if type(inventoryItemAtlasLookup) == "table" then
		inventoryItemAtlasLookup[image] = softresolvefilepath(atlas)
	end
end

local function throw_item(inst, item)
	local down = TheCamera:GetDownVec()
	local spawnangle = math.atan2(down.z, down.x)
	local angle = math.atan2(down.z, down.x) + (math.random()*90-45)*DEGREES
	local sp = math.random()*3+2

	if item.components.inventoryitem and not item.components.health then




		item.Transform:SetPosition(WARGON.get_pos(inst))
		item.Physics:SetVel(sp*math.cos(angle), math.random()*2+9, sp*math.sin(angle))
		item.components.inventoryitem:OnStartFalling()
	end
end

local function gift_throw(inst, loots)
	local delay = GetRandomWithVariance(1,0.8)
	local pt = inst:GetPosition()
	for k, v in pairs(loots) do
		for i = 1, v, 1 do
			inst:DoTaskInTime(delay, function(inst) 
				local down = TheCamera:GetDownVec()
				local spawnangle = math.atan2(down.z, down.x)
				local angle = math.atan2(down.z, down.x) + (math.random()*90-45)*DEGREES
				local sp = math.random()*3+2
				local item = SpawnPrefab(k)
				if item.components.inventoryitem and not item.components.health then
					local pt = Vector3(inst.Transform:GetWorldPosition()) + Vector3(2*math.cos(spawnangle), 3, 2*math.sin(spawnangle))
					inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/slotmachine_reward")
					item.Transform:SetPosition(pt:Get())
					item.Physics:SetVel(sp*math.cos(angle), math.random()*2+9, sp*math.sin(angle))
					item.components.inventoryitem:OnStartFalling()
				else
					local pt = Vector3(inst.Transform:GetWorldPosition()) + Vector3(2*math.cos(spawnangle), 0, 2*math.sin(spawnangle))
					pt = pt + Vector3(sp*math.cos(angle), 0, sp*math.sin(angle))
					item.Transform:SetPosition(pt:Get())
					SpawnPrefab("collapse_small").Transform:SetPosition(pt:Get())
				end
				
			end)
			delay = delay + 0.25
		end
	end
end

local function spawn_creatures(inst, creatures)
	local pos = inst:GetPosition()
	for k, v in pairs(creatures) do
		local offset = FindWalkableOffset(pos, math.random()*2*PI, 4 , 8, true, false)
		local pt = pos + offset
		delay = GetRandomWithVariance(1,0.8)
		inst:DoTaskInTime(delay, function() 
			SpawnPrefab("collapse_small").Transform:SetPosition(pt:Get())
			local spawn = lootdropper:SpawnLootPrefab(v, pt)



		end)
	end
end

local function absorb_ripped(target)
	if target.tp_absorb_ripped_task then
		target.tp_absorb_ripped_task:Cancel()
		target.tp_absorb_ripped_task = nil
	end
	target:AddTag("tp_absorb_ripped")
	target.components.health:RemoveAbsorbModifier("ripper_nil")
	target.tp_absorb_ripped_task = target:do_task(3, function()
		target:RemoveTag("tp_absorb_ripped")
		target.components.health:RemoveAbsorbModifier("ripper_nil")
	end)
end

local function is_kind(inst, target)
	local v = inst
	if v ~= target and ((v.prefab==target.prefab)
	or (v:HasTag("hound") and target:HasTag("hound")) 
	or (v:HasTag("spider") and target:HasTag("spider")) 
	or (v:HasTag("chess") and target:HasTag("chess"))
	or (v:HasTag("bee") and target:HasTag("bee")) )
	and (v:HasTag("werepig")==target:HasTag("werepig")) then
		return true
	end
end

local function split_string(str, words_width)
	local len = #str
	local cnt = 0
	local wid = 0
	for i = 1, len do
		local byte = string.byte(str, i)
		cnt = cnt + 1
		if byte > 127 then
			if cnt >= 3 then
				cnt = 0
				wid = wid + 2
			end
		else
			cnt = 0
			wid = wid + 1
		end
		if wid >= words_width * 2 then
			if i < len then
				return string.sub(str, 1, i), string.sub(str, i+1, -1)
			end
		end
	end
	return str
end

local function get_pos(pos, is_v)
	if pos.Get then
		if is_v then
			return pos
		end
		return pos:Get()
	elseif pos.GetPosition then
		if is_v then
			return pos:GetPosition()
		end
		return pos:GetPosition():Get()
	end
end

local function make_fx2(fn_name, pos, data)
	local fn = WARGON.DATA.tp_data_fx[fn_name]
	if fn then
		local fx = GetWorld().components.tpfxmanager:MakeFx(fn_name)
		fn(fx, data)
		fx.Transform:SetPosition(WARGON.get_pos(pos))
		if data and data.follow and pos.entity then
			pos:AddChild(fx)
			fx.Transform:SetPosition(0, 0, 0)
		end
		return fx
	end
end

local function make_light2(inst, light_type)
	local light = nil
    if inst.enity.Light then
        light = inst.Light
    else
        light = inst.entity:AddLight()
    end
    local lights = {
        miner_hat = {.4, .7, 2.5, {180/255, 195/255, 150/255}},
        lantern = {.9, .6, 5, {180/255, 195/255, 150/255}},
    }
    local data = lights[light_type]
    light:SetFalloff(data[1])
    light:SetIntensity(data[2])
    light:SetRadius(data[3])
    light:SetColour(unpack(data[4]))
    light:Enable(false)
end

local function test_fx(anims)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	trans:SetPosition(TheInput:GetWorldPosition():Get())
	anim:SetBank(anims[1])
	anim:SetBuild(anims[2])
	anim:PlayAnimation(anims[3])
end

local function enable_power(kind)
	return GetWorld().components.tppowermanager:IsEnablePower(kind)
end

local function get_attacked(attacker, target, dmg, weapon, stimuli)
	if attacker.components.combat and target.components.combat
	and attacker.components.combat:CanTarget(target) then
		target.components.combat:GetAttacked(attacker, dmg, weapon, stimuli)
	end
end

GLOBAL.WARGON = {
	add_asset 			= add_asset,
	add_recipe 			= add_recipe,
	add_map				= add_map,
	add_print 			= add_print,
	add_str 			= add_str,
	add_tags 			= add_tags,
	add_listen 			= add_listen,
	add_speed_rate		= add_speed_rate,
	add_dmg_rate		= add_dmg_rate,
	add_hunger_rate		= add_hunger_rate,
	add_san_rate 		= add_san_rate,
	remove_tags 		= remove_tags,
	remove_speed_rate 	= remove_speed_rate,
	remove_dmg_rate 	= remove_dmg_rate,
	remove_hunger_rate 	= remove_hunger_rate,
	remove_san_rate 	= remove_san_rate,
	make_prefab 		= make_prefab,
	create_prefab 		= create_prefab,
	make_placer 		= make_placer,
	make_fx 			= make_fx,
	make_spawn 			= make_fx,
	make_light			= make_light,
	make_map 			= make_map,
	make_burn 			= make_burn,  

	make_prop 			= make_prop,  

	make_free 			= make_free,  

	make_blow 			= make_blow,  
	make_poi 			= make_poi,
	set_scale 			= set_scale,
	set_pos 			= set_pos,
	on_water 			= on_water,
	on_land 			= on_land,
	is_dlc 				= is_dlc,
	can_dlc 			= can_dlc,
	find 				= find,
	finds 				= finds,
	find_close 			= find_close,
	do_task 			= do_task,
	per_task 			= per_task,
	around_land 		= around_land,  
	no_save 			= no_save,
	key_up 				= key_up,
	key_down 			= key_down,
	area_dmg 			= area_dmg,
	area_dmg2 			= area_dmg2,
	other_area_dmg 		= other_area_dmg,
	can_enemy 			= can_enemy,
	shake_camera		= shake_camera,
	fire_prefab 		= fire_prefab,
	frozen_prefab 		= frozen_prefab,
	poison_prefab 		= poison_prefab,
	sleep_prefab		= sleep_prefab,

	pigking_throw		= pigking_throw,  

	burn_save 			= burn_save, 

	burn_load 			= burn_load,  
	seed_save 			= seed_save, 
	seed_load 			= seed_load,

	burn_bait 			= burn_bait,  
	get_tile 			= get_tile,
	can_deploy 			= can_deploy,
	is_monster 			= is_monster,
	play_snd 			= play_snd,

	get_divide_point 	= get_divide_point,  
	get_config 			= get_config,

	has_tag 			= has_tag,  

	has_tags 			= has_tags,  
	get_years			= get_years,
	get_days 			= get_days,
	get_dist 			= get_dist,
	is_full_moon 		= is_full_moon,
	face_target 		= face_target,

	can_thorns 			= can_thorns,  
	resolve_img_path	= resolve_img_path,
	get_screen_size		= get_screen_size,
	spawn_tornado 		= spawn_tornado,
	flc_get 			= flc_get,
	trans_to_boss 		= trans_to_boss,
	around_ground 		= around_ground,
	add_inventory_item_atlas_lookup = add_inventory_item_atlas_lookup,
	add_placer_test 	= add_placer_test,
	gift_throw 			= gift_throw,
	spwan_creatures  	= spawn_creatures,
	not_entity_tags 	= not_entity_tags,
	not_enemy_tags 		= not_enemy_tags,
	absorb_ripped 		= absorb_ripped,
	is_kind 			= is_kind,
	split_string 		= split_string,
	get_pos 			= get_pos,
	make_fx2			= make_fx2,
	make_light2			= make_light2,
	throw_item 			= throw_item,
	test_fx 			= test_fx,
	enable_power		= enable_power,
	get_attacked 		= get_attacked,
}