require "behaviours/wander"
require "behaviours/runaway"
require "behaviours/doaction"
require "behaviours/panic"

local TpEyePlantBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local minionrange = 3.5

local function CanActOn(item)
    return item:IsOnValidGround() and 
    item:GetTimeAlive() > 1 and 
    item:IsValid() and not 
    item:HasTag("irreplaceable") and not
    (item:HasTag("lureplant") or item:HasTag("eyeplant") or item:HasTag("notarget")) and not
    (item.components.inventoryitem and (item.components.container or item.components.inventoryitem:IsHeld())) and not
    (item.components.pickable and not (item.components.pickable:CanBePicked() or item.components.pickable.caninteractwith))
end

local function harvest_item(inst)
    local target = FindEntity(inst, minionrange, function(v)
        if CanActOn(v) then
            if (v.components.crop and v.components.crop:IsReadyForHarvest()) or
            (v.components.stewer and v.components.stewer.done) or
            (v.components.dryer and v.components.dryer:IsDone()) then
                return true
            end
        end
    end)
    if target then

        WARGON.face_target(inst, target)
        local ba = BufferedAction(inst,target,ACTIONS.HARVEST)
        ba.distance = 4
        return ba
    end
end

local function pick_item(inst)
    local target = FindEntity(inst, minionrange, function(v)
        if CanActOn(v) then
            if (v.components.pickable 
            and v.components.pickable:CanBePicked() 
            and v.components.pickable.caninteractwith) then
                return true
            end
        end
    end)
    if target then
        WARGON.face_target(inst, target)
        local ba = BufferedAction(inst,target,ACTIONS.PICK)
        ba.distance = 4
        return ba
    end
end

local function pickup_item(inst)
    local target = FindEntity(inst, minionrange, function(v)
        if CanActOn(v) then
            if (v.components.inventoryitem 
            and v.components.inventoryitem.cangoincontainer 
            and not v.components.container 
            and not v.components.inventoryitem:IsHeld()) then
                return true
            end
        end
    end)
    if target then

        WARGON.face_target(inst, target)
        local ba = BufferedAction(inst,target,ACTIONS.PICKUP)
        ba.distance = 4
        return ba
    end
end

function TpEyePlantBrain:OnStart()
    local root = PriorityNode(
    {
        DoAction(self.inst, harvest_item, "Harvest"),
        DoAction(self.inst, pick_item, "Pick"),
        DoAction(self.inst, pickup_item, "Pick Up"),
    }, .25)
    
    self.bt = BT(self.inst, root)
end

return TpEyePlantBrain