require "behaviours/wander"
require "behaviours/doaction"
require "behaviours/chaseandattack"
require "behaviours/standstill"

local STOP_RUN_DIST = 10
local SEE_PLAYER_DIST = 5
local MAX_WANDER_DIST = 20
local SEE_TARGET_DIST = 6

local MAX_CHASE_DIST = 7
local MAX_CHASE_TIME = 8
local SEE_BAIT_DIST = 20

local LEASH_RETURN_DIST = 10
local LEASH_MAX_DIST = 30

local function GoHomeAction(inst)
	if inst.components.homeseeker and 
		inst.components.homeseeker.home and 
		inst.components.homeseeker.home:IsValid() then
		return BufferedAction(inst, inst.components.homeseeker.home, ACTIONS.GOHOME)
	end
end

local function HomePos(inst)
	if inst.components.homeseeker and 
		inst.components.homeseeker.home and 
		inst.components.homeseeker.home:IsValid() then
		return inst.components.homeseeker:GetHomePos()
	end
end

local TpPigKillerBrain = Class(Brain, function(self, inst)
	Brain._ctor(self, inst)
end)

function TpPigKillerBrain:OnStart()

	local clock = GetClock()

	local root = PriorityNode(
	{
		ChaseAndAttack(self.inst, MAX_CHASE_TIME),
		WhileNode(function() return clock and not clock:IsDay() end, "IsNight",
			DoAction(self.inst, GoHomeAction, "Go Home", true )),
		Leash(self.inst, HomePos, LEASH_MAX_DIST, LEASH_RETURN_DIST),
		Wander(self.inst, HomePos, MAX_WANDER_DIST),
	}, .25)
	
	self.bt = BT(self.inst, root)
		
end

return TpPigKillerBrain
