require "behaviours/follow"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/panic"
require "behaviours/doaction"
require "behaviours/leash"

local MIN_FOLLOW_DIST = 0
local MAX_FOLLOW_DIST = 12
local TARGET_FOLLOW_DIST = 6
local MAX_WANDER_DIST = 3
local SEE_TARGET_DIST = 20
local LEASH_RETURN_DIST = 20
local LEASH_MAX_DIST = 35

local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function PickItem(inst)
    local target = FindEntity(inst, SEE_TARGET_DIST, function(item)
        return item.components.inventoryitem
            and item.components.inventoryitem.owner == nil
            and inst.components.container:Has(item.prefab, 1)
    end)
    if target then
        if not inst.components.container:IsFull()
        and not inst.sg:HasStateTag("open") then
            return BufferedAction(inst, target, ACTIONS.TP_ROBOT_PICK)

        end
    end
end

function LeaderPos(inst)



    if inst.components.homeseeker then
        return inst.components.homeseeker:GetHomePos()
    end
end

local TpRobotCleanerBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)


function TpRobotCleanerBrain:OnStart()
    local root = 
    PriorityNode({

        DoAction(self.inst, PickItem, "pick", true),
        Leash(self.inst, LeaderPos, LEASH_MAX_DIST, LEASH_RETURN_DIST),


        Wander(self.inst, LeaderPos, MAX_WANDER_DIST),
    }, .25)
    self.bt = BT(self.inst, root)
end

return TpRobotCleanerBrain