local function eat_ice_recover(inst, data)
	if data.food and data.food.prefab == "ice"
	and inst:HasTag("ice_prince") then
		inst.components.health:DoDelta(10)
		inst.components.sanity:DoDelta(10)
	end
end

local function wave_happy(inst, data)
	if inst:HasTag("sea_prince") then
		inst.components.sanity:DoDelta(5)
	end
end

local function kill_guard(inst, data)

	if data and data.victim and data.victim.prefab == "pigguard"
	and data.victim.components.tppower then



		if data.victim.components.tppower:GetPowerString() == "sheild"
		and inst:HasTag("tech_prince") 
		then

			inst.components.sciencemorph:Morph("f")

		elseif data.victim.components.tppower:GetPowerString() == "ice"
		and inst:HasTag("tech_prince") 
		then

			inst.components.sciencemorph:Morph("i")
		else


		end
	end
end

local function eat_boss_loot(inst, data)
	if data.food and (data.food.prefab == "deerclops_eyeball" 
	or data.food.prefab == "tigereye" 
	or data.food.prefab == "minotaurhorn"
	or data.food.prefab == "snakebonesoup" )
	and inst:HasTag("tech_prince") then
		inst.components.sciencemorph:Morph("b")
	end
end

local function kill_recover(inst, data)
	if data.victim and data.victim.components.health then
		local max = data.victim.components.health.maxhealth
		local dt = math.min(25, math.max(5, max/10))
		inst.components.health:DoDelta(dt)
	end
end

local ScienceMorph = Class(function(self, inst)
	self.inst = inst
	if inst.Light == nil then
		inst.entity:AddLight()
	end
    inst.Light:Enable(false)
	inst.Light:SetRadius(5)
    inst.Light:SetFalloff(.5)
    inst.Light:SetIntensity(.6)
    inst.Light:SetColour(245/255,40/255,0/255)
    inst:do_task(0, function()
    	inst:RemoveTag("lightsource")
    end)
	self.builds = {
		v = "wickerbottom",
		m = "wilson",

		s = "woodlegs",  

		f = "wathgrithr",  

		i = "wendy",  

		b = "waxwell",  
		w = "esctemplate",  

	}
	self.unlock = {
		v = true,
		m = true,
		s = true,
		f = true,
		i = true,
		b = true,
		w = true,
	}
	self.cur = 'w'
	self.morph_data = {
		v = {
			tags = {"tech_prince"},
			techs = {
				science = 1,

			},
			speed = .1,
			hunger = .25,
			san = TUNING.DAPPERNESS_MED_LARGE,
			fn = function(inst, cmp)
				print("listen kill")
				inst:ListenForEvent("killed", kill_guard)
				inst:ListenForEvent("oneat", eat_boss_loot)
			end,
		},
		m = {
			tags = {"mad_prince"},
			techs = {
				magic = 2,

			},
			speed = .25,
			san = -TUNING.DAPPERNESS_SMALL,
		},
		s = {
			tags = {"sea_prince"},
			dry = 2,
			hunger = .25,

			fn = function(inst, cmp)
				inst:ListenForEvent("boostbywave", wave_happy)
			end,
		},
		f = {
			tags = {"fire_prince"},
			fn = function(inst, cmp)
				if inst.components.inventory:Has("tp_spear_rose", 1) then
					inst.components.inventory:ConsumeByName("tp_spear_rose", 1)
					c_give("tp_spear_fire_rose")
				elseif inst.components.inventory:Has("tp_spear_lance", 1) then
					inst.components.inventory:ConsumeByName("tp_spear_lance", 1)
					c_give("tp_spear_fire")
				end
			end,
			dmg = .2,
			armor = .2,
		},
		i = {
			tags = {"ice_prince"},
			fn = function(inst, cmp)
				if inst.components.inventory:Has("tp_spear_rose", 1) then
					inst.components.inventory:ConsumeByName("tp_spear_rose", 1)
					c_give("tp_spear_ice_rose")
				elseif inst.components.inventory:Has("tp_spear_lance", 1) then
					inst.components.inventory:ConsumeByName("tp_spear_lance", 1)
					c_give("tp_spear_ice")
				end
				inst:ListenForEvent("oneat", eat_ice_recover)
			end,
		},
		b = {
			tags = {"blood_prince"},
			eater_monster = true,
			fn = function(inst, cmp)
				if not cmp.tasks["blood_prince"] then
					cmp.tasks["blood_prince"] = inst:per_task(1, function()
						if not GetWorld():IsCave()
						and not GetClock():IsNight() then
							inst.components.health:DoDelta(-1)
						end
					end)
				end
				inst:PushEvent("tp_morph_ui_show", {ui="blood_prince"})
				inst:ListenForEvent("killed", kill_recover)
			end,
		},
		w = {},
	}
	self.fxs = {}
	self.tasks = {}



end)

function ScienceMorph:Unlock(state)
	if not self.unlock[state] then
		self.unlock[state] = true
		self.inst:PushEvent("tp_unlock_state", {state=state})
	end
end

function ScienceMorph:CanState(state)
	print("ScienceMorph", self.unlock[state])
	return self.unlock[state]
end

function ScienceMorph:SetOriginal(old_state)
	local tags = {
		"tech_prince", "mad_prince", "sea_prince",
		"fire_prince", "ice_prince", "blood_prince",
	}
	for k, v in pairs(tags) do
		self.inst:RemoveTag(v)
	end
	local techs = {
		"science", "magic",

	}
	for k, v in pairs(techs) do
		self.inst.components.builder[v.."_bonus"] = 0
	end
	self.inst.components.sanity.dapperness = 0
	self.inst:rm_hunger_rate("tech_prince")
	self.inst:rm_speed_rate("tech_prince")
	self.inst:rm_dmg_rate("tech_prince")
	self.inst.components.eater.monsterimmune = false
	self.inst.components.health:RemoveAbsorbModifier("tech_prince")
	if self.inst.components.tpbody then
		self.inst.components.tpbody.dry_rate = 0

	end
	if old_state == "v" then
		self.inst:RemoveEventCallback("killed", kill_guard)
		self.inst:RemoveEventCallback("oneat", eat_boss_loot)
	elseif old_state == "i" then
		self.inst:RemoveEventCallback("oneat", eat_ice_recover)
	elseif old_state == "s" then
		self.inst:RemoveEventCallback("boostbywave", wave_happy)
	elseif old_state == "b" then
		if self.tasks["blood_prince_light"] then
			self.tasks["blood_prince_light"]:Cancel()
			self.tasks["blood_prince_light"] = nil
			self.inst.Light:Enable(false)
			GetWorld().components.colourcubemanager:SetOverrideColourCube(nil)
		end
		if self.tasks["blood_prince"] then
			self.tasks["blood_prince"]:Cancel()
			self.tasks["blood_prince"] = nil
		end
		self.inst:PushEvent("tp_morph_ui_hide", {ui="blood_prince"})
		self.inst:RemoveEventCallback("killed", kill_recover)
	end
end

function ScienceMorph:WithChange(old_state)
	local data = self.morph_data[self.cur]
	self:SetOriginal(old_state)
	if data.tags then
		for k, v in pairs(data.tags) do
			self.inst:AddTag(v)
		end
	end
	if data.techs then
		for k, v in pairs(data.techs) do
			self.inst.components.builder[k.."_bonus"] = v
		end
	end
	if data.hunger then
		self.inst:add_hunger_rate("tech_prince", data.hunger)
	end
	if data.speed then
		self.inst:add_speed_rate("tech_prince", data.speed)
	end
	if data.dmg then
		self.inst:add_dmg_rate("tech_prince", data.dmg)
	end
	if data.san then
		self.inst.components.sanity.dapperness = data.san
	end
	if data.eater_monster then
		self.inst.components.eater.monsterimmune = true
	end
	if data.dry and self.inst.components.tpbody then
		self.inst.components.tpbody.dry_rate = data.dry
	end
	if data.armor and self.inst.components.tpbody then

		self.inst.components.health:AddAbsorbModifier("tech_prince", data.armor)
	end
	if data.fn then
		data.fn(self.inst, self)
	end
end

function ScienceMorph:Morph(state)
	if self.inst:HasTag("tp_no_morph") then
		return
	end
	local old = self.cur
    if not state then



		self.cur = "v"
		if old ~= "w" then
			self.cur = "w"
		end
		if self.inst.components.driver:GetIsDriving() then
			self.cur = "s"
		end





		local clock = GetWorld().components.clock
	    if (clock:IsNight() 
	    and clock:GetMoonPhase() == "full") then
	    	self.cur = "m"
	    end
	else
    	self.cur = state
    end
	self.inst:DoTaskInTime(.5, function()
		self:SetBuild(old)
	end)
end

function ScienceMorph:UnMad()
	if self.cur == "m" then
		self.inst.sg:GoToState("science_morph2")
		self:Morph("w")
	end








end

local function fly_do_trail(inst)
    local owner = inst
    if not owner.entity:IsVisible() then
        return
    end

    local x, y, z = owner.Transform:GetWorldPosition()
    if owner.sg ~= nil and owner.sg:HasStateTag("moving") then
        local theta = -owner.Transform:GetRotation() * DEGREES
        local speed = owner.components.locomotor:GetRunSpeed() * .1
        x = x + speed * math.cos(theta)
        z = z + speed * math.sin(theta)
    end
	local map = GetWorld().Map
    local offset = FindValidPositionByFan(
        math.random() * 2 * PI,
         .5 + math.random() * .5,
        4,
        function(offset)
            local pt = Vector3(x + offset.x, 0, z + offset.z)
            return map:GetTileAtPoint(pt:Get())	
                and #TheSim:FindEntities(pt.x, 0, pt.z, .7, { "shadowtrail" }) <= 0 
        end
    )
    if offset ~= nil then
		SpawnPrefab("tp_shadow_fx").Transform:SetPosition(x + offset.x, 0, z + offset.z)
    end
end

function ScienceMorph:SetFx()
	if self.fx then
		self.fx:Remove()
		self.fx = nil
	end
	if self.fx_task then
		self.fx_task:Cancel()
		self.fx_task = nil
	end
	if self.cur == "v" then
		self.fx = SpawnPrefab("tp_sparkle_fx")
		self.fx.entity:AddFollower()
		self.fx.Follower:FollowSymbol(self.inst.GUID, "swap_hat", 0, 0, 0)
	elseif self.cur == "m" then
		self.fx_task = WARGON.per_task(self.inst, 6*FRAMES, fly_do_trail, 2*FRAMES)
	end
end

function ScienceMorph:GetBuild()
	return self.builds[self.cur]
end

function ScienceMorph:SetBuild(old_state)
	if self:CanState(self.cur) then
		self.inst.AnimState:SetBuild(self.builds[self.cur])
		self:WithChange(old_state)
		self.inst:PushEvent("tp_morph", {cur=self.cur})
		print("ScienceMorph", self.cur)
	end
end

function ScienceMorph:OnSave()
	local unlock = deepcopy(self.unlock)
	return {
		cur = self.cur,
		unlock = unlock,
	}
end

function ScienceMorph:OnLoad(data)
	if data then
		if data.unlock then
			self.unlock = deepcopy(data.unlock)
		end
		self:Morph(data.cur)
	end
end

return ScienceMorph