local function on_hit_other(inst, data)
	local target = data and data.target
	if target then
		local weapon = inst:get_equip_item("hand")
		local cmp = weapon and weapon.components.tpenchantment
		if cmp then
			local attr = cmp.attribute
			if attr.redgem > 0 then
				WARGON.fire_prefab(target, nil)
			end
			if attr.bluegem > 0 then
				local cold = math.ceil(attr.bluegem/10)
				WARGON.frozen_prefab(target, nil, cold)
			end
			if attr.orangegem > 0 then
				if inst.components.health then
					inst.components.health:DoDelta(attr.orangegem)
				end
			end
			if attr.greengem > 0 then
				WARGON.poison_prefab(target, inst)
			end
		end
	end
end

local attributes = {
	redgem = 0,
	bluegem = 0,
	purplegem = 0,
	orangegem = 0,
	greengem = 0,
	yellowgem = 0,
}

local TpEnchantment = Class(function(self, inst)
	self.inst = inst
	self.type = "hand"


	self.attribute = deepcopy(attributes)
	self.insulation = nil
	self.inst:ListenForEvent("equipped", function(inst, data)
		if data and data.owner then
			if self.type == "hand" then
				data.owner:ListenForEvent("onhitother", on_hit_other)
			else
				self:StartRegenTask(data.owner)
			end
			self:TurnOnLight()
		end
	end)
	self.inst:ListenForEvent("unequipped", function(inst, data)
		if data and data.owner then
			if self.type == "hand" then
				data.owner:RemoveEventCallback("onhitother", on_hit_other)
			else
				self:StopRegenTask()
			end
		end
	end)
	self.inst:ListenForEvent("ondropped", function(inst, data)
		self:TurnOffLight()
	end)
	self.inst:ListenForEvent("onputininventory", function(inst, data)
		self:TurnOffLight()
	end)
end)

function TpEnchantment:DeltaAttribute(k, amount)
	local val = self.attribute[k]
	self:SetAttribute(k, val+amount)
end

function TpEnchantment:SetAttribute(k, v)
	self.attribute[k] = v
	self.inst:PushEvent("tp_enchantment", {attr=k, val=v})




	if self.attribute.yellowgem > 0 then



		if self.light_fx == nil then
			local fx = CreateEntity()
			fx.entity:AddTransform()
			fx.entity:AddLight()
			fx.Light:SetColour(180/255, 195/255, 150/255)
		    fx.Light:SetFalloff(.9)
		    fx.Light:Enable(true)
			fx:AddTag("FX")
		    self.inst:AddChild(fx)
		    self.light_fx = fx
		end
		local mult = math.min(6, self.attribute.yellowgem)
		self.light_fx.Light:SetIntensity(Lerp(0.1*mult, 0.15*mult, 1))
	    self.light_fx.Light:SetRadius(Lerp(3/4*mult, 5/4*mult, 1))
	end

	if self.inst.components.equippable
	and self.inst.components.equippable.equipslot ~= EQUIPSLOTS.HANDS then


		if self.attribute.redgem > 0 or self.attribute.bluegem > 0 then
			if self.inst.components.insulator == nil then
				self.inst:AddComponent("insulator")
				self.inst.components.insulator:SetInsulation(60)
			end
			if k == "redgem" then
				self.inst.components.insulator:SetWinter()
				self.insulation = "winter"
			elseif k == "bluegem" then
				self.inst.components.insulator:SetSummer()
				self.insulation = "summer"
			end
		end

		if self.attribute.greengem > 0 then

			self.inst.components.equippable.poisonblocker = true
			if v >= 10 then
				self.inst.components.equippable.poisongasblocker = true
			end
		end
	end
end

function TpEnchantment:StartRegenTask(owner)
	if self.regen_task == nil then
		local time = math.max(1, 60/self.attribute.orangegem)
		local amount = 1
		if self.attribute.orangegem > 60 then
			local over = self.attribute.orangegem - 60
			amount = 1 + over/60
		end
		self.regen_task = self.inst:per_task(time, function()
			if owner and owner.components.health then
				owner.components.health:DoDelta(amount, "tp_enchantment", true)
			end
		end)
	end
end

function TpEnchantment:StopRegenTask()
	if self.regen_task then
		self.regen_task:Cancel()
		self.regen_task = nil
	end
end

function TpEnchantment:TurnOnLight()
	if self.light_fx then
		self.light_fx.Light:Enable(true)
	end
end

function TpEnchantment:TurnOffLight()
	if self.light_fx then
		self.light_fx.Light:Enable(false)
	end
end

function TpEnchantment:OnSave()







	return {
		attribute = self.attribute,
		insulation = self.insulation,
	}
end

function TpEnchantment:OnLoad(data)
	if data then
		if data.attribute then
			for k, v in pairs(data.attribute) do
				self:SetAttribute(k, v)
			end
		end
		self.insulation = data.insulation
		if self.insulation and self.inst.components.insulator then
			if self.insulation == "winter" then
				self.inst.components.insulator:SetWinter()
			elseif self.insulation == "summer" then
				self.inst.components.insulator:SetSummer()
			end
		end
	end
end

return TpEnchantment