local TpEnergy = Class(function(self, inst)
	self.inst = inst
	self.max = 100
	self.current = self.max
	self.rate = 1
	self.period = 1
	self.burn_modifier = {}
	self.fuel_rate = 1



end)

function TpEnergy:DoDelta(delta)
	local old = self.current
	self.current = math.max(0, math.min(self.max, self.current+delta))

	if self.current > 0 and old <= 0 then
		if self.income then
			self.income(self.inst)
		end
	end
	if self.current <= 0 and old > 0 then
		if self.run_out then
			self.run_out(self.inst)
		end
	end
	if self.current <= 0 then
		if self.empty then
			self.empty(self.inst)
		end
	end
	if self.current > 0 then
		if self.have then
			self.have(self.inst)
		end
	end
	if self.current > old then
		if self.up then
			self.up(self.inst)
		end
	end
	if self.current < old then
		if self.down then
			self.down(self.inst)
		end
	end

	self.inst:PushEvent("tp_energy_delta", {
		old_p = old/self.max,
		new_p = self.current/self.max,
	})

	if self.inst.components.tpvehicle then
		local owner = self.inst.components.tpvehicle.owner
		if owner then
			owner:PushEvent("tp_vehicle_energy_delta", {
				old_p = old/self.max,
				new_p = self.current/self.max,
				max = self.max,
			})
		end
	end
end

function TpEnergy:TakeFuel(item)
	local fuel = item.components.tpenergysource.fuel
	self:DoDelta(fuel*self.fuel_rate)
end

function TpEnergy:Start()
	if self.task == nil then
		self.task = WARGON.per_task(self.inst, self.period, function()
			self:DoDelta(-self.rate*self:GetBurnModifier())
		end)
		print("TpEnergy Start")
	end
end

function TpEnergy:Stop()
	if self.task then
		self.task:Cancel()
		self.task = nil
		print("TpEnergy Stop")
	end
end

function TpEnergy:IsEmpty()
	return self.current <= 0
end

function TpEnergy:SetMax(max)
	self.max = max
	self.current = self.max
end

function TpEnergy:GetPercent()
	return self.current/self.max
end

function TpEnergy:SetPercent(p)
	local dt = p*self.max - self.current
	self:DoDelta(dt)
end

function TpEnergy:SetRate(rate, period)
	self.rate = rate
	if period then
		self.period = period
	end
end

function TpEnergy:AddBurnModifier(key, mod)
	self.burn_modifier[key] = mod
end

function TpEnergy:RemoveBurnModifier(key, mod)
	self.burn_modifier[key] = nil
end

function TpEnergy:GetBurnModifier()
	local mod = 0
	for k, v in pairs(self.burn_modifier) do
		mod = mod + v
	end
	return mod
end

function TpEnergy:RemoveAllBurnModifier()
	for k, v in pairs(self.burn_modifier) do
		self.burn_modifier[k] = nil
	end
end

function TpEnergy:Toggle(key, mod, on, off)
	local player = self.inst.components.tpvehicle.owner
	if not (player and player.components.talker) then
		player = GetPlayer()
	end
	if self:IsEmpty() then
		player.components.talker:Say("燃料不足")
	else
		if self.burn_modifier[key] then
			off(self.inst, player)
			self:RemoveBurnModifier(key)
		else
			on(self.inst, player)
			self:AddBurnModifier(key, mod)
		end
	end
end

function TpEnergy:OnSave()
	return {current=self.current}
end

function TpEnergy:OnLoad(data)
	if data and data.current then
		self.current = data.current
		self:DoDelta(0)
	end
end

function TpEnergy:GetWargonString()
	return string.format("能量: %d/%d", self.current, self.max)
end

return TpEnergy