local TpFxGroup = Class(function(self, prefab)

    self.fxs = {}
    self.prefab = prefab
    self.inactivate_fx_id = 1
end)

function TpFxGroup:Recycle(fx)
    fx:PushEvent("tp_recycle")
    fx:RemoveAllEventCallbacks()
    fx:CancelAllPendingTasks()
    fx:KillTasks()
    fx:RemoveFromScene()
    fx.activate = false
    self.inactivate_fx_id = fx.fx_id
end

function TpFxGroup:CreateFx()
    local fx = SpawnPrefab("tp_fx_normal")
    fx.fx_group = self
    fx.Recycle = function(fx)
        if fx.fx_group then
            fx.fx_group:Recycle(fx)
        end
    end
    fx:ListenForEvent("onremove", function()

        if fx.fx_id then
            table.remove(self.fxs, fx.fx_id)

            self.inactivate_fx_id = math.max(1, self.inactivate_fx_id - 1)
        end
    end)
    return fx
end

function TpFxGroup:GetInactivateId()
    local len = #self.fxs
    if len > 0 then
        for i = self.inactivate_fx_id, len do
            if not self.fxs[i].activate then
                self.inactivate_fx_id = math.min(len, i + 1)
                return i
            end
        end
        for i = 1, self.inactivate_fx_id do
            if not self.fxs[i].activate then
                self.inactivate_fx_id = math.min(len, i + 1)
                return i
            end
        end
    end

    local fx = self:CreateFx()
    table.insert(self.fxs, fx)
    fx.fx_id = len + 1
    return fx.fx_id
end

function TpFxGroup:MakeFx()
    local id = self:GetInactivateId()
    local fx = self.fxs[id]
    fx.activate = true
    fx:ReturnToScene()
    return fx
end

local TpFxManager = Class(function(self, inst)
    self.inst = inst
    self.groups = {}
end)

function TpFxManager:MakeFx(group)
    if self.groups[group] == nil then
        self.groups[group] = TpFxGroup(group)
    end
    local fx_group = self.groups[group]
    local fx = fx_group:MakeFx()

    return fx
end

return TpFxManager