local TpOrnament = Class(function(self, inst)
    self.inst = inst
    self.level = 1
    self.core = {}
    self.core_num = 0
    self.inst:ListenForEvent("equipped", function(inst, data)
        self:OnEquip(data)
    end)
    self.inst:ListenForEvent("unequipped", function(inst, data)
        self:OnUnequip(data)
    end)
end)

local function lightning_strike(inst, data)
    if inst.components.tpbuff and math.random() < .66 then
        inst.components.tpbuff:AddBuff("tp_armor_thunder")
    end
end

local function hunger_delta(inst, data)
    local p = inst.components.hunger:GetPercent()
    inst:add_hunger_rate("tp_ornament_brave", p*2)
    inst:add_dmg_rate("tp_ornament_brave", p*.4)
    inst.components.health:AddAbsorbModifier("tp_ornament_brave", p*.6)
end

function TpOrnament:OnEquip(data)
    local owner = data.owner
    if self.core["firm"] then
        owner:AddTagNum("not_hit_stunned", 1)
    end
    if self.core["thunder"] then
        owner:ListenForEvent("tp_lightning_strike", lightning_strike)
    end
    if self.core["venting"] then
        owner:AddTag("venting")
    end
    if self.core["health"] then
        if owner.components.tpbody then
            owner.components.tpbody:AddMaxHealthModifier("tp_ornament_health", 100, not self.is_load)
        end
    end
    if self.core["sanity"] then
        if owner.components.tpbody then
            owner.components.tpbody:AddMaxSanityModifier("tp_ornament_sanity", 100, not self.is_load)
        end
    end
    if self.core["hunger"] then
        if owner.components.tpbody then
            owner.components.tpbody:AddMaxHungerModifier("tp_ornament_hunger", 100, not self.is_load)
        end
    end
    if self.core["titan"] then
        if owner.components.tpbody then
            owner.components.tpbody:AddScaleModifier("tp_ornament_titan", .3)
        end
    end





    self.is_load = nil
end

function TpOrnament:OnUnequip(data)
    local owner = data.owner
    if self.core["firm"] then
        owner:AddTagNum("not_hit_stunned", -1)
    end
    if self.core["thunder"] then
        owner:RemoveEventCallback("tp_lightning_strike", lightning_strike)
    end
    if self.core["fog"] then
        owner:RemoveTag("venting")
    end
    if self.core["health"] then
        if owner.components.tpbody then
            owner.components.tpbody:RemoveMaxHealthModifier("tp_ornament_health", true)
        end
    end
    if self.core["sanity"] then
        if owner.components.tpbody then
            owner.components.tpbody:RemoveMaxSanityModifier("tp_ornament_sanity", true)
        end
    end
    if self.core["hunger"] then
        if owner.components.tpbody then
            owner.components.tpbody:RemoveMaxHungerModifier("tp_ornament_hunger", true)
        end
    end
    if self.core["titan"] then
        if owner.components.tpbody then
            owner.components.tpbody:RemoveScaleModifier("tp_ornament_titan")
        end
    end




end

function TpOrnament:TakeDamage(damage, attacker, weapon)
    local owner = self.inst.components.equippable and self.inst.components.equippable.owner
    if self.core["armor"] then
        damage = damage - 5
    end
    if self.core["recover"] then
        if owner and math.random() < .66 then
            if owner.components.health
            and owner.components.health:GetPercent()<.3 then
                owner.components.health:DoDelta(10)
            end
        end
    end
    if self.core["combat"] then
        if owner and math.random() < .2 then
            if owner.components.tpbuff then
                owner.components.tpbuff:AddBuff("tp_ballhat")
            end
        end
    end
    if self.core["ice"] then
        if math.random() < .33 then
            WARGON.frozen_prefab(attacker)
        end
    end
    if self.core["fire"] then
        if math.random() < .66 then
            WARGON.fire_prefab(attacker)
        end
    end
    if self.core["poison"] then
        if math.random() < .66 then
            WARGON.poison_prefab(attacker)
        end
    end
    if self.core["shadow"] then
        if attacker:HasTag("shadowcreature") then
            damage = damage * .1
        end
    end
    if self.core["titan"] then
        damage = damage * .2
    end
    if self.core["thorns"] then
        local data = {
            attacker = attacker,
            weapon = weapon,
            damage = damage,
        }
        if owner then
            if WARGON.can_thorns(owner, data) then
                WARGON.make_fx2("bramblefx", owner)
                WARGON.get_attacked(owner, attacker, 5, nil, "thorns")
            end
        end
    end
    if self.core["speed"] then
        if owner and owner.components.tpbuff then
            owner.components.tpbuff:AddBuff("tp_spear_speed")
        end
    end

    return damage
end

function TpOrnament:CanSetCore()

    return self.core_num < self.level
end

function TpOrnament:SetCore(core)
    if self:CanSetCore() then
        self.core[core] = true
        self.core_num = self.core_num + 1
        if core == "warm" then
            self.inst:AddComponent("insulator")
            self.inst.components.insulator:SetInsulation(TUNING.INSULATION_LARGE)
        end
        if core == "cool" then
            self.inst:AddComponent("insulator")
            self.inst.components.insulator:SetSummer()
            self.inst.components.insulator:SetInsulation(TUNING.INSULATION_LARGE)
        end
        if core == "poison_block" then
            self.inst.components.equippable.poisonblocker = true
        end
        if core == "fog" then
            self.inst:AddTag("fogproof")
            self.inst:AddTag("venting")
        end
    end
end

function TpOrnament:OnSave()
    local data = {}
    data.core = self.core
    return data
end

function TpOrnament:OnLoad(data)
    self.is_load = true
    if data then

        for k, v in pairs(data.core or {}) do
            self:SetCore(k)
        end
    end
end

function TpOrnament:GetWargonString()
    local s = "核心："
    for k, v in pairs(self.core) do
        local w = STRINGS.TP_STR.tp_cores[k]
        if w then
            s = s..w..","
        end
    end
    return s
end

return TpOrnament





