












































local trees = {}
for k, v in pairs(WARGON.PLANTABLE_ITEMS) do
    trees[v[1]] = {
        placer = "tp_"..v[2].."_placer",
        tree = v[2],
        aquatic = v[3],
    }
end
trees["dug_sapling"].tree = "mangrovetree_normal"
trees["dug_sapling"].placer = "tp_mangrovetree_normal_placer"
trees["tp_alloy"] = {
    placer = "tp_alloy_tree_placer",
    tree = "tp_alloy_tree",
}
trees["tp_alloy_red"] = {
    placer = "tp_alloy_red_tree_placer",
    tree = "tp_alloy_red_tree",
}

local function common_plantable_deploy(inst, pt)
	inst = inst.components.stackable and inst.components.stackable:Get() or inst
	inst:Remove()

    local tree = SpawnPrefab(inst.components.tpplantable.tree)
	tree:set_pos(pt:Get())
end

local function common_deploy_test(inst, pt)
	return WARGON.TREE.treeseed_test(inst, pt, 2)
end

local function common_aquatic_deploy_test(inst, pt)
	return WARGON.TREE.treeseed_test(inst, pt, 3, true)
end

local TpPlantable = Class(function(self, inst)
    self.inst = inst
    self.item = nil
    self.item_img = nil
end)

function TpPlantable:CanSet(name)
    return self.item == nil and trees[name]
end

function TpPlantable:SetItem(item)
    self:SetItemByName(item.prefab)
    if item.components.inventoryitem.atlasname
    and item.components.inventoryitem.imagename then
        self:SetItemImage({
            item.components.inventoryitem.atlasname,
            item.components.inventoryitem.imagename..".tex"
        })
    else
        self:SetItemImage(self.item)
    end
end

function TpPlantable:SetItemImage(item_img)
    self.item_img = item_img
end

function TpPlantable:SetItemByName(name)
    if not self:CanSet(name) then
        return
    end
    local data = trees[name]
    if data then
        self.item = name
        self.tree = data.tree
        self.inst:add_cmps({
            dep = {
                test = data.aquatic and common_aquatic_deploy_test or common_deploy_test, 
                deploy = common_plantable_deploy,
            }
        })
        self.inst.components.deployable.placer = data.placer

    end
end

function TpPlantable:GetImage()
    if type(self.item_img) == "table" then
        return unpack(self.item_img)
    else

        return WARGON.resolve_img_path(self.item_img)
    end
end

function TpPlantable:SetImage()
    if self.inst.components.tpfollowimage then

        local atlas, img = self:GetImage()
        local hide = nil
        if self.inst.components.inventoryitem.owner then
            hide = true
        end
        self.inst.components.tpfollowimage:SetImage(
            atlas, img, hide
        )
    end
end

function TpPlantable:OnSave()
    return {
        item = self.item,
        item_img = self.item_img,
    }
end

function TpPlantable:OnLoad(data)
    if data then
        if data.item then
            self:SetItemByName(data.item)
            self:SetItemImage(self.item)
        end
        if data.item_img then
            self:SetItemImage(data.item_img)
        end
    end
end

function TpPlantable:GetWargonString()
    local s = ""
    if self.item_img then
        s = string.format("图片:%s, %s", self:GetImage())
    end
    return s
end

return TpPlantable