local fns = {}
local function add_buff(buff)
    return function(inst, eater)
        if eater.components.tpbuff then
            eater.components.tpbuff:AddBuff(buff)
        end
    end
end
fns.blue_cap = add_buff("tp_potion_health_small")
fns.green_cap = add_buff("tp_potion_sanity_small")
fns.red_cap = function(inst, eater)
    if eater.components.tpmadvalue then
		eater.components.tpmadvalue:DoDelta(-100)
	end
end
fns.petals_evil = add_buff("tp_potion_crazy")
fns.flint = add_buff("tp_potion_dry")
fns.petals = add_buff("tp_potion_smell")
fns.cutlichen = add_buff("tp_potion_shine")
fns.meat_dried = add_buff("tp_potion_warth")
fns.iron = add_buff("tp_potion_iron")
fns.tentaclespots = add_buff("tp_potion_metal")
fns.bluegem = add_buff("tp_potion_shadow")
fns.obsidian = add_buff("tp_potion_killer")
fns.spidergland = function(inst, eater)
    if eater.components.poisonable
    and eater.components.poisonable:IsPoisoned() then
        eater.components.poisonable:Cure()
    end
    if eater.components.tpbuff then
        eater.components.tpbuff:AddBuff("tp_spear_shell")
    end
end
fns.durian = add_buff("tp_potion_warm")
fns.watermelon = add_buff("tp_potion_cool")
fns.cave_banana = add_buff("tp_potion_frozen")
fns.eggplant = add_buff("tp_potion_fire")
fns.slurper_pelt = function(inst, eater)
    local hp_p = eater.components.health:GetPercent()
    local san_p = math.max(eater.components.sanity:GetPercent(), .01)
    eater.components.health:SetPercent(san_p, "tp_potion_horror")
    eater.components.sanity:SetPercent(hp_p)
end
fns.dragonfruit = add_buff("tp_potion_holy")
fns.trunk_summer = add_buff("tp_potion_trunk")
fns.trunk_winter = add_buff("tp_potion_trunk")

local TpPotion = Class(function(self, inst)
    self.inst = inst
    self.item = nil
end)

function TpPotion:OnEat(eater)
    local fn = fns[self.item]
    if fn then
        fn(self.inst, eater)
    end
end

function TpPotion:CanSet(name)

    return self.item == nil and fns[name]
end

function TpPotion:SetItem(item)
    self:SetItemByName(item.prefab) 
    if item.components.inventoryitem.atlasname
    and item.components.inventoryitem.imagename then
        self:SetItemImage({
            item.components.inventoryitem.atlasname,
            item.components.inventoryitem.imagename..".tex"
        })
    else
        self:SetItemImage(self.item)
    end
end

function TpPotion:SetItemByName(name)
    if not self:CanSet(name) then
        return
    end
    local data = fns[name]
    if data then
        self.item = name

    end
end

function TpPotion:SetItemImage(item_img)
    self.item_img = item_img
end

function TpPotion:GetImage()
    if type(self.item_img) == "table" then
        return unpack(self.item_img)
    else

        return WARGON.resolve_img_path(self.item_img)
    end
end

function TpPotion:SetImage()
    if self.inst.components.tpfollowimage then

        local atlas, img = self:GetImage()
        local hide = nil
        if self.inst.components.inventoryitem.owner then
            hide = true
        end
        self.inst.components.tpfollowimage:SetImage(
            atlas, img, hide
        )
    end
end

function TpPotion:OnSave()
    return {
        item = self.item,
        item_img = self.item_img,
    }
end

function TpPotion:OnLoad(data)
    if data then
        if data.item then
            self:SetItem(data.item)
            self:SetItemImage(self.item)
        end
        if data.item_img then
            self:SetItemImage(data.item_img)
        end
    end
end

return TpPotion