local TpPower = Class(function(self, inst)
	self.inst = inst
	self.power = nil
	self.powered = nil
	self.enable = WARGON.CONFIG.elem
	self.limit_day = 0
	self:Init()
	self.inst:ListenForEvent("death", function(inst, data)
		if WARGON.enable_power("elements") then
			local drop = false
			if (self.inst.components.health
			and self.inst.components.health:GetMaxHealth()>=250
			and math.random()<.1) or math.random()<.01 then
				drop = true
			end
			if drop then
				local gift = "tp_alloy_red"
				local n = math.random()
				if n < .01 then
					gift = "tp_alloy_great"
				elseif n < .33 then
					gift = "tp_alloy"
				end
				local loot = SpawnPrefab(gift)
				if loot then
					WARGON.throw_item(self.inst, loot)
				end
			end
		end
	end)
end)

local colours = {
	blood = {1, 0, 0, 1},
	thunder = {0, 0, 1, 1},
	poison = {0, 1, 0, 1},
	sheild = {1, 1, 0, 1},
	shadow = {1, 0, 1, 1},
	ice = {0, 1, 1, 1},
	pure = {1, 1, 1, 1},
}

function TpPower:Init()
	local inst = self.inst
	inst:ListenForEvent("onhitother", function(inst, data)
		if self.enable and data and data.target then
			local target = data.target
			local damage = data.damage
			local attacker = inst

			if self.power == "blood" then
				if damage>0 and attacker.components.health then
					attacker.components.health:DoDelta(damage)
					target.components.health:DoTrueDelta(-damage/2)
				end
			end

			if self.power == "poison" then
				if target:HasTag("player") and math.random() < .5 then
					WARGON.poison_prefab(target)
				else
					WARGON.poison_prefab(target)
				end
			end

			if self.power == "sheild" then

				inst.components.health:SetInvincible(true, "tp_power_yellow")
				if self.sheild_fx == nil then
					self.sheild_fx = WARGON.make_fx2("sheild_element", Vector3(0,0,0))
					self.inst:AddChild(self.sheild_fx)
					self.sheild_fx.Transform:SetPosition(0, 0, 0)
				end
				if self.sheild_fx then
					self.sheild_fx.AnimState:SetMultColour(1, 1, 1, 1)
					self.sheild_fx:Show()
				end
				if self.fx then
					self.fx.AnimState:SetMultColour(1, 1, 1, 1)
				end
				if self.sheild_task == nil then
					self.sheild_task = WARGON.do_task(inst, 4, function()
						if self.power == "sheild" and self.fx then
							self.fx.AnimState:SetMultColour(1, 1, 1, .2)
						end
						if self.sheild_fx and self.power == "sheild" then
							self.sheild_fx:Hide()
						end
						inst.components.health:SetInvincible(false, "tp_power_yellow")
						if self.sheild_task then
							self.sheild_task:Cancel()
							self.sheild_task = nil
						end
					end)
				end
			end

			if self.power == "shadow" then
				local pt = target:GetPosition()
		        local st_pt =  FindWalkableOffset(pt or attacker:GetPosition(), math.random()*2*PI, 2, 3)
		        if st_pt then
		        	if attacker.SoundEmitter then
			            attacker.SoundEmitter:PlaySound("dontstarve/common/shadowTentacleAttack_1")
			            attacker.SoundEmitter:PlaySound("dontstarve/common/shadowTentacleAttack_2")            
		        	end
		            st_pt = st_pt + pt
		            local st = SpawnPrefab("shadowtentacle")

		            st.Transform:SetPosition(st_pt.x, st_pt.y, st_pt.z)
		            st.components.combat:SetTarget(target)
		        end
		    end

	        if self.power == "ice" then

				WARGON.frozen_prefab(target)
	        end
		end
	end)
	if inst.components.combat then
		self.dmg_range = {
			inst.components.combat.attackrange,
			inst.components.combat.hitrange,
		}
	end
end

local elements = {
	"blood", "thunder", "poison", 
	"sheild", "shadow", "ice",
}
function TpPower:Random()

	if WARGON.enable_power("element") then
		self:SetPower(elements[math.random(#elements)])
	else
		self:SetPower("pure")
	end
end

local fxs = {
	blood = "blood_element",
	ice = "ice_element",
	poison = "poison_element",
	shadow = "shadow_element",
	thunder = "thunder_element",
	sheild = "sheild_element",

}

function TpPower:SetPower(n, isload)

	if n then
		if type(n) == "number" then
			n = elements[n]
		end
		local old_power = self.power
		self.power = n
		if not self.enable then
			return
		end

		local colour = colours[n]
		if not self.inst:HasTag("scroll_pig_super") then
			self.inst.AnimState:SetMultColour(unpack(colour))
		end











		self.powered = true
		local inst = self.inst
		if self.power == "thunder" then
			if inst.components.inventory then
				local damage = inst.components.combat.defaultdamage
				WARGON.EQUIP.equip_temp_weapon(inst, damage, {8, 10}, "tp_bishop_charge")
			else
				inst.components.combat:SetRange(8, 10)
			end
		elseif old_power == "thunder" then
			if inst.components.inventory then
				local weapon = inst:get_equip_item("hand")
				if weapon then
					inst.components.inventory:DropItem(weapon)
				end
			else
				local range = self.dmg_range or {}
				inst.components.combat:SetRange(range[1], range[2])
			end
		end
	end
end

function TpPower:OnSave()
	return {
		power = self.power,
		powered = self.powered,
	}
end


function TpPower:OnLoad(data)
	if data then
		self.powered = data.powered
		self.power = data.power
		self.inst:do_task(0, function()
			self:SetPower(self.power)
		end)
	end
end

function TpPower:GetPowerString()



	return self.power
end

return TpPower