local spears = {
	"ice", "fire", "thunder", "poison", "shadow", "blood", "lightning",
	"earth", "speed", "gold", "combat", "beefalo", "tornado", 
	"conqueror", "potion", "shine", "trap", "rose", "heracles", 
	"perseus", "valkyrie", "hermes", "ripper", "shell", "shark",
	"gaea", "hades", "anubis_3", "ice_thunder", "fire_thunder",
}


local power = {}
for k, v in pairs(spears) do

	power["tp_spear_"..v] = false
end

local TpSpearDream = Class(function(self, inst)
	self.inst = inst
	self.power = deepcopy(power)
	self.inst.components.weapon:SetAttackCallback(
		function(inst, attacker, target, projectile)
			self:Attack(inst, attacker, target, projectile)
		end)
	self.inst:ListenForEvent("tp_owner_killed", function(inst, data)
		self:Killed(data.owner, data.victim)
	end)
	self:CreateShine()
end)

function TpSpearDream:CreateShine()
	self.shine_fx = CreateEntity()
	local light = self.shine_fx.entity:AddLight()
	light:SetFalloff(0.4)
	light:SetIntensity(.7)
	light:SetRadius(2.5)
	light:SetColour(180/255, 195/255, 150/255)
	light:Enable(false)
	self.shine_fx.entity:AddTransform()
	self.inst:AddChild(self.shine_fx)
	self.shine_fx:set_pos(0, 0, 0)
end

function TpSpearDream:CanGetItem(item)

	return power[item.prefab] ~= nil
end

function TpSpearDream:Killed(owner, victim)
	if self.power["gold"] then
		if not victim:HasTag("tp_not_reward")
		and not victim:HasTag("tp_not_reward_gold") then
			if victim:HasTag("epic") then
				c_give("oinc10", 5)
			elseif victim:HasTag("largecreature") then
				c_give("oinc10")
			elseif victim:HasTag("monster") then
				c_give("oinc")
			end
			victim:AddTag("tp_not_reward_gold")
		end
	end
	if self.power["anubis_3"] then
		local anubis = self.inst.components.container:FindItem(function(item)
			return item.prefab == "tp_spear_anubis_3"
		end)
		if anubis and anubis.components.tprecharge



		then

			if not victim:HasTag("tp_not_reward")
			and not victim:HasTag("tp_not_reward_anubis") then
				if victim:HasTag("epic") then
					anubis.level = anubis.level + 4
				elseif victim:HasTag("monster") then
					anubis.level = anubis.level + 1
				end
				victim:AddTag("tp_not_reward_anubis")
			end


		end

	end
end

local potions = {
	"health_small",
	"sanity_small",
	"crazy",
	"shine",
	"dry",
	"smell",
	"warth",
	"iron",
}

function TpSpearDream:Attack(inst, attacker, target, projectile)
	local owner = attacker
	if self.power["ice"] then
		WARGON.frozen_prefab(target)
	end
	if self.power["fire"] then
		WARGON.fire_prefab(target)
	end
	if self.power["thunder"] then
		local new_target = WARGON.find(target, 5, function(guy)
	 		return WARGON.can_enemy(owner, guy)
				and guy ~= target
	 	end)	
	    if new_target then
	        local proj = WARGON.make_spawn(target, "tp_charge_proj")
	        proj.master = owner
	        table.insert(proj.no_targets, target)
	        proj.components.tpproj:Throw(owner, new_target, owner)
	    end
	end
	if self.power["combat"] then
		if owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_combat")
		end
	end
	if self.power["poison"] then
		WARGON.poison_prefab(target)
	end
	if self.power["shadow"] then
		if target:HasTag("shadowcreature") then
			if target.components.combat then


				local dmg = 100
				if GetWorld():IsCave() then
					dmg = 300
				end
				target.components.combat:GetAttacked(attacker, dmg, inst, "tp_spear_dream")
			end
		end
	end
	if self.power["blood"] then
		owner.components.health:DoDelta(10, nil, "tp_health_equip")
	end
	if self.power["lightning"] then
		if math.random() < .33 then
	        local pos = Vector3(target.Transform:GetWorldPosition())
	        GetSeasonManager():DoLightningStrike(pos, false, target) 
		end
	end
	if self.power["speed"] then
		if owner and owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_speed")
		end
	end
	if self.power["earth"] then
		WARGON.make_fx(inst, "tp_fx_small_ground_pound")
		WARGON.make_fx(inst, "tp_fx_small_ground_pound_ring")

		local ents = inst:wg_finds(3, nil, {"FX", "NOCLICK", "DECOR", "INLIMBO", "groundpoundimmune", "player"})
		for k,v in pairs(ents) do
	        if v and v.components.workable 
	        and v.components.workable.workleft > 0 
	       	and v.components.workable.action ~= ACTIONS.NET then
        	    v.components.workable:Destroy(self.inst)
			end
	        if v and v.components.crop then
        	    v.components.crop:ForceHarvest()
			end
	    end
	    for k,v in pairs(ents) do
	        if v and v.components.health and 
	        not v.components.health:IsDead() and 
	        owner.components.combat:CanTarget(v) then

	        	v.components.combat:GetAttacked(attacker, 34, inst, "tp_spear_dream")
	        end
	    end
	end
	if self.power["shine"] then
		self.shine_fx.Light:Enable(true)
		if self.shine_task then
			self.shine_task:Cancel()
			self.shine_task = nil
		end
		self.shine_task = self.inst:do_task(20, function()
			self.shine_fx.Light:Enable(false)
		end)
	end
	if self.power["tornado"] then
		if math.random() < .15 then
			WARGON.spawn_tornado(owner, target)
		end
	end	
	if self.power["beefalo"] then
		if target and target:HasTag("beefalo") then
			if target.components.health then
				target.components.health:DoDelta(-25)
			end
		end
	end
	if self.power["conqueror"] then
		if owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_conqueror")
		end
		if owner:HasTag("tp_spear_conqueror") then
			if owner.components.health then
				owner.components.health:DoDelta(5)
			end
			if owner.components.sanity then
				owner.components.sanity:DoDelta(5)
			end
			if target.components.health then
				target.components.health:DoDelta(-5)
			end
		end
	end
	if self.power["potion"] then
		if owner.components.tpbuff then
			if math.random() < .1 then
				local buff = "tp_potion_"..potions[math.random(#potions)]
				owner.components.tpbuff:AddBuff(buff)
			end
		end
	end
	if self.power["rose"] then
		if not self.rose_trigger then
			local fx = WARGON.make_fx(target,"bramblefx")
			fx.AnimState:SetMultColour(.3, 1, .3, 1)
			WARGON.area_dmg(inst, 5, owner, 34, "tp_spear_dream")
			self.rose_trigger = true
			self.rose_task = self.inst:do_task(5, function()
				self.rose_trigger = nil
			end)
		end
	end
	if self.power["trap"] then
		if math.random() < .1 then
			if not target:wg_find(1.5, nil, {"tp_trap_teeth"})
			and target:on_land() then
				local trap = WARGON.make_spawn(target, "trap_teeth")
				trap.AnimState:SetBuild("trap_teeth_tiger")
				trap.components.dsskins.skin = "trap_teeth_tiger"
				trap.components.mine:Reset()
			end
		end
	end
	if self.power["perseus"] then
		if owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_perseus")
		end
	end
	if self.power["valkyrie"] then
		if owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_valkyrie_buff")
		end
	end
	if self.power["hermes"] then
		if math.random() < .15 then
			if target.components.lootdropper then
				target.components.lootdropper:DropSingleLoot()
			end
		end
	end
	if self.power["ripper"] then
		WARGON.absorb_ripped(target)
	end
	if self.power["shell"] then
		if owner.components.tpbuff then
			owner.components.tpbuff:AddBuff("tp_spear_shell")
		end
	end
	if self.power["shark"] then
		if math.random() < .25 then
			local fx = WARGON.make_fx(inst, "tp_fx_water_flower_line")
			WARGON.face_target(fx, target)
			fx.owner = owner
		end
	end
	if self.power["gaea"] then
		WARGON.make_fx(target, "groundpoundring_fx")
		local ents = target:wg_finds(12)
		for k, v in pairs(ents) do
			if WARGON.is_kind(v, target) then
				if WARGON.can_enemy(owner, v) then
					v.components.combat:GetAttacked(owner, 15, inst)
				end
			end
		end
	end
	if self.power["hades"] then
		local hp, san = 10, 10
		if target:HasTag("epic") then
			hp, san = 20, 20
		elseif target:HasTag("largecreature") then
			hp, san = 15, 15
		end
		owner.components.health:DoDelta(hp)
		owner.components.sanity:DoDelta(san)
		if target:HasTag("shadowcreature") 
		or (target.components.health:GetPercent() < .2) then
			target.components.combat:GetAttacked(owner, 99999, inst)
			owner.components.health:DoDelta(30)
			owner.components.sanity:DoDelta(30)
		end
	end
	if self.power["anubis_3"] then
		local anubis = self.inst.components.container:FindItem(function(item)
			return item.prefab == "tp_spear_anubis_3"
		end)
		if anubis and anubis.components.tprecharge
		and anubis.components.tprecharge:IsRecharged() then
			anubis.components.tprecharge:SetRechargeTime(anubis.cooldown)
		end

	end
	if self.power["ice_thunder"] then
		WARGON.frozen_prefab(target, nil)
		local new_target = target:wg_find(5, function(guy)
			return WARGON.can_enemy(owner, guy)
				and guy ~= target
		end)
	    if new_target then
	        local proj = WARGON.make_spawn(target, "tp_ice_charge_proj")
	        proj.master = owner
	        table.insert(proj.no_targets, target)
	        proj.components.tpproj:Throw(owner, new_target, owner)
	    end
	end
	if self.power["fire_thunder"] then
		WARGON.fire_prefab(target, nil)
		local new_target = target:wg_find(5, function(guy)
			return WARGON.can_enemy(owner, guy)
				and guy ~= target
		end)
	    if new_target then
	        local proj = WARGON.make_spawn(target, "tp_fire_charge_proj")
	        proj.master = owner
	        table.insert(proj.no_targets, target)
	        proj.components.tpproj:Throw(owner, new_target, owner)
	    end
	end
end

function TpSpearDream:CheckContainer()
	local items = {}
	for k, v in pairs(self.inst.components.container.slots) do
		if v then
			local item = string.sub(v.prefab, 10, -1)
			table.insert(items, item)
		end
	end
	return items
end

function TpSpearDream:SetPower(items)
	for k, v in pairs(self.power) do
		self.power[k] = false





	end

	for k, v in pairs(items) do
		self.power[v] = true





	end
end

function TpSpearDream:GetPower()
	local items = self:CheckContainer()
	self:SetPower(items)
end

function TpSpearDream:OnSave()
	local data = {
		power = {},
	}
	for k, v in pairs(self.power) do
		if self.power[k] then
			data.power[k] = true
		end
	end
	return data
end

function TpSpearDream:OnLoad(data)
	if data and data.power then
		self.power = deepcopy(data.power)
	end
end

function TpSpearDream:GetWargonString()
	local anubis = self.inst.components.container:FindItem(function(item)
		return item.prefab == "tp_spear_anubis_3"
	end)
	local str = ""
	if anubis and anubis.components.tprecharge then
		local time = anubis.components.tprecharge.current
		str = str.."阿努比斯冷却时间： "..time
		if anubis.components.tprecharge:IsRecharged() then
			str = str.."\n完毕"
		else
			str = str.."\n冷却中"
		end
	end
	return str
end

return TpSpearDream