local TpSuit = Class(function(self, inst)
	self.inst = inst
	self.cur_suit = nil
	self.tasks = {}
	self.fxs = {}
	self.inst:ListenForEvent("equip", function(inst, data)
		self:TriggerEquip()
	end)
	self.inst:ListenForEvent("unequip", function(inst, data)
		self:TriggerUnequip()
	end)
end)

local suits = {
	health = {
		"tp_hat_health",
		"tp_armor_health",
		"tp_spear_blood",
	},
	bag = {
		"tp_hat_bag",
		"tp_armor_bag",
		"tp_spear_bag",
	},
	cool = {
		"tp_hat_cool",
		"tp_armor_cool",
		"tp_spear_ice",
	},
	warm = {
		"tp_hat_warm",
		"tp_armor_warm",
		"tp_spear_fire",
	},
}

local fns = {
	health = {
		add = function(inst, cmp)
			if cmp.tasks["health"] == nil then
				cmp.tasks["health"] = inst:per_task(4, function()
					if inst.components.health then
						inst.components.health:DoDelta(2, true, "tp_health_equip")
					end
				end)
			end	
		end,
		rm = function(inst, cmp)
			if cmp.tasks["health"] then
				cmp.tasks["health"]:Cancel()
				cmp.tasks["health"] = nil
			end
		end,
	},
	bag = {
		add = function(inst, cmp)
			if cmp.tasks["bag"] == nil then
				cmp.tasks["bag"] = inst:per_task(1, function()
					local rate = 0
					if inst.components.inventory then
						local n = inst.components.inventory.maxslots - inst.components.inventory:NumItems()
						rate = n/100
					end
					inst:add_speed_rate("tp_suit_bag", rate)
				end)
			end
		end,
		rm = function(inst, cmp)
			if cmp.tasks["bag"] then
				cmp.tasks["bag"]:Cancel()
				cmp.tasks["bag"] = nil
			end
			inst:rm_speed_rate("tp_suit_bag")
		end,
	},
	cool = {
		add = function(inst, cmp)

			inst:AddTagNum("tp_not_hot_damage", 1)
		end,
		rm = function(inst, cmp)
			inst:AddTagNum("tp_not_hot_damage", -1)

		end,
	},
	warm = {
		add = function(inst, cmp)
			inst:AddTagNum("tp_not_cold_damage", 1)

		end,
		rm = function(inst, cmp)
			inst:AddTagNum("tp_not_cold_damage", -1)

		end,
	},
}

function TpSuit:Check()
	for k, v in pairs(suits) do
		local hat = self.inst:get_equip_item("head")
		local armor = self.inst:get_equip_item("body")
		local weapon = self.inst:get_equip_item("hand")
		if hat and armor and weapon and hat.prefab == v[1]
		and armor.prefab == v[2] and weapon.prefab == v[3] then
			print(hat, armor, weapon)
			print(k)		
			return k
		end
	end
end

function TpSuit:TriggerEquip()
	local suit = self:Check()
	self.cur_suit = suit
	if suit then
		local fn = fns[suit].add
		fn(self.inst, self)
	end
end

function TpSuit:TriggerUnequip()
	local suit = self:Check()
	if not suit and self.cur_suit then
		local fn = fns[self.cur_suit].rm
		fn(self.inst, self)
		self.cur_suit = nil
	end
end

return TpSuit