local function vehicle_push_event(inst, event)
	local vehicle = inst.components.tpvehicleowner and inst.components.tpvehicleowner.vehicle
	if vehicle then
		vehicle:PushEvent(event)
	end
end

local events = {
	"locomote",

}
local listen_fns = {
	locomote = function(inst, data)
		vehicle_push_event(inst, "locomote")
	end,
}

local TpVehicle = Class(function(self, inst)
	self.inst = inst
	self.owner = nil
	self.inst.components.combat.canbeattackedfn = function(inst, attacker)
		return self.owner ~= attacker
	end
	self.inst:ListenForEvent("death", function(inst, data)
		if self.owner then
			self:DownCar()
			if self.ex_death then
				self.ex_death(self.inst, self.owner)
			end
		end
	end)
	self.inst:AddTag("tp_not_move_target")
	self.inst:AddTag("tp_not_proj_target")
end)

function TpVehicle:UpCar(doer, is_load)
	local inst = self.inst
	if self.up_car then
		self.up_car(self.inst, doer)
	else
		if doer:is_alive() then
			doer.components.tpvehicleowner:SetVehicle(self.inst)
			self.owner = doer
			doer:AddTag("tp_not_combat")
			doer:AddTag("riding_vehicle")








			if is_load then
				doer:do_task(0, function()
					doer.HUD.controls.crafttabs:Hide()
					doer:Hide()
					doer.DynamicShadow:Enable(false)
				end)
			else
				doer.HUD.controls.crafttabs:Hide()


				doer:Hide()
				doer.DynamicShadow:Enable(false)

			end
			doer.entity:SetParent(self.inst.entity)
			doer:set_pos(0, 0, 0)
			doer.Physics:SetActive(false)

			self.inst.components.tpvehiclecontroller:Enable(true)




			if self.inst.components.tpenergy then
				self.inst.components.tpenergy:Start()
			end
			doer:PushEvent("tp_up_car", {vehicle = self.inst})
			if self.ex_up_car then
				self.ex_up_car(self.inst, doer)
			end
		end
	end
end

function TpVehicle:DownCar(target_pos)

	local doer = self.owner
	local inst = self.inst
	if self.down_car then
		self.down_car(self.inst, doer)
	else

			doer.components.tpvehicleowner:SetVehicle(nil)
			self.owner = nil








			doer:Show()
			doer.DynamicShadow:Enable(true)
			doer:RemoveTag("tp_not_combat")
			doer:RemoveTag("riding_vehicle")
			doer.HUD.controls.crafttabs:Show()


			doer.entity:SetParent(nil)
			doer.Physics:SetActive(true)


			self.inst.components.tpvehiclecontroller:Enable(false)




			if self.inst.components.tpenergy then
				self.inst.components.tpenergy:Stop()
			end

			if self.inst.components.locomotor then
				self.inst.components.locomotor:Stop()
			end
			doer:PushEvent("tp_down_car", {vehicle = self.inst})
			if self.ex_down_car then
				self.ex_down_car(self.inst, doer)
			end
			if self.jump_into and target_pos then
				local pos = self.inst:GetPosition()
				doer:set_pos(pos:Get())
				doer.sg:GoToState("tp_jump_down_car", target_pos)
			else
				local pos = WARGON.around_land(self.inst, 1)
				if pos then
					doer:set_pos(pos:Get())
				end
			end

	end
end

function TpVehicle:TransmitEvent(event, target)
	self.inst:ListenForEvent(event, listen_fns[event], target)
end

function TpVehicle:ObstructEvent(event, target)
	self.inst:RemoveEventCallback(event, listen_fns[event], target)
end

function TpVehicle:CanUpCar(doer)
	if not (doer.components.rider and doer.components.rider:IsRiding())
	and not (doer.components.driver and doer.components.driver:GetIsDriving())
	and	doer.components.tpvehicleowner 
	and not doer.components.tpvehicleowner:IsDriving()
	and self.owner == nil then
		return true
	end
end

function TpVehicle:CanDownCar()
	if self.owner and self.owner.components.tpvehicleowner
	and self.owner.components.tpvehicleowner.vehicle == self.inst 
	and not self.jump_into then
		return true
	end
end

function TpVehicle:CollectSceneActions(doer, actions, right)
	if right then
		if self:CanUpCar(doer) then
			if self.jump_into then
				table.insert(actions, ACTIONS.TP_JUMP_UP_CAR)
			else
				table.insert(actions, ACTIONS.TP_UP_CAR)
			end
		elseif self:CanDownCar() then
			table.insert(actions, ACTIONS.TP_DOWN_CAR)
		end
	end
end

function TpVehicle:OnSave()
	local data = {}
	local refs = {}
	if self.owner then
		table.insert(refs, self.owner.GUID)
		data.owner = self.owner.GUID
	end
	return data, refs
end

function TpVehicle:LoadPostPass(ents, data)
	if data.owner and ents[data.owner] then
		local owner = ents[data.owner].entity
		self:UpCar(owner, true)
	end
end

function TpVehicle:OnLoad(data)
end

return TpVehicle