local TpVehicleActionPicker = Class(function(self, inst)
	self.inst = inst
	self.pointspecialactionsfn = nil
end)

function TpVehicleActionPicker:SortActionList(actions, target, useitem)

    if #actions > 0 then
        table.sort(actions, function(l, r) return l.priority > r.priority end)
        local ret = {}

        for k,v in ipairs(actions) do
            if not target then
                table.insert(ret, BufferedAction(self.inst, nil, v, useitem))
            elseif target:is_a(EntityScript) then
                table.insert(ret, BufferedAction(self.inst, target, v, useitem))
            elseif target:is_a(Vector3) then
                local quantizedTarget = target 
                local distance = nil 


                if v == ACTIONS.DEPLOY and useitem.components.deployable then 
                    distance = useitem.components.deployable.deploydistance
                    quantizedTarget = useitem.components.deployable:GetQuantizedPosition(target)
                end

                local ba = BufferedAction(self.inst, nil, v, useitem, quantizedTarget)
                if distance then 
                    ba.action.distance = distance 
                end 
                table.insert(ret, ba)
            end
        end
        return ret
    end
end

function TpVehicleActionPicker:GetSceneActions(targetobject, right)
	local actions = {}
    local cansee = true

    if GetPlayer().components.vision 
    and not GetPlayer().components.vision.focused 
	and not GetPlayer().components.vision:testsight(targetobject) then
        cansee = false
    end
    
    for k,v in pairs(targetobject.components) do
        if v.CollectSceneActions and (cansee or v.nearsited_ok ) then
            v:CollectSceneActions(self.inst, actions, right)
        end
    end

	if targetobject.inherentsceneaction and not right then
		table.insert(actions, targetobject.inherentsceneaction)
	end

	if targetobject.inherentscenealtaction and right then
		table.insert(actions, targetobject.inherentscenealtaction)
	end

    if #actions == 0 and targetobject.components.inspectable then
        table.insert(actions, ACTIONS.WALKTO)
    end
    return self:SortActionList(actions, targetobject)
end

function TpVehicleActionPicker:GetClickActions( target_ent, position )

    local isTargetAquatic = false 
    local isCursorWet = false 
    local isBoating = false


    if position then
        isCursorWet = self.inst:GetIsOnWater(position.x, position.y, position.z)
    end 


    local interactingWithBoat = false
    
    if target_ent then 
        isTargetAquatic = target_ent:HasTag("aquatic")
    end 

    if self.leftclickoverride then
        return self.leftclickoverride(self.inst, target_ent, position)
    end

    local actions = nil




    local useitem = nil
    local equipitem = nil
    local equipitemhead = nil
    local boatitem = nil

    local passable = true
    if not self.ground then
        self.ground = GetWorld()
    end

    if position and self.ground and self.ground.Map then
        local tile = self.ground.Map:GetTileAtPoint(position.x, position.y, position.z)
        passable = tile ~= GROUND.IMPASSABLE
    end


    if target_ent then

        local target = target_ent
        if target_ent.playerpickerproxy then
            target = target_ent.playerpickerproxy
        end







        
        if actions == nil or #actions == 0 then
			actions = self:GetSceneActions(target)
        end
    end
    

    if position then
        
    end

    if not actions and position and not target_ent and passable then
		
    end

    return actions or {}
end

function TpVehicleActionPicker:GetRightClickActions( target_ent, position, leftaction )
    local isTargetAquatic = false 
    local isCursorWet = false 

    local isBoating = false

    if position then 
        isCursorWet = self.inst:GetIsOnWater(position.x, position.y, position.z)
    end


    local interactingWithBoat = false

    if self.rightclickoverride then
        return self.rightclickoverride(self.inst, target_ent, position)
    end

    local actions = nil




    local useitem = nil
    local equipitem = nil
    local equipitemhead = nil
    local boatitem = nil

    local passable = true
    if not self.ground then
        self.ground = GetWorld()
    end

    if position and self.ground and self.ground.Map then
        local tile = self.ground.Map:GetTileAtPoint(position.x, position.y, position.z)
        passable = tile ~= GROUND.IMPASSABLE
    end


    if target_ent then

        if not actions then
            actions = self:GetSceneActions(target_ent, true)
        end
    end















    if (actions == nil or #actions <= 0) and target_ent == nil and passable then
        actions = self:GetPointSpecialActions(position, useitem, true)
    end

    return actions or {}
end

function TpVehicleActionPicker:GetPointSpecialActions(pos, useitem, right)
    return self.pointspecialactionsfn ~= nil and self:SortActionList(self.pointspecialactionsfn(self.inst, pos, useitem, right), pos) or {}
end

return TpVehicleActionPicker