local TreeData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local anims = data.anim or {}
	 	local inst = WARGON.make_prefab(anims, anims[4], {"obs", 0.1})
	 	inst:add_cmps({
	 		inspect = {},
	 	})
	 	inst:AddTag("tp_alloy_tree")
	 	if data.fn then
	 		data.fn(inst)
	 	end

	    return inst
	end
end)

local growth_stages = {
    {name="low", time = function(inst) 
        return GetRandomWithVariance(TUNING.EVERGREEN_GROW_TIME[1].base, TUNING.EVERGREEN_GROW_TIME[1].random) 
    end,
    fn = function(inst)
        inst.AnimState:PlayAnimation("full_3")
        inst.components.workable:SetWorkLeft(TUNING.MARBLETREE_MINE)
        inst.components.lootdropper:SetLoot({inst.alloy})
        inst.Transform:SetScale(.6, .6, .6)
    end},
    {name="med", time = function(inst) 
        return GetRandomWithVariance(TUNING.EVERGREEN_GROW_TIME[2].base, TUNING.EVERGREEN_GROW_TIME[2].random) 
    end, fn = function(inst)
        WARGON.make_fx2("collapse_small", inst)
        inst.AnimState:PlayAnimation("full_4")
        inst.components.workable:SetWorkLeft(TUNING.MARBLETREE_MINE)
        inst.components.lootdropper:SetLoot({inst.alloy})
        inst.Transform:SetScale(.8, .8, .8)
    end },
	{name="tall", function(inst) 
        return GetRandomWithVariance(TUNING.EVERGREEN_GROW_TIME[3].base, TUNING.EVERGREEN_GROW_TIME[3].random) 
    end, fn = function(inst)
        WARGON.make_fx2("collapse_small", inst)
        inst.AnimState:PlayAnimation("full_1")
        inst.components.workable:SetWorkLeft(TUNING.MARBLETREE_MINE)
        inst.components.lootdropper:SetLoot({inst.alloy, inst.alloy})
        inst.Transform:SetScale(1, 1, 1)
    end},
}

local alloy_tree = TreeData("tp_alloy_tree", {
    anim = {"tp_alloy_tree", "tp_alloy_tree", "full_3"},
    fn = function(inst)

        inst.alloy = "tp_alloy"
        inst:AddTag("tree")
        inst:AddComponent("lootdropper")
        inst:AddComponent("workable")
		inst.components.workable:SetWorkAction(ACTIONS.MINE)
		inst.components.workable:SetWorkLeft(TUNING.MARBLETREE_MINE)
		inst.components.workable:SetOnWorkCallback(          
			function(inst, worker, workleft)
                local anims = {3, 4, 1}
                local anim = anims[inst.components.growable.stage]
				local pt = Point(inst.Transform:GetWorldPosition())
				if workleft <= 0 then
					inst.components.lootdropper:DropLoot(pt)
					inst:Remove()
                else
					if workleft < TUNING.MARBLETREE_MINE*(1/3) then
						inst.AnimState:PlayAnimation("low_"..anim)
					elseif workleft < TUNING.MARBLETREE_MINE*(2/3) then
						inst.AnimState:PlayAnimation("med_"..anim)
					else
						inst.AnimState:PlayAnimation("full_"..anim)
					end
				end
			end
        )
        inst:AddComponent("growable")
		inst.components.growable.stages = growth_stages
		inst.components.growable:SetStage(1)
		inst.components.growable:StartGrowing()
    end,
})
local alloy_tree_placer = WARGON.make_placer("tp_alloy_tree", alloy_tree.data.anim, nil, nil, nil, .6)

local alloy_blue_tree = deepcopy(alloy_tree)
alloy_blue_tree:HookFn(function(inst)
    inst:SetPrefabName("tp_alloy_tree")
end)

local alloy_red_tree = deepcopy(alloy_tree)
alloy_red_tree:SetName("tp_alloy_red_tree")
alloy_red_tree:HookFn(function(inst)
    inst.AnimState:SetBank("tp_alloy_red_tree")
    inst.AnimState:SetBuild("tp_alloy_red_tree")
    inst:SetPrefabName("tp_alloy_red_tree")
    inst.alloy = "tp_alloy_red"
    inst.components.lootdropper:SetLoot({inst.alloy})
end)
local alloy_red_tree_placer = WARGON.make_placer("tp_alloy_red_tree", alloy_red_tree.data.anim, nil, nil, nil, .6)

return 
alloy_blue_tree,
alloy_tree_placer,
alloy_red_tree,
alloy_red_tree_placer