local Anims = {
	chester_white = {"chester", "chester_snow_build", "idle_loop"},
	chester_black = {"chester", "chester_shadow_build", "idle_loop"},
	robot_finder = {"tp_structures", "tp_structures", "tp_robot_finder"},
	wx78 = {"wilson", "wx78", "idle_loop"},
}
local Phys = {
	chester = {"char", 75, .5},
	wilson = {"char", 75, .5},
}
local Shadows = {
	chester = {2, 1.5},
	wilson = {1.3, .6},
}










































local AnimalData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()



	 	local inst = WARGON.make_prefab(data.anim, nil, data.phy, 
	 		data.shadow, data.faced)
	 	if data.map then
		 	WARGON.make_map(inst, data.map)
	 	end
	 	inst:add_cmps({
	 		inspect = {},
	 	})
	 	if data.fn then
	 		data.fn(inst)
	 	end
	 	return inst
	end
end)

function AnimalData:__tostring()
	return string.format("Pet %s - %s", self.name, self.desc)
end

local robot_cleaner = AnimalData("tp_robot_cleaner", {

	anim = Anims.wx78,

	phy = Phys.wilson,

	shadow = Shadows.wilson,
	faced = 4,
	fn = function(inst)
		WARGON.EQUIP.hat_on(inst, "strawhat_bonnet", nil, true)
		inst.AnimState:Hide("ARM_carry")
		inst:add_tags({
			"companion", "character", "scarytoprey",
			"notraptrigger", "cattoy", "noauradamage", 
		})
		inst:AddTag("tp_robot_cleaner")
		inst:add_cmps({
			combat = {

				symbol = "torso",
			},
			health = {
				max = TUNING.CHESTER_HEALTH,
				regen = {
					TUNING.CHESTER_HEALTH_REGEN_AMOUNT, 
					TUNING.CHESTER_HEALTH_REGEN_PERIOD,
				},
			},
			loco = {


				walk = 4,
				run = 6,
			},

			cont = {typ="3x4"},
		})






		inst.components.container.onopenfn = function(inst)

			inst.brain:Stop()
		end
		inst.components.container.onclosefn = function(inst)

			inst.brain:Start()
			inst:ClearBufferedAction()
		end
		inst:SetBrain(require "brains/tp_robot_cleaner_brain")

		inst:SetStateGraph("SGtp_pig_killer")
	end,
})

local function face_target(inst, target)

	local angle = inst.Transform:GetRotation()












	if inst.arrow == nil then



		inst.arrow = WARGON.make_fx2("pointer_arrow", inst, {follow=true})
		inst.arrow.pos_target = inst
	end

	inst.arrow:UpdateRotation(target)






end

local function is_target(inst, item)
	local prefab = item.prefab
	if not (inst.components.inventoryitem 
	and inst.components.inventoryitem.owner) and not inst:IsInLimbo() then
		if inst.prefab == prefab then
            return true
        elseif inst.components.pickable 
        and inst.components.pickable:CanBePicked() 
        and inst.components.pickable.product == prefab then
            return true
        elseif inst.components.harvestable 
        and inst.components.harvestable:CanBeHarvested() 
       	and inst.components.harvestable.product == prefab then
            return true
        elseif inst.components.dryable 
        and inst.components.dryable.product == prefab then
            return true
        elseif inst.components.shearable 
        and inst.components.shearable:CanShear() 
       	and inst.components.shearable.product == prefab then
            return true
        elseif inst.components.dislodgeable 
        and inst.components.dislodgeable:CanBeDislodged() 
       	and inst.components.dislodgeable.product == prefab then
            return true
        elseif inst.components.cookable 
        and inst.components.cookable.product == prefab then
            return true   
        elseif item.components.deployable 
        and item.components.deployable.product == inst.prefab then
            return true
        elseif inst.components.lootdropper then
            local total_loot = inst.components.lootdropper:GetPotentialLoot()
            for i,v2 in ipairs(total_loot) do
                if v2 == prefab then
                    return true
                end
            end
        end
	end
	return false
end

local function find_item(inst)
	local item = inst.components.container:GetItemInSlot(1)
	if item then
		local ents = inst:wg_finds(1000)
		local target = nil
		for k, v in pairs(ents) do
			if is_target(v, item) then
				target = v
			end
		end
		if target then
			face_target(inst, target)
		else
			GetPlayer().components.talker:Say("没有找到")
		end
	end
end

local robot_finder = AnimalData("tp_robot_finder", {
	anim = Anims.robot_finder,
	phy = Phys.chester,
	shadow = Shadows.chester,

	fn = function(inst)
		inst:add_tags({
			"companion", "character", "scarytoprey",
			"notraptrigger", "cattoy", "noauradamage", 
		})
		inst:AddTag("tp_robot_finder")
		inst:add_cmps({
			combat = {
				symbol = "chester_body",
			},
			health = {
				max = TUNING.CHESTER_HEALTH,
				regen = {
					TUNING.CHESTER_HEALTH_REGEN_AMOUNT, 
					TUNING.CHESTER_HEALTH_REGEN_PERIOD,
				},
			},
			loco = {},
		})
		inst:AddComponent("container")
	    inst.components.container.itemtestfn = function(inst, item, slot)
	    	return true
	   	end
	    inst.components.container:SetNumSlots(1.1)
	    inst.components.container.widgetslotpos = {Vector3(0,0,0)}
	    inst.components.container.widgetanimbank = "ui_bundle_2x2"
	    inst.components.container.widgetanimbuild = "ui_bundle_2x2"
	    inst.components.container.widgetpos = Vector3(200,0,0)
	    inst.components.container.side_align_tip = 100
	    inst.components.container.acceptsstacks = false
		inst.components.container.widgetbuttoninfo = {
			text = STRINGS.TP_STR.tp_sure,
			position = Vector3(0, -100, 0),
			fn = function(inst)
				inst:find_item()	
			end,
			validfn = function(inst)
				return true
			end,
		}







		inst.find_item = find_item
	end
})
local robot_finder_placer = WARGON.make_placer("tp_robot_finder", Anims.robot_finder)

local white_pig = AnimalData("tp_white_pig", {
	anim = {"pigman", "pig_build", "idle_loop"},
	faced = 4,
	shadow = {1.5, .75},
	phy = {"char", 50, .5},
	fn = function(inst)
		inst:add_cmps({
			health = {
				max = 100,
			},
			combat = {},
			loco = {
				walk = 3,
				run = 5,
			},
			hunger = {},
			eater = {},
			trader = {
				test = function(inst, item, giver)
					return inst.components.eater 
						and inst.components.eater:CanEat(item)
				end,
			},
			inv = {},
			loot = {},
		})
		inst.Transform:SetScale(.8, .8, .8)
		inst.components.lootdropper:AddRandomLoot("meat", 3)
		inst.components.lootdropper:AddRandomLoot("pigskin", 1)
		inst.components.lootdropper.numrandomloot = 1
		inst.grow = function(inst)
			if inst.gorwed == nil then
				inst:AddTag("pigman")
				inst.Transform:SetScale(1, 1, 1)
				inst.components.lootdropper:AddLoot("meat")
				inst.components.lootdropper:AddLoot("meat")
				inst.components.lootdropper:AddLoot("pigskin")
				inst.components.health:SetMaxHealth(200)
				inst.growed = true
			end
		end
		inst:AddComponent("tpvalue")
		inst.components.tpvalue:Stop()
		inst.components.tpvalue.empty = function(inst)
			inst:grow()
		end
		inst.components.eater:SetOmnivore()
		inst.components.eater:SetCanEatHorrible()
		table.insert(inst.components.eater.foodprefs, "RAW")
		table.insert(inst.components.eater.ablefoods, "RAW")
		inst.components.eater.strongstomach = true
		inst.components.eater:SetOnEatFn(function(inst, food)





			local val = food.components.edible.hungervalue
			inst.components.tpvalue:DoDelta(-val)
		end)
		inst:SetBrain(require "brains/tp_white_pig_brain")
		inst:SetStateGraph("SGtp_white_pig")











	end,
})

return 
robot_cleaner,
robot_finder,
robot_finder_placer,
white_pig