local Anims = {

	lamellar = {"armor_wood_lamellar", "armor_wood_lamellar", "anim", "idle_water", },
	thunder = {"armor_wood_lamellar", "tp_armor_thunder", "anim", "idle_water", },
	health = {"armor_wood_lamellar", "tp_armor_health", "anim", "idle_water", },
	firm = {"armor_wood_lamellar", "tp_armor_firm", "anim", "idle_water", },
	warm = {"armor_wood_lamellar", "tp_armor_warm", "anim", "idle_water", },
	cool = {"armor_wood_lamellar", "tp_armor_cool", "anim", "idle_water", },
	ruin = {"armor_ruins", "armor_ruins_tusk", "anim", "idle_water", },
	ruin2 = {"armor_ruins", "armor_ruins_arcane", "anim", "idle_water"},
	ruin3 = {"armor_ruins", "armor_ruins_leaf", "anim", "idle_water", },
	ruin4 = {"armor_ruins", "armor_ruins_bulky", "anim", "idle_water", },
	bag = {"armor_wood_lamellar", "tp_armor_bag", "anim", "idle_water"},
	shadow = {"armor_wood_lamellar", "tp_armor_shadow", "anim", "idle_water"},
	airi = {"armor_wood_lamellar", "tp_armor_airi", "anim", "idle_water"},
	cloak = {"armor_grass_cloak", "armor_grass_cloak", "anim", "idle_water"},
	sharp = {"armor_sanity", "armor_sanity_sharp", "anim", "idle_water"},
	rockabs = {"armor_marble", "armor_marble_rockabs", "anim", "idle_water"},
	marble_skin2 = {"armor_marble", "armor_marble_chainmail", "anim", "idle_water"},
}

local function MakeItem(name, anims, item_fn, atlas, img)
	local function fn()
		local the_atlas = atlas and "images/inventoryimages/"..atlas..".xml" 
		local the_img = img or atlas
	 	local inst = WARGON.make_prefab(anims, anims[4], "inv", nil, nil)
	 	WARGON_CMP_EX.add_cmps(inst, {
	 		invitem = {atlas=the_atlas, img=the_img},
	 		equip = {slot="body"},
	 	})
		inst:AddTag("tp_item")
		inst:AddComponent("tpinter")
		inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
			return invitem:HasTag("tp_fix_powder")
				and not inst:HasTag("tp_not_fix")
		end)
	 	if item_fn then
	 		item_fn(inst)
	 	end

	    return inst
	end
	return Prefab("common/inventory/"..name, fn, {})
end

local function common_unequip(inst, owner)
	WARGON.EQUIP.body_off(owner)
end

local function armor_lamellar_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "armor_wood_lamellar", "swap_body")
end

local function armor_lamellar_unequip(inst, owner)
	common_unequip(inst, owner)
end

local function armor_lamellar_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=armor_lamellar_equip, unequip=armor_lamellar_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},
	})
end

local function armor_health_on_attacked(inst, data)
	if inst.components.health and inst.components.health:GetPercent() < .3
	and math.random() < .33 then
		inst.components.health:DoDelta(10, nil, "tp_health_equip")
	end
end

local function armor_health_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_health", "swap_body")
	owner:ListenForEvent("attacked", armor_health_on_attacked)


end

local function armor_health_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveEventCallback("attacked", armor_health_on_attacked)


end

local function armor_health_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=armor_health_equip, unequip=armor_health_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},
	})
end

local function armor_firm_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_firm", "swap_body")

	owner:AddTagNum("not_hit_stunned", 1)
end

local function armor_firm_unequip(inst, owner)
	common_unequip(inst, owner)

	owner:AddTagNum("not_hit_stunned", -1)
end

local function armor_firm_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=armor_firm_equip, unequip=armor_firm_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},
	})
end

local function armor_warm_heat_fn(inst, owner)
	local c = 0
	if owner and owner:HasTag("tp_armor_warm") and owner:HasTag("tp_hat_warm")
	and owner:HasTag("tp_spear_fire") then
		c = 50
	end
	return c
end

local function armor_warm_on_attacked(inst, data)
	if (data.weapon == nil or (not data.weapon:HasTag("projectile") and data.weapon.projectile == nil))
	and data.attacker and data.attacker.components.combat and data.stimuli ~= "thorns" and not data.attacker:HasTag("thorny")
	and (data.attacker.components.combat == nil or (data.attacker.components.combat.defaultdamage > 0))
	and data.damage and data.damage > 0 and math.random() < .5 then
		WARGON.fire_prefab(data.attacker, inst)
	end
end

local function armor_warm_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_warm", "swap_body")
	owner:ListenForEvent("attacked", armor_warm_on_attacked)
	owner:AddTag("tp_armor_warm")
end

local function armor_warm_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveEventCallback("attacked", armor_warm_on_attacked)
	owner:RemoveTag("tp_armor_warm")
end

local function armor_warm_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=armor_warm_equip, unequip=armor_warm_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},

	})
end

local function armor_cool_heat_fn(inst, owner)
	local c = 0
	if owner and owner:HasTag("tp_armor_cool") and owner:HasTag("tp_hat_cool")
	and owner:HasTag("tp_spear_ice") then
		c = 10
	end
	return c
end

local function armor_cool_on_attacked(inst, data)
	if (data.weapon == nil or (not data.weapon:HasTag("projectile") and data.weapon.projectile == nil))
	and data.attacker and data.attacker.components.combat and data.stimuli ~= "thorns" and not data.attacker:HasTag("thorny")
	and (data.attacker.components.combat == nil or (data.attacker.components.combat.defaultdamage > 0))
	and data.damage and data.damage > 0 and math.random() < .5 then
		WARGON.frozen_prefab(data.attacker, inst, 1)
	end
end

local function armor_cool_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_cool", "swap_body")
	owner:ListenForEvent("attacked", armor_cool_on_attacked)
	owner:AddTag("tp_armor_cool")
end

local function armor_cool_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveEventCallback("attacked", armor_cool_on_attacked)
	owner:RemoveTag("tp_armor_cool")
end

local function armor_cool_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=armor_cool_equip, unequip=armor_cool_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},

	})
end





		























local function armor_thunder_turn_on(inst, owner)
	inst.Light:Enable(true)
	local owner = inst.components.inventoryitem.owner
	if owner then
		WARGON.add_speed_rate(owner, "tp_armor_thunder", .25)
	end
end

local function armor_thunder_turn_off(inst, owner)
	inst.Light:Enable(false)
	local owner = inst.components.inventoryitem.owner
	if owner then
		WARGON.remove_speed_rate(owner, "tp_armor_thunder")
	end
end

local function armor_thunder_on_lightning(inst, data)
	if inst.components.health then
		inst.components.health:DoDelta(20)
	end
	if inst.components.sanity then
		inst.components.sanity:DoDelta(-40)
	end
	local armor = inst.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY)
	if armor and armor:HasTag("tp_armor_thunder") then
		if armor.components.tpvalue then
			armor.components.tpvalue:DoDelta(100)
		end
		armor_thunder_turn_on(armor)
	end
end

local function armor_thunder_on_attacked(inst, data)

















end

local function armor_thunder_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_thunder", "swap_body")
	owner:ListenForEvent("tp_lightning_strike", armor_thunder_on_lightning)
	owner:ListenForEvent("attacked", armor_thunder_on_attacked)
	inst.components.tpvalue:Start()
	if inst.components.tpvalue:IsEmpty() then
		armor_thunder_turn_off(inst)
	else
		armor_thunder_turn_on(inst)
	end
end

local function armor_thunder_unequip(inst, owner)
	WARGON.EQUIP.body_off(owner)
	inst.components.tpvalue:Stop()
	owner:RemoveEventCallback("tp_lightning_strike", armor_thunder_on_lightning)
	owner:RemoveEventCallback("attacked", armor_thunder_on_attacked)
	armor_thunder_turn_off(inst)
end

local function armor_thunder_drop(inst)
	print("tp_armor_thunder drop")
	armor_thunder_turn_off(inst)
end

local function armor_thunder_fn(inst)
	inst:AddTag("tp_armor_thunder")
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		invitem = {drop=armor_thunder_drop},
		equip = {equip=armor_thunder_equip, unequip=armor_thunder_unequip},
		armor = {armor=TUNING.ARMORWOOD, absorb=.8},
		tpvalue = {},
	})
	inst.components.tpvalue:SetMax(300)
	inst.components.tpvalue.empty = function(inst)
		armor_thunder_turn_off(inst)
	end
	local light = inst.entity:AddLight()
	light:SetFalloff(0.4)
	light:SetIntensity(.7)
	light:SetRadius(2.5)
	light:SetColour(180/255, 195/255, 150/255)
	light:Enable(false)
end











































































local function armor_shadow_equip(inst, owner)
	WARGON.EQUIP.body_on(owner, "tp_armor_shadow", "swap_body")
end

local function armor_shadow_unequip(inst, owner)
	common_unequip(inst, owner)
end

local function armor_shadow_fn(inst)
	inst:add_cmps({
		inspect = {},
		equip = {equip=armor_shadow_equip, unequip=armor_shadow_unequip},
		armor = {armor=800, absorb=.95},
	})
	inst.components.armor:SetTags({"shadowcreature"})
end

local ArmorData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local atlas, img = WARGON.resolve_img_path(data.img)
		img = string.sub(img, 1, -5)
	 	local inst = WARGON.make_prefab(data.anim, data.anim[4], "inv")
	 	inst:add_cmps({
	 		invitem = { atlas=atlas, img=img },
	 		inspect = {},
	 		equip = {
	 			equip = data.equip_fn,
	 			unequip = data.unequip_fn,
	 			slot = "body",
	 		},
	 		armor = {
	 			armor = data.armor, 
	 			absorb = data.absorb,
	 		},
	 	})
	 	if data.giant then
	 		inst:AddTag("tp_giant")
	 		inst:ListenForEvent("equipped", function(inst, data)
	 			if data and data.owner then
	 				local owner = data.owner
	 				if owner.components.tpbody then
						owner.components.tpbody:AddScaleModifier(inst.prefab, .1)
					end
	 			end
	 		end)
	 		inst:ListenForEvent("unequipped", function(inst, data)
	 			if data and data.owner then
	 				local owner = data.owner
	 				if owner.components.tpbody then
						owner.components.tpbody:RemoveScaleModifier(inst.prefab)
					end
	 			end
	 		end)
	 	end
	 	if data.fn then
	 		data.fn(inst)
	 	end
	 	if data.not_fix then
	 		inst:AddTag("tp_not_fix")
	 	end
		inst:AddTag("tp_item")
		inst:AddComponent("tpinter")
		inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
			return invitem:HasTag("tp_fix_powder") 
				and not inst:HasTag("tp_not_fix")
		end)

	    return inst
	end
end)

function ArmorData:__tostring()
	return string.format("Armor %s - %s", self.name, self.desc)
end

local armor_bag = ArmorData("tp_armor_bag", {
	anim = Anims.bag,
	img = "tp_armor_bag#",
	armor = 450,
	absorb = .8,
	fn = function(inst)
		inst:add_cmps({
			cont = {typ="2x4"},	
		})
		inst.components.container.itemtestfn = function(inst, item, slot)
			return inst ~= item
		end
		inst.components.container.CollectSceneActions = function()
		end
		inst.components.container.CollectInventoryActions = function()
		end
		inst:ListenForEvent("armorbroke", function(inst, data)
			inst.components.container:DropEverything()
		end)
	end,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "tp_armor_bag", "swap_body")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		inst.components.container:Close()
	end,
})

local function armor_ancient_attacked(inst, data)
	if inst.components.tpbuff then
		inst.components.tpbuff:AddBuff("tp_armor_ancient")
	end
end

local function armor_ancient_broke(inst, data)
	local owner = inst.components.equippable.owner
	if owner and owner.components.tpbuff then
		owner.components.tpbuff:AddBuff("tp_armor_ancient4")
	end
end

local function armor_ancient_health_attacked(inst, data)
	if math.random() < .3 then
		if inst.components.tpbuff then
			inst.components.tpbuff:AddBuff("tp_armor_health")
		end
	end
end

local armor_ancient = ArmorData("tp_armor_ancient", {
	anim = Anims.ruin,
	img = "armor_ruins_tusk#",
	armor = 2300,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_ruins_tusk", "swap_body")
		owner.components.tpbody:AddMaxHealthModifier(inst.prefab, 100, not inst.is_load)
		inst.is_load = nil


		inst:ListenForEvent("attacked", armor_ancient_attacked, owner)
		inst:ListenForEvent("attacked", armor_ancient_health_attacked, owner)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		owner.components.tpbody:RemoveMaxHealthModifier(inst.prefab, true)

		inst:RemoveEventCallback("attacked", armor_ancient_attacked, owner)
		inst:RemoveEventCallback("attacked", armor_ancient_health_attacked, owner)
	end,
	fn = function(inst)
		inst:ListenForEvent("armorbroke", armor_ancient_broke)
		inst.OnLoad = function(inst, data)
			inst.is_load = true
		end
	end,
})

local function armor_sanity_attacked(inst, data)
	if math.random() < .3 then
		if inst.components.tpbuff then
			inst.components.tpbuff:AddBuff("tp_armor_shadow")
		end
	end
end

local armor_ancient2 = ArmorData("tp_armor_ancient2", {
	anim = Anims.ruin2,
	img = "armor_ruins_arcane#",
	armor = 2300,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_ruins_bulky", "swap_body")
		owner.components.tpbody:AddMaxSanityModifier(inst.prefab, 100, not inst.is_load)
		inst.is_load = nil

		inst:ListenForEvent("attacked", armor_ancient_attacked, owner)
		inst:ListenForEvent("attacked", armor_sanity_attacked, owner)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		owner.components.tpbody:RemoveMaxSanityModifier(inst.prefab, true)

		inst:RemoveEventCallback("attacked", armor_ancient_attacked, owner)
		inst:RemoveEventCallback("attacked", armor_sanity_attacked, owner)
	end,
	fn = function(inst)
		inst:ListenForEvent("armorbroke", armor_ancient_broke)
		inst.OnLoad = function(inst, data)
			inst.is_load = true
		end
	end,
})

local function armor_ancient_lightning(inst, data)
	if inst.components.tpbuff then
		inst.components.tpbuff:AddBuff("tp_armor_thunder")
	end
end

local armor_ancient3 = ArmorData("tp_armor_ancient3", {
	anim = Anims.ruin3,
	img = "armor_ruins_leaf#",
	armor = 2300,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_ruins_leaf", "swap_body")
		owner.components.tpbody:AddMaxHungerModifier(inst.prefab, 100, not inst.is_load)
		inst.is_load = nil

		inst:ListenForEvent("attacked", armor_ancient_attacked, owner)
		inst:ListenForEvent("tp_lightning_strike", armor_ancient_lightning, owner)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		owner.components.tpbody:RemoveMaxHungerModifier(inst.prefab, true)

		inst:RemoveEventCallback("attacked", armor_ancient_attacked, owner)
		inst:RemoveEventCallback("tp_lightning_strike", armor_ancient_lightning, owner)
	end,
	fn = function(inst)
		inst:ListenForEvent("armorbroke", armor_ancient_broke)
		inst.OnLoad = function(inst, data)
			inst.is_load = true
		end
	end,
})

local armor_ancient4 = ArmorData("tp_armor_ancient4", {
	anim = Anims.ruin4,
	img = "armor_ruins_bulky#",
	absorb = .85,
	armor = 1800,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_ruins_arcane", "swap_body")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
	end,
	fn = function(inst)
		inst:ListenForEvent("armorbroke", armor_ancient_broke)
	end,
})

local function armor_cloak_set(inst, percent)
	local absorb = .9-percent*(.9-.6)
	inst.components.armor.absorb_percent = absorb
	local speed = .3-percent*(.3-.1)
	inst.components.equippable.walkspeedmult = speed
end

local armor_cloak = ArmorData("tp_armor_cloak", {
	anim = Anims.cloak,
	img = "armor_grass_cloak#",	
	armor = 450,
	absorb = .6,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_grass_cloak", "swap_body")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
	end,
	fn = function(inst)
		inst.components.equippable.walkspeedmult = .1
		inst:ListenForEvent("percentusedchange", function(inst, data)
			if data.percent then
				armor_cloak_set(inst, data.percent)
				local owner = inst.components.equippable.owner
				if owner then
					local slot = inst.components.equippable.equipslot
					local rate = inst.components.equippable.walkspeedmult
					owner:add_speed_rate("equipslot_"..slot, rate)
				end
			end
		end)
		inst.OnLoad = function(inst, data)
			local percent = inst.components.armor:GetPercent()
			armor_cloak_set(inst, percent)
		end
	end,
})

local function armor_unreal_blink(inst, data)
	if data and data.stimuli ~= "thorns" then
		local owner = inst.components.equippable.owner
		if owner then
			local pos = owner:around_land(4)
			if pos and not ( owner.components.tpvehicleowner
			and owner.components.tpvehicleowner:IsDriving())
			and WARGON.on_land(owner, pos) == WARGON.on_land(owner) then
				WARGON.make_fx(owner, "sanity_lower")
				owner:set_pos(pos:Get())
			end
		end
	end
end

local armor_unreal = ArmorData("tp_armor_unreal", {
	anim = Anims.sharp,
	img = "armor_sanity_sharp#",
	absorb = .9,
	armor = 450,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_sanity_sharp", "swap_body")
		inst:ListenForEvent("armorhit", armor_unreal_blink)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		inst:RemoveEventCallback("armorhit", armor_unreal_blink)
	end,
	fn = function(inst)
		inst.components.armor.bonussanitydamage = .3
	end,
})

local armor_stone = ArmorData("tp_armor_stone", {
	anim = Anims.rockabs,
	img = "armor_marble_rockabs#",
	absorb = .9,
	armor = 1050,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_marble_rockabs", "swap_body")
		owner.components.tpbody:AddMaxHealthModifier(inst.prefab, 50, not inst.is_load)
		inst.is_load = nil

	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		owner.components.tpbody:RemoveMaxHealthModifier(inst.prefab, true)

	end,
	fn = function(inst)
		inst.components.equippable.walkspeedmult = -.3
		inst.OnLoad = function(inst, data)
			inst.is_load = true
		end
	end,
})

local armor_airi = ArmorData("tp_armor_airi", {
	anim = Anims.airi,
	img = "tp_armor_airi#",
	absorb = .8,
	armor = 450,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "tp_armor_airi", "swap_body")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
	end,
	fn = function(inst)
		inst:AddComponent("tprecharge")
		inst.components.tprecharge:SetRechargeTime(10)
		inst.spawn_pet = function(inst, owner)
			if inst.components.tprecharge:IsRecharged()
			and not owner:HasTag("tp_armor_airi") then
				if owner.components.tpbuff then
					owner.components.tpbuff:AddBuff("tp_armor_airi")
				end
				local pos = WARGON.around_land(owner, math.random(3, 6))
				if pos then
					local pet = WARGON.make_spawn(pos, "stungray")
					if WARGON.on_land(pet) then
						WARGON.make_fx(pos, "collapse_small")
						pet:Remove()
					else
						WARGON.make_fx(pos, "tp_fx_shadow_bat")

						pet:AddComponent("follower")
						pet:SetBrain(require "brains/abigailbrain")
						owner.components.leader:AddFollower(pet)

						pet:no_save()
						pet:AddTag("tp_no_drop_loot")

						pet:RemoveTag("stungray")

						pet.components.combat.targetfn = function(inst)
							local notags = {"FX", "NOCLICK","INLIMBO", "player", "wall"}
							local yestags = {"monster", "character"}

							return FindEntity(inst, TUNING.STINKRAY_TARGET_DIST, function(guy)
								return pet.components.combat:CanTarget(guy)
								and guy:GetIsOnWater(guy:GetPosition():Get())
							end, nil, notags, yestags)
						end

						pet.components.combat.ShareTarget = function() end
						pet:do_task(10, function()
							if pet:is_alive() then
								pet.components.health:Kill()
							end
						end)
						inst.components.tprecharge:SetRechargeTime(10)
					end
				end
			end
		end
	end,
})

local armor_rock = ArmorData("tp_armor_rock", {
	anim = Anims.marble_skin2,
	absorb = .85,
	armor = 1050,
	img = "armor_marble_chainmail#",
	equip_fn = function(inst, owner)
		WARGON.EQUIP.body_on(owner, "armor_marble_chainmail", "swap_body")
		inst:ListenForEvent("attacked", armor_ancient_attacked, owner)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.body_off(owner)
		inst:RemoveEventCallback("attacked", armor_ancient_attacked, owner)
	end,
	fn = function(inst)
		inst.components.equippable.walkspeedmult = -.3
	end,
})

return
MakeItem("tp_armor_lamellar", Anims.lamellar, armor_lamellar_fn, "armor_wood_lamellar"),
MakeItem("tp_armor_health", Anims.health, armor_health_fn, "tp_armor_health"),
MakeItem("tp_armor_firm", Anims.firm, armor_firm_fn, "tp_armor_firm"),
MakeItem("tp_armor_warm", Anims.warm, armor_warm_fn, "tp_armor_warm"),
MakeItem("tp_armor_cool", Anims.cool, armor_cool_fn, "tp_armor_cool"),
MakeItem("tp_armor_thunder", Anims.thunder, armor_thunder_fn, "tp_armor_thunder"),




armor_bag,
armor_ancient,
armor_ancient2,
armor_ancient3,
armor_ancient4,
armor_cloak,
armor_unreal,
armor_stone,
armor_airi,
armor_rock,

MakeItem("tp_armor_shadow", Anims.shadow, armor_shadow_fn, "tp_armor_shadow")
