local assets = {
	Asset("ANIM", "anim/willow.zip"),
}

local Anims = {
	willow = {"wilson", "willow", "idle_loop", },
	wendy = {"wilson", "wendy", "idle_loop", },
	wathgrithr = {"wilson", "wathgrithr", "idle_loop", },
}
local wathgrithrs = {"wilson", "wathgrithr", "idle_loop"}
local willows = {"wilson", "willow", "idle_loop"}
local wendys = {"wilson", "wendy", "idle_loop"}
local phys = {"char", 75, .5}
local shadows = {1.3, .6}

local function combat_lord_combat_re(inst)
	return WARGON.find(inst, 16*1.33, function(guy)
        return inst.components.combat:CanTarget(guy)
           and guy:HasTag("player")
    end)
end

local function combat_lord_combat_keep(inst, target)
	return inst.components.combat:CanTarget(target)

end

local function combat_lord_on_attacked(inst, data)
	if data.attacker then
		inst.components.combat:SetTarget(data.attacker)
		inst.components.combat:ShareTarget(data.attacker, 30, function(dude) 
	    	return dude:HasTag("tp_combat_lord") 
	    end, 5)
	end
end

local function combat_lord_on_hit_other(inst, data)
	if data.target.components.tpbuff then
		data.target.components.tpbuff:AddBuff("tp_wound")
	end
end

local function combat_lord_on_health_delta(inst, data)


















end

local function combat_lord_on_death(inst, data)
	if inst.prefab == "tp_combat_lord" and inst.spawn_friend then
		for i = 1, 2 do
			WARGON.do_task(inst, i, function()
				local pos = WARGON.around_land(inst, 4)
				if pos then
					WARGON.make_fx(pos, "statue_transition")
					local lord = WARGON.make_spawn(pos, "tp_combat_lord"..i)
					local target = inst.components.combat.target
					if target then
						lord.components.combat:SetTarget(target)
					end
				end
			end)
		end
		inst.spawn_friend = false
	end
end

local function combat_lord_can_attacked(inst, attacker)


	return not inst.components.health:IsInvincible()
end

local function close_door(inst)
	local door = inst:wg_find(100, nil, {"interior_door"})
	if door then
		door:AddTag("NOCLICK")
	end
end

local function open_door(inst)
	if inst.prefab ~= "tp_combat_lord" then
		local lords = inst:wg_finds(100, {"tp_combat_lord"})
		local can_open = true
		for k, v in pairs(lords) do
			if v.components.health
			and not v.components.health:IsDead() then
				can_open = false
			end
		end
		if can_open then
			local door = inst:wg_find(100, nil, {"interior_door"})
			if door then
				door:RemoveTag("NOCLICK")
			end
		end
	end
end

local function catch_spear(inst)
	WARGON.EQUIP.object_on(inst, "swap_spear_forge_gungnir", "swap_spear_gungnir")
	inst:RemoveTag("spear_thrown")
	inst.sg:GoToState("tp_ci_start")
end

local function MakeLord(name, anims, boss_loots)
	local function fn()
		local inst = WARGON.make_prefab(anims, nil, phys, shadows, 4)
		inst:AddTag("tp_sign_damage")
		inst:AddTag("epic")
		inst:AddTag("tp_only_player_attack")
		inst:AddTag("tp_combat_lord")
		inst:AddTag("tp_room_boss")
		WARGON.CMP.add_cmps(inst, {
			inspect = {},
			health = {max=1500, fire=0, regen={20, 5}},

			loco = {walk=0, run=0},
			combat = {
				dmg=150, player=.5, per=2, range={10, 3},
				re={time=3, fn=combat_lord_combat_re},
				keep=combat_lord_combat_keep,
			},
			loot = {loot=boss_loots, },
		})
		inst.components.combat.canbeattackedfn = combat_lord_can_attacked
		inst:AddTag("groundpoundimmune")
	    inst:AddComponent("tpgroundpounder")
		inst.components.tpgroundpounder.noTags = {"FX", "NOCLICK", "DECOR", "INLIMBO", "groundpoundimmune"}
	    inst.components.tpgroundpounder.groundpoundfx = "small_ground_pound_ring"
	    inst.components.tpgroundpounder.groundpoundringfx = "small_ground_pound_ring"
	    inst.components.tpgroundpounder.damageRings = 3
	    inst.components.tpgroundpounder.numRings = 5



		WARGON.EQUIP.hat_on(inst, "hat_wathgrithr", nil, true)
		WARGON.EQUIP.object_on(inst, "swap_spear_wathgrithr", "swap_spear_wathgrithr")
		WARGON.add_listen(inst, {
			attacked = combat_lord_on_attacked,
			onhitother = combat_lord_on_hit_other,
			healthdelta = combat_lord_on_health_delta,
			death = combat_lord_on_death,
		})
		if name == "tp_combat_lord" then
			inst.spawn_friend = true
		end

		inst:SetBrain(require "brains/tp_sign_rider_brain")
		inst:SetStateGraph("SGtp_combat_lord")
		inst.OnSave = function(inst, data)
			if data then
				data.spawn_friend = inst.spawn_friend
			end
		end
		inst.OnLoad = function(inst, data)
			if data then
				inst.spawn_friend = data.spawn_friend
			end
		end
		inst:do_task(0, function()
			close_door(inst)
		end)
		inst:ListenForEvent("death", open_door)

		inst.catch_spear = catch_spear

		local can_attack = inst.components.combat.CanAttack
		inst.components.combat.CanAttack = function(cmp, target)
			if inst:HasTag("spear_thrown") then
				return false
			end
			return can_attack(cmp, target)
		end

		return inst
	end
	return Prefab("common/"..name, fn, assets)
end

local loot_1 = {"tp_boss_loot"}
local loot_2 = {"tp_boss_loot"}
local loot_3 = {"tp_boss_loot"}

return
MakeLord("tp_combat_lord", Anims.wathgrithr, loot_1),
MakeLord("tp_combat_lord1", Anims.willow, loot_2),
MakeLord("tp_combat_lord2", Anims.wendy, loot_3)
