require "prefabutil"

local cooking = require("cooking")

local assets=
{
	Asset("ANIM", "anim/cook_pot_warly.zip"),
	Asset("ANIM", "anim/cook_pot_food.zip"),
	Asset("ANIM", "anim/cook_pot_food_2.zip"),
	Asset("MINIMAP_IMAGE", "cookpotwarly")
}

local prefabs = {

}




local slotpos = {	Vector3(0,64+32+8+4,0), 
					Vector3(0,32+4,0),
					Vector3(0,-(32+4),0), 
					Vector3(0,-(64+32+8+4),0)}

local widgetbuttoninfo = {
	text = STRINGS.ACTIONS.COOK.GENERIC,
	position = Vector3(0, -165, 0),
	fn = function(inst)
		inst.components.stewer:StartCooking()	
	end,
	
	validfn = function(inst)
		return inst.components.stewer:CanCook()
	end,
}

local function itemtest(inst, item, slot)
	if cooking.IsCookingIngredient(item.prefab) then
		return true
	end
end



local function ShowProduct(inst)
	local product = inst.components.stewer.product
	if IsModCookingProduct(inst.prefab, product) then
		inst.AnimState:OverrideSymbol("swap_cooked", product, product)
	else
    	if inst.AnimState:HasSymbol("cook_pot_food_2", product) then
    		inst.AnimState:OverrideSymbol("swap_cooked", "cook_pot_food_2", product)
    	else
			inst.AnimState:OverrideSymbol("swap_cooked", "cook_pot_food", product)
		end
	end
end

local function startcookfn(inst)
	inst.AnimState:PlayAnimation("cooking_loop", true)

	inst.SoundEmitter:KillSound("snd")
	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_rattle", "snd")
	inst.Light:Enable(true)
end


local function onopen(inst)
	inst.AnimState:PlayAnimation("cooking_pre_loop", true)
	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_open", "open")
	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot", "snd")
end

local function onclose(inst)
	if not inst.components.stewer.cooking then
		inst.AnimState:PlayAnimation("idle_empty")
		inst.SoundEmitter:KillSound("snd")
	end
	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_close", "close")
end

local function spoilfn(inst)
	inst.components.stewer.product = inst.components.stewer.spoiledproduct
	ShowProduct(inst)
end

local function donecookfn(inst)
	inst.AnimState:PlayAnimation("cooking_pst")
	inst.AnimState:PushAnimation("idle_full")
	ShowProduct(inst)
	inst.SoundEmitter:KillSound("snd")
	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_finish", "snd")
	inst.Light:Enable(false)

end

local function continuedonefn(inst)
	inst.AnimState:PlayAnimation("idle_full")
	ShowProduct(inst)
end

local function continuecookfn(inst)
	inst.AnimState:PlayAnimation("cooking_loop", true)

	inst.Light:Enable(true)

	inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_rattle", "snd")
end

local function harvestfn(inst)
	inst.AnimState:PlayAnimation("idle_empty")
end













local function onfar(inst)
	if inst.components.container then
		inst.components.container:Close()
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle_empty")
end
























local function onFloodedStart(inst)
	if inst.components.container then 
		inst.components.container.canbeopened = false 
	end
	if inst.components.stewer then 
		if inst.components.stewer.cooking then 
			inst.components.stewer.product = "wetgoop"
		end
	end
end


local function onFloodedEnd(inst)
	if inst.components.container then 
		inst.components.container.canbeopened = true 
	end
end


































	












	















local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	local minimap = inst.entity:AddMiniMapEntity()

	minimap:SetIcon( "tp_cook_pot.tex" )
	
	local light = inst.entity:AddLight()
	inst.Light:Enable(false)
	inst.Light:SetRadius(.6)
	inst.Light:SetFalloff(1)
	inst.Light:SetIntensity(.5)
	inst.Light:SetColour(235/255,62/255,12/255)

	inst:AddTag("structure")
	MakeObstaclePhysics(inst, .5)
	
	inst.AnimState:SetBank("cook_pot_warly")

	inst.AnimState:SetBuild("tp_cook_pot")
	inst.AnimState:PlayAnimation("idle_empty")

	inst:AddTag("tp_cook_pot")

	inst:AddComponent("stewer")
	inst.components.stewer.specialcookername = "portablecookpot"
	inst.components.stewer:SetCookerName("cookpot")
	inst.components.stewer.onstartcooking = startcookfn
	inst.components.stewer.oncontinuecooking = continuecookfn
	inst.components.stewer.oncontinuedone = continuedonefn
	inst.components.stewer.ondonecooking = donecookfn
	inst.components.stewer.onharvest = harvestfn
	inst.components.stewer.onspoil = spoilfn
	
	inst:AddComponent("container")
	inst.components.container.itemtestfn = itemtest
	inst.components.container:SetNumSlots(4)
	inst.components.container.widgetslotpos = slotpos
	inst.components.container.widgetanimbank = "ui_cookpot_1x4"
	inst.components.container.widgetanimbuild = "ui_cookpot_1x4"
	inst.components.container.widgetpos = Vector3(200,0,0)
	inst.components.container.side_align_tip = 100
	inst.components.container.widgetbuttoninfo = widgetbuttoninfo
	inst.components.container.acceptsstacks = false
	inst.components.container.type = "cooker"

	inst.components.container.onopenfn = onopen
	inst.components.container.onclosefn = onclose

	inst:AddComponent("inspectable")


	inst:AddComponent("playerprox")
	inst.components.playerprox:SetDist(3,5)
	inst.components.playerprox:SetOnPlayerFar(onfar)

	inst:AddComponent("floodable")
	inst.components.floodable.onStartFlooded = onFloodedStart
	inst.components.floodable.onStopFlooded = onFloodedEnd
	inst.components.floodable.floodEffect = "shock_machines_fx"
	inst.components.floodable.floodSound = "dontstarve_DLC002/creatures/jellyfish/electric_land"

	local function cookpot_use(inst)
		WARGON.make_fx(inst, "collapse_small")
		inst.components.tpbebird:BeBird()
	end

	local function cookpot_test(inst, doer)
		return not (inst.components.stewer.done or inst.components.stewer.cooking)
	end

	WARGON.CMP.add_cmps(inst, {


		tpbebird = {},
		})

	MakeSnowCovered(inst, .01)    
	inst:ListenForEvent( "onbuilt", onbuilt)








	return inst
end

return Prefab( "common/tp_cook_pot", fn, assets, prefabs)
