local Anims = {
    log = {"log", "log", "idle"},
    core_blade = {"tp_core_blade", "tp_core_blade", "core1"},
    core_necklace = {"tp_core_necklace", "tp_core_necklace", "core1"},
    core_ring = {"tp_core_ring", "tp_core_ring", "core1"},
    core_gun = {"tp_core_gun", "tp_core_gun", "core1"},
}

local CoreItem = Class(Prefab, function(self, name, data)
    Prefab._ctor(self, name)
    self.fn = function()
        local atlas, img
        if type(data.img) == "table" then
            atlas = "images/inventoryimages/"..data.img[1]..".xml"
            img = data.img[2]
        else
            atlas, img = WARGON.resolve_img_path(data.img)
            img = string.sub(img, 1, -5)
        end
	 	local inst = WARGON.make_prefab(data.anim, data.anim[4], "inv")
	 	inst:add_cmps({
	 		invitem = { atlas=atlas, img=img },
	 		inspect = {},
	 	})
        inst:AddTag("tp_equip_core")
        inst:AddTag("tp_item")
        if data.fn then
            data.fn(inst)
        end

        return inst
    end
end)

local core_blade_names = {
    "gold", "strength", "speed", "conqueror",
    "ice", "poison", "fire", "earth",
    "blood", "shadow", "combat", "laevatain",
}

local cores = {}
for i = 1, 12 do
    local core_blade = CoreItem("tp_core_blade"..i, {
        anim = Anims.core_blade,
        img = {"tp_core_blade", "tp_core_blade"..i},
        fn = function(inst)
            inst.AnimState:PlayAnimation("core"..i)
            inst:AddTag("tp_core_blade")
            inst.core = core_blade_names[i]
            inst.level = math.ceil(i/4)
            inst:AddComponent("tpinterable")
            inst.components.tpinterable:SetFn(function(inst, target, doer)
                if target.components.tpsword then
                    target.components.tpsword:SetCore(inst.core)
                    inst:Remove()
                end
            end)
            inst:AddComponent("tpinfo")
            inst.components.tpinfo.info = function(inst)
                return string.format("核心：%s，等级：%d", inst.core, inst.level)
            end
        end
    })
    table.insert(cores, core_blade)
end

local core_gun_names = {
    "strength", "weak", "hunter",
    "thunder", "tornado",
    "loot", "fast",
}
for i = 1, 7 do
    local core_gun = CoreItem("tp_core_gun"..i, {
        anim = Anims.core_gun,
        img = {"tp_core_gun", "tp_core_gun"..i},
        fn = function(inst)
            inst.AnimState:PlayAnimation("core"..i.."a")
            inst:AddTag("tp_core_gun")
            inst.core = core_gun_names[i]

            if i <= 3 then
                inst.level = 1
            elseif i <= 5 then
                inst.level = 2
            else
                inst.level = 3
            end
            inst:AddComponent("tpinterable")
            inst.components.tpinterable:SetFn(function(inst, target, doer)
                if target.components.tpsword then
                    target.components.tpsword:SetCore(inst.core)
                    inst:Remove()
                end
            end)
            inst:AddComponent("tpinfo")
            inst.components.tpinfo.info = function(inst)
                return string.format("核心：%s，等级：%d", inst.core, inst.level)
            end
        end
    })
    table.insert(cores, core_gun)
end

local core_necklace_names = {
    "armor", "recover", "thorns", 
    "ice", "fire", "poison",
    "health", "sanity",
}
for i = 1, 8 do
    local core_necklace = CoreItem("tp_core_necklace"..i, {
        anim = Anims.core_necklace,
        img = {"tp_core_necklace", "tp_core_necklace"..i},
        fn = function(inst)
            inst.AnimState:PlayAnimation("core"..i)
            inst:AddTag("tp_core_necklace")
            inst.core = core_necklace_names[i]
            inst.level = math.ceil(i/3)
            inst:AddComponent("tpinterable")
            inst.components.tpinterable:SetFn(function(inst, target, doer)
                if target.components.tpornament then
                    target.components.tpornament:SetCore(inst.core)
                    inst:Remove()
                end
            end)
            inst:AddComponent("tpinfo")
            inst.components.tpinfo.info = function(inst)
                return string.format("核心：%s，等级：%d", inst.core, inst.level)
            end
        end,
    })
    table.insert(cores, core_necklace)
end

local core_ring_names = {
    "speed", "cool", "warm",
    "shadow", "poison_block", "thunder",
    "firm", "titan", 
}
for i = 1, 8 do
    local core_ring = CoreItem("tp_core_ring"..i, {
        anim = Anims.core_ring,
        img = {"tp_core_ring", "tp_core_ring"..i},
        fn = function(inst)
            inst.AnimState:PlayAnimation("core"..i.."a")
            inst:AddTag("tp_core_ring")
            inst.core = core_ring_names[i]
            inst.level = math.ceil(i/3)
            inst:AddComponent("tpinterable")
            inst.components.tpinterable:SetFn(function(inst, target, doer)
                if target.components.tpornament then
                    target.components.tpornament:SetCore(inst.core)
                    inst:Remove()
                end
            end)
            inst:AddComponent("tpinfo")
            inst.components.tpinfo.info = function(inst)
                return string.format("核心：%s，等级：%d", inst.core, inst.level)
            end
        end,
    })
    table.insert(cores, core_ring)
end

return unpack(cores)






