local assets = {
	Asset("ANIM", "anim/wes.zip")
}

local Anims = {
	wes = {"wilson", "wes", "idle_loop"},
}
local weses = {"wilson", "wes", "idle_loop"}
local phys = {"char", 75, .5}
local shadows = {1.3, .6}

local function fake_knight_combat_re(inst)
	return WARGON.find(inst, 16*1.33, function(guy)
        return inst.components.combat:CanTarget(guy)
           and guy:HasTag("player")
    end)
end

local function fake_knight_combat_keep(inst, target)
	return inst.components.combat:CanTarget(target)

end

local function fake_knight_on_attacked(inst, data)
	if data.attacker then
		inst.components.combat:SetTarget(data.attacker)
	end
end

local function fake_knight_on_hit_other(inst, data)
	if data.target.components.tpbuff then
		data.target.components.tpbuff:AddBuff("tp_armor_broken")
	end
end

local function fake_knight_on_health_delta(inst, data)
	if inst.components.health:GetPercent() < .9 then
		WARGON.CMP.add_cmps(inst, {
			health = {absorb=0},
		})
	end
end

local function close_door(inst)
	local door = inst:wg_find(100, nil, {"interior_door"})
	if door then
		door:AddTag("NOCLICK")
	end
end

local function open_door(inst)
	local door = inst:wg_find(100, nil, {"interior_door"})
	if door then
		door:RemoveTag("NOCLICK")
	end
end

local function UpdateShadowSize(inst, height)
	if inst.shadow then
		local scaleFactor = Lerp(0.5, 1.5, height/35)
		inst.shadow.Transform:SetScale(scaleFactor, scaleFactor, scaleFactor)
	end
end

local function GiveDebrisShadow(inst)
	local pt = Vector3(inst.Transform:GetWorldPosition())
	inst.shadow = SpawnPrefab("warningshadow")
	UpdateShadowSize(inst, 35)
	inst.shadow.Transform:SetPosition(pt.x, 0, pt.z)
end

function PlayFallingSound(inst, volume)
	volume = volume or 1	
    local sound = inst.SoundEmitter
    if sound then
        local tile, tileinfo = inst:GetCurrentTileType()        
        if tile and tileinfo then
			local x, y, z = inst.Transform:GetWorldPosition()			
			local size_affix = "_small"			
			sound:PlaySound(tileinfo.walksound .. size_affix, nil, volume)
        end
    end
end

local function grounddetection_update(inst)
	local pt = Point(inst.Transform:GetWorldPosition())
	
	if not inst.shadow then
		GiveDebrisShadow(inst)
	else
		UpdateShadowSize(inst, pt.y)
	end

	if pt.y < 2 then
		inst.fell = true
		inst.Physics:SetMotorVel(0,0,0)
    end

	if pt.y <= .2 then
		PlayFallingSound(inst)
		if inst.shadow then
			inst.shadow:Remove()
		end

		local ents = TheSim:FindEntities(pt.x, 0, pt.z, 2, nil, {'smashable'})
	    for k,v in pairs(ents) do
	    	if v and v.components.combat and v ~= inst then
	    		v.components.combat:GetAttacked(inst, 20, nil)
	    	end
	   	end



	   	inst.Physics:SetDamping(0.9)	   	

	    if inst.updatetask then
			inst.updatetask:Cancel()
			inst.updatetask = nil
		end



			inst.SoundEmitter:PlaySound("dontstarve/common/stone_drop")
			local pt = Vector3(inst.Transform:GetWorldPosition())
			local breaking = SpawnPrefab("ground_chunks_breaking")
			breaking.Transform:SetPosition(pt.x, 0, pt.z)
			inst:Remove()

	end


	if inst.last_y and pt.y > 2 and inst.last_y > 2 and (inst.last_y - pt.y  < 1) and inst:GetTimeAlive() > 1 and not inst.fell then
		if inst.shadow then
			inst.shadow:Remove()
		end
		inst:Remove()
	end
	inst.last_y = pt.y
end

local function start_grounddetection(inst)
	inst.updatetask = inst:DoPeriodicTask(0.1, grounddetection_update, 0.05)
end

local function spawn_fall_stone(inst)
	inst:StartThread(function()
		for i = 1, 10 do
			local pos = inst:around_land(math.random(4,8))
			if pos then
				pos.y = 35
				local rock = WARGON.make_spawn(pos, "rocks")
				start_grounddetection(rock)
			end
			Sleep(.2)
		end
	end)
end

local function fn()
	local inst = WARGON.make_prefab(Anims.wes, nil, phys, shadows, 4)

	inst:AddTag("tp_sign_damage")
	inst:AddTag("epic")
	inst:AddTag("tp_only_player_attack")


	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		health = {max=1750, fire=0, absorb=.8},
		loco = {walk=4, run=6},
		combat = {
			dmg=150, player=.5, per=2,
			re={time=3, fn=fake_knight_combat_re},
			keep=fake_knight_combat_keep,
		},
		loot = {loot={


			"tp_boss_loot",
		}, },
	})
	WARGON.EQUIP.body_on(inst, "armor_metalplate", "swap_body")
	WARGON.EQUIP.hat_on(inst, "hat_metalplate", nil, true)
	WARGON.EQUIP.object_on(inst, "swap_halberd", "swap_halberd")
	WARGON.add_listen(inst, {
		attacked = fake_knight_on_attacked,
		onhitother = fake_knight_on_hit_other,
		healthdelta = fake_knight_on_health_delta,
	})
	WARGON.do_task(inst, 0, function()
		fake_knight_on_health_delta(inst)
	end)
	inst:SetBrain(require "brains/tp_fake_knight_brain")
	inst:SetStateGraph("SGtp_fake_knight")

	inst.spawn_fall_stone = spawn_fall_stone

	inst:do_task(0, function()
		close_door(inst)
	end)
	inst:ListenForEvent("death", open_door)

	return inst
end

return Prefab("common/tp_fake_knight", fn, assets)
