local Anims = {
	health = {"tp_hat_health", "tp_hat_health", "anim", "idle_water"},
	ruin = {"ruinshat", "ruinshat_arcane", "anim", "idle"},
	warm = {"tp_hat_warm", "tp_hat_warm", "anim", "idle_water"},
	antitoxin = {"tp_hat_antitoxin", "tp_hat_antitoxin", "anim", "idle_water"},
	cool = {"tp_hat_cool", "tp_hat_cool", "anim", "idle_water"},
	combathelm2 = {"footballhat", "footballhat_combathelm2", "anim", "idle_water"},
	bag = {"tp_hat_bag", "tp_hat_bag", "anim", "idle_water"},
	frozen = {"tp_hat_frozen", "tp_hat_frozen", "anim", "idle_water"},
	burn = {"tp_hat_burn", "tp_hat_burn", "anim", "idle_water"},
	fog = {"tp_hat_fog", "tp_hat_fog", "anim", "idle_water"},
	rain = {"tp_hat_rain", "tp_hat_rain", "anim", "idle_water"},
	forest = {"tp_hat_forest", "tp_hat_forest", "anim", "idle_water"},
	dog = {"footballhat", "hat_dog_helm", "anim", "idle_water"},
	wrestle = {"wathgrithrhat", "wathgrithrhat_wrestle", "anim", "idle_water"},
	dragon_body = {"footballhat", "dragonbodyhat", "anim", "idle_water"},
	dragon_head = {"footballhat", "dragonheadhat", "anim", "idle_water"},
}

local function MakeItem(name, anims, item_fn, atlas, img)
	local function fn()
		local the_atlas = atlas and "images/inventoryimages/"..atlas..".xml" 
		local the_img = img or atlas
	 	local inst = WARGON.make_prefab(anims, anims[4], "inv", nil, nil)
	 	WARGON_CMP_EX.add_cmps(inst, {
	 		invitem = {atlas=the_atlas, img=the_img},
	 		equip = {slot="head"},
	 	})
		inst:AddTag("tp_item")
		inst:AddComponent("tpinter")
		inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
			return invitem:HasTag("tp_fix_powder")
				and not inst:HasTag("tp_not_fix")
		end)
	 	if item_fn then
	 		item_fn(inst)
	 	end

	    return inst
	end
	return Prefab("common/inventory/"..name, fn, {})
end

local function common_unequip(inst, owner)
	WARGON.EQUIP.hat_off(owner)
end

local function helm_equip(inst, owner)
	WARGON.EQUIP.hat_on(owner, "footballhat_combathelm2")
end

local function helm_unequip(inst, owner)
	common_unequip(inst, owner)
end

local function helm_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=helm_equip, unequip=helm_unequip},
		armor = {armor=TUNING.ARMOR_FOOTBALLHAT, absorb=.8},
	})
end

local function warm_equip(inst, owner)
	WARGON.EQUIP.hat_on(owner, "tp_hat_warm")
	owner:AddTag("tp_hat_warm")
end

local function warm_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveTag("tp_hat_warm")
end

local function warm_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=warm_equip, unequip=warm_unequip},
		armor = {armor=TUNING.ARMOR_FOOTBALLHAT, absorb=.8},
		insu = {value=TUNING.INSULATION_LARGE, typ="winter"},
	})
end

local function cool_equip(inst, owner)
	WARGON.EQUIP.hat_on(owner, "tp_hat_cool")
	owner:AddTag("tp_hat_cool")
end

local function cool_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveTag("tp_hat_cool")
end

local function cool_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=cool_equip, unequip=cool_unequip},
		armor = {armor=TUNING.ARMOR_FOOTBALLHAT, absorb=.8},
		insu = {value=TUNING.INSULATION_MED, typ="summer"},
	})
end

local function hat_health_on_health_delta(inst, data)
	if data.cause and data.cause ~= "tp_health_equip" then
		if data.oldpercent < data.newpercent then
			local dt_p = data.newpercent - data.oldpercent
			local dt = inst.components.health.maxhealth * dt_p
			local recover = dt/5
			inst.components.health:DoDelta(recover, true, "tp_health_equip")
		end
	end
end

local function health_equip(inst, owner)
	WARGON.EQUIP.hat_on(owner, "tp_hat_health")
	owner:ListenForEvent("healthdelta", hat_health_on_health_delta)


end

local function health_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveEventCallback(owner, hat_health_on_health_delta)


end

local function health_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		equip = {equip=health_equip, unequip=health_unequip},
		armor = {armor=TUNING.ARMOR_FOOTBALLHAT, absorb=.8},
	})
end

local function antitoxin_equip(inst, owner)
	owner:anim_hat_on("tp_hat_antitoxin")
end

local function antitoxin_unequip(inst, owner)
	common_unequip(inst, owner)
end

local function antitoxin_fn(inst)
	inst:add_cmps({
		inspect = {},
		equip = {equip=antitoxin_equip, unequip=antitoxin_unequip},
		armor = {armor=TUNING.ARMOR_FOOTBALLHAT, absorb=.8},
	})
	inst.components.equippable.poisongasblocker = true
	inst.components.equippable.poisonblocker = true
end

local function ancient_on_attack(inst, data)

	if data and data.stimuli ~= "tp_hat_ancient"
	and data.target and data.target.components.health
	and data.target.components.combat
	and inst.components.combat:CanTarget(data.target)

	then
		local max = data.target.components.health:GetMaxHealth()
		if max > 3000 then

			local dmg = max/20

			data.target.components.combat:GetAttacked(inst, dmg, nil, "tp_hat_ancient")
		end
	end
end

local function ancient_equip(inst, owner)
	owner:anim_hat_open("ruinshat_arcane")
	owner:ListenForEvent("onhitother", ancient_on_attack)
	if owner.components.tpbody then
		owner.components.tpbody:AddScaleModifier("tp_hat_ancient", .1)
	end
end

local function ancient_unequip(inst, owner)
	common_unequip(inst, owner)
	owner:RemoveEventCallback("onhitother", ancient_on_attack)
	if owner.components.tpbody then
		owner.components.tpbody:RemoveScaleModifier("tp_hat_ancient")
	end
end

local function ancient_fn(inst)
	inst:add_cmps({
		inspect = {},
		equip = {equip=ancient_equip, unequip=ancient_unequip},
		armor = {armor=TUNING.ARMOR_RUINSHAT, absorb=.9},
	})
end























local HatData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local atlas, img = WARGON.resolve_img_path(data.img)
		img = string.sub(img, 1, -5)
	 	local inst = WARGON.make_prefab(data.anim, data.anim[4], "inv")
	 	inst:add_cmps({
	 		invitem = { atlas=atlas, img=img },
	 		inspect = {},
	 		equip = {
	 			equip = data.equip_fn,
	 			unequip = data.unequip_fn,
	 			slot = "head",
	 		},
	 		armor = {
	 			armor = data.armor, 
	 			absorb = data.absorb,
	 		},
	 	})
	 	if data.not_fix then
	 		inst:AddTag("tp_not_fix")
	 	end
	 	if data.fn then
	 		data.fn(inst)
	 	end
	 	if data.giant then
	 		inst:AddTag("tp_giant")
	 		inst:ListenForEvent("equipped", function(inst, data)
	 			if data and data.owner then
	 				local owner = data.owner
	 				if owner.components.tpbody then
						owner.components.tpbody:AddScaleModifier(inst.prefab, .1)
					end
	 			end
	 		end)
	 		inst:ListenForEvent("unequipped", function(inst, data)
	 			if data and data.owner then
	 				local owner = data.owner
	 				if owner.components.tpbody then
						owner.components.tpbody:RemoveScaleModifier(inst.prefab)
					end
	 			end
	 		end)
	 	end
		inst:AddTag("tp_item")
		inst:AddComponent("tpinter")
		inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
			return invitem:HasTag("tp_fix_powder") 
				and not inst:HasTag("tp_not_fix")
		end)

	    return inst
	end
end)

function HatData:__tostring()
	return string.format("Hat %s - %s", self.name, self.desc)
end

local hat_bag = HatData("tp_hat_bag", {
	anim = Anims.bag,
	img = "tp_hat_bag#",
	armor = 450,
	absorb = .8,
	fn = function(inst)
		inst:add_cmps({
			cont = { typ="2x4", },
		})
		inst:AddTag("tp_hat_bag")
		inst.components.container.itemtestfn = function(inst, item, slot)
			return inst ~= item
		end
		inst.components.container.CollectSceneActions = function()
		end
		inst.components.container.CollectInventoryActions = function()
		end
		inst:ListenForEvent("armorbroke", function(inst, data)
			inst.components.container:DropEverything()
		end)
	end,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "tp_hat_bag")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		inst.components.container:Close()
	end,
})

local function spawn_ice_pig(inst, data)
	if math.random() < .33 then
		for i = 1, 2 do
			local pos = inst:around_land(math.random(3,5))
			if pos then
				local pig = WARGON.make_spawn(pos, "pigman")
				if WARGON.on_water(pig) then
					WARGON.make_fx(pos, "splash_water_sink")
					pig:Remove()
				else
					WARGON.make_fx(pos, "tp_fx_shadow_bat")
					WARGON.make_fx(pos, "statue_transition")

					WARGON.make_fx(pos, "vortex_cloak_fx")
					inst.components.leader:AddFollower(pig)
					pig.components.follower:AddLoyaltyTime(1200)
					if pig.components.tppower then
						pig.components.tppower:SetPower(6)
					end
					if data.attacker then
						pig.components.combat:SetTarget(data.attacker)
					end
				end
			end
		end
	end
end

local hat_frozen = HatData("tp_hat_frozen", {
	anim = Anims.frozen,
	img = "tp_hat_frozen#",
	armor = 1200,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_open(owner, "tp_hat_frozen")
		owner:ListenForEvent("attacked", spawn_ice_pig)
		if inst.task then
			inst.task:Cancel()
			inst.task = nil
		end
		inst.task = inst:per_task(1, function()
			if owner.components.temperature then
				owner.components.temperature:DoDelta(-2)
			end
		end)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveEventCallback("attacked", spawn_ice_pig)
		if inst.task then
			inst.task:Cancel()
			inst.task = nil
		end
	end,
})

local hat_burn = HatData("tp_hat_burn", {
	anim = Anims.burn,
	img = "tp_hat_burn#",
	armor = 1200,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_open(owner, "tp_hat_burn")
		if inst.task then
			inst.task:Cancel()
			inst.task = nil
		end
		inst.task = inst:per_task(1, function()
			if owner.components.temperature then
				owner.components.temperature:DoDelta(2)
			end
			if owner:HasTag("tp_hat_burn") then
				WARGON.area_dmg(inst, 4, owner, 10, inst.prefab)
				WARGON.make_fx(owner, "firesplash_fx")
			end
		end)
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("tp_hat_burn")
		if inst.task then
			inst.task:Cancel()
			inst.task = nil
		end
	end,
})

local hat_fog = HatData("tp_hat_fog", {
	anim = Anims.fog,
	img = "tp_hat_fog#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "tp_hat_fog")
		owner:AddTag("venting")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("venting")
	end,
	fn = function(inst)
		inst:AddTag("venting")
		inst:AddTag("fogproof")

		inst:AddComponent("waterproofer")
		inst.components.waterproofer:SetEffectiveness(TUNING.WATERPROOFNESS_MED)
	end,
})

local function rain_hat_delta(inst, data)
	local p = inst.components.moisture:GetMoisturePercent()
	local hat = inst:get_equip_item("head")
	if hat and hat.prefab == "tp_hat_rain" then
		hat.components.armor.absorb_percent = .8+.15*p
		local rate = .05+.25*p
		hat.components.equippable.walkspeedmult = rate
		local slot = hat.components.equippable.equipslot
		inst:add_speed_rate("equipslot_"..slot, rate)
	end
end

local hat_rain = HatData("tp_hat_rain", {
	anim = Anims.rain,
	img = "tp_hat_rain#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "tp_hat_rain")
		owner:ListenForEvent("moisturechange", rain_hat_delta)
	end,	
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveEventCallback("moisturechange", rain_hat_delta)
		inst.components.armor.absorb_percent = .8
		inst.components.equippable.walkspeedmult= .05
	end,
	fn = function(inst)
		inst.components.equippable.walkspeedmult = .05
	end,
})

local function check_tile_eyeplant(tile)
	for k,v in pairs({2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,30}) do
		if v == tile then
			return true
		end
	end
end

local function get_pos_eyeplant(inst)
	local positions = {}
	for i = 1, 100 do
	   	local s = i/32.0
	   	local a = math.sqrt(s*512.0)
	   	local b = math.sqrt(s)
		table.insert(positions, Vector3(math.sin(a)*b, 0, math.cos(a)*b))
	end
	
	local useablepositions = {}
	local pt = Vector3(inst.Transform:GetWorldPosition())
    local ground = GetWorld()
	for k,v in pairs(positions) do
		local offset = Vector3(v.x * 11, 0, v.z * 11)
		local try_pos = offset + pt
		local tile = GetVisualTileType(try_pos.x, try_pos.y, try_pos.z)


		if not ( tile == GROUND.IMPASSABLE or tile > GROUND.UNDERGROUND ) and 
		check_tile_eyeplant(tile) and
		ground.Pathfinder:IsClear(pt.x, pt.y, pt.z, try_pos.x, try_pos.y, try_pos.z, {ignorewalls = true}) and
		#TheSim:FindEntities(try_pos.x, try_pos.y, try_pos.z, 2.5, {"eyeplant"}) <= 0 and 
		#TheSim:FindEntities(try_pos.x, try_pos.y, try_pos.z, 1, nil, {"FX", "NOCLICK", "DECOR", "INLIMBO"}) <= 0 then
			table.insert(useablepositions, try_pos)
			if #useablepositions >= 27 then 
				return useablepositions 
			end
		end
	end
	return useablepositions
end

local function spawn_eyeplant(inst, owner)
	if inst.components.tprecharge:IsRecharged() then
		inst.components.tprecharge:SetRechargeTime(30)
		local positions = get_pos_eyeplant(inst)
		inst:StartThread(function()
			for k, v in pairs(positions) do
				local plant = WARGON.make_spawn(v, "eyeplant")
				plant:no_save()
				plant.minionlord = owner
				plant.sg:GoToState("spawn")
				plant:SetBrain(require "brains/tp_eyeplant_brain")
				plant:do_task(10, function()
					if plant:is_alive() then
						plant.components.inventory:TransferInventory(owner)
						plant.components.health:Kill()
					end
				end)
				Sleep(.1)
			end
		end)
	end
end

local hat_forest = HatData("tp_hat_forest", {
	anim = Anims.forest,
	img = "tp_hat_forest#",
	armor = 1200,
	absorb = .9,
	giant = true,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_open(owner, "tp_hat_forest")

		owner:AddTag("venting")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("venting")

	end,
	fn = function(inst)
		inst:AddTag("venting")
		inst:AddTag("fogproof")
		inst.components.equippable.poisongasblocker = true
		inst.components.equippable.poisonblocker = true
		inst.spawn_eyeplant = spawn_eyeplant
		inst:AddComponent("tprecharge")
		inst.components.tprecharge:SetRechargeTime(30)
	end,
})

local hat_wolf = HatData("tp_hat_wolf", {
	anim = Anims.dog,
	img = "hat_dog_helm#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "hat_dog_helm")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
	end,
	fn = function(inst)
		inst:AddComponent("tprecharge")
		inst.components.tprecharge:SetRechargeTime(30)
		inst.spawn_pet = function(inst, owner)
			if inst.components.tprecharge:IsRecharged()
			and not owner:HasTag("tp_hat_wolf") then
				if owner.components.tpbuff then
					owner.components.tpbuff:AddBuff("tp_hat_wolf")
				end
				local pos = WARGON.around_land(owner, math.random(3, 6))
				if pos then
					local pet = WARGON.make_spawn(pos, "hound")
					if WARGON.on_water(pet) then
						WARGON.make_fx(pos, "splash_water_sink")
						pet:Remove()
					else
						WARGON.make_fx(pos, "tp_fx_shadow_bat")
						owner.components.leader:AddFollower(pet)

						pet:no_save()
						pet:AddTag("tp_no_drop_loot")

						pet:RemoveTag("hound")

						pet.components.combat.targetfn = function(inst)
							local dist = TUNING.HOUND_TARGET_DIST
						    if inst:HasTag("pet_hound") then
						        dist = TUNING.HOUND_FOLLOWER_TARGET_DIST
						    end
						    local notags = {"FX", "NOCLICK","INLIMBO", "wall", "aquatic", "player"}
						    return FindEntity(inst, dist, function(guy) 
								local shouldtarget = inst.components.combat:CanTarget(guy)
						        return shouldtarget
						    end, nil, notags)
						end









						pet.components.combat.ShareTarget = function() end

						pet:do_task(20, function()
							if pet:is_alive() then
								pet.components.health:Kill()
							end
						end)
						inst.components.tprecharge:SetRechargeTime(30)
					end
				end
			end
		end
	end,
})

local hat_wrestle = HatData("tp_hat_wrestle", {
	anim = Anims.wrestle,
	img = "wathgrithrhat_wrestle#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "wathgrithrhat_wrestle")
		owner:AddTag("tp_hat_wrestle")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("tp_hat_wrestle")
	end,
})

local hat_dragon_body = HatData("tp_hat_dragon_body", {
	anim = Anims.dragon_body,
	img = "dragonbodyhat#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "dragonbodyhat")
		owner:AddTag("tp_hat_dragon_body")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("tp_hat_dragon_body")
	end,
})

local hat_dragon_head = HatData("tp_hat_dragon_head", {
	anim = Anims.dragon_head,
	img = "dragonheadhat#",
	armor = 450,
	absorb = .8,
	equip_fn = function(inst, owner)
		WARGON.EQUIP.hat_on(owner, "dragonheadhat")
		owner:AddTag("tp_hat_dragon_head")
	end,
	unequip_fn = function(inst, owner)
		WARGON.EQUIP.hat_off(owner)
		owner:RemoveTag("tp_hat_dragon_head")
	end,
})

return
MakeItem("tp_hat_helm", Anims.combathelm2, helm_fn, "footballhat_combathelm2"),
MakeItem("tp_hat_warm", Anims.warm, warm_fn, "tp_hat_warm"),
MakeItem("tp_hat_cool", Anims.cool, cool_fn, "tp_hat_cool"),
MakeItem("tp_hat_health", Anims.health, health_fn, "tp_hat_health"),
MakeItem("tp_hat_antitoxin", Anims.antitoxin, antitoxin_fn, "tp_hat_antitoxin"),

hat_bag,
hat_frozen,
hat_burn,
hat_fog,
hat_rain,
hat_forest,
hat_wolf,
hat_wrestle,
hat_dragon_head,
hat_dragon_body,

MakeItem("tp_hat_ancient", Anims.ruin, ancient_fn, "ruinshat_arcane")
