local Anims = {}

local ItemData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local atlas, img = nil, nil
		if type(data.img) == "table" then
            atlas = "images/inventoryimages/"..data.img[1]..".xml"
            img = data.img[2]
        else
            atlas, img = WARGON.resolve_img_path(data.img)
            img = string.sub(img, 1, -5)
        end
		local anims = data.anim or {}
	 	local inst = WARGON.make_prefab(anims, anims[4], "inv")
	 	inst:add_cmps({
	 		invitem = {atlas=atlas, img=img},
	 		inspect = {},
	 	})
	 	inst:AddTag("tp_item")
	 	if data.fn then
	 		data.fn(inst)
	 	end

	    return inst
	end
end)

local many_items = {}

local armormarble = require("prefabs/armor_marble")

local armormarble2 = deepcopy(armormarble)
armormarble2:SetName("tp_armormarble_2")
armormarble2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("armormarble")
end)

local armormarble3 = deepcopy(armormarble2)
armormarble3:SetName("tp_armormarble_3")
armormarble3:HookFn(function(inst)
    inst.fix_value = 500
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "marble"
			and inst.components.armor:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.armor:DoDelta(dt)
	end)
end)

local armormarble4 = deepcopy(armormarble3)
armormarble4:SetName("tp_armormarble_4")
armormarble4:HookFn(function(inst)
    inst.components.armor:InitCondition(TUNING.ARMORMARBLE, .95)
end)

local armormarble5 = deepcopy(armormarble4)
armormarble5:SetName("tp_armormarble_5")
armormarble5:HookFn(function(inst)
    inst.components.equippable:AddEquipFn(function(inst, owner)
        owner.components.tpbody:AddMaxHealthModifier(inst.prefab, 50, not inst.is_load)
		inst.is_load = nil
    end)
    inst.components.equippable:AddUnequipFn(function(inst, owner)
        owner.components.tpbody:RemoveMaxHealthModifier(inst.prefab, true)
    end)
    inst.OnLoad = function(inst, data)
        inst.is_load = true
    end
end)

local armorruins = require("prefabs/armor_ruins")

local function armor_ancient_broke(inst, data)
	local owner = inst.components.equippable.owner
	if owner and owner.components.tpbuff then
		owner.components.tpbuff:AddBuff("tp_armor_ancient4")
	end
end

local armorruins_2 = deepcopy(armorruins)
armorruins_2:SetName("tp_armorruins_2")
armorruins_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("armorruins")
    inst:ListenForEvent("armorbroke", armor_ancient_broke)
end)

local armorruins_3 = deepcopy(armorruins_2)
armorruins_3:SetName("tp_armorruins_3")
armorruins_3:HookFn(function(inst)
    inst.fix_value = 500
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "thulecite"
			and inst.components.armor:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.armor:DoDelta(dt)
	end)
end)

local armorruins_4 = deepcopy(armorruins_3)
armorruins_4:SetName("tp_armorruins_4")
armorruins_4:HookFn(function(inst)
    inst.components.armor:InitCondition(TUNING.ARMORRUINS, .9)
end)

local armorruins_5 = deepcopy(armorruins_4)
armorruins_5:SetName("tp_armorruins_5")
armorruins_5:HookFn(function(inst)
    inst:AddTag("tp_not_armor_broken")
end)

local hat_fn = loadfile("prefabs/hats")
local hats = {hat_fn()}
local ruinshat = nil
for k, v in pairs(hats) do
	if v.name == "ruinshat" then
		ruinshat = v
	end
end

local ruinshat_2 = deepcopy(ruinshat)
ruinshat_2:SetName("tp_ruinshat_2")
ruinshat_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("ruinshat")
    inst.components.equippable:AddEquipFn(function(inst, owner)
        owner:AddTag("pigroyalty")
    end)
    inst.components.equippable:AddUnequipFn(function(inst, owner)
        owner:RemoveTag("pigroyalty")
    end)
end)

local ruinshat_3 = deepcopy(ruinshat_2)
ruinshat_3:SetName("tp_ruinshat_3")
ruinshat_3:HookFn(function(inst)
    inst.fix_value = 450
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "thulecite"
			and inst.components.armor:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.armor:DoDelta(dt)
	end)
end)

local ruinshat_4 = deepcopy(ruinshat_3)
ruinshat_4:SetName("tp_ruinshat_4")
ruinshat_4:HookFn(function(inst)
    inst.components.armor:InitCondition(TUNING.ARMOR_RUINSHAT, .9)
end)

local function ruinshat_attacked(inst, data)
    local armor = inst:get_equip_item("body")
    if armor and armor.components.tprecharge
    and armor.components.tprecharge:IsRecharged()
    and math.random() < .33 then
        armor.components.tprecharge:SetRechargeTime(armor.cooldown)
        if inst:is_alive() then
            inst.components.health:DoDelta(20)
        end
        local ents = inst:wg_fins(8)
        for k, v in pairs(ents) do
            if v:is_alive()
            and v.components.follower
            and v.components.follower.leader == inst then
                v.components.health:DoDelta(500)
            end
        end
    end
end

local ruinshat_5 = deepcopy(ruinshat_4)
ruinshat_5:SetName("tp_ruinshat_5")
ruinshat_5:HookFn(function(inst)
    inst.cooldown = 5
    inst:AddComponent("tprecharge")
    inst.components.tprecharge:SetRechargeTime(inst.cooldown)
    inst.components.equippable:AddEquipFn(function(inst, owner)
        inst:ListenForEvent("attacked", ruinshat_attacked, owner)
    end)
    inst.components.equippable:AddUnequipFn(function(inst, owner)
        inst:RemoveEventCallback("attacked", ruinshat_attacked, owner)
    end)
end)

local nightstick = require("prefabs/nightstick")
local nightstick_2 = deepcopy(nightstick)
nightstick_2:SetName("tp_nightstick_2")
nightstick_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("nightstick")
    inst.components.fueled:InitializeFuelLevel(60*8)
end)

local nightstick_3 = deepcopy(nightstick_2)
nightstick_3:SetName("tp_nightstick_3")
nightstick_3:HookFn(function(inst)
    inst.fix_value = 240
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "nitre"
			and inst.components.fueled:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.fueled:DoDelta(dt)
	end)
end)

local nightstick_4 = deepcopy(nightstick_3)
nightstick_4:SetName("tp_nightstick_4")
nightstick_4:HookFn(function(inst)
    local onattack = inst.components.weapon.onattack
    inst.components.weapon.onattack = function(inst, owner, target)
        onattack(inst, owner, target)
        inst.components.fueled:DoDelta(10)
    end
end)

local nightstick_5 = deepcopy(nightstick_4)
nightstick_5:SetName("tp_nightstick_5")
nightstick_5:HookFn(function(inst)
    inst:AddComponent("tprecharge")
    inst.cooldown = 60
    inst.components.tprecharge:SetRechargeTime(inst.cooldown)
    inst:add_cmps({
        caster = {
            test = function(staff, caster, target, pos)
                if staff.components.tprecharge
                and staff.components.tprecharge:IsRecharged() then
                    return true
                end
            end,
            spell = function(staff, target, pos)
                local owner = inst.components.equippable.owner
                if owner then
                    WARGON.make_fx2("lightning", owner)
                    if owner.components.tpbuff then
                        owner.components.tpbuff:AddBuff("tp_nightstick_5")
                    end
                    staff.components.tprecharge:SetRechargeTime(inst.cooldown)
                end
            end,
            can={point=true, inv=false, target=false},
        }
    })






end)

local ruins_bat = require("prefabs/ruins_bat")
local ruins_bat_2 = deepcopy(ruins_bat)
ruins_bat_2:SetName("tp_ruins_bat_2")
ruins_bat_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("ruins_bat")
    inst.components.weapon:SetDamage(65)





end)

local ruins_bat_3 = deepcopy(ruins_bat_2)
ruins_bat_3:SetName("tp_ruins_bat_3")
ruins_bat_3:HookFn(function(inst)
    inst.fix_value = 50
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "thulecite"
			and inst.components.finiteuses:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.finiteuses:DoDelta(dt)
	end)
end)

local ruins_bat_4 = deepcopy(ruins_bat_3)
ruins_bat_4:SetName("tp_ruins_bat_4")
ruins_bat_4:HookFn(function(inst)
    inst.components.weapon.getdamagefn = function(inst)
        local dmg = 65
        if inst:HasTag("slowattack") then
            dmg = dmg+35
        end
        return dmg
    end

    inst:AddComponent("tprecharge")
    inst.components.tprecharge:SetRechargeTime(30)
    inst.skill_fn = function(item, player)
        if item:HasTag("slowattack") then
            item:RemoveTag("slowattack")
        else
            item:AddTag("slowattack")
        end
        item.components.tprecharge:SetRechargeTime()
    end
end)

local ruins_bat_5 = deepcopy(ruins_bat_4)
ruins_bat_5:SetName("tp_ruins_bat_5")
ruins_bat_5:HookFn(function(inst)
    inst.components.weapon:AddAttackFn(function(inst, owner, target)
        if inst:HasTag("projectile") then
            inst.components.finiteuses:Use(1)
        end
    end)
    inst.components.tprecharge:SetRechargeTime(75)
    inst.skill_fn = function(item, player)
        item:AddTag("projectile")
        item.components.weapon:SetProjectile("tp_spear_projectile")
        item.components.weapon:SetRange(8, 10)
        item.components.tprecharge:SetRechargeTime()
        item:do_task(30, function()
            item:RemoveTag("projectile")
            item.components.weapon:SetProjectile()
            item.components.weapon:SetRange()
        end)
    end
end)

local cutlass = require("prefabs/cutlass")
local cutlass_2 = deepcopy(cutlass)
cutlass_2:SetName("tp_cutlass_2")
cutlass_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("cutlass")
    inst.components.finiteuses:SetMaxUses(200)
    inst.components.finiteuses:SetUses(200)
    inst.components.weapon.onattack = function(inst, owner, target)
       if target.prefab == "twister" then
           WARGON.get_attacked(owner, target, 40)
       end 
    end
end)

local cutlass_3 = deepcopy(cutlass_2)
cutlass_3:SetName("tp_cutlass_3")
cutlass_3:HookFn(function(inst)
    inst.fix_value = 50
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)

        return item:HasTag("tp_fish")
			and inst.components.finiteuses:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		if item.prefab == "dead_swordfish" then
            dt = dt+100
        end
        inst.components.finiteuses:DoDelta(dt)
	end)
end)

local function spear_wave_surf(inst, data)
	inst:add_speed_rate("tp_spear_wave", .25, 5)
	if inst.components.tpbuff then
		inst.components.tpbuff:AddBuff("tp_spear_wave", 5)
	end
end

local cutlass_4 = deepcopy(cutlass_3)
cutlass_4:SetName("tp_cutlass_4")
cutlass_4:HookFn(function(inst)
    inst.components.equippable:AddEquipFn(function(inst, owner)
        inst:ListenForEvent("boostbywave", spear_wave_surf, owner)
    end)
    inst.components.equippable:AddUnequipFn(function(inst, owner)
        inst:RemoveEventCallback("boostbywave", spear_wave_surf, owner)
    end)
end)

local cutlass_5 = deepcopy(cutlass_4)
cutlass_5:SetName("tp_cutlass_5")
cutlass_5:HookFn(function(inst)
    inst.components.weapon:AddAttackFn(function(inst, owner, target)
        if inst:HasTag("tp_cutlass_wave_maker") then
            inst.components.finiteuses:Use(1)
            WARGON.make_fx2("cutlass_wave_maker", owner, {owner=owner, target=target})
        end
    end)
    inst.components.weapon.getdamagefn = function(inst)
        if inst:HasTag("tp_cutlass_wave_maker") then
            return 0
        else
            return 68
        end
    end
    inst:AddComponent("tprecharge")
    inst.components.tprecharge:SetRechargeTime(75)
    inst.skill_fn = function(item, player)
        item:AddTag("tp_cutlass_wave_maker")

        item.components.weapon:SetRange(6, 8)
        item.components.tprecharge:SetRechargeTime()
        item:do_task(30, function()
            item:RemoveTag("tp_cutlass_wave_maker")

            item.components.weapon:SetRange()
        end)
    end
end)

local armorgrass = require("prefabs/armor_grass")

local function armor_cloak_set(inst, percent)
	local absorb = .9-percent*(.9-.7)
	inst.components.armor.absorb_percent = absorb
	local speed = .3-percent*(.3-.1)
	inst.components.equippable.walkspeedmult = speed
end

local armorgrass_2 = deepcopy(armorgrass)
armorgrass_2:SetName("tp_armorgrass_2")
armorgrass_2:HookFn(function(inst)
    inst.components.inventory:SetImage("armorgrass")
    inst.components.armor:InitCondition(225, .7)
    inst.components.equippable.walkspeedmult = .1
    inst:ListenForEvent("percentusedchange", function(inst, data)
        if data.percent then
            armor_cloak_set(inst, data.percent)
            local owner = inst.components.equippable.owner
            if owner then
                local slot = inst.components.equippable.equipslot
                local rate = inst.components.equippable.walkspeedmult
                owner:add_speed_rate("equipslot_"..slot, rate)
            end
        end
    end)
    inst.OnLoad = function(inst, data)
        local percent = inst.components.armor:GetPercent()
        armor_cloak_set(inst, percent)
    end
end)

local halberd = require("prefabs/halberd")
local halberd_2 = deepcopy(halberd)
halberd_2:SetName("tp_halberd_2")
halberd_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("halberd")
    inst.components.equippable.walkspeedmult = .1
end)

local halberd_3 = deepcopy(halberd_2)
halberd_3:SetName("tp_halberd_3")
halberd_3:HookFn(function(inst)
    inst.fix_value = 50
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "alloy"
			and inst.components.finiteuses:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.finiteuses:DoDelta(dt)
	end)
end)

local halberd_4 = deepcopy(halberd_3)
halberd_4:SetName("tp_halberd_4")
halberd_4:HookFn(function(inst)
    inst.components.weapon.onattack = function(inst, owner, target)
        WARGON.absorb_ripped(target)
    end
end)

local armor_sanity = require("prefabs/armor_sanity")

local armor_sanity_2 = deepcopy(armor_sanity)
armor_sanity_2:SetName("tp_armor_sanity_2")
armor_sanity_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("armor_sanity")
    inst.components.equippable.dapperness = TUNING.CRAZINESS_SMALL
end)

local armor_sanity_3 = deepcopy(armor_sanity_2)
armor_sanity_3:SetName("tp_armor_sanity_3")
armor_sanity_3:HookFn(function(inst)
    inst.components.equippable.dapperness = -TUNING.DAPPERNESS_LARGE
    inst.fix_value = 75
	inst:AddComponent("tpinter")
	inst.components.tpinter:SetCanFn(function(inst, item, doer)
		return item.prefab == "nightmarefuel"
			and inst.components.armor:GetPercent() < 1
	end)
	inst.components.tpinter:SetFn(function(inst, item, doer)
		local item = item.components.stackable:Get()
		item:Remove()
		local dt = doer.components.tplevel.attr.fixup + inst.fix_value
		inst.components.armor:DoDelta(dt)
	end)
end)

local armor_sanity_4 = deepcopy(armor_sanity_3)
armor_sanity_4:SetName("tp_armor_sanity_4")
armor_sanity_4:HookFn(function(inst)
    inst.components.equippable.dapperness = -TUNING.DAPPERNESS_MED_LARGE
    inst.components.armor:InitCondition(750, .95)
end)

local armor_sanity_5 = deepcopy(armor_sanity_4)
armor_sanity_5:SetName("tp_armor_sanity_5")
armor_sanity_5:HookFn(function(inst)
    inst.components.equippable.dapperness = -TUNING.DAPPERNESS_MED
    inst:AddComponent("tprecharge")
    inst.components.tprecharge:SetRechargeTime(.75)
    inst.skill_fn = function(item, player)
        inst.components.tprecharge:SetRechargeTime()
        local pos = player:around_land(4)
        if pos and not ( player.components.tpvehicleowner
        and player.components.tpvehicleowner:IsDriving())
        and WARGON.on_land(player, pos) == WARGON.on_land(player) then
            WARGON.make_fx2("sanity_lower", player)
            player:set_pos(pos:Get())
            player.components.sanity:DoDelta(-5)
        end
    end
end)

local blunderbuss = require("prefabs/blunderbuss")

local blunderbuss_2 = deepcopy(blunderbuss)
blunderbuss_2:SetName("tp_blunderbuss_2")
blunderbuss_2:HookFn(function(inst)
    inst.components.inventoryitem:SetImage("blunderbuss")
    inst.components.weapon:AddPreAttackFn(function(inst, owner, target)
        if inst:HasTag("blunderbuss") then
            if inst.sanity_recover then
                owner.components.sanity:DoDelta(inst.sanity_recover)
            end
            if inst.health_recover then
                owner.components.health:DoDelta(inst.health_recover)
            end
        end
    end)
    inst.sanity_recover = 5
end)

local blunderbuss_3 = deepcopy(blunderbuss_2)
blunderbuss_3:SetName("tp_blunderbuss_3")
blunderbuss_3:HookFn(function(inst)
    inst.sanity_recover = 10
end)

local blunderbuss_4 = deepcopy(blunderbuss_3)
blunderbuss_4:SetName("tp_blunderbuss_4")
blunderbuss_4:HookFn(function(inst)
    inst.health_recover = 5
end)

local blunderbuss_5 = deepcopy(blunderbuss_4)
blunderbuss_5:SetName("tp_blunderbuss_5")
blunderbuss_5:HookFn(function(inst)
    inst.sanity_recover = nil
    inst.health_recover = nil
    inst:AddComponent("tprecharge")
    inst.components.tprecharge:SetRechargeTime(75)
    inst.skill_fn = function(item, player)
        item.components.tprecharge:SetRechargeTime()
        item:AddTag("tp_blunderbuss_dartle")
        item:do_task(20, function()
            item:RemoveTag("tp_blunderbuss_dartle")
        end)
    end
    inst.components.weapon.onprojectilelaunch = function(inst, attacker, target, proj)
        if inst:HasTag("tp_blunderbuss_dartle") then
            for i = 1, 4 do
                inst:do_task(.1*i, function()
                    local fx = WARGON.make_fx2("blunderbuss_proj", inst)
                    fx.components.tpfxproj:Throw(attacker, target, attacker)
                end)
            end
        end
    end
    inst.components.weapon:AddAttackFn(function(inst, owner, target)
        if inst:HasTag("tp_blunderbuss_dartle") then

                local ammo = owner.components.inventory:FindItem(function(item)
                    return item.prefab == "gunpowder"
                end)
                if ammo then
                    inst.components.trader:AcceptGift(owner, ammo)
                end

        end
    end)
end)

local space_converter = ItemData("tp_space_converter", {
    anim = {"tp_items", "tp_items", "tp_space_converter"},
    img = {"tp_items", "tp_space_converter"},
    fn = function(inst)
        inst:add_cmps({
            inspect = {},
        })
        inst:AddComponent("tpinterable")
        inst.components.tpinterable:SetCanFn(function(inst, target, doer)
            if target.components.door and target.components.trans_door
            and not target:HasTag("predoor") 
            and not target.components.door.hidden then
                return true
            end
        end)
        inst.components.tpinterable:SetFn(function(inst, target, doer)
            target.components.trans_door:PushScreen()
        end)
    end,
})

local inhibitor = ItemData("tp_inhibitor", {
    anim = {"tp_items", "tp_items", "tp_inhibitor"},
    img = {"tp_items", "tp_inhibitor"},
    fn = function(inst)
        inst:add_cmps({
            inspect = {},
            stack = {max=40},
            food = {
				health=0, hunger=0, san=0, 
				fn=function(inst, eater)
					if eater.components.tpbody then
                        eater.components.tpbody.poison_damage_stack=0
                    end
				end,
			},
        })
    end
})

return 
armormarble2,
armormarble3,
armormarble4,
armormarble5,
armorruins_2,
armorruins_3,
armorruins_4,
armorruins_5,
ruinshat_2,
ruinshat_3,
ruinshat_4,
ruinshat_5,
nightstick_2,
nightstick_3,
nightstick_4,
nightstick_5,
ruins_bat_2,
ruins_bat_3,
ruins_bat_4,
ruins_bat_5,
cutlass_2,
cutlass_3,
cutlass_4,
cutlass_5,
armorgrass_2,
halberd_2,
halberd_3,
halberd_4,
armor_sanity_2,
armor_sanity_3,
armor_sanity_4,
armor_sanity_5,
blunderbuss_2,
blunderbuss_3,
blunderbuss_4,
blunderbuss_5,
space_converter,
inhibitor,
unpack(many_items)