local Anims = {
    ring_1 = {"tp_ornaments", "tp_ornaments", "ring_1a",},
    ring_2 = {"tp_ornaments", "tp_ornaments", "ring_2a",},
    ring_3 = {"tp_ornaments", "tp_ornaments", "ring_3a",},
    necklace_1 = {"tp_ornaments", "tp_ornaments", "necklace_1a",},
    necklace_2 = {"tp_ornaments", "tp_ornaments", "necklace_2a",},
    necklace_3 = {"tp_ornaments", "tp_ornaments", "necklace_3a",},
}

local RingItem = Class(Prefab, function(self, name, data)
    Prefab._ctor(self, name)
    self.fn = function()
        local atlas, img = WARGON.resolve_img_path(data.img)
		img = string.sub(img, 1, -5)
	 	local inst = WARGON.make_prefab(data.anim, data.anim[4], "inv")
	 	inst:add_cmps({
	 		invitem = { atlas=atlas, img=img },
	 		inspect = {},
	 		equip = {
                slot = "body",
	 			equip = data.equip_fn,
	 			unequip = data.unequip_fn,
	 		},
	 	})
        inst:AddTag("tp_item")
        if data.necklace then
            inst.components.equippable.equipslot = EQUIPSLOTS.HEAD
            inst:AddTag("tp_core_necklace")
        else
            inst:AddTag("tp_core_ring")
        end
        if data.fn then
            data.fn(inst)
        end
        
        return inst
    end
end)

local NecklaceItem = Class(RingItem, function(self, name, data)
    data.necklace = true
    RingItem._ctor(self, name, data)
end)

local items = {}
for i = 1, 3 do
    local ring = RingItem("tp_ring_"..i, {
        anim = Anims["ring_"..i],
        img = {"tp_ornaments", "tp_ring_"..i},
        equip_fn = function(inst, owner)
            if inst.fx == nil then
                inst.fx = WARGON.make_fx2("star", inst, {owner=owner, num=i})

            end
        end,
        unequip_fn = function(inst, owner)
            if inst.fx then
                inst.fx:kill()
                inst.fx = nil
            end
        end,
        fn = function(inst)
            inst:AddComponent("tpornament")
            inst.components.tpornament.level = i
            inst:AddComponent("tpinter")
            inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
                return invitem:HasTag("tp_core_ring")
                    and invitem.level <= inst.components.tpornament.level
                    and inst.components.tpornament:CanSetCore()
            end)
        end,
    })
    table.insert(items, ring)
end

for i = 1, 3 do
    local necklace = NecklaceItem("tp_necklace_"..i, {
        anim = Anims["necklace_"..i],
        img = {"tp_ornaments", "tp_necklace_"..i},
        equip_fn = function(inst, owner)
            if inst.fx == nil then
                inst.fx = WARGON.make_fx2("spore", inst, {num=i, owner=owner})

            end
        end,
        unequip_fn = function(inst, owner)
            if inst.fx then
                inst.fx:kill()
                inst.fx = nil
            end
        end,
        fn = function(inst)
            inst:AddComponent("tpornament")
            inst.components.tpornament.level = i
            inst:AddComponent("tpinter")
            inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
                return invitem:HasTag("tp_core_necklace")
                    and invitem.level <= inst.components.tpornament.level
                    and inst.components.tpornament:CanSetCore()
            end)
        end,
    })
    table.insert(items, necklace)
end

return unpack(items)