local Anims = {

}

local function plant_retarget_fn(inst)
    local player = GetPlayer()
    local newtarget = FindEntity(inst, 20, function(guy)
        return guy.components.combat 
            and inst.components.combat:CanTarget(guy) 
            and (guy.components.combat.target == player 
            or player.components.combat.target == guy)
            and not guy:HasTag("tp_plant_army")
    end)
    return newtarget
end

local function plant_keep_target(inst, target)
    if target and target:IsValid() 
    and (target.components.health 
    and not target.components.health:IsDead()) then
        local distsq = target:GetDistanceSqToInst(inst)
        return distsq < 20*20
    else
        return false
    end
end

local PlantData = Class(Prefab, function(self, name, data)
    Prefab._ctor(self, name)
    self.fn = function()
        local inst = CreateEntity()
        local trans = inst.entity:AddTransform()
        local sound = inst.entity:AddSoundEmitter()
        local anim = inst.entity:AddAnimState()








        trans:SetFourFaced()
        anim:SetBank("wilson")
        anim:SetBuild("wormwood")
        anim:PlayAnimation("idle_loop")
        anim:SetMultColour(1, 1, 1, .5)

        inst:AddTag("tp_plant_army")
        inst:AddTag("companion")
        inst:AddComponent("combat")
        inst:AddComponent("inventory")
        inst:AddComponent("health")
        inst.components.health:SetMaxHealth(500)
        inst.components.health:StartRegen(20, 10)
        inst.components.combat:SetAttackPeriod(1)
        inst.components.combat:SetRetargetFunction(1, plant_retarget_fn)
        inst.components.combat:SetKeepTargetFunction(plant_keep_target)
        
        inst:SetStateGraph("SGtp_plant_army")
        inst:SetBrain(require "brains/tp_plant_army_brain")
        if data.fn then
            data.fn(inst)
        end

        return inst
    end
end)

local function plant_on_attacked(inst, data)
    local attacker = data and data.attacker
    if attacker == GetPlayer() or attacker:HasTag("tp_plant_army") then
        return
    end
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, 15, function(dude) 
        return dude:HasTag("tp_plant_army") 
    end, 10)
end

local function set_weapon(inst)
    if inst.components.finiteuses then
        inst:RemoveComponent("finiteuses")
    end
    inst:no_save()
    if inst.components.inventoryitem then
        inst.components.inventoryitem:SetOnDroppedFn(function()
            inst:Remove()
        end)
    end
end

local plants = {
    "carrot",
    "corn",
    "radish",
    "asparagus",
    "aloe",
    "pumpkin",
    "eggplant",
    "durian",
    "pomegranate",
    "dragonfruit",
    "watermelon",
    "sweet_potato",
}

local plant_fns = {

    carrot = function(inst)
        inst.components.combat:SetDefaultDamage(34)
        WARGON.EQUIP.object_on(inst, "swap_spear", "swap_spear")
        inst:ListenForEvent("attacked", plant_on_attacked)





    end,

    corn = function(inst)

        inst.components.combat:SetDefaultDamage(51)
        WARGON.EQUIP.object_on(inst, "swap_spike", "swap_spike")
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    radish = function(inst)

        inst.components.combat:SetDefaultDamage(60)
        WARGON.EQUIP.object_on(inst, "swap_ham_bat", "swap_ham_bat")
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    asparagus = function(inst)

        inst.components.combat:SetDefaultDamage(51)
        inst.components.combat:SetAttackPeriod(2)
        WARGON.EQUIP.object_on(inst, "swap_cork_bat", "swap_cork_bat")
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    aloe = function(inst)

        inst.components.combat:SetDefaultDamage(44)
        WARGON.EQUIP.object_on(inst, "swap_halberd", "swap_halberd")
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    pumpkin = function(inst)

        inst.components.combat:SetDefaultDamage(10)
        inst.components.combat:SetAttackPeriod(1)
        WARGON.EQUIP.object_on(inst, "swap_staffs", "bluestaff")
        inst:ListenForEvent("attacked", plant_on_attacked)
        inst:ListenForEvent("onhitother", function(inst, data)
            if data and data.target then
                WARGON.frozen_prefab(data.target)
            end
        end)
        inst:do_task(0, function()
            WARGON.EQUIP.equip_temp_weapon(inst, 10, {8, 12}, "ice_projectile")
        end)
    end,

    eggplant = function(inst)

        inst.components.combat:SetDefaultDamage(10)
        inst.components.combat:SetAttackPeriod(1)
        WARGON.EQUIP.object_on(inst, "swap_staffs", "redstaff")
        inst:ListenForEvent("attacked", plant_on_attacked)
        inst:ListenForEvent("onhitother", function(inst, data)
            if data and data.target then
                WARGON.fire_prefab(data.target)
            end
        end)
        inst:do_task(0, function()
            WARGON.EQUIP.equip_temp_weapon(inst, 10, {8, 12}, "fire_projectile")
        end)
    end,

    durian = function(inst)

        inst:AddTag("shooter")
        inst.components.combat:SetDefaultDamage(100)
        WARGON.EQUIP.object_on(inst, "swap_speargun_spear", "swap_speargun")
        inst:ListenForEvent("attacked", plant_on_attacked)
        inst:do_task(0, function()
            WARGON.EQUIP.equip_temp_weapon(inst, 100, {12, 14}, "speargun")
        end)
    end,

    pomegranate = function(inst)

        inst.components.combat:SetDefaultDamage(0)
        inst.components.combat:SetRange(10, 12)
        inst.components.combat:SetAttackPeriod(3)
        WARGON.EQUIP.object_on(inst, "swap_coconade", "swap_coconade")
        inst:AddComponent("thrower")
        inst.components.thrower.throwable_prefab = "cannonshot"
        inst:ListenForEvent("doattack", function(inst, data)
            if data.target then
                inst.components.thrower:Throw(data.target:GetPosition())
            end
        end)
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    dragonfruit = function(inst)

        inst:AddTag("shooter")
        inst.components.combat:SetDefaultDamage(200)
        WARGON.EQUIP.object_on(inst, "swap_blunderbuss_loaded", "swap_blunderbuss")
        inst:do_task(0, function()
            WARGON.EQUIP.equip_temp_weapon(inst, 200, {12, 14}, "gunpowder_projectile")
        end)
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    watermelon = function(inst)

        WARGON.EQUIP.object_on(inst, "swap_nightmaresword", "swap_nightmaresword")
        inst.components.combat:SetDefaultDamage(68)
        inst:ListenForEvent("attacked", plant_on_attacked)
    end,

    sweet_potato = function(inst)

        WARGON.EQUIP.object_on(inst, "swap_batbat", "swap_batbat")
        inst.components.combat:SetDefaultDamage(42)
        inst:ListenForEvent("attacked", plant_on_attacked)
        inst:ListenForEvent("onhitother", function(inst, data)
            if inst.components.health then
                inst.components.health:DoDelta(6.8)
            end
        end)
    end,
}

local plant_armys = {}
for k, v in pairs(plant_fns) do
    table.insert(plant_armys, PlantData("tp_army_"..k, {
        fn = v
    }))
end

return unpack(plant_armys)