local Anims = {
    bishop_charge = {"bishop_attack", "bishop_attack", "idle"},
    bulb = {"bulb", "bulb", "idle"},
    lance = {"tp_spear_lance", "tp_spear_lance", "idle"},
    boomerang = {"boomerang", "boomerang", "idle"},
}

local function MakeProj(name, anims, data, proj_fn)
    local function fn()
        local inst = CreateEntity()
        local trans = inst.entity:AddTransform()
        inst.Transform:SetFourFaced()
        local anim = inst.entity:AddAnimState()
        local sound = inst.entity:AddSoundEmitter()
        
        MakeInventoryPhysics(inst)
        RemovePhysicsColliders(inst)
        
        anim:SetBank(anims[1])
        anim:SetBuild(anims[2])
        anim:PlayAnimation(anims[3])
        
        inst:AddTag("projectile")
        inst.persists = false
        inst:AddComponent("weapon")
        inst:AddComponent("tpproj")


        inst.components.weapon:SetDamage(data.damage)

        inst.components.tpproj:SetSpeed(data.speed)


        inst.components.tpproj:SetOnHitFn(data.hit)
        inst.components.tpproj:SetOnMissFn(data.miss)
        inst.components.tpproj:SetLaunchOffset(data.offset)
        inst.components.tpproj.oncatch = data.catch
        if proj_fn then
            proj_fn(inst)
        end
        
        return inst
    end

    return Prefab( "common/inventory/"..name, fn, {}) 
end


local function proj_remove(inst)
    inst:Remove()
end

local function charge_proj_can_target(target, inst)
    if target.components.combat and target.components.health
    and inst.master.components.combat:CanTarget(target)
    and target.components.follower
    and target.components.follower.leader ~= GetPlayer() then
        for k, v in pairs(inst.no_targets) do
            if v == target then
                return false
            end
        end
        return true
    end
end

local function charge_proj_miss(inst, owner, target)
    inst:Remove()
    inst.SoundEmitter:PlaySound("dontstarve/creatures/bishop/shotexplo")
end


local function charge_proj_hit(inst, owner, target)
    table.insert(inst.no_targets, target)
    if inst:HasTag("tp_fire_charge") then
        WARGON.fire_prefab(target, nil)
    elseif inst:HasTag("tp_ice_charge") then
        WARGON.frozen_prefab(target, nil)
    end
    if #inst.no_targets <= 10 then


       
        local new_target = inst:wg_find(5, function(guy)
            if WARGON.can_enemy(owner, guy) then

                for k, v in pairs(inst.no_targets) do

                    if v == guy then

                        return false
                    end
                end
                return true
            end
        end)




















        if new_target then
            inst.components.tpproj:Throw(owner, new_target, owner)
        else
            inst:Remove()
        end
    else
        inst:Remove()
    end
    inst.SoundEmitter:PlaySound("dontstarve/creatures/bishop/shotexplo")

end

local charge_proj_data = {
    damage = 10, speed = 20, hit = charge_proj_hit,
    miss = charge_proj_miss, offset = Vector3(0, -1, 0),
}

local function charge_proj_fn(inst)
    inst.no_targets = {}
end

local function fire_charge_proj_fn(inst)
    inst.no_targets = {}
    inst:AddTag("tp_fire_charge")
    inst.AnimState:SetMultColour(1, .1, .1, 1)
end

local function ice_charge_proj_fn(inst)
    inst.no_targets = {}
    inst:AddTag("tp_ice_charge")
    inst.AnimState:SetMultColour(.1, .1, 1, 1)
end

local function soul_charge_hit(inst, owner, target)








    inst:Remove()
end

local soul_charge_data = {
    damage = 75, speed = 20, hit = soul_charge_hit,
    miss = proj_remove, offset = Vector3(0, -1, 0),
}

local function soul_charge_fn(inst)
    inst.AnimState:PlayAnimation("idle", true)
end

local spear_shoot_proj_data = {
    damage = 34, speed = 30, hit = proj_remove,
    miss = proj_remove, offset = Vector3(0, -1, 0),
}

local function spear_shoot_proj_fn(inst)
    inst.AnimState:PlayAnimation("idle", true)
    inst:AddTag("projectile")
    inst:AddComponent("fader")
    inst.glow = inst.entity:AddLight()    
    inst.glow:SetIntensity(.6)
    inst.glow:SetRadius(3)
    inst.glow:SetFalloff(1)

    inst.glow:SetColour(0.3, 0.6, 0.5)
    inst.glow:Enable(true)
end

local function spear_lance_proj_hit(inst, owner, target)
    inst.components.tpproj:Throw(owner, owner)
end

local function spear_lance_proj_catch(inst, owner)
    if owner.catch_spear then
        owner:catch_spear()
    end
    inst:Remove()
end

local spear_lance_proj_data = {
    damage = 75, speed = 20, hit = spear_lance_proj_hit,
    miss = spear_lance_proj_hit, offset = Vector3(0, -1, 0),
    catch = spear_lance_proj_catch,
}

local function spear_lance_proj_fn(inst)
    inst.AnimState:PlayAnimation("throw", true)
end

local boomerang_proj_data = {
    damage = 75, speed = 10, hit = spear_lance_proj_hit,
    miss = spear_lance_proj_hit, offset = Vector3(0, .2, 0),
    catch = spear_lance_proj_catch,
}

local function boomerang_proj_fn(inst)
    inst.AnimState:PlayAnimation("spin_loop", true)
end

return 
    MakeProj("tp_charge_proj", Anims.bishop_charge, charge_proj_data, charge_proj_fn),
    MakeProj("tp_fire_charge_proj", Anims.bishop_charge, charge_proj_data, fire_charge_proj_fn),
    MakeProj("tp_ice_charge_proj", Anims.bishop_charge, charge_proj_data, ice_charge_proj_fn),
    MakeProj("tp_spear_shoot_proj", Anims.bulb, spear_shoot_proj_data, spear_shoot_proj_fn),
    MakeProj("tp_spear_lance_proj", Anims.lance, spear_lance_proj_data, spear_lance_proj_fn),
    MakeProj("tp_boomerang_proj", Anims.boomerang, boomerang_proj_data, boomerang_proj_fn),

    MakeProj("tp_soul_charge", Anims.bishop_charge, soul_charge_data, soul_charge_fn)