WARGON.SCROLL_PIG_FNS = {}

local papyrus = {"papyrus", "papyrus", "idle"}
local Anims = {
	papyrus = {"papyrus", "papyrus", "idle", },
}

local function onfinished(inst)
	inst:Remove()
end

local function on_use(inst)
	print(inst)
	local item = inst.components.stackable and inst.components.stackable:Get() or inst
	item:Remove()
end

local function MakeScroll(name, anims, book_fn)
	WARGON.SCROLL_PIG_FNS[name] = book_fn
	local function fn()
		local inst = WARGON.make_prefab(anims, nil, "inv")
		WARGON_CMP_EX.add_cmps(inst, 
		{
			invitem = {atlas="images/inventoryimages/"..name..'.xml', 
				img=name},
			inspect = {},
			stack = {max=20},
			book = {fn = book_fn, act="map"},

		})
		inst:AddTag("tp_scroll")

	    return inst
	end
	return Prefab("common/inventory/"..name, fn, {})
end

local function search_pigs(reader, num)
	local pigs = WARGON.finds(reader, 40, {'pig'}, {'werepig', 'guard'})
	if pigs then
		num = num or 8
		for k, v in pairs(pigs) do
			if (not (v.components.follower
			and v.components.follower.leader == reader))
			or k > num  then
				pigs[k] = nil
			end
		end
	end
	return pigs
end

local function common_fn(inst, reader, name)
	local pigs = search_pigs(reader)
	if pigs then
		for k, v in pairs(pigs) do
			if v.components.tpbuff then
				v.components.tpbuff:AddBuff(name)
				WARGON.make_fx(v, "tp_fx_"..name)
			end
		end
	end
	on_use(inst)
	return true
end

local function armor_fn(inst, reader)
	local pigs = search_pigs(reader, 8)
	if pigs then
		for k, v in pairs(pigs) do
			if reader.components.inventory:Has("pigskin", 1) then
				reader.components.inventory:ConsumeByName("pigskin", 1)
				if v.components.inventory then
					local current = v.components.inventory:GetEquippedItem(EQUIPSLOTS.HEAD)
					if current then
						v.components.inventory:DropItem(current)
					end
					local hat = SpawnPrefab("footballhat")
					v.components.inventory:Equip(hat)
					v.AnimState:Show("hat")
					WARGON.make_fx(v, "tp_fx_scroll_pig_armor")

				end
			else
				break
			end
		end
	end
	on_use(inst)
	return true
end

local function armorex_fn(inst, reader)
	return common_fn(inst, reader, "scroll_pig_armorex")
end

local function speed_fn(inst, reader)
	return common_fn(inst, reader, "scroll_pig_speed")
end

local function damage_fn(inst, reader)
	return common_fn(inst, reader, "scroll_pig_damage")
end

local function heal_fn(inst, reader)
	return common_fn(inst, reader, "scroll_pig_heal")
end

local TpTransPanel = require "screens/tp_trans_panel"
local function teleport_fn(inst, reader)










	TheFrontEnd:PushScreen(TpTransPanel())
	return true
end

local function leader_fn(inst, reader)
	local pigs = WARGON.finds(reader, 30, {'pig'}, {'werepig', 'guard'})
	for k, v in pairs(pigs) do
		if reader.components.inventory:Has("meat", 1) then
			if v.components.follower then
				if v.components.follower.leader == nil then
					reader.components.leader:AddFollower(v)
				end
				local time = 480
				v.components.follower:AddLoyaltyTime(time)
				WARGON.make_fx(v, "tp_fx_scroll_pig_leader")
				reader.components.inventory:ConsumeByName("meat", 1)
			end
		else
			break
		end
	end
	on_use(inst)
	return true
end

local function health_fn(inst, reader)
	return common_fn(inst, reader, "scroll_pig_health")
end

local function wind_fn(inst, reader)
	if reader.components.tpbuff then
		reader.components.tpbuff:AddBuff("scroll_wind")
		reader.components.sanity:DoDelta(-50)
	end
	on_use(inst)
	return true
end

local function pigman_fn(inst, reader)
	reader.components.sanity:DoDelta(-10)

	local pos = WARGON.around_land(reader, math.random(3, 6))
	if pos then
		local pig = WARGON.make_spawn(pos, "pigman")
		if WARGON.on_water(pig) then
			WARGON.make_fx(pos, "splash_water_sink")
			pig:Remove()
		else
			WARGON.make_fx(pos, "tp_fx_shadow_bat")
			WARGON.make_fx(pos, "statue_transition")

			WARGON.make_fx(pos, "vortex_cloak_fx")
			reader.components.leader:AddFollower(pig)
			pig.components.follower:AddLoyaltyTime(1200)
		end
		on_use(inst)
		return true
	end
end

local function bunnyman_fn(inst, reader)
	reader.components.sanity:DoDelta(-10)

	local pos = WARGON.around_land(reader, math.random(3, 6))
	if pos then
		local pig = WARGON.make_spawn(pos, "bunnyman")
		if WARGON.on_water(pig) then
			WARGON.make_fx(pos, "splash_water_sink")
			pig:Remove()
		else
			WARGON.make_fx(pos, "tp_fx_shadow_bat")
			WARGON.make_fx(pos, "statue_transition")

			WARGON.make_fx(pos, "vortex_cloak_fx")
			reader.components.leader:AddFollower(pig)
			pig.components.follower:AddLoyaltyTime(1200)
		end
		on_use(inst)
		return true
	end
end

local function shadow_fn(inst, reader)
	if reader.components.sanity:GetMaxSanity() >= TUNING.SHADOWWAXWELL_SANITY_PENALTY then
		local pos = reader:around_land(math.random(3,6))
		if pos then

			WARGON.do_task(reader, .5, function()
				local shadow = WARGON.make_spawn(pos, "tp_unreal_wilson")
				if WARGON.on_water(shadow) then
					shadow:SpawnShadowBoat(pos)
				end
				shadow.components.follower:SetLeader(reader)
				reader.components.health:DoDelta(-TUNING.SHADOWWAXWELL_HEALTH_COST)
			    reader.components.sanity:RecalculatePenalty()
			    reader.SoundEmitter:PlaySound("dontstarve/maxwell/shadowmax_appear")
			end)
			on_use(inst)
			return true
		end
	end
end

local function harvest_fn(inst, reader)
	reader.components.sanity:DoDelta(-10)
	local ents = reader:wg_finds(15)
	for k, v in pairs(ents) do
		if v.components.pickable and v.prefab ~= "flower" then
			v.components.pickable:Pick(GetPlayer(), v:GetPosition())
		end
		if v.components.crop then
			v.components.crop:Harvest(GetPlayer(), v:GetPosition())
		end
	end
	on_use(inst)
	return true
end

local function boss_fn(inst, reader, boss_name)
	reader.components.sanity:DoDelta(-50)
	local pos = reader:around_land(4)
	if pos then
		if not reader:HasTag("play_boss_music") or math.random() < .1 then
			if boss_name == "moose" then
				reader.SoundEmitter:PlaySound("tp_boss_scroll_bgm/music/bgm2", "tp_boss_scroll")
			else
				reader.SoundEmitter:PlaySound("tp_boss_scroll_bgm/music/bgm", "tp_boss_scroll")
			end
			reader:AddTag("play_boss_music")
		end

		WARGON.make_fx2("shadow_spawn", pos)
		reader:StartThread(function()
			for i = 1, 5 do
				local angle = DEGREES*360/5*i
				local radius = 6
				local offset = pos + Vector3(math.cos(angle)*radius, 0, math.sin(angle)*radius)

				WARGON.make_fx2("lightning", offset)
				Sleep(.5)
			end

			WARGON.make_fx2("collapse_big", pos)
			local boss_sp = WARGON.make_spawn(pos, "boss_follower_sp")
			boss_sp.boss_name = boss_name
		end)
		local owner = inst.components.inventoryitem.owner
		local other_boss = inst:wg_find(9999, function(item, inst)
			return item.prefab == boss_name
		end, {"tp_summon_boss"})
		if not other_boss and owner 
		and owner.prefab == "tp_spawn_book" then



			if owner.components.container:Has("tp_scroll", 3) then
				if inst.components.container then
					inst.components.container:DropEverything()
				end
				owner.components.container:ConsumeByName("tp_scroll", 3)
			end
		else
			on_use(inst)
		end
		return true
	end
end

local function bearger_fn(inst, reader)
	return boss_fn(inst, reader, "bearger")
end

local function dragonfly_fn(inst, reader)
	return boss_fn(inst, reader, "dragonfly")
end

local function moose_fn(inst, reader)
	return boss_fn(inst, reader, "moose")
end

local function super_fn(inst, reader)
	local pigs = search_pigs(reader, 8)
	if pigs then
		for k, v in pairs(pigs) do
			v.AnimState:SetMultColour(.1, .1, .5, .5)


			WARGON.make_fx2("statue_transition", v)
			WARGON.make_fx2("statue_transition_2", v)
			WARGON.no_save(v)
			v.components.health:SetAbsorptionAmount(.99)
			v:RemoveComponent("burnable")
			v:RemoveComponent("propagator")
			v:RemoveComponent("poisonable")
			v:RemoveComponent("freezable")

			v:AddTag("scroll_pig_super")
		end
	end
	on_use(inst)
	return true
end

local function army_fn(inst, reader)
	local stack = 1
	if inst.components.stackable then
		stack = inst.components.stackable:StackSize()
	end
	reader:StartThread(function()
		for i = 1, stack*4 do
			reader.components.sanity:DoDelta(-10)
			local pos = WARGON.around_land(reader, math.random(3, 6))
			if pos then
				local pig = WARGON.make_spawn(pos, "pigman")
				if WARGON.on_water(pig) then
					WARGON.make_fx(pos, "splash_water_sink")
					pig:Remove()
				else

					WARGON.make_fx(pos, "statue_transition")


					reader.components.leader:AddFollower(pig)
					pig.components.follower:AddLoyaltyTime(1200)
				end
			end
			Sleep(.2)
		end
	end)
	inst:Remove()
	return true
end

local function flower_grow_big(inst)
	local s = .1
	inst.Transform:SetScale(s, s, s)
	inst:StartThread(function()
		for i = 1, 10 do
			s = .1*i
			inst.Transform:SetScale(s, s, s)
			Sleep(.1)
		end
	end)
end

local function flower_fn(inst, reader)
	local pt = inst:GetPosition()
	local valid_tiles = {4,5,6,7,30}
	reader:StartThread(function()
		for i = 1, 6 do
			Sleep(.2)
			local angle = 60*i*PI/180
			local radius = 4
			local x = math.cos(angle) * radius
			local z = math.sin(angle) * radius
			local pos = pt + Vector3(x, 0, z)
			local tile =  WARGON.get_tile(pos) 
			for k, v in pairs(valid_tiles) do
				if tile == v then
					local flower = WARGON.make_spawn(pos, "flower")
					flower_grow_big(flower)
				end
			end
		end
	end)
	on_use(inst)
	return true
end

return
MakeScroll("scroll_pig_armor", Anims.papyrus, armor_fn),
MakeScroll("scroll_pig_armorex", Anims.papyrus, armorex_fn),
MakeScroll("scroll_pig_speed", Anims.papyrus, speed_fn),
MakeScroll("scroll_pig_damage", Anims.papyrus, damage_fn),
MakeScroll("scroll_pig_heal", Anims.papyrus, heal_fn),

MakeScroll("scroll_pig_leader", Anims.papyrus, leader_fn),
MakeScroll("scroll_pig_health", Anims.papyrus, health_fn),
MakeScroll("scroll_wind", Anims.papyrus, wind_fn),
MakeScroll("scroll_pigman", Anims.papyrus, pigman_fn),
MakeScroll("scroll_bunnyman", Anims.papyrus, bunnyman_fn),
MakeScroll("scroll_shadow", Anims.papyrus, shadow_fn),
MakeScroll("scroll_bearger", Anims.papyrus, bearger_fn),
MakeScroll("scroll_dragonfly", Anims.papyrus, dragonfly_fn),
MakeScroll("scroll_moose", Anims.papyrus, moose_fn),
MakeScroll("scroll_pig_super", Anims.papyrus, super_fn),
MakeScroll("scroll_pig_army", Anims.papyrus, army_fn),
MakeScroll("scroll_flower", Anims.papyrus, flower_fn),

MakeScroll("scroll_harvest", Anims.papyrus, harvest_fn)