local assets =
{
    Asset("ANIM", "anim/spat_basic.zip"),
    Asset("ANIM", "anim/spat_actions.zip"),
    Asset("ANIM", "anim/spat_build.zip"),
    Asset("ANIM", "anim/spat_phlegm.zip"),
    Asset("SOUND", "sound/beefalo.fsb"),
}

local prefabs =
{
    "meat",
    "poop",
    "steelwool",
    "phlegm",
    "spat_bomb",
}

local projectile_assets =
{
    Asset("ANIM", "anim/spat_bomb.zip"),
}

local projectile_prefabs =
{
    "spat_splat_fx",
    "spat_splash_fx_full",
    "spat_splash_fx_med",
    "spat_splash_fx_low",
    "spat_splash_fx_melted",
}

local brain = require("brains/spatbrain")

SetSharedLootTable( 'spat',
{
    {'meat',            1.00},
    {'meat',            1.00},
    {'meat',            1.00},
    {'meat',            1.00},
    {'steelwool',       1.00},
    {'steelwool',       1.00},
    {'steelwool',       0.50},
    {'phlegm',          1.00},
    {'phlegm',          0.50},
})

local sounds =
{
    walk = "dontstarve/creatures/spat/walk",
    grunt = "dontstarve/creatures/spat/grunt",
    yell = "dontstarve/creatures/spat/yell",
    hit = "dontstarve/creatures/spat/hurt",
    death = "dontstarve/creatures/spat/death",
    curious = "dontstarve/creatures/spat/curious",
    sleep = "dontstarve/creatures/spat/sleep",
    angry = "dontstarve/creatures/spat/angry",
    spit = "dontstarve/creatures/spat/spit",
    spit_hit = "dontstarve/creatures/spat/spit_hit",
}

local function Retarget(inst)
    return FindEntity(
        inst,
        TUNING.SPAT_TARGET_DIST,
        function(guy)
            return inst.components.combat:CanTarget(guy)
        end,
        nil,
        nil,
        { "player", "monster" }
    )
end

local function KeepTarget(inst, target)
    return target:IsNear(inst, TUNING.SPAT_CHASE_DIST)
end

local function OnAttacked(inst, data)
    local target = inst.components.combat.target
    if target ~= nil and target.components.pinnable ~= nil and target.components.pinnable:IsStuck() then

        return
    end

    inst.components.combat:SetTarget(data.attacker)
end

local function EquipWeapons(inst)
    if inst.components.inventory ~= nil and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local snotbomb = CreateEntity()
        snotbomb.name = "Snotbomb"
        snotbomb.entity:AddTransform()
        MakeInventoryPhysics(snotbomb)
        snotbomb:AddComponent("weapon")
        snotbomb.components.weapon:SetDamage(TUNING.SPAT_PHLEGM_DAMAGE)
        snotbomb.components.weapon:SetRange(TUNING.SPAT_PHLEGM_ATTACKRANGE)
        snotbomb.components.weapon:SetProjectile("spat_bomb")
        snotbomb:AddComponent("inventoryitem")
        snotbomb.persists = false
        snotbomb.components.inventoryitem:SetOnDroppedFn(snotbomb.Remove)
        snotbomb:AddComponent("equippable")
        snotbomb:AddTag("snotbomb")

        inst.components.inventory:GiveItem(snotbomb)
        inst.weaponitems.snotbomb = snotbomb

        local meleeweapon = CreateEntity()
        meleeweapon.name = "Snaut Bash"
        meleeweapon.entity:AddTransform()
        MakeInventoryPhysics(meleeweapon)
        meleeweapon:AddComponent("weapon")
        meleeweapon.components.weapon:SetDamage(TUNING.SPAT_MELEE_DAMAGE)
        meleeweapon.components.weapon:SetRange(TUNING.SPAT_MELEE_ATTACKRANGE)
        meleeweapon:AddComponent("inventoryitem")
        meleeweapon.persists = false
        meleeweapon.components.inventoryitem:SetOnDroppedFn(meleeweapon.Remove)
        meleeweapon:AddComponent("equippable")
        meleeweapon:AddTag("meleeweapon")

        inst.components.inventory:GiveItem(meleeweapon)
        inst.weaponitems.meleeweapon = meleeweapon
    end
end

local function CustomOnHaunt(inst)
    inst.components.periodicspawner:TrySpawn()
    return true
end

local function action_button_picker(inst)
    return BufferedAction(inst, inst.components.tpvehicleowner.vehicle, ACTIONS.TP_DOWN_CAR)
end

local function left_click_picker(inst, target, pos)
    if target and target.components.tpvehicle
    and target.components.tpvehicle:CanDownCar() then
        return inst.components.playeractionpicker:SortActionList({ACTIONS.TP_DOWN_CAR}, target, nil)
    end



end

local function right_click_picker(inst, target, pos)
    return {}
end

local function energy_off(inst)

    inst.components.tpenergy:RemoveBurnModifier("shovel")
    if inst.task then
        inst.task:Cancel()
        inst.task = nil
    end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()

    MakeCharacterPhysics(inst, 100, .5)

    WARGON.make_map(inst, "spat_head.tex")

    inst.DynamicShadow:SetSize(6, 2)
    inst.Transform:SetSixFaced()

    inst.AnimState:SetBank("spat")
    inst.AnimState:SetBuild("spat_build")
    inst.AnimState:PlayAnimation("idle_loop", true)





    inst.sounds = sounds




    

    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "spat_body"


    inst.components.combat:SetAttackPeriod(3)
    inst.components.combat:SetHurtSound(sounds.hit)

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(TUNING.SPAT_HEALTH)



    inst:AddComponent("lootdropper")


    inst:AddComponent("inspectable")













    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 1.5
    inst.components.locomotor.runspeed = 7





    inst:SetStateGraph("SGspat")



    inst:AddTag("scarytoprey")
    inst:AddTag("companion")
    inst:AddTag("tp_spat")

    inst:AddComponent("tpenergy")
    inst.components.tpenergy:SetMax(500)
    inst.components.tpenergy:SetRate(.5)
    inst.components.tpenergy.run_out = function(inst)
        energy_off(inst)
        GetPlayer().components.talker:Say("燃料耗尽，所有效果关闭")
    end

    inst:AddComponent("tpvehicle")
    inst:AddComponent("tpvehiclecontroller")
    inst:AddComponent("tpvehicleactionpicker")
    inst.components.tpvehicle.ex_up_car = function(inst, doer)
        doer.components.playercontroller.actionbuttonoverride = action_button_picker
        doer.components.playeractionpicker.leftclickoverride = left_click_picker
        doer.components.playeractionpicker.rightclickoverride = right_click_picker
    end
    inst.components.tpvehicle.ex_down_car = function(inst, doer)
        doer.components.playercontroller.actionbuttonoverride = nil
        doer.components.playeractionpicker.leftclickoverride = nil
        doer.components.playeractionpicker.rightclickoverride = nil

        energy_off(inst)
    end

    return inst
end





















































































return Prefab("common/object/tp_spat", fn, assets, prefabs),
MakePlacer("common/tp_spat_placer", "spat", "spat_build", "idle_loop")

