local Phys = {
	tent = {1, nil},
	ham = {.5, nil},
	bench = {2, 1.2},
	obelisk = {1, nil},
	birdcage = {.5, nil},
	ice_statue = {1.5, nil},
	smelter = {.5},
	basefan = {.5},
}
local Maps = {

	tent = "tp_tent.tex",

	bench = "tp_bench.tex",
	box = 'treasure_chest_sacred.tex',
	cork_chest = 'cork_chest.png',
	lake = "tp_moon_lake.tex",
	shadow_statue = 'tp_shadow_statue.tex',
	smelter = "smelter.png",
	basefan = "basefan.png",
}
local Anims = {

	tent = {"tp_structures_3", "tp_structures_3", "tp_tent"},
	ham = {"ham_bat", "ham_bat", "idle"},
	bench = {"workbench_obsidian", "workbench_obsidian", "idle"},
	box = {"chest", "treasure_chest_sacred", "closed"},
	cork_chest = {"treasure_chest_cork", "treasure_chest_cork", "closed"},
	lake = {"oasis_tile", "oasis_tile", "idle"},
	wathgrithr = {"wilson", "wathgrithr", "sleep"},
	wendy = {"wilson", "wendy", "sleep"},
	wes = {"wilson", "wes", "sleep"},
	waxwell = {"wilson", "waxwell", "sleep"},

	birdcage = {"tp_structures_3", "tp_structures_3", "tp_dragon_cage"},
	bearger = {"bearger", "bearger_build", "idle_loop"},
	dragonfly = {"dragonfly", "dragonfly_fire_build", "idle"},
	deerclops = {"deerclops", "deerclops_build", "idle_loop"},
	moose = {"goosemoose", "goosemoose_build", "idle"},
	mole_move = {"mole_fx", "mole_move_fx", "move"},
	fertilizer = {"fertilizer", "fertilizer", "idle"},
	shadow_statue = {"tp_shadow_statue", "tp_shadow_statue", "idle_full"},
	sanityrock = {"blocker_sanity", "blocker_sanity", "idle_inactive"},
	smelter = {"smelter", "smelter", "idle_empty"},

	cookpot_tureen = {"tp_structures_3", "tp_structures_3", "tp_fire_pot"},

	cookpot_cauldron = {"tp_structures_3", "tp_structures_3", "tp_rot_pot"},

	cookpot_clay = {"tp_structures_3", "tp_structures_3", "tp_dry_pot"},
	boat_winch = {"boat_winch", "boat_winch", "idle"},
	pig_house = {"pig_house", "pig_house", "idle"},
	lab_hutch = {"researchlab4", "researchlab4_hutch_costume", "idle"},
	sign_factory = {"sign_home", "arrowsign_post_factory", "idle"},
	tech3_skin_tophat = {"researchlab4", "researchlab4_tophat_circus", "idle"},
	tech3_skin_chester = {"researchlab4", "researchlab4_chester_costume", "idle"},
	basefan = {"basefan", "basefan", "off"},
	structures = {"tp_structures", "tp_structures", "tp_auto_farmper"},
	structures2 = {"tp_structures_2", "tp_structures_2", "tp_farm_pile"},
}

local function MakeStructure(name, anims, structure_fn, phy, map)
	local function fn()
		local the_phy = nil
		if phy then
			the_phy = {'obs', phy[1], phy[2]}
		end
		local inst = WARGON.make_prefab(anims, nil, the_phy, nil, nil, nil)
		if type(map) == "table" then
			WARGON.make_map(inst, map[1], map[2])
		elseif map ~= nil then
			WARGON.make_map(inst, map)
		end

		if structure_fn then
			structure_fn(inst)
		end

		return inst
	end

	return Prefab("common/objects/"..name, fn, {})
end

local function common_structure(inst, hit_fn, ham_fn)
	local function hit(inst, worker)
		if hit_fn then
			hit_fn(inst, worker)
		end
	end

	local function ham(inst, worker)
		if inst.components.lootdropper then
			inst.components.lootdropper:DropLoot()
		end
		SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
		if inst:HasTag("tp_snd_wood") then
			inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
		elseif inst:HasTag("tp_snd_metal") then
			inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
		end
		if ham_fn then
			ham_fn(inst, worker)
		end
		inst:Remove()
	end

	inst:AddTag("structure")
	inst:add_cmps({
		inspect = {},
		loot = {},
		work = {act=ACTIONS.HAMMER, num=4, ham=ham, hit=hit},
	})
end

local function make_strucutres_placer(prefab)
	return WARGON.make_placer(prefab, {"tp_structures", "tp_structures", prefab})
end

local function tent_ham(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function tent_hit(inst, worker)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("idle", true)
	end
end

local function tent_finish(inst)
	if not inst:HasTag("burnt") then


		inst.SoundEmitter:PlaySound("dontstarve/common/tent_dis_pre")

		inst:DoTaskInTime(16*FRAMES, function() inst.SoundEmitter:PlaySound("dontstarve/common/tent_dis_twirl") end)
	end
end

local function tent_built(inst)


	inst.SoundEmitter:PlaySound("dontstarve/common/craftable/tent")
end

local function tent_save(inst, data)
	WARGON.burn_save(inst, data)
end

local function tent_load(inst, data)
	WARGON.burn_load(inst, data)
end

local function tent_sleep(inst, sleeper)
	if inst.components.finiteuses:GetUses() <= 0 then
		return
	else
		WARGON_SLEEP_EX.sleep_tent(inst, sleeper)
	end
end

local function tent_fn(inst)
	inst:AddTag("structure")
	WARGON_CMP_EX.add_cmps(inst, {
		inspect = {},
		loot = {},
		work = {act=ACTIONS.HAMMER, num=4, ham=tent_ham, hit=tent_hit},
		finite = {max=10, use=10, fn=tent_finish},
		sleepingbag = {},
		fueled = {time=100, typ="USAGE"},
	})
	inst.components.sleepingbag.onsleep = tent_sleep 
	WARGON.add_listen(inst, {
		onbuilt = tent_built,
		})
	inst.components.fueled:SetPercent(.9)
	inst.components.fueled:StopConsuming()


	WARGON.add_tags(inst, {'tp_tent', 'tp_sea_sleep'})




end

local function bench_fn(inst)
	
end

local function chest_open(inst)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("open")
		inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
		if WARGON.is_full_moon() then
			WARGON.make_fx(inst, "statue_transition")
			local nightmares = {
				"rawlingnightmare", "nightmarebeak",
			}
			for i = 1, math.random(5) do
				local pos = inst:around_land(math.random(2,4))
				if pos then
					WARGON.make_spawn(pos, nightmares[math.random(2)])
				end
			end
		end
	end
end

local function chest_close_update(inst)










end

local function chest_close(inst)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("close")
		inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
		chest_close_update(inst)
	end
end

local function chest_test(inst, item, slot)
	return false
end

local function chest_ham(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
		inst.components.burnable:Extinguish()
	end

	WARGON.make_fx(inst, "collapse_small")
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")	
	inst:Remove()
end

local function chest_hit(inst, worker)
	if not inst:HasTag("burnt") then
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("closed", true)
		if inst.components.container then 
			inst.components.container:Close()
		end
	end
end

local function chest_built(inst, data)
	chest_close_update(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("closed", true)
	inst.SoundEmitter:PlaySound("dontstarve/common/craftable/chest")
end

local function chest_item_lose(inst, data)
	local player = GetPlayer()
	player.components.health:DoDelta(-15)
end

local function chest_save(inst, data)
end

local function chest_load(inst, data)
	chest_close_update(inst)
end

local function chest_fn(inst)
	inst:AddTag("structure")
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		cont = {num=9, open=chest_open, close=chest_close, widgets={}, test=chest_test},
		work = {act=ACTIONS.HAMMER, num=2, ham=chest_ham, hit=chest_hit},
		loot = {},
	})
	WARGON.add_tags(inst, {'chest', 'tp_chest'})




	MakeSnowCovered(inst, 0.01)	
	inst.OnSave = chest_save
	inst.OnLoad = chest_load





end

local function bangalore_ham(inst)

	inst:boom(inst)
end

local function bangalore_onignite(inst)

	inst:boom(inst)
end

local function bangalore_built(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("closed")
	inst.SoundEmitter:PlaySound("dontstarve_DLC003/common/crafted/cork_chest/place")
end

local function bangalore_explode(inst, scale)
	scale = scale or 1
	local pos = Vector3(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")

	local explode = SpawnPrefab("explode_large")
	local ring = SpawnPrefab("explodering_fx")
	local pos = inst:GetPosition()

	ring.Transform:SetPosition(pos.x, pos.y, pos.z)
	ring.Transform:SetScale(scale, scale, scale)

	explode.Transform:SetPosition(pos.x, pos.y, pos.z)
	explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
	explode.AnimState:SetLightOverride(1)
	explode.Transform:SetScale(scale, scale, scale)
end

local function bangalore_boom(inst)
	WARGON.do_task(inst, .3, function()
		inst.components.explosive:OnBurnt()
	end)
end

local function bangalore_fn(inst)
	inst.AnimState:SetMultColour(1,.1,.1,1)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},

	})
	inst:AddComponent("burnable")
	inst.components.burnable.onignite = bangalore_onignite
	inst.components.burnable.nofx = true
	inst:AddComponent("explosive")
	inst.components.explosive:SetOnExplodeFn(bangalore_explode)
	inst.components.explosive.explosivedamage = 100
	inst.components.explosive.explosiverange = TUNING.COCONADE_EXPLOSIONRANGE
	inst.components.explosive.buildingdamage = TUNING.COCONADE_BUILDINGDAMAGE
	WARGON.make_prop(inst, "small")
	WARGON.add_tags(inst, {'tp_bangalore'})
	WARGON.add_listen(inst, {
		onbuilt = bangalore_built,
		})
	WARGON.do_task(inst, 60, function()
		inst.components.explosive:OnBurnt()
	end)
	MakeSnowCovered(inst, 0.01)
	inst.boom = bangalore_boom
end

local function common_spawn_fn(inst, spawn_prefab)
	WARGON.do_task(inst, 0, function(inst)
		if type(spawn_prefab) == "string" then
			spawn_prefab = {spawn_prefab}
		end
		for k, v in pairs(spawn_prefab) do
			if c_find(v) == nil then
				WARGON.make_spawn(inst, v)
				break
			end
		end
		inst:Remove()
	end)
end

local function start_base_fn(inst)
	WARGON.do_task(inst, 0, function()
		WARGON.make_spawn(inst, "tp_fake_knight_room")
		inst:Remove()
	end)
end

local function werepig_start_fn(inst)
	local function crt_rock(x, z)
		local rock = SpawnPrefab("wall_stone")
		rock.Transform:SetPosition(x, 0, z)
	end
	WARGON.do_task(inst, 0, function()

		WARGON.make_spawn(inst, "tp_grass_pigking")
		inst:Remove()
	end)
end

local function unreal_rock_fn(inst)
	inst:AddTag("tp_unreal_rock")
end

local function moon_sea_point_fn(inst)




	inst:AddTag("tp_moon_sea_point")











end

local function moon_sea_handler_near(inst)
	local pos = WARGON.around_land(inst, 3)
	if pos then
		WARGON.make_spawn(pos, "tp_fool_spider")
		WARGON.make_spawn(pos, "tp_moon_sea_point")
		inst:Remove()
	end
end

local function moon_sea_handler_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		near = {dist={2,3}, near=moon_sea_handler_near}
	})
end

local function leif_spawner_fn(inst)






	common_spawn_fn(inst, {
		"tp_leif", "tp_leif_seed", "tp_leif_level", "tp_leif_sale", 
	})
end

local function pot_bird_egg_spawner_fn(inst)
	common_spawn_fn(inst, "tp_pot_bird_egg")
end

local function moon_lake_fn(inst)
	inst.AnimState:PlayAnimation("idle", true)
	inst:AddTag("tp_moon_lake")
	inst:AddTag("NOCLICK")
	inst:AddTag("FX")
	inst.AnimState:SetOrientation( ANIM_ORIENTATION.OnGround )
	inst.AnimState:SetLayer( LAYER_BACKGROUND )
	inst.AnimState:SetSortOrder( 3 )
	local s = 1.5
	inst.Transform:SetScale(s, s, s)
end

local function moon_rock_fn(inst)
	inst:ListenForEvent("nighttime", function()
		if WARGON.is_full_moon() then
			WARGON.make_fx(inst, "sanity_lower")
			inst:Remove()
		end
	end, GetWorld())
end

local function boss_sleep_wake(inst)
	inst.AnimState:PlayAnimation("wakeup")
	inst:ListenForEvent("animover", function()
		if inst.boss then
			WARGON.make_spawn(inst, inst.boss)
		end
		inst:Remove()
	end)
end

local function boss_sleep_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
	})
	inst:AddComponent("tpuse")
	inst.components.tpuse.use = function()
		boss_sleep_wake(inst)
	end
end

local function combat_lord_sleep_fn(inst)
	boss_sleep_fn(inst)
	inst.boss = "tp_combat_lord"
	WARGON.EQUIP.body_on(inst, "armor_wood_fangedcollar", "swap_body")
	WARGON.EQUIP.hat_on(inst, "footballhat_combathelm", nil, true)
	WARGON.EQUIP.object_on(inst, "swap_spear_forge_gungnir", "swap_spear_gungnir")
end

local function hornet_sleep_fn(inst)
	boss_sleep_fn(inst)
	inst.boss = "tp_hornet"
	WARGON.EQUIP.body_on(inst, "armor_vortex_cloak", "swap_body")
	WARGON.EQUIP.hat_on(inst, "hat_bandit", nil, true)
	WARGON.EQUIP.object_on(inst, "tp_spear_lance", "swap_object")
end

local function soul_student_sleep_fn(inst)
	boss_sleep_fn(inst)
	inst.boss = "tp_soul_student"
	WARGON.EQUIP.hat_on(inst, "tophat_witch_pyre", nil, true)
	WARGON.EQUIP.object_on(inst, "swap_firestaff_meteor", "swap_redstaff")
end

local function fake_knight_sleep_fn(inst)
	inst.boss = "tp_fake_knight"
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		near = {dist={4,6}, near=boss_sleep_wake},
	})
	WARGON.EQUIP.body_on(inst, "armor_metalplate", "swap_body")
	WARGON.EQUIP.hat_on(inst, "hat_metalplate", nil, true)
	WARGON.EQUIP.object_on(inst, "swap_halberd", "swap_halberd")
end

local function boss_spawner_fn(inst)

end

local function boss_spawner_fn_2(inst)

end

local function dragon_cage_hit(inst)


	if inst.fx_front then


	end
	if inst.fx_fly then


	end
end

local function dragon_cage_ham(inst)
	WARGON.make_fx(inst, "collapse_small")
	inst.components.lootdropper:DropLoot()
	if inst.fx_fly then
		inst.fx_fly:Remove()
		inst.fx_fly = nil
	end
	if inst.fx_front then
		inst.fx_front:Remove()
		inst.fx_front = nil
	end
	inst:Remove()
end

local function dragon_cage_accept(inst, giver, item)	
	local seedspawnprefab = "seeds_cooked"

	if inst.fx_fly then
		inst.fx_fly.AnimState:PlayAnimation("taunt_pre")
		inst.fx_fly.AnimState:PushAnimation("taunt")
		inst.fx_fly.AnimState:PushAnimation("taunt_pst")
		inst.fx_fly.AnimState:PushAnimation("idle", true)
	end
	inst:DoTaskInTime(60*FRAMES, function()
		local loots = {
			bird_egg = 50,
			tallbirdegg = 35,
			flint = 1,
			obsidian = 1,
			nitre = 1,
			rocks = 1,
			marble = 1,
			iron = 1,
			redgem = 1,
			bluegem = 1,
			purplegem = 1,
			orangegem = 1,
			yellowgem = 1,
			thulecite = 1,
			thulecite_pieces = 1,
			goldnugget = 1,
			dragonfruit_seeds = 1,
			greengem = 1,
		}
		local loot = weighted_random_choice(loots)
		if item.components.edible.foodtype == "MEAT" then
			inst.components.lootdropper:SpawnLootPrefab(loot)
		else
			inst.components.lootdropper:SpawnLootPrefab(seedspawnprefab)
		end
	end)
end

local function dragon_cage_test(inst, item, giver)
	local seed_name = string.lower(item.prefab .. "_seeds")
	local can_accept = item.components.edible and (Prefabs[seed_name] or item.prefab == "seeds" or item.components.edible.foodtype == "MEAT") 
	
	if item.prefab == "egg" or item.prefab == "bird_egg" 
	or item.prefab == "rottenegg" or item.prefab == "monstermeat"
	or item.prefab == "tallbirdegg" then
		can_accept = false
	end
	
	return can_accept
end

local function dragon_cage_add_fx(inst)
	if inst.fx_fly == nil then
		inst.fx_fly = SpawnPrefab("tp_fx_dragon_cage_fly")
		inst:AddChild(inst.fx_fly)
		inst.fx_fly.Transform:SetPosition(0, 0, 0)
	end
	if inst.fx_front == nil then



	end
end

local function dragon_cage_on_built(inst, data)


	inst.SoundEmitter:PlaySound("dontstarve/common/craftable/birdcage_craft")
	dragon_cage_add_fx(inst)
	if inst.fx_front then


	end
end



local function dragon_cage_fn(inst)
	WARGON.CMP.add_cmps(inst, {
		inspect = {},
		work = {act=ACTIONS.HAMMER, num=2, ham=dragon_cage_ham, hit=dragon_cage_hit},
		loot = {},
		trader = {accept=dragon_cage_accept, test=dragon_cage_test},
	})

	WARGON.do_task(inst, 0, function()
		dragon_cage_add_fx(inst)
	end)
	inst:ListenForEvent("onbuilt", dragon_cage_on_built)
end

local function ice_statue_fn(inst)
	inst:AddTag("tp_boss_ice_statue")
	inst.AnimState:PlayAnimation("frozen", true)
	inst.AnimState:OverrideSymbol("swap_frozen", "frozen", "frozen")
end


local function bearger_ice_statue_fn(inst)
	ice_statue_fn(inst)
	WARGON.set_scale(inst, 1/1.65)
end


local function dragonfly_ice_statue_fn(inst)
	ice_statue_fn(inst)
	WARGON.set_scale(inst, 1.3/1.65)
end


local function moose_ice_statue_fn(inst)
	ice_statue_fn(inst)
	WARGON.set_scale(inst, 1.55/1.65)
end


local function deerclops_ice_statue_fn(inst)
	ice_statue_fn(inst)
	WARGON.set_scale(inst, 1)
end

function farm_pile_ham(inst, worker)
	WARGON.make_fx(inst, "collapse_small")
	inst:Remove()
end

function farm_pile_hit(inst, worker)
end

local function farm_pile_set_fertility_fn(inst, fert_percent)











end

local function farm_pile_fn(inst)
	local anim = inst.AnimState



	anim:SetOrientation( ANIM_ORIENTATION.OnGround )
	anim:SetLayer( LAYER_BACKGROUND )
	anim:SetSortOrder( 3 )
	inst:add_cmps({
		inspect = {},
		grower = {},
		work = {act=ACTIONS.HAMMER, num=1, ham=farm_pile_ham, hit=farm_pile_hit},
	})
	inst:AddTag("tp_farm_pile")
	inst:AddTag("structure")
	inst.components.grower.level = 3
    inst.components.grower.onplantfn = function() 
    	inst.SoundEmitter:PlaySound("dontstarve/wilson/plant_seeds") 
   	end
    inst.components.grower.croppoints = {Vector3(0,0,0)}
    inst.components.grower.growrate = TUNING.FARM3_GROW_BONUS
    inst.components.grower.max_cycles_left = 30
    inst.components.grower.cycles_left = inst.components.grower.max_cycles_left
	inst.components.grower.setfertility = farm_pile_set_fertility_fn
end

local function grass_pigking_spawner_fn(inst)
	inst:do_task(0, function()
		WARGON.make_spawn(inst, "tp_grass_pigking")
		local pos = inst:GetPosition()
		for i = -1, 1, 1 do

			WARGON.make_spawn(Vector3(i, 0, -2)+pos, "tp_moon_rock")
			WARGON.make_spawn(Vector3(i, 0, 2)+pos, "tp_moon_rock")
			WARGON.make_spawn(Vector3(2, 0, i)+pos, "tp_moon_rock")
			WARGON.make_spawn(Vector3(-2, 0, i)+pos, "tp_moon_rock")
		end
		inst:Remove()
	end)
end

local function shadow_statue_hit(inst, worker)
end

local function shadow_statue_ham(inst, worker)
	inst.components.lootdropper:DropLoot()
	WARGON.make_fx(inst, "collapse_small")
	inst:Remove()
end

local function shadow_statue_update(inst)
	if WARGON.is_full_moon() then
		if inst.tp_per_task == nil then
			inst.tp_per_task = inst:per_task(5, function()
				local nightmares = {"crawlingnightmare", "nightmarebeak"}
				WARGON.make_spawn(inst, nightmares[math.random(2)])
			end)
		end
	else
		if inst.tp_per_task then
			inst.tp_per_task:Cancel()
			inst.tp_per_task = nil
		end
	end
end

local function shadow_statue_fn(inst)
	inst:add_cmps({
		inspect = {},
		loot = {},
		work = {act=ACTIONS.HAMMER, num=4, 
			ham=shadow_statue_ham, hit=shadow_statue_hit},
	})
	inst:ListenForEvent("nighttime", function()
		shadow_statue_update(inst)
	end, GetWorld())
	inst:ListenForEvent("daytime", function()
		shadow_statue_update(inst)
	end, GetWorld())
	inst.OnLoad = function(inst, data)
		shadow_statue_update(inst)
	end
end

local function soul_student_point_fn(inst)

	inst:do_task(0, function()
		WARGON.make_spawn(inst, "tp_soul_student")
		WARGON.make_spawn(inst, "tp_soul_student_anchor")
		inst:Remove()
	end)
end

local function soul_student_anchor_fn(inst)
	inst:AddTag("tp_soul_student_point")
end

local function egg_cooker_accept(inst, giver, item)



	inst:AddTag("tp_egg_cooker_doing")
	inst:do_task(1.5, function()
		if item.components.stackable then
			inst:RemoveTag("tp_egg_cooker_doing")
			local stack = item.components.stackable:StackSize()
			local egg = SpawnPrefab("rottenegg")
			local egg2 = SpawnPrefab("bird_egg_cooked")
			egg.components.stackable:SetStackSize(stack)
			egg2.components.stackable:SetStackSize(stack)
			WARGON.pigking_throw(inst, egg)
			WARGON.pigking_throw(inst, egg2)

		end
	end)
end

local function egg_cooker_trader_test(inst, item, giver)
	return item.prefab == "bird_egg"
end

local function egg_cooker_hit(inst, worker)
	if not inst:HasTag("tp_egg_cooker_doing") then


	end
end

local function egg_cooker_fn(inst)
	inst.AnimState:PlayAnimation("tp_egg_cooker")
	common_structure(inst, egg_cooker_hit)
	inst:add_cmps({
		trader = {accept=egg_cooker_accept, test=egg_cooker_trader_test,
			stack=true},
		inv = {},
	})
	inst:AddTag("tp_snd_metal")
	inst.components.trader.deleteitemonaccept = true




end

local function defense_plant_shovel(inst, worker)
	inst.components.lootdropper:DropLoot()
	inst:Remove()
end

local function defense_plant_re(inst)
	return inst:wg_find(5, function(target, inst)
		return WARGON.is_monster(target)
	end)
end

local function defense_plant_keep(inst, target)
	return inst.components.combat:CanTarget(target)
		and inst:IsNear(target, 10)
end

local function defense_plant(inst)
	inst:add_cmps({
		work = {act=ACTIONS.SHOVEL, num=4, ham=defense_plant_shovel,},	
		loot = {},
		combat = {},
		health = {max=500, regen={per=5, amount=100}},
		combat = {dmg=50, per=4, re={time=3, fn=defense_plant_re},
			keep=defense_plant_keep, range={5, 6}},
	})
	inst:add_listen("attacked", function(inst, data)
		if data and data.attacker then
			inst.components.combat:SetTarget(data.attacker)
		end
	end)
end

local StructureData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local phy = data.phy and {"obs", data.phy[1], data.phy[2]} or nil
		local inst = WARGON.make_prefab(data.anim, nil, phy)
		if data.map then
			WARGON.make_map(inst, data.map)
		end
		inst:add_cmps({	
			inspect = {},
		})
		if data.structure then
			inst:AddTag("structure")
			inst:add_cmps({
				loot = {},
				work = {act=ACTIONS.HAMMER, num=4, 
					ham = function(inst, worker)
						if inst.components.lootdropper then
							inst.components.lootdropper:DropLoot()
						end

						WARGON.make_fx2("collapse_big", inst)
						if data.snd == "wood" then
							inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
						elseif data.snd == "metal" then
							inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
						end
						if data.ham_fn then
							data.ham_fn(inst, worker)
						end
						inst:Remove()
					end, 
					hit = function(inst, worker)
						if data.hit_fn then
							data.hit_fn(inst, worker)
						end
					end,
				},
			})
		end
		if data.onbuilt then
			inst:ListenForEvent("onbuilt", data.onbuilt)
		end
	 	if data.fn then
	 		data.fn(inst)
	 	end

	    return inst
	end
end)

function StructureData:__tostring()
	return string.format("Structure %s - %s", self.name, self.desc)
end

local cooking = require("cooking")

local fire_pot = StructureData("tp_fire_pot", {
	anim = Anims.cookpot_tureen,
	phy = {.5},


	structure = true,
	hit_fn = function(inst, worker)

		if not inst.components.tpfirepot:IsCooking() then

		else

		end
	end,
	ham_fn = function(inst, worker)
		inst.components.container:DropEverything()
	end,
	fn = function(inst)



		inst:AddComponent("container")
	    inst.components.container.itemtestfn = function(inst, item, slot)
	    	if inst.components.tpfirepot.cooker then
	    		return true
	    	end
	    	if inst.components.tpfirepot:IsCooking() then
		    	return cooking.IsCookingIngredient(item.prefab)
		    else
		    	return item.prefab == "tp_flavour"
			end
	   	end
	    inst.components.container:SetNumSlots(1.1)
	    inst.components.container.widgetslotpos = {Vector3(0,0,0)}
	    inst.components.container.widgetanimbank = "ui_bundle_2x2"
	    inst.components.container.widgetanimbuild = "ui_bundle_2x2"
	    inst.components.container.widgetpos = Vector3(200,0,0)
	    inst.components.container.side_align_tip = 100
	    inst.components.container.acceptsstacks = false
	    inst.components.container.type = "cooker"
		inst.components.container.widgetbuttoninfo = {
			text = STRINGS.ACTIONS.COOK.GENERIC,
			position = Vector3(0, -100, 0),
			fn = function(inst)
				inst.components.tpfirepot:Cook()	
			end,
			validfn = function(inst)
				return true
			end,
		}
		inst.components.container.onopenfn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_open", "open")
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot", "snd")
		end
		inst.components.container.onclosefn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_close", "close")
		end
		inst:AddComponent("tpfirepot")
		inst.components.tpfirepot.start = function(inst)
			inst.Light:Enable(true)

		end
		inst.components.tpfirepot.stop = function(inst)
			inst.Light:Enable(false)

		end
		local light = inst.entity:AddLight()
	    inst.Light:Enable(false)
		inst.Light:SetRadius(.6)
	    inst.Light:SetFalloff(1)
	    inst.Light:SetIntensity(.5)
	    inst.Light:SetColour(235/255,62/255,12/255)




	end,
	onbuilt = function(inst)


	end
})
local fire_pot_placer = WARGON.make_placer("tp_fire_pot", Anims.cookpot_tureen)

local rot_pot = StructureData("tp_rot_pot", {
	anim = Anims.cookpot_cauldron,
	phy = {.5},


	structure = true,
	hit_fn = function(inst)


	end,
	ham_fn = function(inst)
		inst.components.container:DropEverything()
	end,
	fn = function(inst)
		inst:add_cmps({
			cont = {typ="chest"},	
		})
		inst.components.container.onopenfn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_open", "open")
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot", "snd")
		end
		inst.components.container.onclosefn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_close", "close")
		end
		inst.task = WARGON.per_task(inst, 1, function()
	        for i, v in pairs(inst.components.container.slots) do
	            if v and v.components.perishable then
	                local perishable = v.components.perishable
	                local old_val = perishable.perishremainingtime
	                perishable.perishremainingtime = perishable.perishremainingtime - 10
	            end
	        end
	    end)




	end,
	onbuilt = function(inst)


	end
})
local rot_pot_placer = WARGON.make_placer("tp_rot_pot", Anims.cookpot_cauldron)

local dry_pot = StructureData("tp_dry_pot", {
	anim = Anims.cookpot_clay,
	phy = {.5},


	structure = true,
	hit_fn = function(inst)

		if inst.components.fueled:GetCurrentSection() <= 0 then

		else

		end
	end,
	ham_fn = function(inst)
		inst.components.container:DropEverything()
	end,
	fn = function(inst)
		inst:add_cmps({
			cont = {typ="chest"},	
		})
		inst.components.container.acceptsstacks = false
		inst.components.container.onopenfn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_open", "open")
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot", "snd")
		end
		inst.components.container.onclosefn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/common/cookingpot_close", "close")
		end

		local light = inst.entity:AddLight()
	    inst.Light:Enable(false)
		inst.Light:SetRadius(.6)
	    inst.Light:SetFalloff(1)
	    inst.Light:SetIntensity(.5)
	    inst.Light:SetColour(235/255,62/255,12/255)

		inst:AddComponent("fueled")
	    inst.components.fueled.maxfuel = TUNING.FIREPIT_FUEL_MAX
	    inst.components.fueled.accepting = true
	    inst.components.fueled.bonusmult = TUNING.FIREPIT_BONUS_MULT
	    inst.components.fueled.ontakefuelfn = function() 
		    inst.SoundEmitter:PlaySound("dontstarve/common/fireAddFuel") 
			inst.components.fueled:StartConsuming()
			inst.Light:Enable(true)

		end 
		inst.components.fueled:SetUpdateFn( function()
			if inst.components.fueled:GetPercent() > 0 then
				for k, v in pairs(inst.components.container.slots) do
					if v.components.tpdryable then
						v.components.tpdryable:DoDelta(-2)

						if v.components.tpdryable.time <= 0 then
							local name = v.components.dryable:GetProduct()
							local product = SpawnPrefab(name)
							local stuff = inst.components.container:RemoveItemBySlot(k)
							if stuff then
								stuff:Remove()
							end
							inst.components.container:GiveItem(product, k)						
						end
					end
				end

			end
		end)
		inst.components.fueled.maxfuel = TUNING.FIREPIT_FUEL_START





		inst.OnLoad = function(inst, data)
			if inst.components.fueled:IsEmpty() then
				inst.Light:Enable(false)

			else
				inst.Light:Enable(true)

				inst.components.fueled:StartConsuming()
			end
		end
	end,
	onbuilt = function(inst)


	end
})
local dry_pot_placer = WARGON.make_placer("tp_dry_pot",Anims.cookpot_clay)

local mine_machine = StructureData("tp_mine_machine", {
	anim = Anims.boat_winch,

	map = "tp_mine_machine#",
	structure = true,
	hit_fn = function(inst, worker)
		if inst.AnimState:IsCurrentAnimation("idle") then
			inst.AnimState:PlayAnimation("hit")
			inst.AnimState:PushAnimation("idle")
		end
	end,
	fn = function(inst)









		inst:ListenForEvent("animover", function(inst, data)
			if inst.AnimState:IsCurrentAnimation("pull_pst") then
				local stones = {
					rocks = 6,
					flint = 4,
					goldnugget = 1,
					nitre = 1,
				}
				inst:do_task(.4, function()
					for i = 1, math.random(6, 8) do
						local name = weighted_random_choice(stones)
						if name then
							local stone = SpawnPrefab(name)
							WARGON.pigking_throw(inst, stone)
						else
							print("error mine_machine")
						end
					end
				end)
			end
		end)
		inst:ListenForEvent("daytime", function(world, data)
			inst.AnimState:PlayAnimation("drop_pre")
			inst.AnimState:PushAnimation("drop_loop", false)
			inst.AnimState:PushAnimation("drop_success_pst", false)
			inst.AnimState:PushAnimation("pull_pre", false)
			inst.AnimState:PushAnimation("pull_loop", false)
			inst.AnimState:PushAnimation("pull_pst", false)
			inst.AnimState:PushAnimation("idle", false)
		end, GetWorld())
	end,
	onbuilt = function(inst)
		inst.AnimState:PlayAnimation("place")
		inst.AnimState:PushAnimation("idle", false)
		local pos = inst:GetPosition()
		pos.y = pos.y + .1
		inst.Transform:SetPosition(pos:Get())
	end
})
local mine_machine_place_test = function(inst, pt)
	local range = 1
	local canbuild = false
	
	local tarpits = TheSim:FindEntities(pt.x, pt.y, pt.z, range, {"tp_mine_source"}, nil)

	if #tarpits > 0 and tarpits[1].open then
		local pt2 = Vector3(tarpits[1].Transform:GetWorldPosition())
		local structures = TheSim:FindEntities(pt2.x, pt2.y, pt2.z, range, {"structure"}, nil)
		if #structures == 0 then
			if inst.parent then
				inst.parent:RemoveChild(inst)
			end
			canbuild = true
			inst.Transform:SetPosition(tarpits[1].Transform:GetWorldPosition())			
		end
	end
	return canbuild
end

local mine_machine_placer = WARGON.make_placer("tp_mine_machine", Anims.boat_winch)

local slaughter_house = StructureData("tp_slaughter_house", {
	anim = Anims.structures,
	phy = {.5},

	map = "tp_slaughter_house.tex",
	structure = true,
	hit_fn = function(inst, worker)


	end,
	ham_fn = function(inst, worker)

		if inst.components.spawner
		and inst.components.spawner.child then
			inst.components.spawner.child:Remove()
		end
	end,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_slaughter_house")
		inst:AddComponent("electricity")









		inst:AddComponent( "spawner" )
	    inst.components.spawner:Configure("tp_pig_killer", 30*16*4)
	    inst.components.spawner.onoccupied = function(inst, child)
	    	inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
		end
		inst.components.spawner.onvacate = function(inst, child)
			inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
		end


















		inst:ListenForEvent("daytime", function(world, data)
			if inst.components.spawner:IsOccupied()
			and inst.components.electricity.current >= 10 then
				inst.components.electricity:DoDelta(-10)
				inst.components.spawner:ReleaseChild()
			end
		end, GetWorld())
	end,




})
local slaughter_house_placer = make_strucutres_placer("tp_slaughter_house")


local food_changer = StructureData("tp_food_changer", {
	anim = Anims.structures,

	map = "tp_food_changer.tex",
	phy = {.5},
	structure = true,
	hit_fn = function(inst, worker)


	end,
	ham_fn = function(inst, worker)
		inst.components.container:DropEverything()
	end,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_food_changer")
		inst:AddComponent("container")
		inst.components.container.itemtestfn = function(inst, item, slot)
	    	if inst.components.container:FindItem(function(nug)
	    		return nug.components.tppowerfruit
	    	end) then
	    		return item.prefab == "goldnugget"
	 		elseif inst.components.container:Has("goldnugget", 1) then
	 			return item.components.tppowerfruit
	    	end
	    	return item.prefab == "goldnugget" or item.components.tppowerfruit
	   	end
	    inst.components.container:SetNumSlots(2)
	    inst.components.container.widgetanimbank = "ui_cookpot_1x4"
	    inst.components.container.widgetanimbuild = "ui_cookpot_1x4"
	    inst.components.container.widgetslotpos = {Vector3(0,40,0), Vector3(0,-40,0)}
	    inst.components.container.widgetpos = Vector3(200,0,0)
	    inst.components.container.side_align_tip = 100
		inst.components.container.widgetbuttoninfo = {
			text = STRINGS.ACTIONS.COOK.GENERIC,
			position = Vector3(0, -140, 0),
			fn = function(inst)
				if inst.components.container:Has("goldnugget", 1) then
					inst.components.container:ConsumeByName("goldnugget", 1)
					local food = inst.components.container:FindItem(function(nug)
						return nug.components.tppowerfruit
					end)
					if food then
						food.components.tppowerfruit:Random()
					end
				end
			end,
			validfn = function(inst)
				local food = inst.components.container:FindItem(function(nug)
					return nug.components.tppowerfruit
				end)
				if food then
					return inst.components.container:Has("goldnugget", 1)
				end
			end,
		}
		inst.components.container.onopenfn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_open")
		end
		inst.components.container.onclosefn = function(inst, doer)
			inst.SoundEmitter:PlaySound("dontstarve/wilson/chest_close")
		end




	end,




})
local food_changer_placer = make_strucutres_placer("tp_food_changer")


local station = StructureData("tp_station", {
	anim = Anims.sign_factory,
	map = "arrowsign_post_factory.tex",
	structure = true,
	hit_fn = function(inst, worker)
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("idle")
	end,
	ham_fn = function(inst, worker)

	end,
	fn = function(inst)

	end,
})

local signal_tower = StructureData("tp_signal_tower", {
	anim = Anims.structures,

	map = "tp_signal_tower.tex",
	structure = true,
	phy = {.5},
	hit_fn = function(inst, worker)


	end,
	onbuilt = function(inst)


	end,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_signal_tower")
		inst:do_task(0, function()
			local x, y, z = inst:GetPosition():Get()
			GetWorld().minimap.MiniMap:ShowArea(x, y, z, 500)
		end)
	end,
})
local signal_tower_placer = WARGON.make_placer("tp_signal_tower", {"tp_structures", "tp_structures", "tp_signal_tower"})

local robot_cleaner_home = StructureData("tp_robot_cleaner_home", {
	anim = Anims.structures,

	map = "tp_robot_cleaner_home.tex",
	structure = true,
	phy = {.5},
	hit_fn = function(inst, worker)


	end,
	onbuilt = function(inst)


	end,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_robot_cleaner_home")
		inst:AddComponent("spawner")
		inst.components.spawner:Configure("tp_robot_cleaner", 30*16)
	end,
})
local robot_cleaner_home_placer = WARGON.make_placer("tp_robot_cleaner_home", {"tp_structures", "tp_structures", "tp_robot_cleaner_home"})

local function small_sun_energy_off(inst)
	inst.components.tpenergy:RemoveBurnModifier("hot")
	inst.sg:GoToState("turn_off")
	inst.components.machine:TurnOn()
	inst.Light:Enable(false)
end

local small_sun = StructureData("tp_small_sun", {
	anim = Anims.basefan,

	map = "tp_small_sun.tex",
	structure = true,
	phy = {.5},
	hit_fn = function(inst, worker)



	end,
	onbuilt = function(inst)


	end,
	fn = function(inst)
		inst:AddComponent("tpenergy")
		inst.components.tpenergy:SetMax(500)
		inst.components.tpenergy:SetRate(1)
		inst:AddComponent("heater")
		inst.components.heater.heatfn = function(inst)
			if inst.components.tpenergy.burn_modifier["hot"] then
				return 100
			end
			return 0
		end
		inst:AddComponent("machine")
		inst.components.machine.turnonfn = function(inst)
			inst.components.tpenergy:AddBurnModifier("hot", 1)

			inst.Light:Enable(true)
		end
		inst.components.machine.turnofffn = function(inst)
			small_sun_energy_off(inst)
		end
		inst.components.machine.caninteractfn = function(inst, doer)
			return not inst.components.tpenergy:IsEmpty()
		end
		inst.components.tpenergy.run_out = function(inst)
			small_sun_energy_off(inst)
	    end
	    inst.entity:AddLight()
	    inst.Light:SetColour(180/255, 50/255, 50/255)
	    inst.Light:SetIntensity(.6)
	    inst.Light:SetRadius(5)
	    inst.Light:SetFalloff(.9)
	    inst.Light:Enable(false)

	end,
})

local small_sun_placer = make_strucutres_placer("tp_small_sun")

local white_pig_house = StructureData("tp_white_pig_house", {
	anim = Anims.pig_house,
	phy = {.5},
	map = "pighouse.png",
	structure = true,
	hit_fn = function(inst, worker)
		inst.AnimState:PlayAnimation("hit")
		inst.AnimState:PushAnimation("idle")
	end,
	fn = function(inst)
		inst:AddComponent( "spawner" )
	    inst.components.spawner:Configure("tp_white_pig", 30*16)
	    inst.components.spawner.onoccupied = function(inst, child)
	    	inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
		end
		inst.components.spawner.onvacate = function(inst, child)
			inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
		end
		inst:ListenForEvent("daytime", function(world, data)
			if inst.components.spawner:IsOccupied() then
				inst.components.spawner:ReleaseChild()
			end
		end, GetWorld())
	end,
	onbuilt = function(inst)
		inst.AnimState:PlayAnimation("place")
		inst.AnimState:PushAnimation("idle")
	end,
})
local white_pig_house_placer = WARGON.make_placer("tp_white_pig_house", Anims.pig_house)

local alloy_tree_spawner = StructureData("tp_alloy_tree_spawner", {
	fn = function(inst)
		common_spawn_fn(inst, "tp_alloy_tree")
	end,
})
local alloy_red_tree_spawner = StructureData("tp_alloy_red_tree_spawner", {
	fn = function(inst)
		common_spawn_fn(inst, "tp_alloy_red_tree")
	end,
})

return 
MakeStructure("tp_tent", Anims.tent, tent_fn, Phys.tent, Maps.tent),
MakePlacer("common/tp_tent_placer", Anims.tent[1], Anims.tent[2], Anims.tent[3], nil, nil, nil, 1.2),
MakeStructure("tp_chest", Anims.box, chest_fn, nil, Maps.box),
MakePlacer("common/tp_chest_placer", Anims.box[1], Anims.box[2], Anims.box[3]),
MakeStructure("tp_bangalore", Anims.cork_chest, bangalore_fn, nil, Maps.cork_chest),
MakeStructure("tp_start_base", {}, start_base_fn, nil, nil),
MakeStructure("tp_unreal_rock", {}, unreal_rock_fn, Phys.obelisk, nil),
MakeStructure("tp_moon_rock", Anims.sanityrock, moon_rock_fn, Phys.obelisk, nil),
MakeStructure("tp_moon_sea_point", {}, moon_sea_point_fn, nil, nil),
MakeStructure("tp_moon_sea_handler", {}, moon_sea_handler_fn, nil, nil),
MakeStructure("tp_leif_spawner", {}, leif_spawner_fn, nil, nil),
MakeStructure("tp_boss_spawner", {}, boss_spawner_fn),
MakeStructure("tp_boss_spawner2", {}, boss_spawner_fn_2),
MakeStructure("tp_werepig_start", {}, werepig_start_fn),
MakeStructure("tp_moon_lake", Anims.lake, moon_lake_fn, nil, Maps.lake),
MakeStructure("tp_combat_lord_sleep", Anims.wathgrithr, combat_lord_sleep_fn),
MakeStructure("tp_hornet_sleep", Anims.wendy, hornet_sleep_fn),
MakeStructure("tp_fake_knight_sleep", Anims.wes, fake_knight_sleep_fn),
MakeStructure("tp_soul_student_sleep", Anims.waxwell, soul_student_sleep_fn),
MakeStructure("tp_dragon_cage", Anims.birdcage, dragon_cage_fn, Phys.birdcage, nil),
MakePlacer("common/tp_dragon_cage_placer", Anims.birdcage[1], Anims.birdcage[2], Anims.birdcage[3]),
MakeStructure("tp_bearger_ice_statue", Anims.bearger, bearger_ice_statue_fn, Phys.ice_statue, nil),
MakePlacer("common/tp_bearger_ice_statue_placer", Anims.bearger[1], Anims.bearger[2], "frozen", nil, nil, nil, 1/1.65),
MakeStructure("tp_moose_ice_statue", Anims.moose, ice_statue_fn, Phys.ice_statue, nil),
MakePlacer("common/tp_moose_ice_statue_placer", Anims.moose[1], Anims.moose[2], "frozen", nil, nil, nil, 1.55/1.65),
MakeStructure("tp_dragonfly_ice_statue", Anims.dragonfly, ice_statue_fn, Phys.ice_statue, nil),
MakePlacer("common/tp_dragonfly_ice_statue_placer", Anims.dragonfly[1], Anims.dragonfly[2], "frozen", nil, nil, nil, 1.3/1.65),
MakeStructure("tp_deerclops_ice_statue", Anims.deerclops, ice_statue_fn, Phys.ice_statue, nil),
MakePlacer("common/tp_deerclops_ice_statue_placer", Anims.deerclops[1], Anims.deerclops[2], "frozen", nil, nil, nil, 1),
MakeStructure("tp_farm_pile", Anims.structures2, farm_pile_fn),
MakePlacer("common/tp_farm_pile_placer", Anims.structures2[1], Anims.structures2[2], Anims.structures2[3], true),
MakeStructure("common/tp_grass_pigking_spawner", {}, grass_pigking_spawner_fn),
MakeStructure("common/tp_shadow_statue", Anims.shadow_statue, shadow_statue_fn, Phys.obelisk, Maps.shadow_statue),
MakePlacer("common/tp_shadow_statue_placer", Anims.shadow_statue[1], Anims.shadow_statue[2], Anims.shadow_statue[3]),
MakeStructure("tp_soul_student_point", {}, soul_student_point_fn),
MakeStructure("tp_soul_student_anchor", {}, soul_student_anchor_fn),
MakeStructure("tp_egg_cooker", Anims.structures, egg_cooker_fn, Phys.smelter, Maps.smelter),
MakePlacer("common/tp_egg_cooker_placer", "tp_structures", "tp_structures", "tp_egg_cooker"),
fire_pot,
fire_pot_placer,
rot_pot,
rot_pot_placer,
dry_pot,
dry_pot_placer,
mine_machine,
mine_machine_placer,
slaughter_house,
slaughter_house_placer,
food_changer,
food_changer_placer,
signal_tower,
signal_tower_placer,
robot_cleaner_home,
robot_cleaner_home_placer,
small_sun,
small_sun_placer,
white_pig_house,
white_pig_house_placer,
alloy_tree_spawner,
alloy_red_tree_spawner,

MakeStructure("tp_pot_bird_egg_spawner", {}, pot_bird_egg_spawner_fn)