local Anims = {
    tent = {"tent", "tent", "idle"},
	structures = {"tp_structures", "tp_structures", "tp_auto_chopper"},
	structures2 = {"tp_structures_2", "tp_structures_2", "tp_resource_transporter"},
}

local StructureData = Class(Prefab, function(self, name, data)
	Prefab._ctor(self, name)
	self.data = data
	self.fn = function()
		local phy = data.phy and {"obs", data.phy[1], data.phy[2]} or nil
		local inst = WARGON.make_prefab(data.anim, nil, phy)
		if data.map then
			WARGON.make_map(inst, data.map)
		end
		inst:add_cmps({	
			inspect = {},
		})
		if data.structure then
			inst:AddTag("structure")
			inst:add_cmps({

				work = {act=ACTIONS.HAMMER, num=4, 
					ham = function(inst, worker)
						if inst.components.lootdropper then
							inst.components.lootdropper:DropLoot()
						end

						WARGON.make_fx2("collapse_big", inst)
                        if data.snd == "wood" then
							inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
						elseif data.snd == "metal" then
							inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
						end
						if data.ham_fn then
							data.ham_fn(inst, worker)
						end
						inst:Remove()
					end, 
					hit = function(inst, worker)
						if data.hit_fn then
							data.hit_fn(inst, worker)
						end
					end,
				},
			})
		end
		if data.electricity then
			inst:AddComponent("tpelectricity")
			inst.components.tpelectricity:SetMax(data.electricity)
			inst.components.tpelectricity.run_out = function(inst)
				if inst.components.machine then
					inst.components.machine:TurnOff()
				end
			end
		end
		if data.machine then
			inst:add_cmps({
				machine = {
					on = function(inst)
						if inst.task == nil then
							if data.machine.on then
								data.machine.on(inst)
							end
						end
					end,
					off = function(inst)
						if inst.task then
							inst.task:Cancel()
							inst.task = nil
						end
						if data.machine.off then
							data.machine.off(inst)
						end
					end,
					test = function(inst)
						if inst.components.tpelectricity then
							return not inst.components.tpelectricity:IsEmpty()
						end
						return true
					end,
				}
			})
		end
		if data.onbuilt then
			inst:ListenForEvent("onbuilt", data.onbuilt)
		end
	 	if data.fn then
	 		data.fn(inst)
	 	end

	    return inst
	end
end)

local auto_chopper = StructureData("tp_auto_chopper", {
    anim = Anims.structures,
	phy = {.5},
    structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(1, function()
				local tree = inst:wg_find(15, function(item, inst)
					if item.components.workable
					and ( (item.components.workable.action == ACTIONS.CHOP
					and item.components.growable
					and item.components.growable.stage >= 3)
					or (item.components.workable.action == ACTIONS.DIG
					and item:HasTag("stump")) ) then


						return true
					end
				end, {"tree"}, {"fire", "burned"})
				if tree then
					inst.components.tpelectricity:DoDelta(-1)
					tree.components.workable:WorkedBy(inst, 1)
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_chopper")
	end,
})

local auto_picker = StructureData("tp_auto_picker", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(2, function()
				local ent = inst:wg_find(15, function(item, inst)
					if item.components.pickable 
					and item.components.pickable:CanBePicked() then
						return item.prefab == "berrybush"
						or item.prefab == "grass"
						or item.prefab == "sapling"
					end
				end, nil, {"fire", "burned"})
				if ent and ent.components.pickable then
					inst.components.tpelectricity:DoDelta(-1)
					ent.components.pickable:TpDropProduct()
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_picker")
	end
})


local auto_planter = StructureData("tp_auto_planter", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(2, function()
				local ent = inst:wg_find(15, function(item, inst)
					if item.components.deployable
					and item.components.inventoryitem then


						return item:HasTag("tp_tree_seed")
					end
				end, nil, {"fire", "burned"})
				if ent then
					local pt = WARGON.get_pos(ent, true)
					if ent.components.deployable:CanDeploy(pt) then
						WARGON.make_fx2("small_puff", ent)
						inst.components.tpelectricity:DoDelta(-1)
						ent.components.deployable:Deploy(pt, inst)






					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_planter")
	end,
})


local auto_harvester = StructureData("tp_auto_harvester", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(3, function()
				local ent = inst:wg_find(15, function(item, inst)
					if item.prefab == "beebox" 
					and item.components.harvestable
					and item.components.harvestable.produce >= 6
					and item.components.harvestable:CanBeHarvested() then
						return true
					end
					if item.components.dryer
					and item.components.dryer:IsDone() then
						return true
					end
				end, {"tp_auto_harvester_target"}, {"burned", "fire"})
				if ent then
					if ent.components.harvestable then
						inst.components.tpelectricity:DoDelta(-1)
						ent.components.harvestable:TpDropProduct()
					elseif ent.components.dryer then
						inst.components.tpelectricity:DoDelta(-1)
						ent.components.dryer:TpDropProduct()
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_harverster")
	end,
})


local auto_fertilizer = StructureData("tp_auto_fertilizer", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(5, function()
				local f = inst.components.container:FindItem(function(item)
					return item.components.fertilizer
						and not item.components.fertilizer.volcanic
				end)
				if f then
					local ent = inst:wg_find(15, function(item, inst)
						if item.components.pickable 
						and item.components.pickable:CanBeFertilized() then
							return item.prefab ~= "coffeebush"
						end
					end, nil, {"burned", "fire"})
					if ent then
						inst.components.tpelectricity:DoDelta(-1)
						ent.components.pickable:Fertilize(f)
						WARGON.make_fx2("small_puff", ent)
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_fertilizer")
		inst:AddTag("tp_auto_fertilizer")
		inst:add_cmps({
			cont = {typ="chest"},
		})
	end,
})


local auto_dryer = StructureData("tp_auto_dryer", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(2, function()
				local dryable = inst.components.container:FindItem(function(item)
					return item.components.dryable
				end)
				if dryable then
					local ent = inst:wg_find(15, function(item, inst)
						if item.components.dryer
						and not item.components.dryer:IsDone()
						and not item.components.dryer:IsDrying() then
							return true
						end
					end, nil, {"burned", "fire"})
					if ent then
						if ent.components.dryer:CanDry(dryable) then
							if dryable.components.stackable then
								dryable = dryable.components.stackable:Get()
							end
							WARGON.make_fx2("collapse_small", ent)
							ent.components.dryer:StartDrying(dryable)
							inst.components.tpelectricity:DoDelta(-1)
						end
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_dryer")
		inst:AddTag("tp_auto_dryer")
		inst:add_cmps({
			cont = {typ="chest"},
		})
	end,
})

local auto_miner = StructureData("tp_auto_miner", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(1, function()
				local ent = inst:wg_find(15, function(item, inst)
					if item.components.workable
					and item.components.workable.action == ACTIONS.MINE then
						return true
					end
				end)
				if ent then
					ent.components.workable:WorkedBy(inst, 1)
					inst.components.tpelectricity:DoDelta(-1)
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_miner")
	end,
})

local auto_farmer = StructureData("tp_auto_farmer", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(3, function()
				local seed = inst.components.container:FindItem(function(item)

					return item:HasTag("tp_seed")
				end)
				if seed then
					local ent = inst:wg_find(15, function(item, inst)
						if item.components.grower
						and item.components.grower:IsEmpty() then
							return true
						end
					end, nil, {"burned", "fire"})
					if ent then
						if seed.components.stackable then
							seed = seed.components.stackable:Get()
						end
						WARGON.make_fx2("small_puff", ent)
						ent.components.grower:PlantItem(seed)
						inst.components.tpelectricity:DoDelta(-1)
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_farmer")
		inst:AddTag("tp_auto_farmer")
		inst:add_cmps({
			cont = {typ="chest"},
		})
	end,
})

local auto_transporter = StructureData("tp_auto_transporter", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(2, function()
				local source = inst:wg_find(15, function(item, inst)
					if item.components.inventoryitem then
						if inst.check_tag == "tp_auto_dryer" then
							return item.components.dryable
						end
						if inst.check_tag == "tp_auto_fertilizer" then
							return item.components.fertilizer
								and not item.components.fertilizer.volcanic
						end
						if inst.check_tag == "tp_auto_farmer" then

							return item:HasTag("tp_seed")
						end
					end
				end, nil, {"burned", "fire"})
				if source then
					local ent = inst:wg_find(15, function(item, inst)
						if item.components.container
						and item.components.container:TpCanGiveItem(source, 1) then
							return true
						end
					end, {inst.check_tag}, {"burned", "fire"})
					if ent then
						WARGON.make_fx2("small_puff", source)
						if source.components.stackable then
							source = source.components.stackable:Get()
						end
						ent.components.container:GiveItem(source)
						inst.components.tpelectricity:DoDelta(-1)
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_transporter")
		inst.check_tags = {
			"tp_auto_fertilizer",
			"tp_auto_dryer",
			"tp_auto_farmer",
		}
		inst.check_num = 1
		inst.check_tag = inst.check_tags[inst.check_num]
		inst:AddComponent("tpuse")
		inst.components.tpuse.right = false
		inst.components.tpuse.use = function(inst, doer)
			inst.check_num = inst.check_num + 1
			if inst.check_num > #inst.check_tags then
				inst.check_num = 1
			end
			inst.check_tag = inst.check_tags[inst.check_num]
			print(inst.check_tag)
			if doer.components.talker then
				doer.components.talker:Say(STRINGS.TP_STR.tp_auto_transporter_strs[inst.check_num])
			end
		end
		inst.OnSave = function(inst, data)
			data.check_num = inst.check_num
		end
		inst.OnLoad = function(inst, data)
			inst.check_num = data and data.check_num or 1
			inst.check_tag = inst.check_tags[inst.check_num]
		end
	end,
})

local auto_food_mover = StructureData("tp_auto_food_mover", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	machine = {
		on = function(inst)
			inst.task = inst:per_task(2, function()
				local food = inst:wg_find(15, function(item, inst)
					return (item.components.edible 
						and item.components.perishable) 
						or item:HasTag("frozen") 
						or item:HasTag("icebox_valid")
				end)
				if food then
					local ent = inst:wg_find(15, function(item, inst)
						if item.prefab == "icebox"
						and item.components.container:TpCanGiveItem(food, 1) then
							return true
						end
					end, nil, {"burned", "fire"})
					if ent then
						WARGON.make_fx2("small_puff", food)
						if food.components.stackable then
							food = food.components.stackable:Get()
						end
						ent.components.container:GiveItem(food)
						inst.components.tpelectricity:DoDelta(-1)
					end
				end
			end)
		end,
	},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_auto_food_mover")
	end,
})

local small_battery = StructureData("tp_small_battery", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_small_battery")
		inst:AddComponent("tpbattery")
		inst:add_cmps({
			machine = {
				on = function(inst)
					inst.components.tpbattery:StartPowerSupply()
					inst.components.tpbattery:StartGetPower()
				end,
				off = function(inst)
					inst.components.tpbattery:StopPowerSupply()
					inst.components.tpbattery:StopGetPower()
				end,
			}
		})
	end,
})

local big_battery = deepcopy(small_battery)
big_battery:SetName("tp_big_battery")
big_battery:HookFn(function(inst)
	inst.AnimState:PlayAnimation("tp_big_battery")
	inst.components.tpbattery:SetMax(500)
	inst.components.tpbattery:Init(20, 100)
end)

local love_generator = StructureData("tp_love_generator", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_love_generator")
		inst:AddComponent("tpvalue")
		inst.components.tpvalue:SetMax(300)
		inst.components.tpvalue:Stop()
		inst:AddTag("tp_generator")
	end,
})

local charcoal_generator = StructureData("tp_charcoal_generator", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_charcoal_generator")
		inst:AddComponent("tpvalue")
		inst.components.tpvalue:SetMax(500)
		inst.components.tpvalue:Stop()
		inst:AddTag("tp_generator")
		inst.tp_charcoal_num = 0
		inst:add_cmps({
			machine = {
				on = function(inst)
					inst:AddTag("tp_not_get_electricity")
					if inst.task == nil then
						inst.task = inst:per_task(1, function()
							local cost = inst.components.tpvalue.max - inst.components.tpvalue.current
							if cost >= 50 then
								local n = inst.tp_charcoal_num
								if n > 0 then
									inst.components.tpvalue:DoDelta(50)
								end
								n = math.max(0, n - 1)
								inst.tp_charcoal_num = n
							end
						end)
					end
				end,
				off = function(inst)
					inst:RemoveTag("tp_not_get_electricity")
					if inst.task then
						inst.task:Cancel()
						inst.task = nil
					end
				end,
			},
			trader = {
				accept = function(inst, giver, item)
					local n = inst.tp_charcoal_num
					n = math.min(20, n+1)



					inst.tp_charcoal_num = n
				end,
				test = function(inst, item, giver)
					return item.prefab == "charcoal"
						and inst.tp_charcoal_num < 20
				end,
			}
		})
		inst.OnSave = function(inst, data)
			data.tp_charcoal_num = inst.tp_charcoal_num
		end
		inst.OnLoad = function(inst, data)
			inst.tp_charcoal_num = data and data.tp_charcoal_num or 0
		end
	end,
})

local electric_coster = StructureData("tp_electric_coster", {
	anim = Anims.tent,
	structure = true,
	fn = function(inst)
		inst:AddComponent("tpelectricity")
		inst.components.tpelectricity:SetRate(3)
		inst:add_cmps({
			machine = {
				on = function(inst)
					inst.components.tpelectricity:Start()
				end,
				off = function(inst)
					inst.components.tpelectricity:Stop()
				end,
			}
		})
	end,
})

local light2 = StructureData("tp_light2", {
	anim = Anims.structures,
	structures = true,
	phy = {.5},
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_light2")
		local light = inst.entity:AddLight()
		inst.Light:SetIntensity(0.6)
		inst.Light:SetColour(197/255, 197/255, 100/255)
		inst.Light:SetFalloff( 0.9 )
		inst.Light:SetRadius( 5 )
		inst.Light:Enable(false)
		inst:AddComponent("tpelectricity")
		inst.components.tpelectricity:SetMax(100)
		inst.components.tpelectricity:SetRate(.3)
		inst:add_cmps({
			machine = {
				on = function(inst)
					if GetClock():IsNight() 
					or GetWorld():IsCave() then
						inst.components.tpelectricity:Start()
						inst.Light:Enable(true)
					end
				end,
				off = function(inst)
					inst.components.tpelectricity:Stop()
					inst.Light:Enable(false)
				end,
			}
		})
		inst:ListenForEvent("nighttime", function(world, data)

			if inst.components.machine:IsOn() then
				inst.Light:Enable(true)
				inst.components.tpelectricity:Start()
			end

		end, GetWorld())
		inst:ListenForEvent("daytime", function(world, data)
			if inst.components.machine:IsOn() then
				if not world:IsCave() then
					inst.Light:Enable(false)
					inst.components.tpelectricity:Stop()
				end
			end
		end, GetWorld())
	end,
})
local light2_placer = WARGON.make_placer("tp_light2", {"tp_structures", "tp_structures", "tp_light2"})

local oil_refinery = StructureData("tp_oil_refinery", {
	anim = Anims.structures,
	phy = {.5},
	structure = true,
	electricity = 100,
	fn = function(inst)
		inst.AnimState:PlayAnimation("tp_oil_refinery")
		inst.tp_stone_num = 0
		inst:add_cmps({
			machine = {
				on = function(inst)
					if inst.task == nil then
						inst.task = inst:per_task(5, function()
							local n = inst.tp_stone_num
							if n > 0 then
								local tar = SpawnPrefab("tar")
								WARGON.throw_item(inst, tar)
								inst.components.tpelectricity:DoDelta(-10)
							end
							n = math.max(0, n - 1)
							inst.tp_stone_num = n
						end)
					end
				end,
				off = function(inst)
					if inst.task then
						inst.task:Cancel()
						inst.task = nil
					end
				end,
			},
			trader = {
				accept = function(inst, giver, item)
					local n = inst.tp_stone_num
					n = n + 1
					inst.tp_stone_num = n
				end,
				test = function(inst, item, giver)
					return item.prefab == "rocks"
						and inst.tp_stone_num < 20
				end,
			},
		})
		inst.OnSave = function(inst, data)
			data.tp_stone_num = inst.tp_stone_num
		end
		inst.OnLoad = function(inst, data)
			inst.tp_stone_num = data and data.tp_stone_num or 0
		end
	end,
})
local oil_refinery_placer = WARGON.make_placer("tp_oil_refinery", {"tp_structures", "tp_structures", "tp_oil_refinery"})

local resource_transporter = StructureData("tp_resource_transporter", {
	anim = Anims.structures2,
	phy = {.5},
	structure = true,
	electricity = 100,
	fn = function(inst)
		inst:AddTag("tp_not_auto_close")
		inst.AnimState:PlayAnimation("tp_resource_transporter")
		inst:add_cmps({
			cont = {typ="3x4"},
		})
		inst.transport = function(inst, slot)


				local item = inst.components.container:RemoveItemBySlot(slot)
				if item then


					local pt = GetPlayer():GetPosition()
					pt.y = pt.y + 20

					item:set_pos(pt:Get())
					item.components.inventoryitem:OnDropped()

				end




		end
		inst:ListenForEvent("onbuilt", function(inst, data)
			if c_countprefabs("tp_resource_transporter") > 1 then
				inst:Remove()
				GetPlayer().components.talker:Say("只能存在一个物资发射中心")
			end
		end)
	end,
})
local resource_transporter_placer = WARGON.make_placer("tp_resource_transporter", {"tp_structures_2", "tp_structures_2", "tp_resource_transporter"})

local cure_desk = StructureData("tp_cure_desk", {
	anim = {"tp_structures_3", "tp_structures_3", "tp_cure_desk"},
	structure = true,
	phy = {.5},
	electricity = 100,
	fn = function(inst)
		inst:AddComponent("tpuse")
		inst.components.tpuse.use = function(inst, doer)
			if doer.components.poisonable 
			and doer.components.poisonable:IsPoisoned() then
				doer.components.poisonable:Cure()
				inst.components.tpelectricity:DoDelta(-10)
			end
		end
	end,
})
local cure_desk_placer = WARGON.make_placer("tp_cure_desk", cure_desk.data.anim)

local shop = StructureData("tp_shop", {
	anim = {"tp_structures_3", "tp_structures_3", "tp_shop"},
	structure = true,
	phy = {.5},
	fn = function(inst)
		inst:AddTag("tp_shop")
	end,
})
local shop_placer = WARGON.make_placer("tp_shop", shop.data.anim)

local train_center = StructureData("tp_train_center", {
	anim = {"tp_structures_3", "tp_structures_3", "tp_train_center"},
	structure = true,
	phy = {.5},
	fn = function(inst)
		inst:AddTag("tp_train_center")
	end,
})
local train_center_placer = WARGON.make_placer("tp_train_center", train_center.data.anim)

local fridge = StructureData("tp_fridge", {
	anim = {"tp_structures_3", "tp_structures_3", "tp_fridge"},
	structure = true,
	electricity = 100,
	fn = function(inst)
		inst.components.tpelectricity:SetRate(.5)
		inst.components.tpelectricity:Start()
		inst:add_cmps({
			cont = {typ="chest"},
		})
		inst:AddTag("tp_pack_deerclops")
		inst.components.tpelectricity.run_out = function(inst)
			inst:RemoveTag("tp_pack_deerclops")
			inst.components.tpelectricity:Stop()
		end
		inst.components.tpelectricity.income = function(inst)
			inst:AddTag("tp_pack_deerclops")
			inst.components.tpelectricity:Start()
		end
	end,
})
local fridge_placer = WARGON.make_placer("tp_fridge", fridge.data.anim)

local super_calculator = StructureData("tp_super_calculator", {
	anim = {"tp_structures", "tp_structures", "tp_super_calculator"},
	structure = true,
	fn = function(inst)
		inst:AddTag("prototyper")
		inst:AddComponent("prototyper")
		inst.components.prototyper.trees = {
			SCIENCE = 2,
			MAGIC = 3,
			ANCIENT = 0,
			OBSIDIAN = 0,
			WATER = 0,
			HOME = 0,	
			CITY = 0,	
			LOST = 0,
		}
		inst.far = 4
		inst.near = 3
		inst:per_task(.5, function()
			local player = GetPlayer()
			if player and inst and not player:HasTag("notarget") then
				local close = nil
				local distsq = player:GetDistanceSqToInst(inst)
				if inst.isclose then
					close = distsq < inst.far*inst.far
				else
					close = distsq < inst.near*inst.near
				end
				if inst.isclose ~= close then
					inst.isclose = close
					if inst.isclose then
						if WARGON.is_full_moon() then
							player:AddTag("mad_prince")
						else
							player:AddTag("tech_prince")
						end
						player:AddTag("tp_fast_builder")
					end
					if not inst.isclose then
						player:RemoveTag("tech_prince")
						player:RemoveTag("mad_prince")
						player:RemoveTag("tp_fast_builder")
					end
				end
			end
		end)
	end,
})
local super_calculator_placer = WARGON.make_placer("tp_super_calculator", super_calculator.data.anim)

return 
auto_chopper,
auto_planter,
auto_harvester,
auto_fertilizer,
auto_dryer,
auto_miner,
auto_farmer,
auto_transporter,
auto_food_mover,
small_battery,
big_battery,
love_generator,
charcoal_generator,
light2,
light2_placer,
oil_refinery,
oil_refinery_placer,
resource_transporter,
resource_transporter_placer,
cure_desk,
cure_desk_placer,
shop,
shop_placer,
train_center,
train_center_placer,
fridge,
fridge_placer,
super_calculator,
super_calculator_placer,

auto_picker