local Anims = {
    sword1 = {"tp_swords", "tp_swords", "tp_sword_1", "tp_sword_1_water"},
    sword2 = {"tp_swords", "tp_swords", "tp_sword_2", "tp_sword_2_water"},
    sword3 = {"tp_swords", "tp_swords", "tp_sword_3", "tp_sword_3_water"},
    gun1 = {"tp_swords", "tp_swords", "tp_gun_1", "tp_gun_1_water"},
    gun2 = {"tp_swords", "tp_swords", "tp_gun_2", "tp_gun_2_water"},
    gun3 = {"tp_swords", "tp_swords", "tp_gun_3", "tp_gun_3_water"},
    ice_dragon_sword = {"tp_swords", "tp_swords", "tp_ice_dragon_sword", "tp_ice_dragon_sword_water"},
    demon_strike = {"tp_swords", "tp_swords", "tp_demon_strike", "tp_demon_strike_water"},
    half_moon_blade = {"tp_swords", "tp_swords", "tp_half_moon_blade", "tp_half_moon_blade_water"},
    holy_gun = {"tp_swords", "tp_swords", "tp_forest_gun", "tp_forest_gun_water"},
}

local SwordItem = Class(Prefab, function(self, name, data)
    Prefab._ctor(self, name)
    self.fn = function()
        local atlas, img
        if type(data.img) == "table" then
            atlas = "images/inventoryimages/"..data.img[1]..".xml"
            img = data.img[2]
        else
            atlas, img = WARGON.resolve_img_path(data.img)
            img = string.sub(img, 1, -5)
        end
	 	local inst = WARGON.make_prefab(data.anim, data.anim[4], "inv")
	 	inst:add_cmps({
	 		invitem = { atlas=atlas, img=img },
	 		inspect = {},
	 		weapon = {
	 			dmg = data.dmg,
	 			fn = data.weapon_fn,
	 			range = data.range,
	 		},
	 		equip = {
	 			equip = data.equip_fn,
	 			unequip = data.unequip_fn,
	 		},
	 	})
        if data.finite then
            inst:add_cmps({
                finite = {
                   use=data.finite, max=data.finite, 
                   fn=function(inst)
                       inst:Remove()
                   end
               },
            })
        end
        inst:AddTag("tp_item")
        if data.fn then
            data.fn(inst)
        end
        
        return inst
    end
end)

local items = {}

for i = 1, 3 do
    local sword = SwordItem("tp_sword_"..i, {
        anim = Anims["sword"..i],
        img = {"tp_swords", "tp_sword_"..i},
        finite = 75*(i+1),

        weapon_fn = function(inst, owner, target)
            inst.components.tpsword:OnAttack(owner, target)
        end,
        equip_fn = function(inst, owner)
            WARGON.EQUIP.object_on(owner, "tp_sword_"..i, "swap_object")
        end,
        unequip_fn = function(inst, owner)
            WARGON.EQUIP.object_off(owner)
        end,
        fn = function(inst)
            inst:AddComponent("tpsword")
            inst.components.tpsword.level = i
            inst.components.weapon.getdamagefn = function(inst)
                return inst.components.tpsword:CalculateDamage()
            end
            inst:AddComponent("tpinter")
            inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
                if invitem:HasTag("tp_core_blade")
                and invitem.level <= inst.components.tpsword.level
                and inst.components.tpsword:CanSetCore() then
                    return true
                end
                if invitem:HasTag("tp_fix_powder")
                and inst.components.finiteuses:GetPercent() < 1 then
                    return true
                end
            end)
        end,
    })
    table.insert(items, sword)
end

local ice_dragon_sword = SwordItem("tp_ice_dragon_sword", {
    anim = Anims.ice_dragon_sword,
    img = {"tp_swords", "tp_ice_dragon_sword"},

    dmg = 34,
    weapon_fn = function(inst, owner, target)
        WARGON.frozen_prefab(target, nil, 2)
    end,
    equip_fn = function(inst, owner)
        WARGON.EQUIP.object_on(owner, "tp_ice_dragon_sword", "swap_object")
    end,
    unequip_fn = function(inst, owner)
        WARGON.EQUIP.object_off(owner)
    end,
    fn = function(inst)
        inst:AddTag("show_spoilage")
		inst:AddComponent("perishable")
		inst.components.perishable:SetPerishTime(TUNING.PERISH_FAST)
		inst.components.perishable:StartPerishing()
		inst.components.perishable:SetOnPerishFn(function(inst)
			inst:Remove()
		end)
        inst:AddComponent("heater")
		inst.components.heater.iscooler = true
		inst.components.heater.equippedheat = 20
    end,
})
table.insert(items, ice_dragon_sword)

local function demon_strike_killed(inst, data)
    if data.victim and inst.components.tpdemonstrike then
        inst.components.tpdemonstrike:Kill(data.victim)
    end
end

local demon_strike = SwordItem("tp_demon_strike", {
    anim = Anims.demon_strike,
    img = {"tp_swords", "tp_demon_strike"},
    finite = 300,
    dmg = 34,
    equip_fn = function(inst, owner)

        WARGON.EQUIP.object_on(owner, "tp_demon_strike", "swap_object")
        inst:ListenForEvent("killed", demon_strike_killed, owner)
    end,
    unequip_fn = function(inst, owner)

        WARGON.EQUIP.object_off(owner)
        inst:RemoveEventCallback("killed", demon_strike_killed, owner)
    end,
    fn = function(inst)
        inst.components.weapon.getdamagefn = function(inst)
            local dmg = 34
            local owner = inst.components.equippable.owner
            if owner and owner.components.tpdemonstrike then
                dmg = dmg + owner.components.tpdemonstrike:GetDamage()
            end
            return dmg
        end
        inst.fix_value = 10
        inst:AddComponent("tpinter")
        inst.components.tpinter:SetCanFn(function(inst, item, doer)
            return item.prefab == "nightmarefuel"
                and inst.components.finiteuses:GetPercent() < 1
        end)
        inst.components.tpinter:SetFn(function(inst, item, doer)
            if doer.components.sanity then
                doer.components.sanity:DoDelta(-3)
            end
            local item = item.components.stackable:Get()
            item:Remove()
            local dt = doer.components.tplevel.attr.fixup + inst.fix_value
            inst.components.finiteuses:DoDelta(dt)
        end)
    end,
})
table.insert(items, demon_strike)

local half_moon_blade = SwordItem("tp_half_moon_blade", {
    anim = Anims.half_moon_blade,
    img = {"tp_swords", "tp_half_moon_blade"},
    finite = 300,
    dmg = 42,
    weapon_fn = function(inst, owner, target)
        if inst:HasTag("tp_full_moon_blade") then
            WARGON.make_fx2("lightning", target)
        end
    end,
    equip_fn = function(inst, owner)
        WARGON.EQUIP.object_on(owner, "tp_half_moon_blade", "swap_object")
    end,
    unequip_fn = function(inst, owner)
        WARGON.EQUIP.object_off(owner)
    end,
    fn = function(inst)
        inst.update_damage = function(inst)
            if WARGON.is_full_moon() then
                inst.components.weapon:SetDamage(68)
                inst.components.weapon:SetRange(10, 12)
                inst:AddTag("tp_full_moon_blade")
            end
        end

        inst:ListenForEvent("daytime", function()
            inst.components.weapon:SetDamage(42)
            inst.components.weapon:SetRange()
            inst:RemoveTag("tp_full_moon_blade")
        end, GetWorld())
        inst:ListenForEvent("nighttime", function()
            if WARGON.is_full_moon() then
                inst:AddTag("tp_full_moon_blade")
                inst.components.weapon:SetDamage(68)
                inst.components.weapon:SetRange(8, 8)
            end
        end, GetWorld())
        inst:do_task(0, function()
            inst:update_damage()
        end)
    end,
})
table.insert(items, half_moon_blade)

local good_sword = SwordItem("tp_good_sword", {
    anim = {"tp_swords", "tp_swords", "tp_good_sword", "tp_good_sword_water"},
    img = {"tp_swords", "tp_good_sword"},
    finite = 300,
    dmg = 68,
    equip_fn = function(inst, owner)
        WARGON.EQUIP.object_on(owner, "tp_good_sword", "swap_object")
    end,
    unequip_fn = function(inst, owner)
        WARGON.EQUIP.object_off(owner)
    end,
    weapon_fn = function(inst, owner, target)
        if owner.components.tpbuff then
            owner.components.tpbuff:AddBuff("tp_spear_perseus")
        end
    end,
    fn = function(inst)
        
    end,
})
table.insert(items, good_sword)

local hell_fire = SwordItem("tp_hell_fire", {
    anim = {"tp_swords", "tp_swords", "tp_hell_fire", "tp_hell_fire"},
    img = {"tp_swords", "tp_hell_fire"},
    dmg = 68,
    finite = 450,
    equip_fn = function(inst, owner)
        WARGON.EQUIP.object_on(owner, "tp_hell_fire", "swap_object")
    end,
    unequip_fn = function(inst, owner)
        WARGON.EQUIP.object_off(owner)
    end,
    weapon_fn = function(inst, owner, target)

        if math.random() < .5 then
            local angle = owner.Transform:GetRotation()
            local fx = WARGON.make_fx2("laser_line", inst)
            fx.Transform:SetRotation(angle)
            local fx2 = WARGON.make_fx2("hell_fire_ignite_line", inst)
            fx2.Transform:SetRotation(angle)
        end
        if target.components.burnable
        and target.components.burnable:IsBurning() then
            WARGON.get_attacked(owner, target, 68, nil, inst.prefab)
        end
    end,
    fn = function(inst)
        inst:ListenForEvent("tp_owner_killed", function(inst,data)
            local victim = data and data.victim
            if victim then
                if victim.components.lootdropper 
                and victim.components.burnable
                and victim.components.burnable:IsBurning() then




                    victim.components.lootdropper:GiveLoot(data.owner)
                end
            end
        end)
    end,
})
table.insert(items, hell_fire)


for i = 1, 3 do
    local gun = SwordItem("tp_gun_"..i, {
        anim = Anims["gun"..i],
        img = {"tp_swords", "tp_gun_"..i},
        dmg = 10,
        finite = 50*(i+1),
        range = {10, 12},
        equip_fn = function(inst, owner)
            WARGON.EQUIP.object_on(owner, "tp_gun_"..i, "swap_object")
        end,
        unequip_fn = function(inst, owner)
            WARGON.EQUIP.object_off(owner)
        end,
        weapon_fn = function(inst, owner, target)
            inst.components.tpsword:OnAttack(owner, target)
        end,
        fn = function(inst)
            inst:AddTag("speargun")

            inst.components.weapon:SetProjectile("gunpowder_projectile")
            inst.components.weapon.heightoffset = 1
            inst:AddComponent("tpsword")
            inst.components.tpsword.level = i
            inst.components.weapon.getdamagefn = function(inst)
                return inst.components.tpsword:CalculateDamage()
            end
            inst:AddComponent("tpinter")
            inst.components.tpinter:SetCanFn(function(inst, invitem, doer)
                if invitem:HasTag("tp_core_gun")
                and invitem.level <= inst.components.tpsword.level
                and inst.components.tpsword:CanSetCore() then
                    return true
                end
                if invitem:HasTag("tp_gun_battery")
                and inst.components.finiteuses:GetPercent() < 1 then
                    return true
                end
            end)
        end,
    })
    table.insert(items, gun)
end

local holy_gun = SwordItem("tp_holy_gun", {
    anim = Anims.holy_gun,
    img = {"tp_swords", "tp_forest_gun"},
    range = {10, 12},
    dmg = 40,
    finite = 300,
    equip_fn = function(inst, owner)
        WARGON.EQUIP.object_on(owner, "tp_forest_gun", "swap_object")
    end,
    unequip_fn = function(inst, owner)
        WARGON.EQUIP.object_off(owner)
    end,
    fn = function(inst)
        inst.components.weapon:SetProjectile("bishop_charge")
        inst.components.weapon.heightoffset = -2.5
        inst.components.weapon:SetElectric()
        inst:AddTag("gun")
        inst:AddTag("hand_gun")
    end,
})
table.insert(items, holy_gun)

return unpack(items)