local Image = require "widgets/image"
local Screen = require "widgets/screen"
local Button = require "widgets/button"
local Text = require "widgets/text"
local UIAnim = require "widgets/uianim"
local Widget = require "widgets/widget"
local Menu = require "widgets/menu"
local ImageButton = require "widgets/imagebutton"

local TpMenu = require "widgets/tp_menu"
local TpBlackboard = require "widgets/tp_blackboard"
local TpIngdSign = require "widgets/tp_ingd_sign"
local TpTestMenu = require "widgets/tp_test_menu"

local TpItem = Class(ImageButton, function(self, atlas, normal)
	ImageButton._ctor(self, atlas, normal)
	self.anim = self:AddChild(UIAnim())
	self.word = self:AddChild(Text(TITLEFONT, 30))
	self.word:SetPosition(0, -50, 0)
end)

function TpItem:OnGainFocus()
	TpItem._base.OnGainFocus(self)
	if self.anim then
		self.anim:SetScale(1.2, 1.2, 1.2)
	end


	if self.gain_focus_fn then
		self.gain_focus_fn()
	end
end

function TpItem:OnLoseFocus()
	TpItem._base.OnLoseFocus(self)
	if self.anim then
		self.anim:SetScale(1, 1, 1)
	end

	if self.lose_focus_fn then
		self.lose_focus_fn()
	end
end

function TpItem:SetAnim(anims)
	if anims then



		self.anim:GetAnimState():SetBank(anims[1])
		self.anim:GetAnimState():SetBuild(anims[2])
		self.anim:GetAnimState():PlayAnimation(anims[3])
		self.anim:Show()





	else
		self.anim:Hide()
	end
end

function TpItem:SetString(str)
	self.word:SetString(str)
end

function TpItem:Test()
	self.word:SetString(1000)
	local strs = {
		"ChildPosY",
		"TextSize",
		"IconPosX",
		"IconSize",
	}
	local test_data = {
		-30, 30, -20, .4,
	}
	local dts = {
		10, 10, 10, .1,
	}
	local function test_fn()
		self.word:SetPosition(0, self.test_menu.data[1], 0)
		self.word:SetSize(self.test_menu.data[2])
		self.icon:SetPosition(self.test_menu.data[3], self.test_menu.data[1], 0)
		self.icon:SetScale(self.test_menu.data[4])
	end
	self.test_menu = self:AddChild(TpTestMenu(strs, test_data, dts, test_fn))
	self.test_menu:AddAnchor(self, 0, 0, 0)
end

local TpPriceSign = Class(TpIngdSign, function(self)
	TpIngdSign._ctor(self)
	for i = 1, 4 do
		self.slots[i]:Hide()
		self.ingds[i]:Hide()
	end
	self.icon = self:AddChild(Image(
		"images/hud.xml", "inventory_bg_single.tex"
	))
	self.icon:AddChild(Image(
		"images/inventoryimages.xml", "oinc.tex"
	))
	self.price = self:AddChild(Text(TITLEFONT, 30))
	self.price:SetPosition(0, -40, 0)
	self.name = self:AddChild(Text(TITLEFONT, 40))
	self.name:SetPosition(0, 75, 0)
end)

function TpPriceSign:SetPrice(price)
	self.price:SetString(price)
end

function TpPriceSign:SetName(name)
	self.name:SetString(name)
end

function TpPriceSign:Test()
	local test_data = { 0, 30, -40, 40, 75 }
	local dts = { 10, 10, 10, 10, 10 }
	local strs = {
		"IconPosY",
		"PriceSize",
		"PricePosY",
		"NameSize",
		"NamePosY",
	}
	local function test_fn()

		self.icon:SetPosition(0, self.test_menu.data[1], 0)
		self.price:SetSize(self.test_menu.data[2])
		self.price:SetPosition(0, self.test_menu.data[3], 0)
		self.name:SetSize(self.test_menu.data[4])
		self.name:SetPosition(0, self.test_menu.data[5], 0)
	end
	self.test_menu = self:AddChild(
		TpTestMenu(strs, test_data, dts, test_fn) 
	)


	self.test_menu:SetScale(.8)
	self.test_menu:AddAnchor(self, 200, 0, 0)
end


local TpInfoBoard = Class(TpBlackboard, function(self)
	TpBlackboard._ctor(self)
	self.back_ground:SetTexture(
		"images/hud.xml", "craftingsubmenu_fullvertical.tex"
	)
	self.back_ground:SetRotation(270)
	self.back_ground:SetScale(.9, 0.7, 0)
	self.page_up:SetPosition(-260, 20, 0)
	self.page_up:SetScale(.6)
	self.page_up:Hide()
	self.page_down:SetPosition(260, 20, 0)
	self.page_down:SetScale(.6)
	self.page_down:Hide()
	self.text:SetPosition(110, -20, 0)
	self.text:SetSize(25)
	self.text:SetRegionSize(450, 450, 0)
	self.label = self:AddChild(Image(
		"images/hud.xml", "craft_slot.tex"
	))
	self.name = self:AddChild(Text(TITLEFONT, 45))
	self.name:SetPosition(30, 50, 0)
	self.label:SetPosition(30, 50, 0)
	self.label:SetScale(1, 1, 0)


end)

function TpInfoBoard:SetName(str)
	local len = string.len(str)/3
	local scale = math.min(2.8, math.max(1, len/2 ) )
	self.label:SetScale(scale, 1, 0)
	self.name:SetString(str)
end

function TpInfoBoard:TestItem()
	local aaa = ""
	for i = 1, 3 do
		for j = 1, 20 do
			aaa = aaa.."个"
		end
		aaa = aaa.."\n"
	end
	self:SetString(aaa)
	self:SetName("个")
	local strs = {
		"BgWidth", "BgHight", "PosX",
		"TextPosX", "TextPosY",
		"FontSize", "RegionSize",
		"NameSize", "NamePosX", "NamePosY",
		"LabelWidth", "LabelHight", 
	}
	local test_data = {
		.9, .7, 230,
		110, -20,
		25, 450,
		45, 30, 50,
		1, 1,
	}
	local dts = {
		.1, .1, 10,
		10, 10,
		5, 10,
		5, 10, 10,
		.1, .1
	}
	local function test_fn()
		local data = self.test_menu.data
		self.back_ground:SetScale(data[1], data[2], 0)
		self:SetPosition(data[3], 0, 0)
		self.text:SetPosition(data[4], data[5], 0)
		self.text:SetSize(data[6])
		self.text:SetRegionSize(450, data[7])
		self.name:SetSize(data[8])
		self.name:SetPosition(data[9], data[10], 0)
		self.label:SetPosition(data[9], data[10], 0)
		self.label:SetScale(data[11], data[12], 0)
	end
	self.test_menu = self:AddChild(TpTestMenu(strs, test_data, dts, test_fn))
	self.test_menu:AddAnchor(self, -350, -250, 0)
end

local TpItemBoard = Class(TpBlackboard, function(self)
	TpBlackboard._ctor(self)
	self.back_ground:SetTexture(
		"images/hud.xml", "craftingsubmenu_fullhorizontal.tex"
	)
	self.back_ground:SetScale(2.2, 1.4, 0)
	self.page_up:SetPosition(-235, 0, 0)
	self.page_down:SetPosition(370, 0, 0)

	self.slots = {}
	self.items = {}
	local count = 1
	local slot_x = 150
	local slot_y = 170
	local slot_pad = 110
	for i = 0, 4  do
		for j = 0, 3 do
			self.slots[count] = self:AddChild(Image(
				"images/hud.xml", "inventory_bg_single.tex"
			))
			self.slots[count]:SetScale(.8)
			self.slots[count]:SetPosition(-slot_x+slot_pad*i, slot_y-slot_pad*j)
			self.slots[count]:Hide()
			self.items[count] = self:AddChild(TpItem(
				"images/inventoryimages.xml", "ash.tex"
			))
			self.items[count]:SetScale(.8)
			self.items[count]:SetPosition(-slot_x+slot_pad*i, slot_y-slot_pad*j)
			self.items[count]:Hide()
			self.items[count].gain_focus_fn = function()
				self.sign:Show()
				self.sign:SetPosition(slot_x+slot_pad*i, slot_y+140-slot_pad*j)
			end
			self.items[count].lose_focus_fn = function()
				self.sign:Hide()
			end
			count = count + 1
		end
	end


	self.sign = self:AddChild(TpInfoBoard())

	self.sign:Hide()

	self.item_word = self:AddChild(Text(TITLEFONT, 30))
	self.item_word:Hide()














end)

function TpItemBoard:Test()
	local strs = {
		"BgWidth",
		"BgHight",
		"LArrowPosX",
		"LArrowPosY",
		"RArrowPosX",
		"RArrowPosY",
		"PosX",
		"PosY",
	}
	local test_data = {
		2.2, 1.4, 
		-235, 0,
		370, 0,
		0, -20,
	}
	local dts = {
		.1, .1,
		10, 10,
		10, 10,
		10, 10,
	}
	local function test_fn()
		self.back_ground:SetScale(self.test_menu.data[1], self.test_menu.data[2], 0)
		self.page_up:SetPosition(self.test_menu.data[3], self.test_menu.data[4], 0)
		self.page_down:SetPosition(self.test_menu.data[5], self.test_menu.data[6], 0)
		self:SetPosition(self.test_menu.data[7], self.test_menu.data[8])
	end
	self.test_menu = self:AddChild(TpTestMenu(strs, test_data, dts, test_fn))
	self.test_menu:AddAnchor(self, 250, -250, 0)
end

function TpItemBoard:TestItem()
	local strs = {
		"SlotPosX",
		"SlotPosY",
		"Padding",
		"SlotSize",
	}
	local test_data = { -150, 170, 110, .8 }
	local dts = { 10, 10, 10, .1 }
	local function test_fn()
		local count = 1
		for i = 0, 4  do
			for j = 0, 3 do
				self.slots[count]:SetPosition(
					self.test_menu.data[1]+self.test_menu.data[3]*i, 
					self.test_menu.data[2]-self.test_menu.data[3]*j
				)
				self.slots[count]:SetScale(self.test_menu.data[4])
				self.items[count]:SetPosition(
					self.test_menu.data[1]+self.test_menu.data[3]*i, 
					self.test_menu.data[2]-self.test_menu.data[3]*j
				)
				self.items[count]:SetScale(self.test_menu.data[4])
				count = count + 1
			end
		end
	end
	self.test_menu = self:AddChild(TpTestMenu(strs, test_data, dts, test_fn))
	self.test_menu:AddAnchor(self, 350, -250, 0)
end

local DATA = WARGON.DATA.level_panel_data
local TpLevelPanel = Class(Screen, function(self)
	SetPause(true)
	self.data = DATA
	Screen._ctor(self, "TpLevelPanel")
	self.root = self:AddChild(Widget("ROOT"))
	self.root:SetVAnchor(ANCHOR_MIDDLE)
	self.root:SetHAnchor(ANCHOR_MIDDLE)
	self.root:SetPosition(0, 0, 0)
	self.root:SetScaleMode(SCALEMODE_PROPORTIONAL)
	self.bg = self.root:AddChild(
		Image("images/fepanels.xml", "panel_saveslots.tex")
	)
	self.bg:SetScale(2, 1, 1)

	self.exit = self.root:AddChild(ImageButton(
		"minimap/minimap_data.xml", "xspot.png"
	))
	self.exit:SetPosition(350, 300)
	self.exit:SetOnClick(function()
		TheFrontEnd:PopScreen(self)
		SetPause(false)
	end)

	self.label = self.root:AddChild(Image(
		"images/ui.xml", "update_banner.tex"
	))
	self.label:SetPosition(0, 300, 0)
	self.title = self.label:AddChild(Text(TITLEFONT, 40))
	self.title:SetString(self.data.title)
	self.title:SetPosition(0, 10, 0)

	self.item_board = self.root:AddChild(TpItemBoard())
	self.item_board:SetPosition(0, -20, 0)

















	self.balance_bg = self.root:AddChild(Image(
		"images/hud.xml", "inventory_bg_single.tex"
	))
	self.balance_bg:SetPosition(-330, 0, 0)
	self.balance_bg:AddChild(Image(
		WARGON.resolve_img_path("tp_epic#")
	))
	self.balance = self.balance_bg:AddChild(Text(TITLEFONT, 40))
	self.balance:SetPosition(0, -40, 0)
	self.level_tip = self.balance_bg:AddChild(Text(TITLEFONT, 30))
	self.level_tip:SetPosition(0, 145, 0)
	self.level_tip:SetString(self.data.tip)

	self.level_bg = self.root:AddChild(Image(
		"images/hud.xml", "inventory_bg_single.tex"
	))
	self.level_bg:SetPosition(-330, -140, 0)
	self.level_sure = self.level_bg:AddChild(ImageButton(
		"images/inventoryimages.xml", "accomplishment_shrine.tex"
	))
	self.level_sure:SetOnClick(function()
		self.data:Sure()
		TheFrontEnd:PopScreen(self)
		SetPause(false)
	end)
	self.level = self.level_bg:AddChild(Text(TITLEFONT, 40))
	self.level:SetPosition(0, -40, 0)
	self.level_tip2 = self.level_bg:AddChild(Text(TITLEFONT, 30))
	self.level_tip2:SetString(self.data.tip2)
	self.level_tip2:Hide()
	self.level_bg.OnGainFocus = function()
		self.level_tip2:Show()
	end
	self.level_bg.OnLoseFocus = function()
		self.level_tip2:Hide()
	end

	self:Init()

end)

function TpLevelPanel:Init()

	self.data:Init()
	self:ItemBoardPageTurn(0)






	self.item_board.page_up:SetOnClick(function()
		self:ItemBoardPageTurn(-1)
	end)
	self.item_board.page_down:SetOnClick(function()
		self:ItemBoardPageTurn(1)
	end)
	self:SetBalance()
	self:SetLevel()
end






function TpLevelPanel:ItemBoardPageTurn(dt)

	self.data:PageTurn(dt)
	self:SetItemBoard()
end




































function TpLevelPanel:SetItemBoard()

	local attr_shelf = self.data:GetItems()
	for k, v in pairs(attr_shelf) do
		local item = self.item_board.items[k]
		local slot = self.item_board.slots[k]
		if slot and item then

			item:SetTextures(WARGON.resolve_img_path(v.img))
			item:SetAnim(v.anim)
			item:SetString(v:GetValue())
			item:SetOnClick(function()
				if TheInput:IsKeyDown(KEY_CTRL) then
					self.data:LevelDelta(v, true)
				else
					self.data:LevelDelta(v)
				end
				item:SetString(v:GetValue())
				self:SetBalance()
				self:SetLevel()
			end)
			item.gain_focus_fn = function()

				local pos = item:GetPosition()



				pos.x = math.min(490, pos.x + 230)
				self.item_board.sign:Show()
				self.item_board.sign:SetPosition(pos)


				self.item_board.sign:SetName(v.name)
				self.item_board.sign:SetString(v:GetString())





			end
			item.lose_focus_fn = function()
				self.item_board.sign:Hide()

			end
			if not item.shown then
				item:Show()
			end
			if not slot.shown then
				slot:Show()
			end
		end
	end
	for i = #attr_shelf+1, self.data.unit do
		local item = self.item_board.items[i]
		local slot = self.item_board.slots[i]
		if item then
			item:Hide()
		end
		if slot then
			slot:Hide()
		end
	end

	local shelf = self.data
	if shelf.cur <= 1 then
		self.item_board.page_up:Disable()
	else
		self.item_board.page_up:Enable()
	end
	if shelf.cur >= shelf.max then
		self.item_board.page_down:Disable()
	else
		self.item_board.page_down:Enable()
	end
end

function TpLevelPanel:SetBalance()
	local balance = self.data:GetEssence()

	self.balance:SetString(balance)	
end

function TpLevelPanel:SetLevel()
	local level = self.data:GetLevel()
	self.level:SetString(level)
end

function TpLevelPanel:Test()
	local strs = {
		"LevelPosY",
		"TipPosY",
		"TipSize",
	}
	local test_data = {
		-140, 145, 40,
	}
	local dts = {
		5, 5, 5,
	}
	local function test_fn()
		local data = self.test_menu.data
		self.level_bg:SetPosition(-330, data[1], 0)
		self.balance_tip:SetPosition(0, data[2], 0)
		self.balance_tip:SetSize(data[3])
	end
	self.test_menu = self.root:AddChild(TpTestMenu(strs, test_data, dts, test_fn))
	self.test_menu:AddAnchor(self, 350, -250, 0)
end

return TpLevelPanel