require("stategraphs/commonstates")

local actionhandlers = 
{
}

local events =
{
    CommonHandlers.OnStep(),
    CommonHandlers.OnLocomote(true,false),

    CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    EventHandler("doattack", function(inst)
        if inst.components.health and not inst.components.health:IsDead()
        and (inst.sg:HasStateTag("hit") or not inst.sg:HasStateTag("busy")) then
            if inst:HasTag("want_throw") then
                inst.sg:GoToState("throw")
            else
                if math.random() < .33 then

                        inst.sg:GoToState("stand_aoe")



                else
                    inst.sg:GoToState("attack")
                end
            end
        end
    end)
}

local states=
{
    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
            inst.components.locomotor:StopMoving()
            RemovePhysicsColliders(inst)            
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("death")
        end,
    },
	
	State{
        name = "idle",
        tags = {"idle", "canrotate"},
        
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:PushAnimation("idle_loop", true)
        end,
    },
    
    State{
        name = "attack",
        tags = {"attack", "busy"},
        
        onenter = function(inst)
            
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")


            inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
        end,
        
        timeline=
        {
            TimeEvent(8*FRAMES, function(inst) 
            	inst.components.combat:DoAttack() 
            end),
            TimeEvent(12*FRAMES, function(inst) 
				inst.sg:RemoveStateTag("busy")
				inst.sg:RemoveStateTag("attack")
			end),
        },
        
        events=
        {
            EventHandler("animqueueover", function(inst)
                inst.sg:GoToState("idle")
                if math.random() < .5 then

                    inst:AddTag("want_throw")
                    inst.components.combat:SetRange(10, 12)
                end
            end ),
        },
    },

    State{
        name = "stand_aoe",
        tags = {"attack", "busy"},
        onenter = function(inst)

            inst.AnimState:PlayAnimation("staff") 



            inst.components.locomotor:Stop()

            inst.stafffx = SpawnPrefab("staffcastfx")            

            local pos = inst:GetPosition()

            inst.stafffx.Transform:SetPosition(pos.x, pos.y, pos.z)

            local colour = {1,1,1}
            inst.stafffx.Transform:SetRotation(inst.Transform:GetRotation())
            inst.stafffx.AnimState:SetMultColour(colour[1], colour[2], colour[3], 1)
        end,
        onexit = function(inst)

            if inst.stafffx then
                inst.stafffx:Remove()
            end
        end,
        timeline = 
        {
            TimeEvent(0*FRAMES, function(inst)
                inst.stafflight = SpawnPrefab("staff_castinglight")

                local pos = inst:GetPosition()

                local colour = {1,1,1}
                inst.stafflight.Transform:SetPosition(pos.x, pos.y, pos.z)
                inst.stafflight.setupfn(inst.stafflight, colour, 1.9, .33)                

            end),
            TimeEvent(20*FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/use_gemstaff") 
                WARGON.other_area_dmg(inst, 4, inst, 75, nil, nil)




            end),
            TimeEvent(40*FRAMES, function(inst) 
                WARGON.other_area_dmg(inst, 4, inst, 75, nil, nil)




                end),

            TimeEvent(60*FRAMES, function(inst) 
                WARGON.other_area_dmg(inst, 4, inst, 75, nil, nil)




                end),            
        },
        events = {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle") 
            end ),
        },
    },

    State{
        name = "throw",
        tags = {"busy", "attack"},
        onenter = function(inst)
            inst.components.locomotor:Stop()

            local target = inst.components.combat.target
            if target then
                WARGON.face_target(inst, target)
            end
            inst.AnimState:PlayAnimation("atk")
            inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
        end,
        timeline = {
            TimeEvent(8*FRAMES, function(inst)
                local target = inst.components.combat.target
                if target then

                    local proj = WARGON.make_spawn(inst, "tp_spear_lance_proj")
                    proj.components.tpproj:Throw(inst, target)
                    WARGON.EQUIP.object_off(inst)

                    inst.components.combat:SetRange(3, 3)
                    inst:RemoveTag("want_throw")
                    inst:AddTag("spear_thrown")
                end
            end)
        },
        events = {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end)
        },
    },
    
    State{
        name = "tp_hua_start",
        tags = {"busy", "evade","no_stun","canrotate"},
        onenter = function(inst)





            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("slide_pre")
            inst:start_slide()
        end,
        events = {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("tp_hua")
            end),
        },
    },

    State{
        name = "tp_hua",
        tags = {"busy", "evade","no_stun", "runing", "moving"},
        onenter =   function(inst)
            inst.AnimState:PushAnimation("slide_loop")
            inst.SoundEmitter:PlaySound("dontstarve_DLC003/characters/wheeler/slide")

            inst.components.locomotor.runspeed = inst:HasTag("tp_hornet_fast") and 30 or 20
            inst.components.locomotor:RunForward()

            inst.components.health:SetInvincible(true)
        end,
        events = {
            EventHandler("animover", function(inst)
                inst.components.health:SetInvincible(false)
                inst.sg:GoToState("tp_hua_pst")
            end),
        },
        onexit = function(inst)


            inst.components.locomotor:Stop()
            
            inst.components.locomotor:SetBufferedAction(nil)
            inst.components.health:SetInvincible(false)
            inst.components.locomotor.runspeed = 6
        end,
    },

    State{
        name = "tp_hua_pst",
        tags = {"evade","no_stun"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("slide_pst")
        end,

        events =
        {
            EventHandler("animover", function(inst)
                inst:stop_slide()
                inst.sg:GoToState("idle")
            end ),
        },
    },

	State{
		name = "run_start",
		tags = {"moving", "running", "canrotate"},
	    
		onenter = function(inst)
            inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_pre")

        end,
        
        timeline = 
        {
            TimeEvent(4*FRAMES, PlayFootstep ),
            TimeEvent(5*FRAMES, PlayFootstep ),
        },

        events=
        {   
            EventHandler("animover", function(inst) 
                if inst.components.combat.target 
                and inst.components.combat:InCooldown() then
                    inst.sg:GoToState("tp_hua_start")
                else
                    inst.sg:GoToState("run") 
                end
            end ),        
		},
	},
    
	State{
		name = "run",
		tags = {"moving", "running", "canrotate"},
	    
		onenter = function(inst) 
			inst.components.locomotor:RunForward()
            inst.AnimState:PlayAnimation("run_loop")

		end,
		
		events=
		{   
			EventHandler("animover", function(inst) 
                inst.sg:GoToState("run") 
            end ),        
		},
	},
        
	State{
		name = "run_stop",
		tags = {"canrotate"},
	    
		onenter = function(inst) 
			inst.Physics:Stop()
            inst.AnimState:PlayAnimation("run_pst")

		end,
	    
		events=
		{   
			EventHandler("animover", function(inst) 
                inst.sg:GoToState("idle") 
            end ),        
		},
	},
	
    State{
        name = "hit",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit") 
            inst.AnimState:PlayAnimation("hit")
            inst.Physics:Stop()            
        end,
        
        events=
        {
            EventHandler("animover", function(inst) 
                inst.sg:GoToState("idle") 
            end ),
        },        
    },    
}
    
return StateGraph("tp_hornet", states, events, "idle", actionhandlers)

