require("stategraphs/commonstates")

local actionhandlers = 
{
}

local events =
{
    CommonHandlers.OnStep(),
    CommonHandlers.OnLocomote(true,false),

    CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    EventHandler("doattack", function(inst)
        if inst.components.health and not inst.components.health:IsDead()
        and (inst.sg:HasStateTag("hit") or not inst.sg:HasStateTag("busy")) then
            if math.random() < .33 then
                inst.sg:GoToState("jump_attack_pre")
            else
                inst.sg:GoToState("attack")
            end
        end
    end)
}

local states=
{
    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
            inst.components.locomotor:StopMoving()
            RemovePhysicsColliders(inst)            
            inst.AnimState:Hide("swap_arm_carry")
            inst.AnimState:PlayAnimation("death")
        end,
    },
	
	State{
        name = "idle",
        tags = {"idle", "canrotate"},
        
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:PushAnimation("idle_loop", true)
        end,
    },
    
    State{
        name = "attack",
        tags = {"attack", "busy"},
        
        onenter = function(inst)
            
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")


            inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_weapon")
        end,
        
        timeline=
        {
            TimeEvent(8*FRAMES, function(inst) 
            	inst.components.combat:DoAttack() 
                local target = inst.components.combat.target
                if target then
                    local soul = inst:wg_find(100, nil, {"ghost"})
                    if soul and soul:is_alive() then
                        local proj = WARGON.make_spawn(soul, "tp_soul_charge")
                        proj.components.tpproj:Throw(inst, target)
                        soul.components.health:Kill()
                    end
                end
            end),
            TimeEvent(12*FRAMES, function(inst) 
				inst.sg:RemoveStateTag("busy")
				inst.sg:RemoveStateTag("attack")
			end),
        },
        
        events=
        {
            EventHandler("animqueueover", function(inst)











                    inst.sg:GoToState("idle")

            end ),
        },
    },

    State{
        name = "jump_attack_pre",
        tags = {"attack", "busy"},
        onenter = function(inst)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("jumpboat")
            inst.sg.statemem.startpos = inst:GetPosition()
            inst.sg.statemem.targetpos = inst:GetPosition()
            local target = inst.components.combat.target
            if target then
                inst.sg.statemem.targetpos = target:GetPosition()
            end
        end,
        onexit = function(inst)
            inst.components.locomotor:Stop()
        end,
        timeline = {
             TimeEvent(7*FRAMES, function(inst)
                inst:ForceFacePoint(inst.sg.statemem.targetpos:Get())
                local dist = inst:GetPosition():Dist(inst.sg.statemem.targetpos)
                local speed = dist / (18/30)
                inst.Physics:SetMotorVelOverride(1 * speed, 0, 0)
            end),
        },
        events =
        {
            EventHandler("animover", function(inst)

                inst.Physics:Stop()
                inst.components.locomotor:Stop()
                inst.sg:GoToState("jump_attack_pst")
            end),
        },
    },

    State{
        name = "jump_attack_pst",
        tags = {"attack", "busy"},
        onenter = function(inst)
            inst.Physics:Stop()
            inst.AnimState:PushAnimation("land", false)

                inst.components.combat:StartAttack()



            local target = inst.components.combat.target
            if target then
                WARGON.make_fx(inst, "statue_transition")
                WARGON.make_fx(inst, "statue_transition_2")
                inst:set_pos(target:GetPosition():Get())
            end
        end,
        timeline = {
            TimeEvent(4*FRAMES, function(inst)

                    WARGON.other_area_dmg(inst, 3, inst, 75, nil, nil)




            end)
        },
        onexit = function(inst)

        end,    
        events = {
            EventHandler("animqueueover", function(inst)
                inst.sg:GoToState("idle")
            end)
        },
    },

    State{
        name = "staff",
        tags = {"doing", "busy",  "canrotate", "spell"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("staff") 
            inst.components.locomotor:Stop()

            inst.stafffx = SpawnPrefab("staffcastfx")            

            local pos = inst:GetPosition()
            inst.stafffx.Transform:SetPosition(pos.x, pos.y, pos.z)

            inst.stafffx.Transform:SetRotation(inst.Transform:GetRotation())
            inst.stafffx.AnimState:SetMultColour(1, 1, 1, 1)
        end,
        onexit = function(inst)
            if inst.stafffx then
                inst.stafffx:Remove()
            end
        end,
        timeline = 
        {
            TimeEvent(13*FRAMES, function(inst)
                inst.SoundEmitter:PlaySound("dontstarve/wilson/use_gemstaff") 
            end),
            TimeEvent(0*FRAMES, function(inst)
                inst.stafflight = SpawnPrefab("staff_castinglight")
                local pos = inst:GetPosition()
                local colour = {1,1,1}
                inst.stafflight.Transform:SetPosition(pos.x, pos.y, pos.z)
                inst.stafflight.setupfn(inst.stafflight, colour, 1.9, .33)                

            end),
            TimeEvent(53*FRAMES, function(inst) 
                local ents = WARGON.finds(inst, 20, {"ghost"})
                local pos = nil
                for k, v in pairs(ents) do
                    if v and v:IsValid() then
                        pos = v:GetPosition()
                        v:Remove()
                    end
                end
                WARGON.make_fx(pos, "tp_fx_ground_pound")
            end),
        },

        events = {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle") 
            end ),
        },
    },
    
	State{
		name = "run_start",
		tags = {"moving", "running", "canrotate"},
	    
		onenter = function(inst) 
			inst.components.locomotor:RunForward()
			inst.AnimState:PlayAnimation("run_pre")
		end,
	    
		timeline = 
		{
		    TimeEvent(4*FRAMES, PlayFootstep ),
		    TimeEvent(5*FRAMES, PlayFootstep ),
		},

		events=
		{   
			EventHandler("animover", function(inst) 
                inst.sg:GoToState("run") 
            end ),        
		},
	},
    
	State{
		name = "run",
		tags = {"moving", "running", "canrotate"},
	    
		onenter = function(inst) 
			inst.components.locomotor:RunForward()
			inst.AnimState:PlayAnimation("run_loop")
		end,
		
		events=
		{   
			EventHandler("animover", function(inst) 
                inst.sg:GoToState("run") 
            end ),        
		},
	},
        
	State{
		name = "run_stop",
		tags = {"canrotate"},
	    
		onenter = function(inst) 
			inst.Physics:Stop()
			inst.AnimState:PlayAnimation("run_pst")
		end,
	    
		events=
		{   
			EventHandler("animover", function(inst) 
                inst.sg:GoToState("idle") 
            end ),        
		},
	},
	
    State{
        name = "hit",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.SoundEmitter:PlaySound("dontstarve/wilson/hit") 
            inst.AnimState:PlayAnimation("hit")
            inst.Physics:Stop()            
        end,
        
        events=
        {
            EventHandler("animover", function(inst) 
                inst.sg:GoToState("idle") 
            end ),
        },        
    },    
}
    
return StateGraph("tp_soul_student", states, events, "idle", actionhandlers)

