local Widget = require "widgets/widget"
local Text = require "widgets/text"
local Button = require "widgets/button"
local Image = require "widgets/image"
local ImageButton = require "widgets/imagebutton"
local UIAnim = require "widgets/uianim"


local TpBuffSlot = Class(Widget, function(self, data)
    Widget._ctor(self, "TpBuffSlot")

    self.name = data.name
    self.max_time = data.time
    self.time = data.time
    self.spoilage = self:AddChild(UIAnim())
    self.spoilage:GetAnimState():SetBank("spoiled_meter")
    self.spoilage:GetAnimState():SetBuild("spoiled_meter")
    self.spoilage:GetAnimState():SetPercent("anim", 0)
    if data.debuff then
        self.spoilage:GetAnimState():SetMultColour(1, .1, .1, 1)
    else
        self.spoilage:GetAnimState():SetMultColour(1, 1, 1, 1)
    end
    self.spoilage:SetClickable(false)
    self.image = self:AddChild(Image(
        WARGON.resolve_img_path(data.img)

    ))
    self:SetScale(.5)
    self.next = nil
    self.stack = self:AddChild(Text(NUMBERFONT, 42))
    self.stack:SetPosition(0, -16, 0)
    self:SetStack(data.stack)
    self.word = self:AddChild(Text(TITLEFONT, 60))
    self.word:SetPosition(0, 60, 0)
    self.word:SetString(data:GetString() or "")
    self.word:Hide()
end)

function TpBuffSlot:Start()
    self.task = GetPlayer():per_task(1, function()
        self.time = math.max( self.time-1, 0 )
        local p = self.time/self.max_time
        self:SetPercent(p)
        if self.time <= 0 then
            if self.task then
                self.task:Cancel()
                self.task = nil
            end
        end
    end)
end

function TpBuffSlot:SetPercent(p)

    self.spoilage:GetAnimState():SetPercent("anim", math.min(1-p, .99))   
end

function TpBuffSlot:SetTime(time)
    self.time = time
    local p = self.time/self.max_time
    self:SetPercent(p)
end

function TpBuffSlot:SetStack(stack)
    if stack then
        self.stack:SetString(stack)
    end
end

function TpBuffSlot:OnGainFocus()
    self.word:Show()
end

function TpBuffSlot:OnLoseFocus()
    self.word:Hide()
end

return TpBuffSlot